import {Component, OnInit} from '@angular/core';
import {Router} from '@angular/router';
import {Select, Store} from '@ngxs/store';
import {TranslateService} from '@ngx-translate/core';
import {Observable} from 'rxjs';
import * as moment from 'moment';
import * as _ from 'lodash-es';
import ChartDataLabels from 'chartjs-plugin-datalabels';
import {Chart} from 'chart.js';
import {FilterMatchMode, FilterService, PrimeNGConfig} from 'primeng/api';

import {GlobalSelectors, WebsocketsActions} from 'data-access/store';
import {UserSettingsService} from '@core-project/services/user-settings.service';

import {AuthenticationService} from './services/authentication.service';
import {SidemenuService} from './services/sidemenu.service';
import {environment} from '../environments/environment';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss'],
})
export class AppComponent implements OnInit {
  @Select(GlobalSelectors.hasSidebarMenuItems)
  hasSidebarMenuItems$: Observable<boolean>;

  layout = 'static';

  constructor(
    private readonly authenticationService: AuthenticationService,
    private readonly sidemenuService: SidemenuService,
    private readonly router: Router,
    private readonly translateService: TranslateService,
    private readonly primeNGConfig: PrimeNGConfig,
    private readonly filterService: FilterService,
    private readonly userSettingsService: UserSettingsService,
    private readonly store: Store
  ) {}

  ngOnInit(): void {
    this.primeNGConfig.ripple = true;

    this.filterService.register('anyOfId', (value: any[], filter: any[]): boolean => {
      if (filter === undefined || filter === null || filter.length === 0) {
        return true;
      }
      if (value === undefined || value === null || value.length === 0) {
        return false;
      }

      const valueIds = _.flatMap(value, 'id');
      return valueIds.some((v) => filter.includes(v));
    });

    this.filterService.register('anyOf', (value: any[], filter: any[]): boolean => {
      if (filter === undefined || filter === null || filter.length === 0) {
        return true;
      }
      if (value === undefined || value === null || value.length === 0) {
        return false;
      }

      return value.some((v) => filter.includes(v));
    });

    this.filterService.register('stringDateBefore', (value: string, filter: string): boolean => {
      if (filter === undefined || filter === null || value === '') {
        return true;
      }

      if (value === undefined || value === null || value === '') {
        return false;
      }

      return moment.utc(value, 'YYYY-MM-DD').isSameOrBefore(moment.utc(filter, 'YYYY-MM-DD'), 'day');
    });

    this.filterService.register('stringDateAfter', (value: string, filter: string): boolean => {
      if (filter === undefined || filter === null || value === '') {
        return true;
      }

      if (value === undefined || value === null || value === '') {
        return false;
      }

      return moment.utc(value, 'YYYY-MM-DD').isSameOrAfter(moment.utc(filter, 'YYYY-MM-DD'), 'day');
    });

    this.primeNGConfig.filterMatchModeOptions = {
      text: [FilterMatchMode.CONTAINS],
      numeric: [FilterMatchMode.GREATER_THAN_OR_EQUAL_TO, FilterMatchMode.LESS_THAN_OR_EQUAL_TO],
      date: ['stringDateBefore', 'stringDateAfter'],
    };

    this.authenticationService.user$.subscribe((user) => {
      if (user) {
        this.sidemenuService.setDefaultItems();
        this.store.dispatch(new WebsocketsActions.Connect(environment.websocketURL));
      } else {
        this.store.dispatch(new WebsocketsActions.Disconnect());
      }
    });
    this.translateService.stream('PRIMENG').subscribe((data) => this.primeNGConfig.setTranslation(data));
    moment.locale('ru');

    this.userSettingsService.layout.subscribe((newLayout) => (this.layout = newLayout));
    Chart.register(ChartDataLabels);
  }

  logout() {
    this.authenticationService.logout();
    this.router.navigate(['/login']);
  }
}
