import { HttpEvent, HttpHandler, HttpInterceptor, HttpRequest } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { MessageService } from 'primeng/api';
import { Observable, throwError } from 'rxjs';
import { catchError } from 'rxjs/operators';
import { AuthenticationService } from '@core-project/services/authentication.service';
import { TranslateService } from '@ngx-translate/core';

@Injectable()
export class ErrorInterceptor implements HttpInterceptor {
  constructor(
    private router: Router,
    private authenticationService: AuthenticationService,
    private messageService: MessageService,
    private translateService: TranslateService
  ) {}

  intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
    if (this.translateService.currentLang) {
      request = request.clone({
        setHeaders: {
          'Accept-Language': this.translateService.currentLang,
        },
      });
    }

    if (this.authenticationService.token) {
      request = request.clone({
        setHeaders: {
          Authorization: 'Token ' + this.authenticationService.token,
        },
      });
    }

    return next.handle(request).pipe(
      catchError((err) => {
        console.error('INTERCEPTOR', err);
        this.messageService.clear();

        if (err.status === 401) {
          // auto logout if 401 response returned from api
          this.authenticationService.logout();
          this.router.navigate(['/login'], { queryParams: { reason: '401' } });
        } else if (err.status === 402) {
          this.messageService.add({
            severity: 'error',
            summary: this.translateService.instant('ERRORS.402_ERROR'),
            detail: this.translateService.instant('ERRORS.402_ERROR_MSG'),
          });
        } else if (err.status === 403) {
          this.messageService.add({
            severity: 'error',
            summary: this.translateService.instant('ERRORS.403_ERROR'),
            detail: this.translateService.instant('ERRORS.403_ERROR_MSG'),
          });
        } else if (err.status === 404) {
          this.messageService.add({
            severity: 'error',
            summary: this.translateService.instant('ERRORS.404_ERROR'),
            detail: err.url,
          });
        } else if (err.status === 500) {
          this.messageService.add({
            severity: 'error',
            summary: this.translateService.instant('ERRORS.500_ERROR'),
            detail: err.url,
          });
        } else {
          for (const e in err.error) {
            if (err.error[e]) {
              if (typeof err.error[e] === 'string') {
                this.messageService.add({
                  severity: 'error',
                  summary: this.translateService.instant('ERRORS.ERROR'),
                  detail: err.error[e],
                });
              } else {
                for (const msg of err.error[e]) {
                  this.messageService.add({
                    severity: 'error',
                    summary: this.translateService.instant('ERRORS.ERROR'),
                    detail: msg,
                  });
                }
              }
            }
          }
        }

        return throwError(err);
      })
    );
  }
}
