import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { AuthenticationService } from '@core-project/services/authentication.service';
import { MessageService } from 'primeng/api';
import { SidemenuService } from '@core-project/services/sidemenu.service';
import { TranslateService } from '@ngx-translate/core';
import { AuthService } from 'data-access/sdk/api2';

@Component({
  selector: 'app-login',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.scss'],
})
export class LoginComponent implements OnInit {
  loginForm: FormGroup;
  loading = false;
  returnUrl: string;
  error = '';

  constructor(
    private formBuilder: FormBuilder,
    private route: ActivatedRoute,
    private router: Router,
    private authService: AuthService,
    private authenticationService: AuthenticationService,
    private messageService: MessageService,
    private sidemenuService: SidemenuService,
    private translateService: TranslateService
  ) {}

  // convenience getter for easy access to form fields
  get f() {
    return this.loginForm.controls;
  }

  ngOnInit() {
    this.loginForm = this.formBuilder.group({
      email: ['', Validators.compose([Validators.required, Validators.email])],
      password: ['', Validators.required],
      remember: [(localStorage.getItem('rememberMe') || true) === 'true'],
    });

    this.authenticationService.logout();

    this.returnUrl = this.route.snapshot.queryParams['returnUrl'] || '/';
  }

  onSubmit() {
    // stop here if form is invalid
    if (this.loginForm.invalid) {
      return;
    }

    this.loading = true;
    this.authenticationService.login(this.f.email.value, this.f.password.value, this.f.remember.value).subscribe({
      next: (_data) => {
        localStorage.setItem('rememberMe', this.f.remember.value);
        this.messageService.clear();
        this.sidemenuService.setDefaultItems();
        this.router.navigate([this.returnUrl]);
      },
      error: (error) => {
        this.error = error;
        this.loading = false;
        this.messageService.clear();
        this.messageService.add({
          severity: 'error',
          summary: this.translateService.instant('LOGIN.AUTH_ERROR_HEADER'),
          detail: this.translateService.instant('LOGIN.AUTH_ERROR_MSG'),
        });
      },
    });
  }

  resetPassword() {
    if (!this.f.email.value || this.f.email.invalid) {
      this.messageService.add({
        severity: 'warn',
        summary: this.translateService.instant('ERRORS.ERROR'),
        detail: this.translateService.instant('LOGIN.ENTER_EMAIL_ERROR'),
      });
      return;
    }

    this.authService.authUsersResetPassword({ email: this.f.email.value }).subscribe({
      next: (_r) =>
        this.messageService.add({
          severity: 'info',
          summary: this.translateService.instant('TOPBAR.USER.RESET_PASSWORD_HEADER'),
          detail: this.translateService.instant('TOPBAR.USER.RESET_PASSWORD_MSG'),
        }),
      error: (_e) =>
        this.messageService.add({
          severity: 'error',
          summary: this.translateService.instant('ERRORS.ERROR'),
          detail: this.translateService.instant('LOGIN.WRONG_EMAIL_ERROR'),
        }),
    });
  }
}
