import { Component, OnDestroy, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { AuthService } from 'data-access/sdk/api2';
import { ActivatedRoute, Router } from '@angular/router';
import { AuthenticationService } from '@core-project/services/authentication.service';
import { MessageService } from 'primeng/api';
import { TranslateService } from '@ngx-translate/core';

@Component({
  selector: 'app-reset-password',
  templateUrl: './reset-password.component.html',
  styleUrls: ['./reset-password.component.scss'],
})
export class ResetPasswordComponent implements OnInit {
  form: FormGroup;
  uid: string;
  token: string;
  loading = false;

  constructor(
    private formBuilder: FormBuilder,
    private authService: AuthService,
    private route: ActivatedRoute,
    private authenticationService: AuthenticationService,
    private messageService: MessageService,
    private translateService: TranslateService,
    private router: Router
  ) {}

  static checkPasswordsValidator(group: FormGroup) {
    const pass = group.controls.new_password.value;
    const confirmPass = group.controls.confirmPass.value;
    return pass === confirmPass ? null : { noMatch: true };
  }

  ngOnInit(): void {
    if (this.authenticationService.user) {
      this.authenticationService.logout();
    }
    this.uid = this.route.snapshot.queryParamMap.get('uid');
    this.token = this.route.snapshot.queryParamMap.get('token');
    this.initForm();
  }

  initForm() {
    this.form = this.formBuilder.group(
      {
        uid: [this.uid, Validators.required],
        token: [this.token, Validators.required],
        new_password: ['', [Validators.required, Validators.minLength(8)]],
        confirmPass: [''],
      },
      { validators: ResetPasswordComponent.checkPasswordsValidator }
    );
  }

  async save() {
    if (this.form.invalid) {
      return;
    }

    this.loading = true;
    this.authService.authUsersResetPasswordConfirmWithLogin(this.form.value).subscribe({
      next: (token) => {
        this.authenticationService.loginByToken(token.auth_token).subscribe((_r) => {
          this.messageService.add({
            severity: 'success',
            summary: this.translateService.instant('LOGIN.PASSWORD_RESET.SUCC_RESET_HEADER'),
            detail: this.translateService.instant('LOGIN.PASSWORD_RESET.SUCC_RESET_MSG'),
          });
          this.router.navigate(['/']);
        });
      },
      error: (_r) => {
        this.loading = false;
      },
    });
  }
}
