import { Component, OnInit, ViewChild, HostListener } from '@angular/core';
import { MenuItem } from 'primeng/api';
import { ScrollPanel } from 'primeng/scrollpanel';
import { SidemenuService } from '@core-project/services/sidemenu.service';
import { UserSettingsService } from '@core-project/services/user-settings.service';
import { UserSettingsEnums } from '@core-project/enums';
import { NavigationEnd, Router } from '@angular/router';
import { filter, takeUntil } from 'rxjs/operators';
import { Unsubscriber } from '@core-project/services/unsubscriber.service';
import { BehaviorSubject, debounceTime, distinctUntilChanged, Observable, of } from 'rxjs';
import { map, switchMap } from 'rxjs/operators';

@Component({
  selector: 'app-sidemenu',
  templateUrl: './sidemenu.component.html',
  styleUrls: ['./sidemenu.component.scss'],
  providers: [Unsubscriber],
})
export class SidemenuComponent implements OnInit {
  private readonly sidebarActiveSubject$ = new BehaviorSubject<boolean>(false);

  @ViewChild('scrollPanel', { static: false }) layoutMenuScrollerViewChild: ScrollPanel;

  items: MenuItem[] = [];
  sidebarActive = false;
  menuClick = false;
  mobileMenuActive = false;
  menuHoverActive = false;
  resetMenu = false;

  readonly sidebarHover$ = this.sidebarActiveSubject$.asObservable().pipe(distinctUntilChanged(), debounceTime(100));
  readonly layoutStatic$: Observable<boolean> = this.userSettingsService.layout.pipe(
    map((val) => val === UserSettingsEnums.LayoutEnum.static)
  );
  readonly sidebarActive$: Observable<boolean> = this.layoutStatic$.pipe(switchMap((val) => (val ? of(false) : this.sidebarHover$)));

  constructor(
    readonly sidemenuService: SidemenuService,
    private readonly userSettingsService: UserSettingsService,
    private readonly router: Router,
    private readonly unsubscriber: Unsubscriber
  ) {}

  get darkMenu(): boolean {
    return this.userSettingsService.theme.value === UserSettingsEnums.ThemeEnum.dark;
  }

  @HostListener('mouseover', ['$event', '$event.target'])
  onMouseOver(_event: MouseEvent, target: HTMLElement): void {
    if (target.closest('.sidebar-anchor-icon')) {
      return;
    }
    this.sidebarActiveSubject$.next(true);
  }

  @HostListener('mouseleave')
  onMouseLeave(): void {
    this.sidebarActiveSubject$.next(false);
  }

  ngOnInit() {
    this.router.events
      .pipe(
        filter((event) => event instanceof NavigationEnd),
        takeUntil(this.unsubscriber.destroy$)
      )
      .subscribe((event: NavigationEnd) => {
        const firstURLSegment = event.url.split('/')[1];
        this.sidemenuService.setActiveItemByFirstSegment(firstURLSegment);
      });
  }

  onSidebarClick() {
    this.menuClick = true;
    this.resetMenu = false;
  }

  onToggleMenuClick() {
    const newLayout =
      this.userSettingsService.layout.value === UserSettingsEnums.LayoutEnum.static
        ? UserSettingsEnums.LayoutEnum.overlay
        : UserSettingsEnums.LayoutEnum.static;
    this.userSettingsService.setLayout(newLayout);
  }

  isOverlay() {
    return this.userSettingsService.layout.value === UserSettingsEnums.LayoutEnum.overlay;
  }

  isMobile() {
    return window.innerWidth <= 1024;
  }

  isTablet() {
    const width = window.innerWidth;
    return width <= 1024 && width > 640;
  }

  isHorizontal() {
    return this.userSettingsService.layout.value === UserSettingsEnums.LayoutEnum.horizontal;
  }
}
