import {Component, HostListener, OnInit} from '@angular/core';
import {AuthenticationService} from '@core-project/services/authentication.service';
import {AuthService, SearchResult, SearchService, User} from 'data-access/sdk/api2';
import {Router} from '@angular/router';
import {Observable} from 'rxjs';
import {TranslateService} from '@ngx-translate/core';
import {MenuItem, MessageService} from 'primeng/api';
import {UserSettingsService} from '@core-project/services/user-settings.service';
import {UserSettingsEnums} from '@core-project/enums';
import {GlobalSelectors, IAppTitle, NotificationsSelectors} from 'data-access/store';
import {Select, Store} from '@ngxs/store';
import {map} from 'rxjs/operators';

@Component({
  selector: 'app-topbar',
  templateUrl: './app.topbar.component.html',
  styleUrls: ['./app-topbar.component.scss'],
})
export class AppTopBarComponent implements OnInit {
  @Select(GlobalSelectors.appTitle)
  title$: Observable<IAppTitle | undefined>;

  @Select(GlobalSelectors.topMenuItems)
  menuItems$: Observable<MenuItem[]>;

  unreadNotificationsNumber$: Observable<string | undefined> = this.store.select(NotificationsSelectors.unreadNotificationsNumber)
    .pipe(
      map((number) => number && number > 0 ? number.toString() : undefined),
    );

  topbarMenuActive = false;
  activeTopbarItem: Element | null;
  user: User;
  searchText: string;
  searchResults$: Observable<Array<SearchResult>>;
  themeEnum = UserSettingsEnums.ThemeEnum;

  constructor(
    readonly userSettingsService: UserSettingsService,
    private readonly authenticationService: AuthenticationService,
    private readonly router: Router,
    private readonly searchService: SearchService,
    private readonly translateService: TranslateService,
    private readonly authService: AuthService,
    private readonly messageService: MessageService,
    private readonly store: Store,
  ) {
  }

  get isImpersonated(): boolean {
    return this.authenticationService.isImpersonated;
  }

  get isMobile() {
    return window.innerWidth <= 1024;
  }

  get isHorizontal() {
    return this.userSettingsService.layout.value === 'horizontal';
  }

  @HostListener('document:click', ['$event', '$event.target'])
  onClick(event: MouseEvent, target: HTMLElement): void {
    if (target.closest('.active-topmenuitem')) {
      return;
    }
    this.activeTopbarItem = null;
  }

  ngOnInit() {
    this.authenticationService.user$
      .subscribe((user) => {
        this.user = user;
      });
  }

  onTopbarRootItemClick($event: Event, item: Element, link?: string) {
    if (this.activeTopbarItem === item) {
      this.activeTopbarItem = null;
    } else {
      this.activeTopbarItem = item;
    }
    if (link) {
      this.router.navigate([link]);
    }
    $event.preventDefault();
  }

  search() {
    if (this.searchText.length > 2) {
      this.searchResults$ = this.searchService.searchList(this.searchText);
    }
  }

  searchSelect(event) {
    this.searchText = null;
    switch (event.object_type) {
      case 'project':
        this.router.navigate(['/project', event.object_id.toString(), 'details']);
        break;
    }
  }

  resetPassword(event: Event) {
    this.activeTopbarItem = null;
    this.authService.authUsersResetPassword({email: this.authenticationService.user.email}).subscribe((_r) =>
      this.messageService.add({
        severity: 'info',
        summary: this.translateService.instant('TOPBAR.USER.RESET_PASSWORD_HEADER'),
        detail: this.translateService.instant('TOPBAR.USER.RESET_PASSWORD_MSG'),
      }),
    );
    event.preventDefault();
  }

  toggleTheme(event: Event) {
    const theme =
      this.userSettingsService.theme.value === UserSettingsEnums.ThemeEnum.dark
        ? UserSettingsEnums.ThemeEnum.light
        : UserSettingsEnums.ThemeEnum.dark;
    this.userSettingsService.setTheme(theme);
    this.activeTopbarItem = null;
    event.preventDefault();
  }

  stopImpersonation(event: Event) {
    event.stopPropagation();
    this.authenticationService.stopImpersonation();
    this.router.navigate(['/profile']);
  }
}
