import {Component, OnInit, ViewEncapsulation} from '@angular/core';
import {Select, Store} from '@ngxs/store';
import {BehaviorSubject, Observable} from 'rxjs';
import {IsActiveMatchOptions, Router} from '@angular/router';

import {Dashboard, DashboardsService, ReportPlugin, ReportPluginsService} from 'data-access/sdk/api2';
import {CustomDashboardsActionsShared, CustomDashboardsSelectorsShared} from 'features/custom-dashboards';
import {ReportDownloaderService} from '@core-project/services/report-downloader.service';
import {ReportConfigComponent} from '@core-project/modules/shared-components/report-config/report-config.component';
import {RightSidebarActions} from 'data-access/store';
import {TranslateService} from '@ngx-translate/core';
import {MessageService} from 'primeng/api';
import {ReportPluginEnums} from '@core-project/enums/report-plugins';
import {DashboardPluginModel} from 'data-access/sdk/internal-api';


interface BuiltinDashboard {
  color: string;
  icon: string;
  name: string;
  url?: string;
  inactive: boolean;
}

@Component({
  selector: 'app-dashboard-list',
  templateUrl: './dashboard-list.component.html',
  styleUrls: ['./dashboard-list.component.scss'],
  encapsulation: ViewEncapsulation.None,
})
export class DashboardListComponent implements OnInit {
  @Select(CustomDashboardsSelectorsShared.plugins) plugins$!: Observable<DashboardPluginModel[]>;

  readonly dashboardsSubject: BehaviorSubject<Dashboard[]> = new BehaviorSubject([]);
  readonly dashboards$: Observable<Dashboard[]> = this.dashboardsSubject.asObservable();

  readonly reportsSubject: BehaviorSubject<ReportPlugin[]> = new BehaviorSubject([]);
  readonly reports$: Observable<ReportPlugin[]> = this.reportsSubject.asObservable();

  builtinDashboards: BuiltinDashboard[];

  reportColorMap = {
    [ReportPluginEnums.FormatEnum.PPTX]: 'orange',
    [ReportPluginEnums.FormatEnum.PDF]: 'red',
    [ReportPluginEnums.FormatEnum.DOCX]: 'blue',
    [ReportPluginEnums.FormatEnum.XSLX]: 'green',
    [ReportPluginEnums.FormatEnum.PNG]: 'amber',
  };

  reportIconMap = {
    [ReportPluginEnums.FormatEnum.PPTX]: 'fa-light fa-file-ppt',
    [ReportPluginEnums.FormatEnum.PDF]: 'fa-light fa-file-pdf',
    [ReportPluginEnums.FormatEnum.DOCX]: 'fa-light fa-file-word',
    [ReportPluginEnums.FormatEnum.XSLX]: 'fa-light fa-file-excel',
    [ReportPluginEnums.FormatEnum.PNG]: 'fa-light fa-file-png',
  };

  constructor(
    private readonly dashboardsService: DashboardsService,
    private readonly reportPluginsService: ReportPluginsService,
    private readonly router: Router,
    private readonly reportDownloaderService: ReportDownloaderService,
    private readonly store: Store,
    private readonly ts: TranslateService,
    private readonly messageService: MessageService,
  ) {
  }

  /**
   * Небольшой хак для ленивой загрузки кастомных плагинов
   */
  get isDashboardListActive() {
    const matchOptions: IsActiveMatchOptions = {
      paths: 'exact',
      queryParams: 'exact',
      fragment: 'exact',
      matrixParams: 'ignored',
    };
    return this.router.isActive('dashboards', matchOptions);
  }

  ngOnInit() {
    this.getReports();
    this.getDashboards();

    this.store.dispatch(new CustomDashboardsActionsShared.ReadList())
      .subscribe();

    this.builtinDashboards = [
      {name: this.ts.instant('DASHBOARD.BUILTIN.STATUS_REPORT'), icon: 'ci-heartbeat', color: '#47C654', url: 'status', inactive: false},
      {name: this.ts.instant('DASHBOARD.BUILTIN.BUDGET'), icon: 'ci-moneybag', color: '#DA6CFF', url: 'budget', inactive: false},
      {name: this.ts.instant('DASHBOARD.BUILTIN.EFFECTS'), icon: 'ci-checklist', color: '#FFA551', inactive: true},
      {name: this.ts.instant('DASHBOARD.BUILTIN.USER_LOAD'), icon: 'ci-person-load', color: '#1E88E5', inactive: true},
      {name: this.ts.instant('DASHBOARD.BUILTIN.PG_DYNAMICS'), icon: 'ci-project-group-arrow', color: '#0DC2DB', inactive: true},
      {name: this.ts.instant('DASHBOARD.BUILTIN.STAGE_LENGTH'), icon: 'ci-calendar-v2', color: '#F1D900', inactive: true},
      {name: this.ts.instant('DASHBOARD.BUILTIN.FIRE_REPORT'), icon: 'ci-fire-search', color: '#FF6465', inactive: true},
    ];
  }

  openDashboard(id: number) {
    this.router.navigate(['dashboard', id.toString()]);
  }

  openBuiltinDashboard(btDashboard: BuiltinDashboard) {
    if (btDashboard.inactive) {
      this.messageService.add({
        severity: 'info',
        summary: this.ts.instant('DASHBOARD.BUILTIN.UNDER_CONSTRUCTION_HEADER'),
        detail: this.ts.instant('DASHBOARD.BUILTIN.UNDER_CONSTRUCTION_MSG'),
      });
    } else {
      this.router.navigate(['bt-dashboard', btDashboard.url]);
    }

  }

  getReport(plugin: ReportPlugin) {
    if (plugin.options.length === 0) {
      this.reportDownloaderService.getReport(plugin);
    } else {
      this.store.dispatch(new RightSidebarActions.Open({getComponentFn: () => ReportConfigComponent, data: {reportPlugin: plugin}}));
    }
  }

  private getDashboards() {
    this.dashboardsService.dashboardsDashboardsList()
      .subscribe((dashboards) => {
        this.dashboardsSubject.next(dashboards);
      });
  }

  private getReports() {
    this.reportPluginsService.reportPluginsList('options')
      .subscribe((reportPlugins) => {
        this.reportsSubject.next(reportPlugins);
      });
  }

}
