import {Component, OnInit, ViewEncapsulation} from '@angular/core';
import {Dashboard, DashboardsService, UserWidget} from 'data-access/sdk/api2';
import {firstValueFrom} from 'rxjs';
import {TranslateService} from '@ngx-translate/core';
import {switchMap} from 'rxjs/operators';
import * as _ from 'lodash-es';
import {DialogService} from 'primeng/dynamicdialog';
import {ConfirmationService, MessageService, SelectItem} from 'primeng/api';
import {UserWidgetEnums} from '@core-project/enums';
import {DashboardFormComponent} from '@core-project/modules/forms/dashboard-form/dashboard-form.component';
import {widgetsParams} from '@core-project/helpers/widget';
import {WidgetParams} from '@core-project/interfaces';
import WidgetEnum = UserWidgetEnums.WidgetEnum;

@Component({
  selector: 'app-dashboard',
  templateUrl: './dashboard.component.html',
  styleUrls: ['./dashboard.component.scss'],
  encapsulation: ViewEncapsulation.None,
})
export class DashboardComponent implements OnInit {
  dashboards: Dashboard[];
  currentDashboard: Dashboard;
  currentDashboardId: number;
  allWidgets: SelectItem[];
  selectedWidget: WidgetEnum;

  constructor(
    private dashboardsService: DashboardsService,
    private translateService: TranslateService,
    private dialogService: DialogService,
    private confirmationService: ConfirmationService,
    private messageService: MessageService,
  ) {
  }

  ngOnInit() {
    this.allWidgets = Object.values(WidgetEnum).map((w) => ({label: this.translateService.instant('WIDGETS.NAMES.' + w), value: w}));

    const lsDashboardId = this.getLastDashboardId();
    firstValueFrom(this.dashboardsService.dashboardsDashboardsList('user_widgets')).then((ds) => {
      if (ds.length === 0) {
        firstValueFrom(
          this.dashboardsService
            .dashboardsDashboardsCreate({name: this.translateService.instant('DASHBOARDS.DEFAULT_DASHBOARD_NAME')})
            .pipe(switchMap((d) => this.dashboardsService.dashboardsDashboardsRead({id: d.id, include: 'user_widgets'}))),
        ).then((d) => {
          this.dashboards = [d];
          this.switchDashboard(d.id);
        });
      } else {
        this.dashboards = ds;
        const d = _.find(ds, ['id', lsDashboardId]);
        if (d) {
          this.switchDashboard(d.id);
        } else {
          this.switchDashboard(ds[0].id);
        }
      }
    });
  }

  switchDashboard(id?: number) {
    if (!id) {
      id = this.currentDashboardId;
    } else {
      this.currentDashboardId = id;
    }
    this.setLastDashboardId(id);
    this.currentDashboard = _.find(this.dashboards, ['id', id]);
  }

  editDashboard(dashboard?: Dashboard) {
    const ref = this.dialogService.open(DashboardFormComponent, {
      header: dashboard ? this.translateService.instant('DASHBOARD.EDIT_HEADER') : this.translateService.instant('DASHBOARD.ADD_HEADER'),
      width: '40%',
      data: {dashboard},
    });

    firstValueFrom(ref.onClose).then((newDashboard) => {
      if (newDashboard) {
        if (dashboard) {
          Object.assign(dashboard, newDashboard);
        } else {
          this.dashboards.push(newDashboard);
          this.dashboards = [...this.dashboards];
          this.switchDashboard(newDashboard.id);
        }
      }
    });
  }

  deleteDashboard(dashboard: Dashboard) {
    if (this.dashboards.length === 1) {
      this.messageService.add({
        severity: 'error',
        summary: this.translateService.instant('ERRORS.ERROR'),
        detail: this.translateService.instant('DASHBOARD.LAST_DASHBOARD_DELETE_ERROR'),
      });
      return;
    }

    this.confirmationService.confirm({
      header: this.translateService.instant('DASHBOARD.DELETE_HEADER'),
      message: this.translateService.instant('DASHBOARD.DELETE_MSG'),
      accept: () => {
        firstValueFrom(this.dashboardsService.dashboardsDashboardsDelete(dashboard.id)).then((_d) => {
          _.remove(this.dashboards, ['id', dashboard.id]);
          this.switchDashboard(this.dashboards[0].id);
        });
      },
    });
  }

  addWidget() {
    const params: WidgetParams = _.find(widgetsParams, ['name', this.selectedWidget]);
    if (!params) {
      console.error('NO PARAMS FOR WIDGET', this.selectedWidget);
      return;
    }
    firstValueFrom(
      this.dashboardsService.dashboardsUserWidgetsCreate({
        widget: this.selectedWidget,
        dashboard: this.currentDashboardId,
        width: params.minWidth,
        height: params.minHeight,
        position_x: 200,
        position_y: 200,
      }),
    ).then((uw) => {
      this.currentDashboard.rel_user_widgets.push(uw);
      this.currentDashboard.rel_user_widgets = [...this.currentDashboard.rel_user_widgets];
    });
  }

  editWidget(userWidget: UserWidget, newUserWidget: UserWidget) {
    firstValueFrom(this.dashboardsService.dashboardsUserWidgetsPartialUpdate({id: userWidget.id, data: newUserWidget})).then((uw) =>
      Object.assign(userWidget, uw),
    );
  }

  removeWidget(userWidget: UserWidget) {
    firstValueFrom(this.dashboardsService.dashboardsUserWidgetsDelete(userWidget.id)).then((_uw) =>
      _.remove(this.currentDashboard.rel_user_widgets, ['id', userWidget.id]),
    );
  }

  private getLastDashboardId(): number {
    const ld = localStorage.getItem('lastDashboard');
    if (ld) {
      return Number(ld);
    }
    return null;
  }

  private setLastDashboardId(id: number) {
    localStorage.setItem('lastDashboard', id.toString());
  }
}
