import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { User } from 'data-access/sdk/api2';
import { firstValueFrom, Observable } from 'rxjs';
import { ProjectGroup } from 'data-access/sdk/api2';
import { CompaniesService } from 'data-access/sdk/api2';
import { AuthenticationService } from '@core-project/services/authentication.service';
import { map } from 'rxjs/operators';
import * as _ from 'lodash-es';
import { TranslateService } from '@ngx-translate/core';
import { dbStringToMoment, momentToDbString } from '@core-project/helpers';
import { SelectItem } from 'primeng/api';
import * as moment from 'moment';
import { ProjectGroupsService } from 'data-access/sdk/api2';
import { ProjectGroupBudgetStats } from 'data-access/sdk/api2';

@Component({
  selector: 'app-project-group-budget',
  templateUrl: './project-group-budget.component.html',
  styleUrls: ['./project-group-budget.component.scss'],
})
export class ProjectGroupBudgetComponent implements OnInit {
  private user: User;
  private serverData: ProjectGroupBudgetStats[];

  @Input() projectGroupId: number;
  @Input() cumulative = true;
  @Input() selectedYear = momentToDbString(moment());
  @Output() onChange: EventEmitter<any> = new EventEmitter<any>();

  projectGroups$: Observable<ProjectGroup[]>;

  chartData: any;
  chartOptions: any;
  loading = true;

  cumulativeOptions: SelectItem[];

  constructor(
    private companiesService: CompaniesService,
    private authenticationService: AuthenticationService,
    private ts: TranslateService,
    private projectGroupsService: ProjectGroupsService
  ) {}

  ngOnInit() {
    this.cumulativeOptions = [
      { value: true, label: this.ts.instant('WIDGETS.BUDGET.WITH_CUMULATIVE') },
      { value: false, label: this.ts.instant('WIDGETS.BUDGET.NO_CUMULATIVE') },
    ];

    this.user = this.authenticationService.user;
    this.projectGroups$ = this.companiesService.companiesProjectGroupTree(this.user.company).pipe(
      map((pgs) =>
        pgs.map((pg) => {
          pg.name = '-'.repeat(pg.level) + ' ' + pg.name;
          return pg;
        })
      )
    );

    if (this.projectGroupId) {
      this.loadData();
    } else {
      this.projectGroups$.subscribe(pgs => {
        if (pgs.length > 0) {
          this.projectGroupId = pgs[0].id;
          this.loadData();
        }
      });
    }
  }

  public changeProjectGroup() {
    if (this.projectGroupId) {
      this.loadData();
      this.changed();
    }
  }

  public changedYear() {
    console.log('YEAR', this.selectedYear);
    this.loadData();
    this.changed();
  }

  public changed() {
    this.onChange.emit({
      projectGroupId: this.projectGroupId,
      cumulative: this.cumulative,
      selectedYear: this.selectedYear,
    });
  }

  private loadData() {
    this.loading = true;
    firstValueFrom(
      this.projectGroupsService.projectGroupsBudgetStats({
        id: this.projectGroupId,
        year: Number(dbStringToMoment(this.selectedYear).format('YYYY')),
      })
    ).then((stats) => {
      this.serverData = stats;
      this.parseData();
      this.loading = false;
    });
  }

  public parseData() {
    this.chartData = {
      labels: _.flatMap(this.serverData, (item) => dbStringToMoment(item.date).format('MMM YYYY')),
      datasets: [
        {
          label: this.ts.instant('WIDGETS.BUDGET.PLAN'),
          data: this.cumulative ? _.flatMap(this.serverData, 'cum_plan') : _.flatMap(this.serverData, 'plan'),
          fill: true,
          stepped: 'middle',
          pointStyle: false,
        },
        {
          label: this.ts.instant('WIDGETS.BUDGET.FACTFC'),
          data: this.cumulative ? _.flatMap(this.serverData, 'cum_fact_fc') : _.flatMap(this.serverData, 'fact_fc'),
          fill: true,
          stepped: 'middle',
          pointStyle: false,
        },
      ],
    };

    this.chartOptions = {
      responsive: true,
      interaction: {
        intersect: false,
        axis: 'x',
      },
      scales: {
        x: {
          offset: true,
          grid: { offset: true },
        },
      },
    };
  }
}
