import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { ProjectGroupsService } from 'data-access/sdk/api2';
import { map, switchMap } from 'rxjs/operators';
import { CompaniesService } from 'data-access/sdk/api2';
import { firstValueFrom, forkJoin, Observable } from 'rxjs';
import { ProjectGroup } from 'data-access/sdk/api2';
import { User } from 'data-access/sdk/api2';
import { AuthenticationService } from '@core-project/services/authentication.service';
import { WorksService } from 'data-access/sdk/api2';
import { ProjectsService } from 'data-access/sdk/api2';
import { WorklinksService } from 'data-access/sdk/api2';
import * as _ from 'lodash-es';
import { WorkEnums } from '@core-project/enums';
import WorkTypeEnum = WorkEnums.WorkTypeEnum;
import {
  GanttWork,
  GanttWorkLink,
  ProjectGroupToGanttAdapter,
  ProjectToGanttAdapter,
  PWorkToGanttAdapter,
  WorkLinkToGanttAdapter,
} from '@core-project/helpers/gantt-adapter';
import GanttViewModeEnum = WorkEnums.GanttViewModeEnum;
import GanttModeEnum = WorkEnums.GanttModeEnum;
import GanttScaleEnum = WorkEnums.GanttScaleEnum;
import { getGanttIdFromProjectGroupId } from '@core-project/helpers/gantt-common';

@Component({
  selector: 'app-project-group-roadmap',
  templateUrl: './project-group-roadmap.component.html',
  styleUrls: ['./project-group-roadmap.component.scss'],
})
export class ProjectGroupRoadmapComponent implements OnInit {
  @Input() projectGroupId: number;
  @Input() scale: GanttScaleEnum;
  @Input() viewMode: GanttViewModeEnum;
  @Output() onChange: EventEmitter<any> = new EventEmitter<any>();
  private user: User;
  public projectGroups$: Observable<ProjectGroup[]>;

  public loading = true;
  public works: GanttWork[];
  public workLinks: GanttWorkLink[];
  public ganttModes = GanttModeEnum;

  constructor(
    private companiesService: CompaniesService,
    private authenticationService: AuthenticationService,
    private worksService: WorksService,
    private workLinksService: WorklinksService,
    private projectGroupsService: ProjectGroupsService,
    private projectsService: ProjectsService
  ) {}

  ngOnInit() {
    if (!this.viewMode) {
      this.viewMode = GanttViewModeEnum.CURR;
    }
    if (!this.scale) {
      this.scale = GanttScaleEnum.QUARTER;
    }

    this.user = this.authenticationService.user;
    this.projectGroups$ = this.companiesService.companiesProjectGroupTree(this.user.company).pipe(
      map((pgs) =>
        pgs.map((pg) => {
          pg.name = '-'.repeat(pg.level) + ' ' + pg.name;
          return pg;
        })
      )
    );

    if (this.projectGroupId) {
      this.loadData();
    }
  }

  public changeProjectGroup() {
    if (this.projectGroupId) {
      this.loadData();
      this.changed();
    }
  }

  private loadData() {
    this.loading = true;
    firstValueFrom(
      forkJoin([
        this.worksService.worksList({
          projectProjectGroup: this.projectGroupId.toString(),
          workType: WorkTypeEnum.MS,
          milestoneLevel: 1,
        }),
        this.projectGroupsService.projectGroupsProjects(this.projectGroupId),
        this.projectGroupsService.projectGroupsGroupTree(this.projectGroupId),
      ]).pipe(
        switchMap((res) => {
          this.works = res[0].map((work) => PWorkToGanttAdapter(work));
          const res$ = this.workLinksService.worklinksList({ source: _.flatMap(this.works, 'id').join(',') });
          this.works = this.works.concat(
            res[1].map((project) => ProjectToGanttAdapter(project)),
            res[2].map((pg) => {
              const pgtg = ProjectGroupToGanttAdapter(pg);
              if (pgtg.id === getGanttIdFromProjectGroupId(this.projectGroupId)) {
                pgtg.parent = 0;
              }
              return pgtg;
            })
          );
          return res$;
        })
      )
    ).then((workLinks) => {
      this.workLinks = workLinks.map((link) => WorkLinkToGanttAdapter(link));
      this.loading = false;
    });
  }

  public changed() {
    this.onChange.emit({
      projectGroupId: this.projectGroupId,
      scale: this.scale,
      viewMode: this.viewMode,
    });
  }
}
