import {Component, EventEmitter, Input, OnInit, Output, ViewChild} from '@angular/core';
import {
  CompaniesService,
  CompanyIndicatorConstraint,
  FinanceService,
  Issue,
  IssuesService,
  ProjectBudget,
  ProjectGroup,
  ProjectGroupsService,
  ProjectStatusReport,
  User,
  WorkIndicator,
  WorksService,
} from 'data-access/sdk/api2';
import {map} from 'rxjs/operators';
import {firstValueFrom, Observable} from 'rxjs';
import {AuthenticationService} from '@core-project/services/authentication.service';
import {WorkEnums} from '@core-project/enums';
import {ProjectEnums} from '@core-project/enums/project';
import {Router} from '@angular/router';
import WorkTypeEnum = WorkEnums.WorkTypeEnum;
import IndicatorEnum = ProjectEnums.IndicatorEnum;
import StatusEnum = WorkEnums.StatusEnum;

@Component({
  selector: 'app-project-group-status',
  templateUrl: './project-group-status.component.html',
  styleUrls: ['./project-group-status.component.scss'],
})
export class ProjectGroupStatusComponent implements OnInit {
  @Input() projectGroupId: number;
  @Output() onChange: EventEmitter<any> = new EventEmitter<any>();

  @ViewChild('worksOp') worksOp;
  @ViewChild('budgetOp') budgetOp;
  @ViewChild('issuesOp') issuesOp;

  indicators = IndicatorEnum;

  projectGroups$: Observable<ProjectGroup[]>;
  projectStatusReports: ProjectStatusReport[];

  milestones: WorkIndicator[];
  milestonesSelectedProjectId: number;

  issues: Issue[];
  issuesSelectedProjectId: number;

  projectBudget: ProjectBudget;
  budgetsSelectedProjectId: number;
  constr: CompanyIndicatorConstraint;
  private user: User;

  constructor(
    private projectGroupsService: ProjectGroupsService,
    private companiesService: CompaniesService,
    private authenticationService: AuthenticationService,
    private worksService: WorksService,
    private issuesService: IssuesService,
    private router: Router,
    private financeService: FinanceService,
  ) {
  }

  ngOnInit(): void {
    this.user = this.authenticationService.user;
    this.constr = this.user.rel_company.rel_indicator_constraints;

    this.projectGroups$ = this.companiesService.companiesProjectGroupTree(this.user.company).pipe(
      map((pgs) =>
        pgs.map((pg) => {
          pg.name = '-'.repeat(pg.level) + ' ' + pg.name;
          return pg;
        }),
      ),
    );

    if (this.projectGroupId) {
      this.loadData();
    }
  }

  changeProjectGroup() {
    if (this.projectGroupId) {
      this.loadData();
      this.changed();
    }
  }

  showWorksOp(project: ProjectStatusReport, event: Event) {
    if (project.works_indicator !== IndicatorEnum.NA) {
      this.worksOp.hide();
      this.milestonesSelectedProjectId = project.id;
      firstValueFrom(
        this.worksService.worksIndicators({
          project: project.id.toString(),
          workType: WorkTypeEnum.MS,
          milestoneLevel: 1,
          status: [StatusEnum.CR, StatusEnum.WP].join(','),
        }),
      ).then((ms) => {
        this.milestones = ms;
        this.worksOp.show(event);
      });
    }
  }

  showIssuesOp(project: ProjectStatusReport, event: Event) {
    if (project.issues_indicator !== IndicatorEnum.NA) {
      this.issuesOp.hide();
      this.issuesSelectedProjectId = project.id;
      firstValueFrom(this.issuesService.issuesIssuesList({project: project.id.toString(), include: 'domains, works__to_user'})).then(
        (issues) => {
          this.issues = issues;
          this.issuesOp.show(event);
        },
      );
    }
  }

  showBudgetOp(project: ProjectStatusReport, event: Event) {
    if (project.budget_indicator !== IndicatorEnum.NA) {
      this.budgetOp.hide();
      this.budgetsSelectedProjectId = project.id;
      firstValueFrom(this.financeService.financeProjectBudgetsRead({id: project.project_budget})).then((pb) => {
        this.projectBudget = pb;
        this.budgetOp.show(event);
      });
    }
  }

  goToProject(projectId: number, module?: string) {
    if (!module) {
      module = 'details';
    }
    this.router.navigate(['/project', projectId, module]);
  }

  private loadData() {
    firstValueFrom(this.projectGroupsService.projectGroupsStatusReport(this.projectGroupId)).then(
      (srs) => (this.projectStatusReports = srs),
    );
  }

  private changed() {
    this.onChange.emit({projectGroupId: this.projectGroupId});
  }
}
