import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { Store } from '@ngxs/store';
import { forkJoin, Observable, of, tap } from 'rxjs';
import { catchError, map, switchMap } from 'rxjs/operators';

import { CompaniesService, ProjectGroup, ProjectGroupsService, ProjectStatusReport, User } from 'data-access/sdk/api2';
import { RightSidebarActions } from 'data-access/store';
import { ProjectStageModel, ProjectsApiService } from 'data-access/sdk/internal-api';
import { Unsubscriber } from 'utils/shared-services';

import { ProjectStatusReportComponent } from '@core-project/modules/shared-components/project-status-report/project-status-report.component';
import { AuthenticationService } from '@core-project/services/authentication.service';

@Component({
  selector: 'app-status-report',
  templateUrl: './status-report.component.html',
  styleUrls: ['./status-report.component.scss'],
  providers: [Unsubscriber],
})
export class StatusReportComponent implements OnInit {
  private user: User;

  @Input() projectGroupId: number;
  @Output() onChange: EventEmitter<any> = new EventEmitter<any>();

  projectGroups$: Observable<ProjectGroup[]>;
  projectStatusReports: (ProjectStatusReport & { timeline: ProjectStageModel[] })[];

  readonly dayInPx = 2.2;
  readonly timelineWidth = 800;
  readonly currentMarkX = this.timelineWidth / 2;
  readonly timelineBgColor = 'rgb(255, 255, 255)';

  constructor(
    private readonly projectGroupsService: ProjectGroupsService,
    private readonly companiesService: CompaniesService,
    private readonly authenticationService: AuthenticationService,
    private readonly store: Store,
    private readonly projectsApi: ProjectsApiService
  ) {}

  ngOnInit(): void {
    this.user = this.authenticationService.user;

    this.projectGroups$ = this.companiesService.companiesProjectGroupTree(this.user.company).pipe(
      map((pgs) =>
        pgs.map((pg) => {
          pg.name = '-'.repeat(pg.level) + ' ' + pg.name;
          return pg;
        })
      )
    );

    if (this.projectGroupId) {
      this.loadData();
    } else {
      this.projectGroups$.subscribe((pgs) => {
        if (pgs.length > 0) {
          this.projectGroupId = pgs[0].id;
          this.loadData();
        }
      });
    }
  }

  changeProjectGroup() {
    if (this.projectGroupId) {
      this.loadData();
      this.changed();
    }
  }

  onRowSelect(projectStatusReport: ProjectStatusReport) {
    this.store.dispatch(
      new RightSidebarActions.Open({ getComponentFn: () => ProjectStatusReportComponent, data: { projectStatusReport } })
    );
  }

  private loadData() {
    this.projectGroupsService
      .projectGroupsStatusReport(this.projectGroupId)
      .pipe(
        switchMap((reports) => {
          const reports$: Observable<ProjectStageModel[]>[] = reports.map((report) =>
            this.projectsApi.readProjectTimeline(report.id).pipe(catchError(() => {
              console.error(`Failed to read project ${report.id} timeline`);
              
              return of([]);
            }))
          );
          
          return forkJoin(reports$).pipe(map((timelines) => timelines.map((timeline, i) => ({ timeline, ...reports[i] }))));
        }),
      )
      .subscribe({
        next: (projectStatusReports) => {
          this.projectStatusReports = projectStatusReports;
        },
        error: () => {
          this.projectStatusReports = [];
        },
      });
  }

  private changed() {
    this.onChange.emit({ projectGroupId: this.projectGroupId });
  }
}
