import {Component, OnInit} from '@angular/core';
import {Department, DepartmentsService, User, WorklinksService, WorksService} from 'data-access/sdk/api2';
import {BehaviorSubject, forkJoin, Observable, of} from 'rxjs';
import {ActivatedRoute} from '@angular/router';
import {switchMap} from 'rxjs/operators';
import * as _ from 'lodash-es';
import {WorkEnums} from '@core-project/enums';
import * as moment from 'moment';
import {dbStringToMoment, momentToDbString} from '@core-project/helpers';
import {
  DepartmentToGanttAdapter,
  GanttWork,
  GanttWorkLink,
  UserToGanttAdapter,
  UWorkToGanttAdapter,
  WorkLinkToGanttAdapter,
} from '@core-project/helpers/gantt-adapter';
import {Store} from '@ngxs/store';
import {GlobalRulesSelectors} from 'data-access/store';

@Component({
  selector: 'app-plan',
  templateUrl: './department-plan.component.html',
  styleUrls: ['./department-plan.component.scss'],
})
export class DepartmentPlanComponent implements OnInit {
  readonly worksSubject: BehaviorSubject<GanttWork[]> = new BehaviorSubject([]);
  readonly works$: Observable<Array<GanttWork>> = this.worksSubject.asObservable();

  readonly links$: Observable<GanttWorkLink[]> = this.works$.pipe(
    switchMap((works) => {
      const workIds = _.flatMap(works, 'id').filter((id) => typeof id === 'number');
      return this.workLinksService.worklinksList({source: workIds.join(',')});
    }),
    switchMap((links) => of(links.map((link) => WorkLinkToGanttAdapter(link)))),
  );

  readonly$: Observable<boolean> = this.store.select(GlobalRulesSelectors.canNotUpdate('projectworkplan'));

  users: User[];
  departments: Department[];
  modes = WorkEnums.GanttModeEnum;
  dateRange: string[];
  startR: string;
  endR: string;
  viewMode: WorkEnums.GanttViewModeEnum = WorkEnums.GanttViewModeEnum.CURR;

  private departmentId: number;

  constructor(
    private readonly worksService: WorksService,
    private readonly workLinksService: WorklinksService,
    private readonly departmentsService: DepartmentsService,
    private readonly activatedRoute: ActivatedRoute,
    private readonly store: Store,
  ) {
  }

  ngOnInit() {
    this.departmentId = Number(this.activatedRoute.parent.snapshot.paramMap.get('departmentId'));
    const now = moment();
    const weekStart = now.startOf('week');
    this.dateRange = [momentToDbString(weekStart), momentToDbString(weekStart.add(13, 'days'))];
    this.copyRange();
    this.loadData();
  }

  loadData() {
    const uWorks = this.departmentsService.departmentsAllUsers(this.departmentId).pipe(
      switchMap((users) => {
        const userIds = _.flatMap(users, 'id');
        this.users = users;
        return this.worksService.worksList({
          workType: 'AS,TS,MS',
          include: 'to_user, to_department, project',
          dateAfter: this.dateRange[0],
          dateBefore: this.dateRange[1],
          toUser: userIds.join(','),
        });
      }),
    );

    const dWorks = this.departmentsService.departmentsSubdepartments(this.departmentId).pipe(
      switchMap((deps) => {
        const depIds = _.flatMap(deps, 'id');
        this.departments = deps;
        return this.worksService.worksList({
          workType: 'AS,TS,MS',
          include: 'to_user, to_department, project',
          dateAfter: this.dateRange[0],
          dateBefore: this.dateRange[1],
          toDepartment: depIds.join(','),
        });
      }),
    );

    forkJoin([uWorks, dWorks])
      .pipe(
        switchMap((res) => of(res[0].concat(res[1]))),
        switchMap((works) =>
          of(
            works
              .map((work) => UWorkToGanttAdapter(work))
              .concat(
                this.users.map((u) => UserToGanttAdapter(u)),
                this.departments.map((d) => DepartmentToGanttAdapter(d)),
              ),
          ),
        ),
      )
      .subscribe((works) => this.worksSubject.next(works));
  }

  plusWeek() {
    const start = dbStringToMoment(this.dateRange[0]);
    const end = dbStringToMoment(this.dateRange[1]);
    this.dateRange = [momentToDbString(start.add(1, 'week')), momentToDbString(end.add(1, 'week'))];
    this.copyRange();
    this.loadData();
  }

  minusWeek() {
    const start = dbStringToMoment(this.dateRange[0]);
    const end = dbStringToMoment(this.dateRange[1]);
    this.dateRange = [momentToDbString(start.subtract(1, 'week')), momentToDbString(end.subtract(1, 'week'))];
    this.copyRange();
    this.loadData();
  }

  onRangeChange() {
    if (this.dateRange[0] && this.dateRange[1]) {
      this.copyRange();
      this.loadData();
    }
  }

  private copyRange() {
    this.startR = this.dateRange[0];
    this.endR = this.dateRange[1];
  }
}
