import { toSignal } from '@angular/core/rxjs-interop';
import { Component, OnInit, Signal } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { Actions, ofActionSuccessful, Store } from '@ngxs/store';
import { BehaviorSubject, Observable, pipe } from 'rxjs';
import { switchMap, takeUntil } from 'rxjs/operators';
import * as _ from 'lodash-es';

import { AuthenticationService } from '@core-project/services/authentication.service';

import { DepartmentsService, User } from 'data-access/sdk/api2';
import { GlobalRulesSelectors, WorksActions } from 'data-access/store';
import { Unsubscriber } from 'utils/shared-services';
import { WorkApiService, WorkModel, WorkStatusEnum } from 'data-access/sdk/internal-api';
import { KanbanChangeInterface } from 'ui/kanban-board';

@Component({
  selector: 'app-department-tasks',
  templateUrl: './department-tasks.component.html',
  styleUrls: ['./department-tasks.component.scss'],
  providers: [Unsubscriber],
})
export class DepartmentTasksComponent implements OnInit {
  private readonly workObserverSub = this.actions
    .pipe(
      takeUntil(this.unsubscriber.destroy$),
      ofActionSuccessful(WorksActions.CreateWork, WorksActions.UpdateWork, WorksActions.ChangeWorkStatus, WorksActions.DeleteWork)
    )
    .subscribe(() => {
      this.loadData();
    });

  private readonly worksSubject: BehaviorSubject<WorkModel[]> = new BehaviorSubject<WorkModel[]>([]);

  readonly canNotUpdateWork: Signal<boolean> = toSignal(this.store.select(GlobalRulesSelectors.canNotUpdate('work')));
  readonly works: Signal<WorkModel[]> = toSignal(this.worksSubject.asObservable());

  currentUser: User;
  departmentId: number;

  constructor(
    private readonly departmentsService: DepartmentsService,
    private readonly authenticationService: AuthenticationService,
    private readonly activatedRoute: ActivatedRoute,
    private readonly store: Store,
    private readonly actions: Actions,
    private readonly unsubscriber: Unsubscriber,
    private readonly workApi: WorkApiService
  ) {}

  ngOnInit(): void {
    this.currentUser = this.authenticationService.user;
    this.departmentId = Number(this.activatedRoute.parent.snapshot.paramMap.get('departmentId'));
    this.loadData();
  }

  openCard(work: WorkModel) {
    this.store.dispatch(new WorksActions.ShowWorkInSidebar(work.id)).subscribe();
  }

  moveCard(change: KanbanChangeInterface) {
    const currentWorkId = change.card.rawModel.id;

    this.workApi
      .changeWorkStatusAndOrderDepartment(
        currentWorkId,
        change.nextCard?.rawModel.id,
        change.previousCard?.rawModel.id,
        change.card.rawModel.status
      )
      .pipe(switchMap(() => this.store.dispatch(new WorksActions.RefreshWork(currentWorkId))))
      .subscribe();
  }

  loadData() {
    this.departmentsService
      .departmentsAllUsers(this.departmentId)
      .pipe(
        switchMap((users) => {
          const userIds = _.flatMap(users, 'id');

          return this.workApi.readList({
            workType: 'TS,MS',
            include: 'to_user, to_department',
            toUser: userIds.join(','),
          });
        })
      )
      .subscribe((works) => {
        this.worksSubject.next(works);
      });
  }

  addAssignment() {
    this.store
      .dispatch(
        new WorksActions.ShowCreateWorkInSidebar({
          startBase: new Date().toISOString(),
          status: WorkStatusEnum.CR,
          toDepartmentId: this.departmentId.toString(),
        })
      )
      .subscribe();
  }
}
