import {Component, OnInit, ViewChild} from '@angular/core';
import {BehaviorSubject, firstValueFrom, Observable} from 'rxjs';
import {DialogService} from 'primeng/dynamicdialog';
import {TranslateService} from '@ngx-translate/core';
import {ConfirmationService, MessageService} from 'primeng/api';
import {FileUpload} from 'primeng/fileupload';
import {CompaniesService, Company, License, LicenseCreate, LicensesService, User, UserRole, UserRolesService} from 'data-access/sdk/api2';
import {AuthenticationService} from '@core-project/services/authentication.service';
import {CompanyFormComponent} from '@core-project/modules/forms/company-form/company-form.component';
import {
  UserRoleFormComponent,
  UserRoleFormComponentData,
} from '@core-project/modules/forms/role-forms/user-role-form/user-role-form.component';

@Component({
  selector: 'app-company',
  templateUrl: './company.component.html',
  styleUrls: ['./company.component.scss'],
})
export class CompanyComponent implements OnInit {
  @ViewChild('fileUpload') fileUpload: FileUpload;

  readonly companySubject: BehaviorSubject<Company> = new BehaviorSubject<Company>(null);
  readonly company$: Observable<Company> = this.companySubject.asObservable();

  readonly userRolesSubject: BehaviorSubject<UserRole[]> = new BehaviorSubject<UserRole[]>([]);
  readonly userRoles$: Observable<UserRole[]> = this.userRolesSubject.asObservable();

  readonly licensesSubject: BehaviorSubject<License[]> = new BehaviorSubject<License[]>([]);
  readonly licenses$: Observable<License[]> = this.licensesSubject.asObservable();

  private user: User;

  constructor(
    private readonly companiesService: CompaniesService,
    private readonly authenticationService: AuthenticationService,
    private readonly userRolesService: UserRolesService,
    private readonly dialogService: DialogService,
    private readonly translateService: TranslateService,
    private readonly confirmationService: ConfirmationService,
    private readonly licensesService: LicensesService,
    private readonly messageService: MessageService,
  ) {
  }

  ngOnInit() {
    this.user = this.authenticationService.user;
    this.loadData();
  }

  editCompany(company: Company) {
    const ref = this.dialogService.open(CompanyFormComponent, {
      header: this.translateService.instant('KB.COMPANY.EDIT_COMPANY'),
      width: '40%',
      data: {company: company},
    });

    ref.onClose.subscribe(_c => this.loadCompany());
  }

  editUserRole(userRole?: UserRole) {
    const data: UserRoleFormComponentData = {
      type: 'company',
      userRole,
      companyId: this.user.company,
      isDepartmentsResponsible: true,
    };

    const ref = this.dialogService.open(UserRoleFormComponent, {
      header: userRole ? this.translateService.instant('KB.COMPANY.EDIT_USER_ROLE') :
        this.translateService.instant('KB.COMPANY.ADD_USER_ROLE'),
      width: '50%',
      data,
    });

    ref.onClose.subscribe(ur => {
      if (ur) {
        this.loadUserRoles();
      }
    });
  }

  deleteUserRole(userRole: UserRole) {
    this.confirmationService.confirm({
      header: this.translateService.instant('KB.COMPANY.TEAM_DELETE_HEADER'),
      message: this.translateService.instant('KB.COMPANY.TEAM_DELETE_MSG'),
      accept: () => this.userRolesService.userRolesDelete(userRole.id)
        .subscribe(_r => this.loadUserRoles()),
    });

  }

  addLicense(event) {
    const file = event.files[0];
    const reader = new FileReader();
    reader.onload = () => {
      const fl = reader.result.toString();
      let flJson = {};
      try {
        flJson = JSON.parse(fl);
      } catch (e) {
        this.messageService.add({
          severity: 'error',
          summary: this.translateService.instant('ERRORS.ERROR'),
          detail: this.translateService.instant('KB.COMPANY.LICENSE.INVALID_LICENSE_MSG'),
        });
        this.fileUpload.clear();
        return;
      }
      flJson['company'] = this.authenticationService.user.company;
      firstValueFrom(this.licensesService.licensesCreate(flJson as LicenseCreate))
        .then(_l => {
          this.loadLicenses();
          this.messageService.add({
            severity: 'success',
            detail: this.translateService.instant('KB.COMPANY.LICENSE.LICENSE_ADDED_MSG'),
            summary: this.translateService.instant('KB.COMPANY.LICENSE.LICENSE_ADDED'),
          });
          this.fileUpload.clear();
        })
        .catch(_err => {
          this.messageService.add({
            severity: 'error',
            summary: this.translateService.instant('ERRORS.ERROR'),
            detail: this.translateService.instant('KB.COMPANY.LICENSE.INVALID_LICENSE_MSG'),
          });
          this.fileUpload.clear();
        });
    };
    reader.readAsText(file);
  }

  private loadData() {
    this.loadCompany();
    this.loadUserRoles();
    this.loadLicenses();
  }

  private loadCompany() {
    this.companiesService.companiesRead({id: this.user.company})
      .subscribe(company => this.companySubject.next(company));
  }

  private loadUserRoles() {
    this.userRolesService.userRolesList({
      company: this.user.company.toString(),
      include: 'user__telephones, role, department',
    })
      .subscribe(userRoles => this.userRolesSubject.next(userRoles));
  }

  private loadLicenses() {
    this.licensesService.licensesCurrent()
      .subscribe(licenses => this.licensesSubject.next(licenses));
  }

}
