import { ChangeDetectorRef, Component, OnInit } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { DialogService } from 'primeng/dynamicdialog';
import { DocumentsService } from 'data-access/sdk/api2';
import { DocumentMetaField } from 'data-access/sdk/api2';
import { DocumentType } from 'data-access/sdk/api2';
import { BehaviorSubject, finalize, forkJoin, Observable } from 'rxjs';
import { DocumentTypeFormComponent } from '@core-project/modules/forms/document-forms/document-type-form/document-type-form.component';
import { DocumentMetaFormComponent } from '@core-project/modules/forms/document-forms/document-meta-form/document-meta-form.component';
import { ConfirmationService, MessageService } from 'primeng/api';
import { takeUntil } from 'rxjs/operators';
import { Unsubscriber } from '@core-project/services/unsubscriber.service';

@Component({
  selector: 'app-documents',
  templateUrl: './documents.component.html',
  styleUrls: ['./documents.component.scss'],
  providers: [Unsubscriber],
})
export class DocumentsComponent implements OnInit {
  readonly metasSubject: BehaviorSubject<DocumentMetaField[]> = new BehaviorSubject<DocumentMetaField[]>([]);
  readonly metas$: Observable<DocumentMetaField[]> = this.metasSubject.asObservable();

  types: DocumentType[] = [];

  private readonly typesSubject: BehaviorSubject<DocumentType[]> = new BehaviorSubject<DocumentType[]>([]);

  constructor(
    private readonly translateService: TranslateService,
    private readonly dialogService: DialogService,
    private readonly documentsService: DocumentsService,
    private readonly messageService: MessageService,
    private readonly confirmationService: ConfirmationService,
    private readonly unsubscriber: Unsubscriber,
    private readonly changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit(): void {
    this.loadMetas();
    this.loadTypes();

    this.typesSubject.pipe(takeUntil(this.unsubscriber.destroy$)).subscribe((types) => {
      this.types = types;
      this.changeDetectorRef.markForCheck();
    });
  }

  /**
   * Сохранение порядка документов по признаку order. Сортировка реализована вне функции.
   *
   * Метод используется для DocumentType и для зависимых DocumentType внутри модели, т.к. модели идентичны.
   *
   * @param documentTypes список типов документов с измененным индексом (для индексации order)
   */
  onDocTypesRowReorder(documentTypes: DocumentType[] = this.types) {
    forkJoin(
      documentTypes.map((docType, index) =>
        this.documentsService.documentsTypesPartialUpdate({
          id: docType.id,
          data: {
            ...docType,
            order: index,
          },
        })
      )
    )
      .pipe(finalize(() => this.loadTypes()))
      .subscribe();
  }

  editType(category: boolean, type?: DocumentType) {
    const header = type
      ? category
        ? this.translateService.instant('KB.METHODOLOGY.DOCUMENTS.TYPE_FORM.HEADER_EDIT_CAT')
        : this.translateService.instant('KB.METHODOLOGY.DOCUMENTS.TYPE_FORM.HEADER_EDIT')
      : category
      ? this.translateService.instant('KB.METHODOLOGY.DOCUMENTS.TYPE_FORM.HEADER_ADD_CAT')
      : this.translateService.instant('KB.METHODOLOGY.DOCUMENTS.TYPE_FORM.HEADER_ADD');
    const ref = this.dialogService.open(DocumentTypeFormComponent, {
      header: header,
      width: '40%',
      data: {
        type: type,
        category: category,
      },
    });

    ref.onClose.subscribe((editedType) => {
      if (editedType) {
        if (type) {
          const obs$: Array<Observable<any>> = [this.documentsService.documentsTypesPartialUpdate({ id: type.id, data: editedType })];

          if (!editedType.parent_type && editedType.group_by) {
            const subs = type.rel_subtypes;
            subs.forEach((st) => {
              if (!st.meta_fields.includes(editedType.group_by)) {
                st.meta_fields.push(editedType.group_by);
                obs$.push(this.documentsService.documentsTypesPartialUpdate({ id: st.id, data: st }));
              }
            });
          }
          forkJoin(obs$).subscribe((_r) => this.loadTypes());
        } else {
          this.documentsService.documentsTypesCreate(editedType).subscribe((_r) => this.loadTypes());
        }
      }
    });
  }

  deleteType(type: DocumentType) {
    if (type.subtypes && type.subtypes.length > 0) {
      this.messageService.add({
        severity: 'error',
        summary: this.translateService.instant('ERRORS.DELETE_ERROR'),
        detail: this.translateService.instant('KB.METHODOLOGY.DOCUMENTS.TYPE_DELETE_HAS_SUBS_ERROR'),
      });
      return;
    }

    if (type.documents && type.documents.length > 0) {
      this.messageService.add({
        severity: 'error',
        summary: this.translateService.instant('ERRORS.DELETE_ERROR'),
        detail: this.translateService.instant('KB.METHODOLOGY.DOCUMENTS.TYPE_DELETE_HAS_DOCS_ERROR'),
      });
      return;
    }
    this.documentsService.documentsTypesDelete(type.id).subscribe((_r) => this.loadTypes());
  }

  editMeta(meta?: DocumentMetaField) {
    const header = meta
      ? this.translateService.instant('KB.METHODOLOGY.DOCUMENTS.META_FORM.HEADER_EDIT')
      : this.translateService.instant('KB.METHODOLOGY.DOCUMENTS.META_FORM.HEADER_ADD');
    const ref = this.dialogService.open(DocumentMetaFormComponent, {
      header: header,
      width: '40%',
      data: {
        meta: meta,
      },
    });

    ref.onClose.subscribe((editedMeta) => {
      if (editedMeta) {
        if (meta) {
          this.documentsService.documentsMetaFieldsPartialUpdate({ id: meta.id, data: editedMeta }).subscribe((_r) => {
            this.loadTypes();
            this.loadMetas();
          });
        } else {
          this.documentsService.documentsMetaFieldsCreate(editedMeta).subscribe((_r) => this.loadMetas());
        }
      }
    });
  }

  deleteMeta(meta: DocumentMetaField) {
    this.confirmationService.confirm({
      message: this.translateService.instant('KB.METHODOLOGY.DOCUMENTS.META_DELETE_MSG'),
      accept: () => {
        this.documentsService.documentsMetaFieldsDelete(meta.id).subscribe((_r) => {
          this.loadTypes();
          this.loadMetas();
        });
      },
    });
  }

  private loadMetas() {
    this.documentsService.documentsMetaFieldsList().subscribe((list) => this.metasSubject.next(list));
  }

  private loadTypes() {
    this.documentsService.documentsTypesList({ uncategorized: 'true', include: 'subtypes__meta_fields, group_by' }).subscribe((list) => {
      this.typesSubject.next(this.sortDocumentTypes(this.sortDocumentSubTypes(list)));
    });
  }

  private sortDocumentSubTypes(types: DocumentType[]) {
    return types.map((type) => {
      type.rel_subtypes = this.sortDocumentTypes(type.rel_subtypes);
      return type;
    });
  }

  private sortDocumentTypes(types: DocumentType[]) {
    return types.sort((a, b) => a.order - b.order);
  }
}
