import { ChangeDetectionStrategy, Component, Input, OnChanges, OnInit, SimpleChanges, ViewChild } from '@angular/core';
import { FormControl } from '@angular/forms';
import { BehaviorSubject, Observable } from 'rxjs';
import { filter, switchMap, takeUntil } from 'rxjs/operators';

import { ProjectType } from 'data-access/sdk/api2';
import { StageModel } from 'data-access/sdk/internal-api';
import { DeleteStageOverlayComponent } from '@core-project/containers/knowledgebase/methodology/stages/components/delete-stage-overlay/delete-stage-overlay.component';

import { ProjectStagesStoreService } from '../../services/project-stages-store.service';
import {Unsubscriber} from 'utils/shared-services';

@Component({
  selector: 'app-project-categories-stages',
  templateUrl: './project-stages.component.html',
  styleUrls: ['./project-stages.component.scss'],
  providers: [Unsubscriber],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class ProjectStagesComponent implements OnChanges, OnInit {
  @ViewChild('deleteStageOverlayRef') deleteStageOverlayRef: DeleteStageOverlayComponent;

  @Input() selectedType: ProjectType;

  addStageFC = new FormControl<StageModel>(undefined);

  selectedStageForDeleteIdSubject: BehaviorSubject<number> = new BehaviorSubject<number>(undefined);
  selectedStageForDeleteId$: Observable<number> = this.selectedStageForDeleteIdSubject.asObservable();

  constructor(readonly projectStagesService: ProjectStagesStoreService, private readonly unsubscriber: Unsubscriber) {}

  ngOnInit(): void {
    this.initAddProjectTypeObs();
  }

  ngOnChanges(changes: SimpleChanges): void {
    const selectedType: ProjectType | undefined = changes.selectedType.currentValue;
    if (selectedType) {
      this.projectStagesService.readProjectStages(selectedType.id);
    }
  }

  deleteProjectStage(replaceWithStage: StageModel['id']) {
    const selectedStageForDeleteId = this.selectedStageForDeleteIdSubject.getValue();
    this.projectStagesService
      .deleteProjectStage(replaceWithStage, selectedStageForDeleteId)
      .subscribe(() => this.deleteStageOverlayRef.hide());
  }

  onDeleteSelectionChange({ event, stage }: { event: MouseEvent; stage: StageModel }) {
    this.deleteStageOverlayRef.show(event);
    this.changeSelectedForDeleteStageId(stage);
  }

  onTableRowReorder(stages: StageModel[]) {
    this.projectStagesService.updateProjectStages(stages).subscribe();
  }

  changeSelectedForDeleteStageId(stage: StageModel) {
    this.selectedStageForDeleteIdSubject.next(stage.id);
  }

  private initAddProjectTypeObs() {
    this.addStageFC.valueChanges
      .pipe(
        filter((value) => !!value),
        takeUntil(this.unsubscriber.destroy$),
        switchMap((value) => {
          this.addStageFC.reset();
          return this.projectStagesService.createProjectStage(value.id, this.selectedType.id);
        })
      )
      .subscribe();
  }
}
