import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { map, takeUntil } from 'rxjs/operators';
import { BehaviorSubject, combineLatest, Observable, tap } from 'rxjs';
import { ConfirmationService, MessageService, TreeNode } from 'primeng/api';
import { DialogService } from 'primeng/dynamicdialog';
import { TreeNodeSelectEvent } from 'primeng/tree';

import { ProjectCategoriesService, ProjectMetaGroupsService, ProjectTypesService } from 'data-access/sdk/api2';
import { ProjectCategory, ProjectMetaGroup, ProjectType } from 'data-access/sdk/api2';
import { Unsubscriber } from 'utils/shared-services';

import { ProjectTypeFormComponent } from '@core-project/modules/forms/project-type-form/project-type-form.component';

@Component({
  selector: 'app-project-categories',
  templateUrl: './project-categories-facade.component.html',
  styleUrls: ['./project-categories-facade.component.scss'],
  providers: [Unsubscriber],
})
export class ProjectCategoriesFacadeComponent implements OnInit {
  private readonly _category: string = 'category';
  private readonly _categoryPrefix: string = 'cat_';
  private readonly _projectType: string = 'type';
  private readonly selectedTypeMetaGroupsBSubject: BehaviorSubject<Array<ProjectMetaGroup>> = new BehaviorSubject<Array<ProjectMetaGroup>>(
    []
  );
  private readonly selectedTypeBSubject: BehaviorSubject<ProjectType> = new BehaviorSubject<ProjectType>(undefined);

  categories: Array<ProjectCategory>;
  categoriesTree: Array<TreeNode>;
  selectedCat: ProjectCategory;
  selectedNode: TreeNode | TreeNode[];

  readonly selectedTypeMetaGroups$: Observable<Array<ProjectMetaGroup>> = this.selectedTypeMetaGroupsBSubject.asObservable();
  readonly selectedType$: Observable<ProjectType> = this.selectedTypeBSubject.asObservable();

  constructor(
    private readonly translateService: TranslateService,
    private readonly projectCategoriesService: ProjectCategoriesService,
    private readonly projectTypesService: ProjectTypesService,
    private readonly projectMetaGroupsService: ProjectMetaGroupsService,
    private readonly messageService: MessageService,
    private readonly dialogService: DialogService,
    private readonly confirmationService: ConfirmationService,
    private readonly activatedRoute: ActivatedRoute,
    private readonly router: Router,
    private readonly unsubscriber: Unsubscriber
  ) {}

  get selectedType(): ProjectType {
    return this.selectedTypeBSubject.getValue();
  }

  set selectedType(value: ProjectType) {
    this.selectedTypeBSubject.next(value);
  }

  set selectedTypeMetaGroups(value: Array<ProjectMetaGroup>) {
    this.selectedTypeMetaGroupsBSubject.next(value);
  }

  ngOnInit(): void {
    this.loadCategories();
  }

  changeNode(event: TreeNodeSelectEvent) {
    let projectType, category;
    if (event.node.type === this._projectType) {
      category = event.node.parent.data.id;
      projectType = event.node.data.id;
    } else if (event.node.type === this._category) {
      category = event.node.data.id;
    }
    this.router.navigate(['settings', 'project-cats', category, projectType].filter((r) => !!r));
  }

  createCategory() {
    const ref = this.dialogService.open(ProjectTypeFormComponent, {
      header: this.translateService.instant('KB.METHODOLOGY.PROJECT_CATEGORIES.ADD_HEADER'),
      width: '40%',
    });

    ref.onClose.subscribe((editedCat) => {
      if (editedCat) {
        let obs$;
        if (editedCat.category) {
          obs$ = this.projectTypesService.projectTypesCreate(editedCat);
        } else {
          obs$ = this.projectCategoriesService.projectCategoriesCreate(editedCat);
        }
        obs$.subscribe(() => this.loadCategories());
      }
    });
  }

  editCategory(projectCategory: ProjectCategory) {
    const ref = this.dialogService.open(ProjectTypeFormComponent, {
      header: this.translateService.instant('KB.METHODOLOGY.PROJECT_CATEGORIES.EDIT_HEADER'),
      width: '40%',
      data: { cat: projectCategory },
    });

    ref.onClose.subscribe((editedCat) => {
      if (editedCat) {
        this.projectCategoriesService
          .projectCategoriesUpdate({ id: projectCategory.id, data: editedCat })
          .subscribe(() => this.loadCategories());
      }
    });
  }

  editProjectType(projectType: ProjectType) {
    const ref = this.dialogService.open(ProjectTypeFormComponent, {
      header: this.translateService.instant('KB.METHODOLOGY.PROJECT_CATEGORIES.EDIT_HEADER'),
      width: '40%',
      data: { cat: projectType },
    });

    ref.onClose.subscribe((editedType) => {
      if (editedType) {
        this.projectTypesService.projectTypesPartialUpdate({ id: projectType.id, data: editedType }).subscribe(() => this.loadCategories());
      }
    });
  }

  deleteCategory(cat: ProjectCategory) {
    if (cat.types.length > 0) {
      this.messageService.add({
        severity: 'error',
        summary: this.translateService.instant('ERRORS.DELETE_ERROR'),
        detail: this.translateService.instant('KB.METHODOLOGY.PROJECT_CATEGORIES.CAT_HAS_TYPES_ERROR'),
      });
      return;
    }
    this.confirmationService.confirm({
      header: this.translateService.instant('KB.METHODOLOGY.PROJECT_CATEGORIES.DELETE_CAT_HEADER'),
      message: this.translateService.instant('KB.METHODOLOGY.PROJECT_CATEGORIES.DELETE_CAT_MSG'),
      accept: () =>
        this.projectCategoriesService.projectCategoriesDelete(cat.id).subscribe((_r) => {
          this.selectedCat = null;
          this.loadCategories();
        }),
    });
  }

  deleteType(type: ProjectType) {
    if (type.projects.length > 0) {
      this.messageService.add({
        severity: 'error',
        summary: this.translateService.instant('ERRORS.DELETE_ERROR'),
        detail: this.translateService.instant('KB.METHODOLOGY.PROJECT_CATEGORIES.TYPE_HAS_PROJECTS_ERROR'),
      });
      return;
    }
    this.confirmationService.confirm({
      header: this.translateService.instant('KB.METHODOLOGY.PROJECT_CATEGORIES.DELETE_CAT_HEADER'),
      message: this.translateService.instant('KB.METHODOLOGY.PROJECT_CATEGORIES.DELETE_CAT_MSG'),
      accept: () =>
        this.projectTypesService.projectTypesDelete(type.id).subscribe((_r) => {
          this.selectedType = null;
          this.selectedTypeMetaGroups = [];
          this.loadCategories();
        }),
    });
  }

  private typeSelect() {
    if ((this.selectedNode as TreeNode).type === this._category) {
      this.selectedCat = (this.selectedNode as TreeNode).data;
      this.selectedType = null;
      this.selectedTypeMetaGroups = [];
    } else {
      this.selectedType = (this.selectedNode as TreeNode).data;
      this.selectedCat = null;
      this.projectMetaGroupsService
        .projectMetaGroupsList('meta_fields')
        .pipe(
          map((mgs) =>
            mgs
              .map((mg) => {
                mg.rel_meta_fields = mg.rel_meta_fields.filter((mf) => mf.all_types || mf.project_types.includes(this.selectedType.id));
                return mg;
              })
              .filter((mg) => mg.rel_meta_fields.length > 0)
          ),
          tap((mgs) => (this.selectedTypeMetaGroups = mgs))
        )
        .subscribe();
    }
  }

  private loadCategories() {
    combineLatest([this.projectCategoriesService.projectCategoriesList('types'), this.activatedRoute.params])
      .pipe(takeUntil(this.unsubscriber.destroy$))
      .subscribe(([cats, params]) => {
        this.convertCatsToTree(cats);

        if (params.category) {
          const categoryNode: TreeNode<ProjectCategory> = this.categoriesTree.find((item) => item.data.id.toString() === params.category);

          if (params.projectType) {
            const categoryProjects: TreeNode<ProjectType>[] = categoryNode.children as TreeNode<ProjectType>[];
            const projectTypeNode: TreeNode<ProjectType> = categoryProjects.find((item) => item.data.id.toString() === params.projectType);

            this.selectedNode = projectTypeNode;
            this.typeSelect();
          } else {
            this.selectedNode = categoryNode;
            this.typeSelect();
          }
        }
      });
  }

  private convertCatsToTree(cats: Array<ProjectCategory>) {
    this.categoriesTree = cats.map((projectCategory) => ({
      label: projectCategory.name,
      data: projectCategory,
      expanded: true,
      type: this._category,
      key: this._categoryPrefix + projectCategory.id.toString(),
      children: projectCategory.rel_types.map((projectType) => ({
        label: projectType.name,
        data: projectType,
        selectable: true,
        type: this._projectType,
        key: projectType.id.toString(),
      })),
    }));
  }
}
