import {Component, OnInit} from '@angular/core';
import {ConfirmationService, MessageService, TreeNode} from 'primeng/api';
import {TranslateService} from '@ngx-translate/core';
import {DialogService} from 'primeng/dynamicdialog';
import {BehaviorSubject, Observable} from 'rxjs';
import {switchMap} from 'rxjs/operators';
import {TreeNodeExpandEvent} from 'primeng/tree';
import {ProjectGroup, ProjectGroupsService, UserRole, UserRolesService} from 'data-access/sdk/api2';
import {ProjectGroupFormComponent} from '@core-project/modules/forms/project-group-form/project-group-form.component';
import {
  UserRoleFormComponent,
  UserRoleFormComponentData,
} from '@core-project/modules/forms/role-forms/user-role-form/user-role-form.component';

@Component({
  selector: 'app-project-groups',
  templateUrl: './project-groups.component.html',
  styleUrls: ['./project-groups.component.scss'],
})
export class ProjectGroupsComponent implements OnInit {
  rootGroups: Array<ProjectGroup>;
  rootGroupTree: Array<TreeNode>;
  selectedNode: TreeNode | TreeNode[];
  selectedGroup: ProjectGroup;

  readonly groupTeamSubject: BehaviorSubject<UserRole[]> = new BehaviorSubject<UserRole[]>([]);
  readonly groupTeam$: Observable<UserRole[]> = this.groupTeamSubject.asObservable();

  constructor(
    private translateService: TranslateService,
    private dialogService: DialogService,
    private projectGroupsService: ProjectGroupsService,
    private userRolesService: UserRolesService,
    private confirmationService: ConfirmationService,
    private messageService: MessageService,
  ) {
  }

  ngOnInit(): void {
    this.loadData();
  }

  loadData() {
    this.loadGroups().subscribe(groups => {
      this.rootGroups = groups;
      this.rootGroupTree = this.convertGroupsToTree(groups);
    });
  }

  groupExpand($event: TreeNodeExpandEvent) {
    const node = $event.node;
    if (!node.leaf && node.children?.length === 0) {
      this.loadGroups(node.data.id).subscribe(
        groups => node.children = this.convertGroupsToTree(groups),
      );
    }
  }

  loadGroups(parent: number = null): Observable<Array<ProjectGroup>> {
    const queryParams: ProjectGroupsService.ProjectGroupsListParams = {
      include: 'subgroups',
    };
    if (parent) {
      queryParams.parentGroup = String(parent);
    } else {
      queryParams.parentGroupIsnull = 'true';
    }
    return this.projectGroupsService.projectGroupsList(queryParams);
  }

  convertGroupsToTree(groups: Array<ProjectGroup>): Array<TreeNode> {
    return groups.map(g => this.convertGroupToNode(g));
  }

  convertGroupToNode(group: ProjectGroup): TreeNode {
    return {
      label: group.name,
      data: group,
      children: [],
      leaf: !(group.subgroups.length > 0),
      expanded: false,
      key: String(group.id),
    };
  }

  groupSelect() {
    this.selectedGroup = (this.selectedNode as TreeNode)?.data;
    if (this.selectedGroup) {
      this.loadUserRoles(this.selectedGroup)
        .subscribe(roles => this.groupTeamSubject.next(roles));
    } else {
      this.groupTeamSubject.next([]);
    }
  }

  editGroup(group: ProjectGroup) {
    const ref = this.dialogService.open(ProjectGroupFormComponent, {
      header: this.translateService.instant('KB.METHODOLOGY.PROJECT_GROUPS.FORM.EDIT_HEADER'),
      width: '40%',
      data: {
        group: group,
      },
    });

    ref.onClose.subscribe(editedGroup => {
      if (editedGroup) {
        this.projectGroupsService.projectGroupsPartialUpdate({id: group.id, data: editedGroup})
          .subscribe(_r => {
            this.loadData();
            Object.assign(group, editedGroup);
          });
      }
    });
  }

  createProjectGroup() {
    const ref = this.dialogService.open(ProjectGroupFormComponent, {
      header: this.translateService.instant('KB.METHODOLOGY.PROJECT_GROUPS.FORM.ADD_HEADER'),
      width: '40%',
    });

    ref.onClose.subscribe(editedGroup => {
      if (editedGroup) {
        this.projectGroupsService.projectGroupsCreate(editedGroup).subscribe(_r => this.loadData());
      }
    });
  }

  deleteGroup(group: ProjectGroup) {
    if (group.subgroups && group.subgroups.length > 0) {
      this.messageService.add({
        severity: 'error',
        summary: this.translateService.instant('ERRORS.DELETE_ERROR'),
        detail: this.translateService.instant('KB.METHODOLOGY.PROJECT_GROUPS.DELETE_ERROR_HAS_SUBS'),
      });
      return;
    }
    if (group.projects && group.projects.length > 0) {
      this.messageService.add({
        severity: 'error',
        summary: this.translateService.instant('ERRORS.DELETE_ERROR'),
        detail: this.translateService.instant('KB.METHODOLOGY.PROJECT_GROUPS.DELETE_ERROR_HAS_PROJECTS'),
      });
      return;
    }

    this.confirmationService.confirm({
      header: this.translateService.instant('KB.METHODOLOGY.PROJECT_GROUPS.DELETE_HEADER'),
      message: this.translateService.instant('KB.METHODOLOGY.PROJECT_GROUPS.DELETE_MSG'),
      accept: () => this.projectGroupsService.projectGroupsDelete(group.id)
        .subscribe(_r => {
          this.loadData();
          this.selectedNode = null;
          this.selectedGroup = null;
        }),
    });

  }

  editUserRole(userRole?: UserRole) {
    const data: UserRoleFormComponentData = {
      type: 'group',
      userRole,
      projectGroupId: this.selectedGroup.id,
      isDepartmentsResponsible: true,
    };

    const ref = this.dialogService.open(UserRoleFormComponent, {
      header: userRole ? this.translateService.instant('KB.METHODOLOGY.PROJECT_GROUPS.USER_FORM.EDIT_HEADER') :
        this.translateService.instant('KB.METHODOLOGY.PROJECT_GROUPS.USER_FORM.ADD_HEADER'),
      width: '50%',
      data,
    });

    ref.onClose.subscribe(editedUserRole => {
      if (editedUserRole) {
        this.loadUserRoles(this.selectedGroup)
          .subscribe(roles => this.groupTeamSubject.next(roles));
      }
    });
  }

  deleteUserRole(userRole: UserRole) {
    this.confirmationService.confirm({
      header: this.translateService.instant('KB.METHODOLOGY.PROJECT_GROUPS.TEAM_DELETE_HEADER'),
      message: this.translateService.instant('KB.METHODOLOGY.PROJECT_GROUPS.TEAM_DELETE_MSG'),
      accept: () => this.userRolesService.userRolesDelete(userRole.id)
        .pipe(
          switchMap(() => this.loadUserRoles(this.selectedGroup)),
        )
        .subscribe(roles => this.groupTeamSubject.next(roles)),
    });

  }

  loadUserRoles(group: ProjectGroup): Observable<Array<UserRole>> {
    return this.userRolesService.userRolesList({projectGroup: group.id.toString(), include: 'user__telephones, role, department'});
  }

}
