import {ChangeDetectorRef, Component, OnInit} from '@angular/core';
import {BehaviorSubject, finalize, forkJoin} from 'rxjs';
import {TranslateService} from '@ngx-translate/core';
import {DialogService} from 'primeng/dynamicdialog';
import {ConfirmationService, MessageService} from 'primeng/api';
import {map, takeUntil} from 'rxjs/operators';
import {Unsubscriber} from '@core-project/services/unsubscriber.service';
import {ProjectMeta, ProjectMetaGroup, ProjectMetaGroupsService, ProjectMetaService} from 'data-access/sdk/api2';
import {ProjectMetaGroupFormComponent} from '@core-project/modules/forms/project-meta-group-form/project-meta-group-form.component';
import {ProjectMetaFormComponent} from '@core-project/modules/forms/project-meta-form/project-meta-form.component';


@Component({
  selector: 'app-project-metas',
  templateUrl: './project-metas.component.html',
  styleUrls: ['./project-metas.component.scss'],
  providers: [Unsubscriber],
})
export class ProjectMetasComponent implements OnInit {
  readonly groupsSubject: BehaviorSubject<ProjectMetaGroup[]> = new BehaviorSubject([]);
  groups: ProjectMetaGroup[] = [];

  constructor(
    private readonly translateService: TranslateService,
    private readonly projectMetaService: ProjectMetaService,
    private readonly projectMetaGroupsService: ProjectMetaGroupsService,
    private readonly dialogService: DialogService,
    private readonly messageService: MessageService,
    private readonly confirmationService: ConfirmationService,
    private readonly unsubscriber: Unsubscriber,
    private readonly changeDetectorRef: ChangeDetectorRef,
  ) {
  }

  ngOnInit(): void {
    this.loadGroups();

    this.groupsSubject
      .pipe(
        takeUntil(this.unsubscriber.destroy$),
      )
      .subscribe(groups => {
        this.groups = groups;
        this.changeDetectorRef.markForCheck();
      });
  }

  reorderGroups() {
    forkJoin(
      this.groups.map((group, index) => this.projectMetaGroupsService.projectMetaGroupsPartialUpdate({
          id: group.id, data: {
            ...group,
            order: index,
          },
        })),
    )
      .pipe(
        finalize(() => {
          this.loadGroups();
        }),
      )
      .subscribe();
  }

  reorderMetaFields(metaFields: ProjectMeta[]) {
    forkJoin(
      metaFields.map((metaField, index) => this.projectMetaService.projectMetaPartialUpdate({
          id: metaField.id, data: {
            ...metaField,
            order: index,
          },
        })),
    )
      .pipe(
        finalize(() => {
          this.loadGroups();
        }),
      )
      .subscribe();

  }

  editGroup(group?: ProjectMetaGroup) {
    const ref = this.dialogService.open(ProjectMetaGroupFormComponent, {
      header: group ? this.translateService.instant('KB.METHODOLOGY.PROJECT_METAS.GROUP_FORM.EDIT_HEADER') :
        this.translateService.instant('KB.METHODOLOGY.PROJECT_METAS.GROUP_FORM.ADD_HEADER'),
      width: '40%',
      data: {group},
    });

    ref.onClose.subscribe(editedGroup => {
      if (editedGroup) {
        if (group) {
          this.projectMetaGroupsService.projectMetaGroupsPartialUpdate({id: group.id, data: editedGroup})
            .subscribe(() => this.loadGroups());
        } else {
          this.projectMetaGroupsService.projectMetaGroupsCreate(editedGroup)
            .subscribe(() => this.loadGroups());
        }
      }
    });
  }

  deleteGroup(group: ProjectMetaGroup) {
    if (group.meta_fields?.length > 0) {
      this.messageService.add({
        severity: 'error',
        summary: this.translateService.instant('ERRORS.DELETE_ERROR'),
        detail: this.translateService.instant('KB.METHODOLOGY.PROJECT_METAS.GROUP_NOT_EMPTY_ERROR'),
      });
      return;
    }
    this.confirmationService.confirm({
      header: this.translateService.instant('KB.METHODOLOGY.PROJECT_METAS.GROUP_DELETE_HEADER'),
      message: this.translateService.instant('KB.METHODOLOGY.PROJECT_METAS.GROUP_DELETE_MSG'),
      accept: () => this.projectMetaGroupsService.projectMetaGroupsDelete(group.id).subscribe(_r => this.loadGroups()),
    });
  }

  editMeta(meta?: ProjectMeta) {
    const ref = this.dialogService.open(ProjectMetaFormComponent, {
      header: meta ? this.translateService.instant('KB.METHODOLOGY.PROJECT_METAS.META_FORM.EDIT_HEADER') :
        this.translateService.instant('KB.METHODOLOGY.PROJECT_METAS.META_FORM.ADD_HEADER'),
      width: '60%',
      data: {meta},
    });

    ref.onClose.subscribe(editedMeta => {
      if (editedMeta) {
        if (meta) {
          this.projectMetaService.projectMetaPartialUpdate({id: meta.id, data: editedMeta})
            .subscribe(() => this.loadGroups());
        } else {
          this.projectMetaService.projectMetaCreate(editedMeta)
            .subscribe(() => this.loadGroups());
        }
      }
    });
  }

  deleteMeta(meta: ProjectMeta) {
    this.confirmationService.confirm({
      header: this.translateService.instant('KB.METHODOLOGY.PROJECT_METAS.META_DELETE_HEADER'),
      message: this.translateService.instant('KB.METHODOLOGY.PROJECT_METAS.META_DELETE_MSG'),
      accept: () => this.projectMetaService.projectMetaDelete(meta.id)
        .subscribe(() => this.loadGroups()),
    });
  }

  private loadGroups() {
    this.projectMetaGroupsService.projectMetaGroupsList('meta_fields__project_types')
      .pipe(
        map(groups => this.sortGroups(groups)),
      )
      .subscribe((groups => this.groupsSubject.next(groups)));
  }

  private sortGroups(groups: ProjectMetaGroup[]) {
    groups.sort((a, b) => a.order - b.order);
    groups.forEach(group => {
      group.rel_meta_fields = this.sortMetaFields(group.rel_meta_fields);
    });
    return groups;
  }

  private sortMetaFields(metaFields: ProjectMeta[]) {
    metaFields.sort((a, b) => a.order - b.order);
    return metaFields;
  }
}
