import { ChangeDetectionStrategy, Component, OnInit } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';

import { ProjectEnums } from '@core-project/enums/project';

import { StageModel } from 'data-access/sdk/internal-api';

interface StageFGInterface {
  id: FormControl<number>;
  name: FormControl<string>;
  status: FormControl<ProjectEnums.StatusEnum>;
  order: FormControl<number>;
  company: FormControl<number>;
  oldName: FormControl<null | ProjectEnums.StageEnum>;
}

export interface EditStageModalData {
  stage: StageModel;
  stagesForReplace?: StageModel[];
  deleteFn?: (stageId: number, replaceWithStage: number) => void;
  updateFn?: (stage: StageModel) => void;
}

@Component({
  selector: 'app-edit-stage-modal',
  templateUrl: './edit-stage-modal.component.html',
  styleUrls: ['./edit-stage-modal.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class EditStageModalComponent implements OnInit {
  formGroup: FormGroup<StageFGInterface>;
  stagesForReplace: StageModel[] = [];
  canDelete = !!this.dialogConfig.data.deleteFn;

  constructor(
    private readonly dynamicDialogRef: DynamicDialogRef,
    private readonly dialogConfig: DynamicDialogConfig<EditStageModalData>
  ) {}

  get statusFC() {
    return this.formGroup.get('status') as FormControl<ProjectEnums.StatusEnum>;
  }

  ngOnInit() {
    this.formGroup = this.createFG();
    this.stagesForReplace = this.dialogConfig.data.stagesForReplace;
  }

  createFG(stage = this.dialogConfig.data.stage) {
    if (!this.dialogConfig.data.stage) {
      console.error('No stage passed to EditStageModalComponent');
      this.close();
      return;
    }

    return new FormGroup<StageFGInterface>({
      id: new FormControl(stage.id, [Validators.required]),
      status: new FormControl<ProjectEnums.StatusEnum>(stage.status, [Validators.required]),
      name: new FormControl<string>(stage.name, [Validators.required]),
      company: new FormControl<number>(stage.companyId),
      oldName: new FormControl<null | ProjectEnums.StageEnum>(stage.oldName),
      order: new FormControl<number>(stage.order),
    });
  }

  close() {
    this.dynamicDialogRef.close();
  }

  save() {
    if (this.formGroup.invalid) {
      return;
    }

    this.dialogConfig.data.updateFn({
      id: this.formGroup.value.id,
      companyId: this.formGroup.value.company,
      name: this.formGroup.value.name,
      status: this.formGroup.value.status,
      order: this.formGroup.value.order,
      oldName: this.formGroup.value.oldName,
    });
  }

  delete(replaceWithStage: StageModel['id']) {
    this.dialogConfig.data.deleteFn(this.formGroup.value.id, replaceWithStage);
  }
}
