import {Component, ElementRef, Inject, LOCALE_ID, OnDestroy, OnInit, ViewChild, ViewEncapsulation} from '@angular/core';
import {WorkTemplate} from 'data-access/sdk/api2';
import {ProjectType} from 'data-access/sdk/api2';
import {Gantt, GanttStatic} from '../../../../../assets/dhtmlxgantt/dhtmlxgantt';
import {TranslateService} from '@ngx-translate/core';
import {DialogService} from 'primeng/dynamicdialog';
import {WorktemplatesService} from 'data-access/sdk/api2';
import {WorklinktemplatesService} from 'data-access/sdk/api2';
import {formatDate} from '@angular/common';
import {DialogEnums} from '@core-project/enums';
import {defaultIfEmpty, firstValueFrom, forkJoin, Observable} from 'rxjs';
import {
  GanttToWorkLinkTemplateAdapter,
  GanttToWorkTemplateAdapter,
  GanttWork,
  WorkLinkTemplateToGanttAdapter,
  WorkLinkToGanttAdapter,
  WorkTemplateToGanttAdapter,
} from '@core-project/helpers/gantt-adapter';
import {ProjectTypesService} from 'data-access/sdk/api2';
import {templateScales} from '@core-project/helpers/gantt-common';
import {dbStringToJsDate} from '@core-project/helpers';
import {WorkTemplateFormComponent} from '@core-project/modules/shared-components/works/work-template-form/work-template-form.component';
import {
  WorklinktemplateInfoComponent,
} from '@core-project/modules/shared-components/works/worklinktemplate-info/worklinktemplate-info.component';
import {Store} from '@ngxs/store';
import {GlobalRulesSelectors} from 'data-access/store';

@Component({
  selector: 'app-work-templates',
  templateUrl: './work-templates.component.html',
  styleUrls: ['../../../../../assets/dhtmlxgantt/dhtmlxgantt.css', './work-templates.component.scss'],
  encapsulation: ViewEncapsulation.None,
  providers: [],
})
export class WorkTemplatesComponent implements OnInit, OnDestroy {
  @ViewChild('ganttChart') ganttContainer: ElementRef;

  projectTypeId: number;
  projectTypes$: Observable<ProjectType[]>;

  private gi: GanttStatic;
  private scales = templateScales;

  constructor(
    @Inject(LOCALE_ID) private readonly locale: string,
    private readonly translateService: TranslateService,
    private readonly dialogService: DialogService,
    private readonly workTemplatesService: WorktemplatesService,
    private readonly workLinkTemplatesService: WorklinktemplatesService,
    private readonly projectTypesService: ProjectTypesService,
    private readonly store: Store,
  ) {
  }

  private get canEdit() {
    return this.store.selectSnapshot(GlobalRulesSelectors.canUpdate('worklinktemplate'));
  }

  ngOnInit() {
    this.projectTypes$ = this.projectTypesService.projectTypesList();
    this.gi = Gantt.getGanttInstance({
      config: {
        date_format: '%Y-%m-%d %H:%i',
        auto_scheduling: true,
        auto_scheduling_strict: false,
        auto_scheduling_initial: true,
        auto_types: true,
        autoscroll: true,
        autosize: true,
        drag_links: this.canEdit,
        drag_move: this.canEdit,
        drag_mode: this.canEdit ? {resize: 'resize', progress: 'progress', move: 'move', ignore: 'ignore'} : 'ignore',
        drag_progress: false,
        order_branch: this.canEdit,
        order_branch_free: this.canEdit,
        placeholder_task: false,
        preserve_scroll: true,
        readonly: false,
        scales: this.scales.day, // TODO
        scale_height: 54,
        start_date: dbStringToJsDate('1970-01-01'),
        end_date: dbStringToJsDate('1971-01-01'),
        show_tasks_outside_timescale: true,
      },
      templates: {
        grid_date_format: (date, _column) => formatDate(date, 'shortDate', this.locale),

        task_end_date: (date) => this.gi.templates.task_date(new Date(date.valueOf() - 1)),

        grid_folder: (item) =>
          '<div class="gantt_tree_icon text-center"><i class="fa-regular fa-folder' + (item.$open ? '-open' : '') + '"></i></div>',

        grid_file: (item) => {
          if (item.type === 'milestone') {
            return '<div class="gantt_tree_icon text-center"><i class="fa-regular fa-flag-swallowtail"></i></div>';
          }
          return '<div class="gantt_tree_icon text-center"><i class="fa-regular fa-file"></i></div>';
        },
      },
      plugins: {
        auto_scheduling: true,
        drag_timeline: true,
      },
    });
    this.configureGantt();
  }

  ngOnDestroy() {
    this.gi.destructor();
  }

  loadData() {
    const w$ = this.workTemplatesService.worktemplatesList({projectType: this.projectTypeId.toString()});
    const l$ = this.workLinkTemplatesService.worklinktemplatesList({sourceProjectType: this.projectTypeId.toString()});

    firstValueFrom(forkJoin([w$, l$])).then((res) => {
      const works = res[0].map((work) => WorkTemplateToGanttAdapter(work));
      const links = res[1].map((link) => WorkLinkTemplateToGanttAdapter(link));
      this.gi.clearAll();
      this.gi.parse({tasks: works, links: links});
      this.initGantt();
    });
  }

  private configureGantt() {
    this.gi.i18n.setLocale(this.locale);
    const textEditor = {type: 'text', map_to: 'text'};
    const durationEditor = {type: 'number', map_to: 'duration', min: 0};

    if (!this.canEdit) {
      this.gi.config.columns = [
        {
          name: 'wbs',
          label: '#',
          width: 60,
          align: 'center',
          template: this.gi.getWBSCode,
        },
        {
          name: 'text',
          label: this.translateService.instant('GANTT.COLUMNS.TASK'),
          tree: true,
          width: 300,
          resize: true,
        },
        {
          name: 'duration',
          label: this.translateService.instant('GANTT.COLUMNS.DURATION'),
          width: 80,
          align: 'center',
          resize: true,
        },
      ];

      // force empty event
      this.gi.attachEvent('onTaskClick', (id, _e) => {
      }, null);
    } else {
      this.gi.config.columns = [
        {
          name: 'wbs',
          label: '#',
          width: 60,
          align: 'center',
          template: this.gi.getWBSCode,
        },
        {
          name: 'text',
          label: this.translateService.instant('GANTT.COLUMNS.TASK'),
          tree: true,
          width: 300,
          resize: true,
          editor: textEditor,
        },
        {
          name: 'duration',
          label: this.translateService.instant('GANTT.COLUMNS.DURATION'),
          width: 80,
          align: 'center',
          resize: true,
          editor: durationEditor,
        },
        {name: 'add'},
      ];

      this.gi.attachEvent(
        'onTaskClick',
        (id, _e) => {
          if (this.gi.getTask(id).type !== 'project') {
            this.openWork(id);
          }
          return false;
        },
        null,
      );

      this.gi.attachEvent(
        'onBeforeLightbox',
        (id) => {
          this.openWork(id);
          return false;
        },
        null,
      );

      this.gi.attachEvent(
        'onAfterTaskUpdate',
        (id, task) => {
          this.updateWork(task);
        },
        null,
      );

      this.gi.attachEvent(
        'onRowDragEnd',
        (id, _target) => {
          const task = this.gi.getTask(id);
          this.updateWork(task);

          const children = this.gi.getChildren(task.parentWorkId);
          const updates$ = [];
          children.forEach((c) => {
            if (c === id) {
              return;
            }
            const tsk = this.gi.getTask(c);
            const index = tsk.$local_index;
            if (index !== tsk.order) {
              // @ts-ignore
              updates$.push(this.workTemplatesService.worktemplatesPartialUpdate({id: tsk.id, data: {order: index}}));
            }
          });
          firstValueFrom(forkJoin(updates$).pipe(defaultIfEmpty(null))).then();
        },
        null,
      );

      this.gi.attachEvent(
        'onAfterLinkAdd',
        (id, item) => {
          firstValueFrom(this.workLinkTemplatesService.worklinktemplatesCreate(GanttToWorkLinkTemplateAdapter(item))).then((wl) =>
            this.gi.changeLinkId(id, wl.id),
          );
        },
        null,
      );

      this.gi.attachEvent(
        'onAfterLinkUpdate',
        (id, item) => {
          firstValueFrom(
            this.workLinkTemplatesService.worklinktemplatesPartialUpdate({id: id, data: GanttToWorkLinkTemplateAdapter(item)}),
          ).then();
        },
        null,
      );

      this.gi.attachEvent(
        'onLinkClick',
        (linkId, _e) => {
          const ref = this.dialogService.open(WorklinktemplateInfoComponent, {
            header: this.translateService.instant('GANTT.EDIT_LINK'),
            width: '40%',
            data: {
              linkId: linkId,
            },
          });

          firstValueFrom(ref.onClose).then((action) => {
            if (action === DialogEnums.ActionEnum.DEL) {
              this.gi.deleteLink(linkId);
            } else {
              firstValueFrom(this.workLinkTemplatesService.worklinktemplatesRead({id: linkId})).then((wl) => {
                const ganttLink = this.gi.getLink(linkId);
                Object.assign(ganttLink, WorkLinkToGanttAdapter(wl));
                this.gi.refreshLink(linkId);
              });
            }
          });
          return false;
        },
        null,
      );
    }
  }

  private openWork(id: string) {
    const ganttWork = this.gi.getTask(id);
    const oldWT = GanttToWorkTemplateAdapter(ganttWork);

    const ref = this.dialogService.open(WorkTemplateFormComponent, {
      header: this.translateService.instant('WORKS.ADD_WORK'),
      width: '50%',
      data: {
        workTemplate: oldWT,
        projectTypeId: this.projectTypeId,
      },
    });

    firstValueFrom(ref.onClose).then((newWT) => {
      if (newWT) {
        if (newWT === DialogEnums.ActionEnum.DEL) {
          this.gi.deleteTask(id);
          return;
        }
        if (!oldWT.id) {
          this.gi.changeTaskId(id, newWT.id);
        }
        this.updateGanttWork(newWT);
        this.gi.setSizes();
      }
    });
  }

  private updateWork(ganttWork: GanttWork) {
    let obs$: Observable<WorkTemplate>;
    if (ganttWork.workTemplate) {
      obs$ = this.workTemplatesService.worktemplatesPartialUpdate({
        id: Number(ganttWork.id),
        data: GanttToWorkTemplateAdapter(ganttWork),
      });
    } else {
      ganttWork.workTemplate = {project_type: this.projectTypeId, name: ''};
      obs$ = this.workTemplatesService.worktemplatesCreate(GanttToWorkTemplateAdapter(ganttWork));
    }

    firstValueFrom(obs$).then((w) => {
      if (!ganttWork.workTemplate.id) {
        this.gi.changeTaskId(ganttWork.id, w.id);
      }
      this.updateGanttWork(w, ganttWork);
    });
  }

  private updateGanttWork(work: WorkTemplate, ganttWork: GanttWork = null) {
    if (!ganttWork) {
      ganttWork = this.gi.getTask(work.id.toString());
    }

    if (!ganttWork) {
      return;
    }
    Object.assign(ganttWork, WorkTemplateToGanttAdapter(work));
    ganttWork.workTemplate = work;
    this.gi.refreshTask(ganttWork.id);
    if (work.group) {
      const gr = this.gi.getTask(work.group.toString());
      this.updateGroupDates(gr);
    }
  }

  private updateGroupDates(group: GanttWork) {
    this.gi.resetProjectDates(group);
    this.gi.refreshTask(group.id);
    if (group.parent) {
      this.updateGroupDates(this.gi.getTask(group.parent.toString()));
    }
  }

  private initGantt() {
    if (this.ganttContainer) {
      this.gi.init(this.ganttContainer.nativeElement);
    }
  }
}
