import {Component, OnInit} from '@angular/core';
import {TranslateService} from '@ngx-translate/core';
import {AuthService, Department, DepartmentsService, User, UsersService} from 'data-access/sdk/api2';
import {ConfirmationService, MessageService, TreeNode} from 'primeng/api';
import {firstValueFrom, forkJoin, Observable} from 'rxjs';
import {BreadcrumbService} from '@core-project/services/breadcrumb.service';
import {DialogService} from 'primeng/dynamicdialog';
import {DepartmentFormComponent} from '@core-project/modules/forms/department-form/department-form.component';
import {UserFormComponent} from '@core-project/modules/forms/user-forms/user-form/user-form.component';
import {AuthenticationService} from '@core-project/services/authentication.service';
import * as _ from 'lodash-es';
import {Router} from '@angular/router';

@Component({
  selector: 'app-company-structure',
  templateUrl: './company-structure.component.html',
  styleUrls: ['./company-structure.component.scss'],
})
export class CompanyStructureComponent implements OnInit {
  user: User;
  permissions: string[];
  orgTree: Array<TreeNode>;
  selectedNode: TreeNode | TreeNode[];
  selectedDepartment: Department;
  selectedUser: User;
  defaultAvatar = 'assets/images/default_photo.png';

  constructor(
    private translateService: TranslateService,
    private usersService: UsersService,
    private departmentsService: DepartmentsService,
    private breadcrumbService: BreadcrumbService,
    private dialogService: DialogService,
    private messageService: MessageService,
    private confirmationService: ConfirmationService,
    private authService: AuthService,
    private authenticationService: AuthenticationService,
    private router: Router,
  ) {
  }

  ngOnInit() {
    this.breadcrumbService.addItems([{label: this.translateService.instant('KB.MENU.STRUCTURE')}]);
    this.loadRootData();
    this.user = this.authenticationService.user;
    this.usersService
      .usersPermissions({id: this.user.id, company: this.user.company})
      .subscribe((perms) => (this.permissions = perms.permissions));
  }

  getTLink(link: string) {
    return `https://t.me/${link}`;
  }

  loadRootData() {
    this.loadNode();
  }

  async loadNode(node?: TreeNode) {
    const res = await firstValueFrom(forkJoin([this.getDepartments(node?.data), this.getUsers(node?.data)]));
    if (node) {
      node.children = [...res[0].map((d) => this.depToNode(d)), ...res[1].map((u) => this.userToNode(u))];
    } else {
      this.orgTree = [...res[0].map((d) => this.depToNode(d)), ...res[1].map((u) => this.userToNode(u))];
    }
    return;
  }

  getDepartments(parent: Department = null): Observable<Array<Department>> {
    const params: DepartmentsService.DepartmentsListParams = parent
      ? {parentDepartment: parent.id.toString()}
      : {parentDepartmentIsnull: 'true'};
    params.include = 'employees__telephones';
    return this.departmentsService.departmentsList(params);
  }

  getUsers(department: Department = null): Observable<Array<User>> {
    const params: UsersService.UsersListParams = department ? {department: department.id.toString()} : {departmentIsnull: 'true'};
    params.include = 'telephones';
    return this.usersService.usersListAll(params);
  }

  depToNode(department: Department): TreeNode {
    return {
      label: department.name,
      type: 'department',
      data: department,
      expanded: false,
      key: 'dep_' + department.id.toString(),
      leaf: !(department.employees.length > 0 || department.subdepartments.length > 0),
    };
  }

  userToNode(user: User): TreeNode {
    return {
      label: user.full_name,
      data: user,
      type: 'user',
      leaf: true,
      key: 'user_' + user.id.toString(),
    };
  }

  onExpand($event) {
    const node: TreeNode = $event.node;
    this.loadNode(node);
  }

  onSelect() {
    if ((this.selectedNode as TreeNode).type === 'department') {
      this.selectedDepartment = (this.selectedNode as TreeNode).data;
      this.selectedUser = null;
    } else {
      this.selectedDepartment = null;
      this.selectedUser = (this.selectedNode as TreeNode).data;
    }
  }

  editDepartment(dep?: Department) {
    const ref = this.dialogService.open(DepartmentFormComponent, {
      header: dep
        ? this.translateService.instant('KB.STRUCTURE.DEP_FORM.HEADER_EDIT')
        : this.translateService.instant('KB.STRUCTURE.DEP_FORM.HEADER_ADD'),
      width: '40%',
      data: {
        department: dep,
      },
    });

    ref.onClose.subscribe((editedDep) => {
      if (editedDep) {
        if (dep) {
          this.departmentsService.departmentsPartialUpdate({id: dep.id, data: editedDep}).subscribe((_r) => {
            if (dep.parent_department !== editedDep.parent_department) {
              this.removeNode((this.selectedNode as TreeNode));
              if (editedDep.parent_department) {
                const depNode = this.findObjectNodeById(editedDep.parent_department, this.orgTree, 'department');
                if (depNode.leaf) {
                  depNode.leaf = false;
                } else {
                  this.loadNode(depNode).then(
                    (_rr) => (this.selectedNode = this.findObjectNodeById(editedDep.id, depNode.children, 'department')),
                  );
                }
              } else {
                this.loadRootData();
              }
            }

            Object.assign(this.selectedDepartment, editedDep);
            Object.assign((this.selectedNode as TreeNode), editedDep);
            (this.selectedNode as TreeNode).label = editedDep.name;
          });
        } else {
          this.departmentsService.departmentsCreate(editedDep).subscribe((_r) => {
            if (!editedDep.parent_department) {
              this.loadRootData();
            } else {
              const depNode = this.findObjectNodeById(editedDep.parent_department, this.orgTree, 'department');
              if (depNode.leaf) {
                depNode.leaf = false;
              } else {
                this.loadNode(depNode).then(
                  (_rr) => (this.selectedNode = this.findObjectNodeById(editedDep.id, depNode.children, 'department')),
                );
              }
            }
          });
        }
      }
    });
  }

  deleteDepartment(dep: Department) {
    if (dep.subdepartments.length > 0) {
      this.messageService.add({
        severity: 'error',
        summary: this.translateService.instant('ERRORS.DELETE_ERROR'),
        detail: this.translateService.instant('KB.STRUCTURE.DEP_HAS_DEPS_ERROR'),
      });
      return;
    }

    if (dep.employees.length > 0) {
      this.messageService.add({
        severity: 'error',
        summary: this.translateService.instant('ERRORS.DELETE_ERROR'),
        detail: this.translateService.instant('KB.STRUCTURE.DEP_HAS_USERS_ERROR'),
      });
      return;
    }

    this.confirmationService.confirm({
      header: this.translateService.instant('KB.STRUCTURE.DEP_DEL_HEADER'),
      message: this.translateService.instant('KB.STRUCTURE.DEP_DEL_MSG'),
      accept: () => {
        this.departmentsService.departmentsDelete(dep.id).subscribe((_r) => {
          this.removeNode((this.selectedNode as TreeNode));
          this.selectedDepartment = null;
          this.selectedNode = null;
        });
      },
    });
  }

  editUser(user?: User) {
    const ref = this.dialogService.open(UserFormComponent, {
      header: user
        ? this.translateService.instant('KB.STRUCTURE.USER_FORM.HEADER_EDIT')
        : this.translateService.instant('KB.STRUCTURE.USER_FORM.HEADER_ADD'),
      width: '60%',
      data: {
        user: user,
        telephones: user?.rel_telephones,
      },
    });

    ref.onClose.subscribe((editedUser) => {
      if (!editedUser) {
        return;
      }
      if (user) {
        if (user.department !== editedUser.department) {
          // delete user from current parent children
          this.removeNode((this.selectedNode as TreeNode));
          if (editedUser.department) {
            const depNode = this.findObjectNodeById(editedUser.department, this.orgTree, 'department');
            if (depNode.leaf) {
              depNode.leaf = false;
            } else {
              this.loadNode(depNode).then((_r) => (this.selectedNode = this.findObjectNodeById(editedUser.id, depNode.children, 'user')));
            }
          } else {
            this.loadRootData();
          }
        }

        Object.assign(this.selectedUser, editedUser);
        Object.assign((this.selectedNode as TreeNode).data, editedUser);
        (this.selectedNode as TreeNode).label = editedUser.full_name;
      } else {
        if (editedUser.department) {
          const depNode = this.findObjectNodeById(editedUser.department, this.orgTree, 'department');
          if (depNode.leaf) {
            depNode.leaf = false;
          } else {
            this.loadNode(depNode);
          }
        } else {
          this.loadRootData();
        }
      }
    });
  }

  deleteUser(user: User) {
    if (this.authenticationService.user.id === user.id) {
      this.messageService.add({
        severity: 'error',
        summary: this.translateService.instant('ERRORS.ERROR'),
        detail: this.translateService.instant('KB.STRUCTURE.USER_DEL_SELF_ERROR_MSG'),
      });
      return;
    }

    this.confirmationService.confirm({
      header: this.translateService.instant('KB.STRUCTURE.USER_DEL_HEADER'),
      message: this.translateService.instant('KB.STRUCTURE.USER_DEL_MSG'),
      accept: () => {
        this.authService.authUsersDelete(user.id).subscribe((_r) => {
          this.messageService.add({
            severity: 'success',
            summary: this.translateService.instant('KB.STRUCTURE.USER_DELETED_HEADER'),
            detail: this.translateService.instant('KB.STRUCTURE.USER_DELETED_MSG', {name: user.full_name}),
          });
          this.removeNode((this.selectedNode as TreeNode));
          this.selectedNode = null;
          this.selectedUser = null;
        });
      },
    });
  }

  toggleActiveUser(user: User) {
    if (this.authenticationService.user.id === user.id) {
      this.messageService.add({
        severity: 'error',
        summary: this.translateService.instant('ERRORS.ERROR'),
        detail: this.translateService.instant('KB.STRUCTURE.USER_DEACT_SELF_ERROR_MSG'),
      });
      return;
    }
    const header = user.is_active
      ? this.translateService.instant('KB.STRUCTURE.USER_DEACT_HEADER')
      : this.translateService.instant('KB.STRUCTURE.USER_ACT_HEADER');
    const message = user.is_active
      ? this.translateService.instant('KB.STRUCTURE.USER_DEACT_MSG')
      : this.translateService.instant('KB.STRUCTURE.USER_ACT_MSG');
    this.confirmationService.confirm({
      header: header,
      message: message,
      accept: () =>
        this.authService
          .authUsersPartialUpdate({id: user.id, data: {email: user.email, is_active: !user.is_active}})
          .subscribe((_u) => (this.selectedUser.is_active = !this.selectedUser.is_active)),
    });
  }

  impersonateUser(userId: number) {
    this.authenticationService.impersonate(userId)
      .subscribe(() => this.router.navigate(['/profile']));
  }

  private removeNode(node: TreeNode) {
    if (node.parent) {
      node.parent.children = _.reject(node.parent.children, ['key', node.key]);
      if (node.parent.children.length === 0) {
        node.parent.leaf = true;
      }
    } else {
      this.orgTree = _.reject(this.orgTree, ['key', node.key]);
    }
  }

  private findObjectNodeById(id: number, nodes: TreeNode[], type: 'department' | 'user'): TreeNode {
    let res: TreeNode = _.find(nodes, (n) => n.type === type && n.data.id === id);
    if (res === undefined) {
      nodes.forEach((n) => {
        if (n.children && n.children.length > 0) {
          res = this.findObjectNodeById(id, n.children, type);
          if (res !== undefined) {
            return res;
          }
        }
      });
    }
    return res;
  }
}
