import {Component, Inject, LOCALE_ID, OnInit, ViewEncapsulation} from '@angular/core';
import {ActivatedRoute, Router} from '@angular/router';
import {MessageService} from 'primeng/api';
import {ProjectFormComponent} from '@core-project/modules/forms/project-form/project-form.component';
import {BehaviorSubject, Observable, take} from 'rxjs';
import {DialogService} from 'primeng/dynamicdialog';

import {TranslateService} from '@ngx-translate/core';
import {
  FinanceService,
  Project,
  ProjectBudget,
  ProjectMember,
  ProjectMembersService,
  ProjectMeta,
  ProjectMetaGroup,
  ProjectMetaGroupsService,
  ProjectsService,
  ProjectState,
  ProjectStatesService,
  ProjectTypeStage,
  ProjectTypeStagesService,
  ProjectWorkPlan,
  UsersService,
  Work,
  WorkplansService,
  WorksService,
} from 'data-access/sdk/api2';
import {ProjectExtra} from '@core-project/interfaces/projects';
import {dbStringToJsDate, jsDateToDbString, projectToProjectExtra} from '@core-project/helpers';
import {formatDate} from '@angular/common';
import {ProjectMetaValuesFormComponent} from '@core-project/modules/forms/project-meta-values-form/project-meta-values-form.component';
import {AuthenticationService} from '@core-project/services/authentication.service';
import {ProjectEnums} from '@core-project/enums/project';
import * as _ from 'lodash-es';
import {WorkEnums} from '@core-project/enums';
import {ProjectStateFormComponent} from '@core-project/modules/forms/project-state-form/project-state-form.component';
import {ProjectStateHistoryComponent} from '@core-project/modules/shared-components/project-state-history/project-state-history.component';
import StatusEnum = ProjectEnums.StatusEnum;
import WorkTypeEnum = WorkEnums.WorkTypeEnum;

/**
 * @deprecated use `ProjectDetailsModule`
 * Будет Удален после завершения рефакторинга
 */

@Component({
  selector: 'app-project-details',
  templateUrl: './project-details-old.component.html',
  styleUrls: ['./project-details-old.component.scss'],
  encapsulation: ViewEncapsulation.None,
})
export class ProjectDetailsOldComponent implements OnInit {
  projectId: number;
  project: ProjectExtra;
  metaGroups: Array<ProjectMetaGroup>;
  perms: Array<string>;

  projectBudgetSubject: BehaviorSubject<ProjectBudget> = new BehaviorSubject(undefined);
  projectBudget$: Observable<ProjectBudget> = this.projectBudgetSubject.asObservable();

  workPlanSubject: BehaviorSubject<ProjectWorkPlan> = new BehaviorSubject(undefined);
  workPlan$: Observable<ProjectWorkPlan> = this.workPlanSubject.asObservable();

  membersSubject: BehaviorSubject<ProjectMember[]> = new BehaviorSubject([]);
  members$: Observable<ProjectMember[]> = this.membersSubject.asObservable();

  currentStateSubject: BehaviorSubject<ProjectState> = new BehaviorSubject(undefined);
  currentState$: Observable<ProjectState> = this.currentStateSubject.asObservable();

  statuses = StatusEnum;
  ptStages: ProjectTypeStage[];
  events: any[];

  private milestones: Work[];

  constructor(
    @Inject(LOCALE_ID) private readonly locale: string,
    private readonly activatedRoute: ActivatedRoute,
    private readonly router: Router,
    private readonly projectsService: ProjectsService,
    private readonly projectMetaGroupsService: ProjectMetaGroupsService,
    private readonly financeService: FinanceService,
    private readonly messageService: MessageService,
    private readonly dialogService: DialogService,
    private readonly translateService: TranslateService,
    private readonly usersService: UsersService,
    private readonly authenticationService: AuthenticationService,
    private readonly worksService: WorksService,
    private readonly workplansService: WorkplansService,
    private readonly projectMembersService: ProjectMembersService,
    private readonly projectStatesService: ProjectStatesService,
    private readonly projectTypeStagesService: ProjectTypeStagesService,
  ) {
  }

  get currentStageIndex(): number {
    if (!this.ptStages) {
      return 0;
    }
    return _.findIndex(this.ptStages, ['stage', this.project.stage]);
  }

  get isFirstStage(): boolean {
    return this.currentStageIndex === 0;
  }

  get isLastStage(): boolean {
    return this.currentStageIndex === this.ptStages?.length - 1;
  }

  ngOnInit() {
    this.activatedRoute.parent.params.subscribe((params) => {
      this.projectId = params.projectId;
      this.getProject();

      this.usersService
        .usersPermissions({
          id: this.authenticationService.user.id,
          project: this.projectId,
        })
        .subscribe((perms) => (this.perms = perms.permissions));
    });
  }

  editProject(project: Project) {
    const ref = this.dialogService.open(ProjectFormComponent, {
      header: this.translateService.instant('PROJECT.EDIT'),
      width: '50%',
      data: {
        project: project,
      },
    });

    ref.onClose.pipe(take(1)).subscribe((editedProject) => {
      if (editedProject) {
        this.projectsService.projectsPartialUpdate({id: project.id, data: editedProject}).subscribe((data) => {
          this.messageService.add({
            severity: 'success',
            summary: this.translateService.instant('PROJECTS.EDIT_SUCC.MSG', {name: data.name}),
          });

          this.getProject();
        });
      }
    });
  }

  editGroup(group: ProjectMetaGroup) {
    const ref = this.dialogService.open(ProjectMetaValuesFormComponent, {
      header: this.translateService.instant('PROJECT.EDIT_METAS_HEADER'),
      width: '50%',
      data: {
        projectId: this.projectId,
        metaFields: group.rel_meta_fields,
        metaValues: _.filter(this.project.rel_metas, (m) => group.meta_fields.includes(m.meta)),
      },
    });

    ref.onClose.subscribe((_r) => this.getProject());
  }

  getMetaValue(metaField: ProjectMeta) {
    const mfv = this.project.metas_by_id[metaField.id];
    if (mfv === null || mfv === undefined) {
      return null;
    }
    let value = mfv['value_' + metaField.data_type];
    if (value === null || value === undefined) {
      return null;
    }

    switch (metaField.data_type) {
      case ProjectEnums.DataTypeEnum.DT:
        value = formatDate(value, 'shortDate', this.locale);
        break;
      case ProjectEnums.DataTypeEnum.MCHS:
        value = value.join(', ');
        break;
      case ProjectEnums.DataTypeEnum.FLAG:
        value = value
          ? '<i class="fa-regular fa-check-circle text-green-500"></i>'
          : '<i class="fa-regular fa-times-circle text-red-500"></i>';
        break;
    }

    value = metaField.prefix ? metaField.prefix + ' ' + value : value;
    value = metaField.postfix ? value + ' ' + metaField.postfix : value;
    return value;
  }

  cancelProject(project: Project) {
    this.projectsService.projectsCancel({id: project.id, data: project}).subscribe((p) => Object.assign(this.project, p));
  }

  deleteProject(project: Project) {
    this.projectsService.projectsDelete(project.id).subscribe({
      complete: () => this.router.navigate(['projects']),
      error: () =>
        this.messageService.add({
          severity: 'error',
          summary: this.translateService.instant('PROJECT.DETAILS.DELETE_FAILURE'),
        }),
    });
  }

  pauseProject(project: Project) {
    this.projectsService.projectsPause({id: project.id, data: project}).subscribe((p) => Object.assign(this.project, p));
  }

  resumeProject(project: Project) {
    if (project.status === StatusEnum.PSP) {
      this.pauseProject(project);
    }
    if (project.status === StatusEnum.CNC) {
      this.cancelProject(project);
    }
  }

  moveStage(forward: boolean) {
    const newStage = this.ptStages[forward ? this.currentStageIndex + 1 : this.currentStageIndex - 1].stage;
    this.projectsService
       // @ts-ignore
      .projectsPartialUpdate({id: this.projectId, data: {stage: newStage}})
      .subscribe((p) => (this.project = Object.assign({}, this.project, p)));
  }

  editState(newState: boolean = false) {
    this.currentState$.pipe(take(1)).subscribe((state) => {
      let stateToForm = state;
      if (!state) {
        stateToForm = {
          project: this.projectId,
          state: '',
        };
      }

      if (state && newState) {
        delete stateToForm.id;
      }

      const ref = this.dialogService.open(ProjectStateFormComponent, {
        header: stateToForm.id
          ? this.translateService.instant('PROJECT.DETAILS.EDIT_STATE_FORM_HEADER')
          : this.translateService.instant('PROJECT.DETAILS.NEW_STATE_FORM_HEADER'),
        width: '40%',
        data: {state: stateToForm},
      });

      ref.onClose.pipe(take(1)).subscribe((_ps) => this.getCurrentState());
    });
  }

  showStateHistory() {
    this.dialogService.open(ProjectStateHistoryComponent, {
      header: this.translateService.instant('PROJECT.DETAILS.STATE_HISTORY'),
      width: '60%',
      data: {projectId: this.projectId},
    });
  }

  private getProject() {
    this.projectsService
      .projectsRead({
        id: this.projectId,
        include: 'customer, project_type, metas, project_group',
      })
      .subscribe((p) => {
        this.project = projectToProjectExtra(p);

        this.financeService.financeProjectBudgetsRead({id: this.project.project_budget}).subscribe((budget) => {
          this.projectBudgetSubject.next(budget);
        });

        if (this.project.work_plan) {
          this.workplansService.workplansRead({id: Number(this.project.work_plan)}).subscribe((workPlan) => {
            this.workPlanSubject.next(workPlan);
          });
        }
        this.projectMembersService
          .projectMembersList({project: this.project.id.toString(), include: 'user, department, organization'})
          .subscribe((members) => {
            this.membersSubject.next(members);
          });

        this.getStages();
        this.getMetaGroups();
        this.getMilestones();
        this.getCurrentState();
      });
  }

  private getMetaGroups() {
    this.projectMetaGroupsService.projectMetaGroupsList('meta_fields').subscribe((mgs) => {
      mgs.forEach((mg) => {
        mg.rel_meta_fields = _.filter(mg.rel_meta_fields, (mf) => mf.all_types || mf.project_types.includes(this.project.project_type));
        mg.meta_fields = mg.rel_meta_fields.map((mf) => mf.id);
      });
      _.remove(mgs, (mg) => mg.meta_fields.length === 0);
      this.metaGroups = mgs;
    });
  }

  private getMilestones() {
    this.worksService
      .worksList({
        project: this.projectId.toString(),
        workType: WorkTypeEnum.MS,
        milestoneLevel: 1,
      })
      .subscribe((milestones) => {
        this.milestones = milestones;
        this.createTimeline();
      });
  }

  private getCurrentState() {
    this.projectStatesService.projectStatesCurrent(this.projectId.toString()).subscribe((projectState) => {
      this.currentStateSubject.next(projectState);
    });
  }

  private getStages() {
    this.projectTypeStagesService
      .projectTypeStagesList({
        projectType: this.project.project_type.toString(),
        include: 'stage',
      })
      .subscribe((ptss) => (this.ptStages = ptss));
  }

  private createTimeline() {
    this.events = [];
    this.milestones.forEach((ms) =>
      this.events.push({
        status: ms.name,
        date: ms.start_eff,
        icon: (ms.status === WorkEnums.StatusEnum.AC ? 'fa-solid ' : 'fa-regular ') + 'fa-square text-purple-500 rotate-45',
      }),
    );
    if (this.project.status === StatusEnum.ACT) {
      this.events.push({
        status: this.translateService.instant('PROJECT.DETAILS.TIMELINE.TODAY'),
        date: jsDateToDbString(new Date()),
        icon: 'fa-solid fa-dash text-blue-500',
      });
    }
    this.events = _.sortBy(this.events, (m) => dbStringToJsDate(m.date));
  }
}
