import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  EventEmitter,
  Input,
  OnInit,
  Output,
  ViewChild,
} from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { BehaviorSubject, Observable, take } from 'rxjs';
import { InputNumber } from 'primeng/inputnumber';

import { ProjectMetaValue } from 'data-access/sdk/api2';

import { ProjectEnums } from '@core-project/enums/project';

import { IProjectDetailsMetaField } from '../../../containers/project-details-container/project-details-container.component';
import { InplaceDropdownOptionsInterface } from 'ui/forms-ui';

export interface IMetaData {
  metaField: IProjectDetailsMetaField;
  form: FormGroup;
}

@Component({
  selector: 'app-attributes-form',
  templateUrl: './attributes-form.component.html',
  styleUrls: ['./attributes-form.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AttributesFormComponent implements OnInit {
  private loadingDataSubject: BehaviorSubject<boolean> = new BehaviorSubject<boolean>(true);

  @ViewChild('inputNumberRef') inputNumberRef: InputNumber;
  @ViewChild('inputTextRef') inputTextRef: ElementRef;

  @Output() valueChange: EventEmitter<ProjectMetaValue> = new EventEmitter<ProjectMetaValue>();

  @Input() content: string;
  @Input() label: string;

  @Input() projectId: number;
  @Input() metaValues: ProjectMetaValue[];
  @Input() relMetaField: IProjectDetailsMetaField;

  loadingData$: Observable<boolean> = this.loadingDataSubject.asObservable();
  metaData?: IMetaData;
  dataTypes: typeof ProjectEnums.DataTypeEnum = ProjectEnums.DataTypeEnum;

  constructor(private readonly changeDetectorRef: ChangeDetectorRef) {}

  ngOnInit() {
    setTimeout(() => {
      this.createForm(this.relMetaField);
      this.loadingDataSubject.next(false);
      this.changeDetectorRef.markForCheck();
    });
  }

  convertOptions(strOptions: string[]): InplaceDropdownOptionsInterface[] {
    return strOptions.map((str) => ({
        label: str,
        value: str,
      }));
  }

  onEditNumber(): void {
    this.metaData.form.valueChanges.pipe(take(1)).subscribe((value: string) => {
      if (value) {
        this.inputNumberRef.input.nativeElement.addEventListener('focusout', this.onFocusoutEventListenerForInputNumber);
        this.inputNumberRef.input.nativeElement.addEventListener('keyup', this.onkeydownEnterEventListenerForInputNumber);
      }
    });
  }

  onEditText(): void {
    this.metaData.form.valueChanges.pipe(take(1)).subscribe((val: string) => {
      if (val) {
        this.inputTextRef.nativeElement.addEventListener('focusout', this.onFocusoutEventListener);
        this.inputTextRef.nativeElement.addEventListener('keyup', this.onkeydownEnterEventListener);
      }
    });
  }

  onSave() {
    this.valueChange.emit(this.metaData.form.value);
    this.loadingDataSubject.next(true);
    this.changeDetectorRef.detectChanges();
  }

  checkValue(val: boolean | null): string {
    if (val === null) {
      return 'не установлено';
    } else {
      return val ? 'да' : 'нет';
    }
  }

  private onFocusoutEventListenerForInputNumber = (): void => {
    this.inputNumberRef.input.nativeElement.removeEventListener('keyup', this.onkeydownEnterEventListenerForInputNumber);
    this.onSave();
    this.inputNumberRef.input.nativeElement.removeEventListener('focusout', this.onFocusoutEventListenerForInputNumber);
  };

  private onkeydownEnterEventListenerForInputNumber = (event: KeyboardEvent): void => {
    event.stopPropagation();
    if (event.key === 'Enter') {
      this.inputNumberRef.input.nativeElement.removeEventListener('focusout', this.onFocusoutEventListenerForInputNumber);
      this.onSave();
      this.inputNumberRef.input.nativeElement.removeEventListener('keyup', this.onkeydownEnterEventListenerForInputNumber);
    }
  };

  private onFocusoutEventListener = (): void => {
    this.inputTextRef.nativeElement.removeEventListener('keyup', this.onkeydownEnterEventListener);
    this.onSave();
    this.inputTextRef.nativeElement.removeEventListener('focusout', this.onFocusoutEventListener);
  };

  private onkeydownEnterEventListener = (event: KeyboardEvent): void => {
    event.stopPropagation();
    if (event.key === 'Enter') {
      this.inputTextRef.nativeElement.removeEventListener('focusout', this.onFocusoutEventListener);
      this.onSave();
      this.inputTextRef.nativeElement.removeEventListener('keyup', this.onkeydownEnterEventListener);
    }
  };

  private createForm(relMetaField: IProjectDetailsMetaField): void {
    this.metaData = {
      metaField: relMetaField,
      form: this.createFormGroup(relMetaField, this.findValueForMeta(this.metaValues, relMetaField)),
    };
  }

  private findValueForMeta(metaValues: ProjectMetaValue[], metaField: IProjectDetailsMetaField): ProjectMetaValue {
    return metaValues.find((projectMetaValue) => projectMetaValue.meta === metaField.id);
  }

  private createFormGroup(metaField: IProjectDetailsMetaField, metaValue?: ProjectMetaValue): FormGroup {
    return new FormGroup({
      id: new FormControl(metaValue?.id),
      meta: new FormControl(metaField.id, Validators.required),
      project: new FormControl(this.projectId, Validators.required),
      value_CH: new FormControl(metaValue?.value_CH),
      value_CHS: new FormControl(metaValue?.value_CHS),
      value_DT: new FormControl(metaValue?.value_DT),
      value_FLAG: new FormControl(metaValue?.value_FLAG),
      value_FLT: new FormControl(metaValue?.value_FLT),
      value_INT: new FormControl(metaValue?.value_INT),
      value_MCHS: new FormControl(metaValue?.value_MCHS),
      value_TXT: new FormControl(metaValue?.value_TXT),
    });
  }
}
