import { ChangeDetectionStrategy, Component, OnInit, Signal, ViewChild, WritableSignal, computed, signal } from '@angular/core';
import { Params } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { BehaviorSubject, Observable } from 'rxjs';
import { map, takeUntil } from 'rxjs/operators';
import { MessageService } from 'primeng/api';
import { OverlayPanel } from 'primeng/overlaypanel';

import { ProjectStageModel, ProjectStageStateEnum, ProjectsApiService } from 'data-access/sdk/internal-api';
import { Project, ProjectMetaGroup, ProjectMetaValue, ProjectMetaValueService, ProjectState } from 'data-access/sdk/api2';
import { Unsubscriber } from 'utils/shared-services';

import {
  IProjectDetailsBudgetChartItem,
  IProjectDetailsBudgetChartModel,
} from '@core-project/containers/project/project-details/models/budget-chart.model';
import { RouterStateService } from '@core-project/containers/project/services/router-state.service';
import { ProjectEnums } from '@core-project/enums/project';
import { ProjectExtra } from '@core-project/interfaces/projects';

import { ProjectDetailsService } from '../../services/project-details.service';
import { PERCENT_TOTAL } from '../../constants/budget-chart.constant';

export interface IProjectDetailsGroup {
  name: string;
  relMetaFields: IProjectDetailsMetaField[];
  metaFields: number[];
  metaValues: ProjectMetaValue[];
  projectId: number;
}

export interface IProjectDetailsMetaField {
  name: string;
  content: string;
  id: number;
  choices: string[];
  dataTypes: string;
  loading: boolean;
}

export interface IProjectDetailsBudgetChart {
  planned: number;
  forecast: number;
  fact: number;
}

@Component({
  selector: 'app-project-details-container',
  templateUrl: './project-details-container.component.html',
  styleUrls: ['./project-details-container.component.scss'],
  providers: [Unsubscriber],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class ProjectDetailsContainerComponent implements OnInit {
  private _timelineStages: WritableSignal<ProjectStageModel[]> = signal<ProjectStageModel[]>([]);

  @ViewChild('panel') overlayPanel: OverlayPanel;

  projectId: number;

  timelineStages: Signal<ProjectStageModel[]> = this._timelineStages.asReadonly();
  canCreateTimeline: Signal<boolean> = computed(() => {
    const timelineMilestones = this.timelineStages().map((stage) => stage.endMilestone);
    timelineMilestones.pop(); // last stage have not milestone (normal)
    const allStagesHaveMilestones = timelineMilestones.every((milestone) => milestone);
    return this.timelineStages()?.length > 0 && allStagesHaveMilestones;
  });
  timelineCurrentStage: Signal<ProjectStageModel | undefined> = computed(() => {
    const timelineStages = this.timelineStages();
    const currentStage = timelineStages.find((stage) => stage.state === ProjectStageStateEnum.CURRENT);
    return currentStage;
  });
  timelineCurrentStageState: Signal<string | undefined> = computed(() => this.timelineCurrentStage()?.name);
  timelineCurrentStageEnd: Signal<string | undefined> = computed(() => {
    const timelineCurrentStage = this.timelineCurrentStage();
    return timelineCurrentStage?.endMilestone?.endEff;
  });
  showTimelineCurrentStageEnd: Signal<boolean> = computed(() => this.canCreateTimeline());

  currentState$: Observable<ProjectState>;
  project$: Observable<ProjectExtra>;
  budgetChart$: Observable<IProjectDetailsBudgetChartModel>;
  metaGroupsSubject: BehaviorSubject<IProjectDetailsGroup[]> = new BehaviorSubject<IProjectDetailsGroup[]>([]);
  metaGroups$: Observable<IProjectDetailsGroup[]> = this.metaGroupsSubject.asObservable();

  constructor(
    private readonly projectDetailsService: ProjectDetailsService,
    private readonly projectMetaValueService: ProjectMetaValueService,
    private readonly messageService: MessageService,
    private readonly translateService: TranslateService,
    private readonly routerStateService: RouterStateService,
    private readonly unsubscriber: Unsubscriber,
    private readonly projectsApi: ProjectsApiService
  ) {}

  ngOnInit() {
    this.routerStateService.params$.pipe(takeUntil(this.unsubscriber.destroy$)).subscribe((params: Params) => {
      this.projectId = params.projectId;
      this.projectDetailsService.getUsersPermissions(this.projectId);
      this.projectDetailsService.getProject(this.projectId);
      this.projectsApi.readProjectTimeline(this.projectId).subscribe((stages) => this._timelineStages.set(stages));
      this.project$ = this.projectDetailsService.project$;

      this.projectDetailsService.metaGroups$.subscribe((metaGroups) => {
        this.metaGroupsSubject.next(this.convertMetaGroup(metaGroups));
      });
    });

    this.updateCurrentState();
    this.budgetChart$ = this.projectDetailsService
      .getBudgetChart$()
      .pipe(map((chartData: IProjectDetailsBudgetChart) => this.chartConverter(chartData)));
  }

  // update attributes
  onSave(event: ProjectMetaValue): void {
    if (event.id) {
      this.projectMetaValueService
        .projectMetaValuePartialUpdate({
          id: event.id,
          data: event,
        })
        .subscribe(() => this.projectDetailsService.getProject(this.projectId));
    } else {
      this.projectMetaValueService.projectMetaValueCreate(event).subscribe(() => this.projectDetailsService.getProject(this.projectId));
    }
  }

  onSaveProject(event: unknown, project: Project): void {
    this.projectDetailsService.projectsPartialUpdate(project.id, event as Project).subscribe((data) => {
      this.messageService.add({
        severity: 'success',
        summary: this.translateService.instant('PROJECTS.EDIT_SUCC.MSG', { name: data.name }),
      });

      this.projectDetailsService.getProject(this.projectId);
    });
  }

  onEditState(state: boolean): void {
    this.projectDetailsService.editState(this.projectId, state);
  }

  onShowStateHistory(): void {
    this.projectDetailsService.showStateHistory(this.projectId);
  }

  cancel(project: Project): void {
    this.projectDetailsService.cancelProject(project);
    this.overlayPanel.hide();
  }

  delete(project: Project): void {
    this.projectDetailsService.deleteProject(project);
    this.overlayPanel.hide();
  }

  resume(project: Project): void {
    if (project.status === ProjectEnums.StatusEnum.PSP) {
      this.pause(project);
    }
    if (project.status === ProjectEnums.StatusEnum.CNC) {
      this.cancel(project);
    }
  }

  pause(project: Project): void {
    this.projectDetailsService.pauseProject(project);
    this.overlayPanel.hide();
  }

  private updateCurrentState(): void {
    this.projectDetailsService.currentStateSubject.next(null);
    this.currentState$ = this.projectDetailsService.currentState$;
  }

  private convertMetaGroup(metaGroups: ProjectMetaGroup[]): IProjectDetailsGroup[] {
    return metaGroups.map((projectMetaGroup) => ({
      name: projectMetaGroup.name,
      relMetaFields: projectMetaGroup.rel_meta_fields.map((projectMeta) => ({
        name: projectMeta.name,
        content: this.projectDetailsService.getMetaValue(projectMeta),
        id: projectMeta.id,
        choices: projectMeta.choices,
        dataTypes: projectMeta.data_type,
        loading: false,
      })),
      metaFields: projectMetaGroup.meta_fields,
      metaValues: this.projectDetailsService.project.rel_metas.filter((projectMetaValue) =>
        projectMetaGroup.meta_fields.includes(projectMetaValue.meta)
      ),
      projectId: this.projectId,
    }));
  }

  private chartConverter(chartData: IProjectDetailsBudgetChart): IProjectDetailsBudgetChartModel {
    const isFactMorePlanned = this.isFactMorePlan(chartData.fact, chartData.planned);
    const isPositiveForecast = this.isPositive(chartData.planned, chartData.forecast);
    return {
      planned: this.setPlannedChart(chartData.planned, chartData.fact, isFactMorePlanned),
      forecast: this.setForecastChart(chartData.forecast, chartData.planned, chartData.fact, isPositiveForecast, isFactMorePlanned),
      expenses: this.setExpensesChart(chartData.forecast, chartData.fact, chartData.planned, isPositiveForecast),
      isPositiveForecast,
      isFactMorePlanned,
    };
  }

  private setPlannedChart(planned: number, fact: number, isFactMorePlanned: boolean): IProjectDetailsBudgetChartItem {
    return {
      value: planned,
      percent: undefined,
      width: this.getPlannedWidth(planned, fact, isFactMorePlanned),
    };
  }

  private getPlannedWidth(planned: number, fact: number, isFactMorePlanned: boolean): number {
    if (isFactMorePlanned) {
      return (planned * PERCENT_TOTAL) / fact;
    }

    return PERCENT_TOTAL;
  }

  private setForecastChart(
    forecast: number,
    planned: number,
    fact: number,
    isPositiveForecast: boolean,
    isFactMorePlanned: boolean
  ): IProjectDetailsBudgetChartItem {
    const forecastDiffValue = this.getForecastValue(forecast, planned, isPositiveForecast);
    return {
      value: forecastDiffValue,
      percent: this.getForecastPercent(forecast, planned, isPositiveForecast),
      width: this.getForecastWidth(fact, planned, forecastDiffValue, isFactMorePlanned),
    };
  }

  private getForecastValue(forecast: number, planned: number, isPositiveForecast: boolean): number {
    if (isPositiveForecast) {
      return planned - forecast;
    }
    return forecast - planned;
  }

  private getForecastWidth(fact: number, planned: number, forecastDiffValue: number, isFactMorePlanned: boolean): number {
    if (isFactMorePlanned) {
      return (forecastDiffValue * PERCENT_TOTAL) / fact;
    }

    return (forecastDiffValue * PERCENT_TOTAL) / planned;
  }

  private getForecastPercent(forecast: number, planned: number, isPositiveForecast: boolean): number {
    if (isPositiveForecast) {
      return Math.round(((planned - forecast) / planned) * PERCENT_TOTAL);
    }
    return Math.round(((forecast - planned) / planned) * PERCENT_TOTAL);
  }

  private setExpensesChart(forecast: number, fact: number, planned: number, isPositiveForecast: boolean): IProjectDetailsBudgetChartItem {
    return {
      value: fact,
      percent: this.getExpensesPercent(fact, planned),
      width: this.getExpensesWidth(forecast, fact, planned, isPositiveForecast),
    };
  }

  private getExpensesWidth(forecast: number, fact: number, planned: number, isPositiveForecast: boolean): number {
    const forecastDiffValue = this.getForecastValue(forecast, planned, isPositiveForecast);
    if (isPositiveForecast) {
      return (fact / planned) * PERCENT_TOTAL;
    }

    return (fact / (planned + forecastDiffValue)) * PERCENT_TOTAL;
  }

  private getExpensesPercent(fact: number, planned: number): number {
    return Math.round((fact / planned) * PERCENT_TOTAL);
  }

  private isFactMorePlan(fact: number, planned: number): boolean {
    return fact > planned;
  }

  private isPositive(planned: number, forecast: number): boolean {
    return planned > forecast;
  }
}
