import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  EventEmitter,
  Input,
  OnChanges,
  OnInit,
  Output,
  SimpleChanges,
  ViewChild,
} from '@angular/core';
import {AuthenticationService} from '@core-project/services/authentication.service';
import {
  CompaniesService,
  ProjectGroup,
  ProjectGroupsService,
  ProjectType,
  ProjectTypesService,
  ProjectTypeStage,
  ProjectTypeStagesService,
  User,
} from 'data-access/sdk/api2';
import {FormControl, FormGroup, Validators} from '@angular/forms';
import {ProjectExtra} from '@core-project/interfaces/projects';
import {BehaviorSubject, Observable, take} from 'rxjs';
import {map, takeUntil} from 'rxjs/operators';
import {Unsubscriber} from 'utils/shared-services';
import {ProjectEnums} from '@core-project/enums/project';

@Component({
  selector: 'app-project-info-form-container',
  templateUrl: './project-info-form-container.component.html',
  styleUrls: ['./project-info-form-container.component.scss'],
  providers: [Unsubscriber],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class ProjectInfoFormContainerComponent implements OnInit, OnChanges {
  @ViewChild('inputTextAreaRef')
  inputTextArea: ElementRef;
  @ViewChild('simpleInputRef')
  simpleInput: ElementRef;

  @Output()
  valueChange: EventEmitter<unknown> = new EventEmitter<unknown>();

  @Input()
  project: ProjectExtra;

  @Input()
  status: ProjectEnums.StatusEnum;

  loadingDataSubject: BehaviorSubject<boolean> = new BehaviorSubject<boolean>(false);
  loadingData$: Observable<boolean> = this.loadingDataSubject.asObservable();

  user: User;
  projectForm: FormGroup;
  projectGroups$: Observable<ProjectGroup[]>;
  projectGroupsTree$: Observable<ProjectGroup[]>;
  projectTypes$: Observable<ProjectType[]>;

  ptStages$: Observable<ProjectTypeStage[]>;

  maxAbbrLen = 5;

  constructor(
    private authenticationService: AuthenticationService,
    private projectTypesService: ProjectTypesService,
    private projectGroupsService: ProjectGroupsService,
    private companiesService: CompaniesService,
    private projectTypeStagesService: ProjectTypeStagesService,
    private changeDetectorRef: ChangeDetectorRef,
    private readonly unsubscriber: Unsubscriber,
  ) {
  }

  ngOnChanges(changes: SimpleChanges) {
    this.initForm();
    if (changes.project) {
      this.loadingDataSubject.next(false);
    }
  }

  ngOnInit() {
    this.getOptions();

    this.projectForm.controls.name.valueChanges.pipe(takeUntil(this.unsubscriber.destroy$)).subscribe((name: string) => {
      if (this.projectForm.controls.abbr.pristine || this.projectForm.controls.abbr.value === '') {
        let abbr: string;
        const words = name.split(' ');
        if (words.length > 1) {
          let abbrArr: any[];
          if (words.length >= this.maxAbbrLen) {
            abbrArr = words.map((w) => w[0]);
          } else {
            const minLetters = Math.floor(this.maxAbbrLen / words.length);
            const addLetterInd = this.maxAbbrLen - minLetters;
            abbrArr = words.map((w, i) => {
              let subInd = minLetters;
              if (i < addLetterInd) {
                subInd += 1;
              }
              return w.substring(0, subInd);
            });
          }
          abbr = abbrArr.join('').substring(0, this.maxAbbrLen);
        } else {
          abbr = name.substring(0, this.maxAbbrLen);
        }

        this.projectForm.controls.abbr.setValue(abbr.toUpperCase());
      }
    });

    this.projectForm.controls.project_type.valueChanges.pipe(takeUntil(this.unsubscriber.destroy$)).subscribe((pt) => {
      if (pt) {
        this.ptStages$ = this.projectTypeStagesService
          .projectTypeStagesList({
            projectType: pt,
            include: 'stage',
          })
          .pipe(
            map((ptStages) => {
              this.projectForm.controls.stage.setValue(ptStages[0].rel_stage.id);
              return ptStages;
            }),
          );
      }
    });
  }

  onSave(): void {
    this.valueChange.emit(this.projectForm.value);
    this.loadingDataSubject.next(true);
    this.changeDetectorRef.detectChanges();

    this.inputTextArea.nativeElement.removeEventListener('focusout', this.onFocusoutEventListener);
    this.inputTextArea.nativeElement.removeEventListener('keyup', this.onkeydownEnterEventListener);

    this.simpleInput.nativeElement.removeEventListener('focusout', this.onFocusoutEventListener);
    this.simpleInput.nativeElement.removeEventListener('keyup', this.onkeydownEnterEventListener);
  }

  onEdit(el: HTMLElement): void {
    this.projectForm.valueChanges.pipe(take(1)).subscribe((val: string) => {
      if (val) {
        el.addEventListener('focusout', this.onFocusoutEventListener);
        el.addEventListener('keyup', this.onkeydownEnterEventListener);
      }
    });
  }

  private initForm() {
    this.user = this.authenticationService.user;
    // TODO: dates validation - end > start

    const projectExtra = this.project;
    this.projectForm = new FormGroup({
      id: new FormControl(projectExtra?.id),
      project_group: new FormControl(projectExtra?.project_group, Validators.required),
      project_type: new FormControl(projectExtra?.project_type, Validators.required),
      name: new FormControl(projectExtra?.name, Validators.required),
      abbr: new FormControl(projectExtra?.abbr, [Validators.required, Validators.maxLength(5)]),
      stage: new FormControl(projectExtra?.stage, Validators.required),
    });
  }

  private getOptions(): void {
    this.projectTypes$ = this.projectTypesService.projectTypesList().pipe(
      map((pts) => {
        if (pts.length === 1) {
          this.projectForm.controls.project_type.setValue(pts[0].id);
        }
        return pts;
      }),
    );
    this.projectGroups$ = this.projectGroupsService.projectGroupsList({}).pipe(
      map((pgs) => {
        if (pgs.length === 1) {
          this.projectForm.controls.project_group.setValue(pgs[0].id);
        }
        return pgs;
      }),
    );

    this.projectGroupsTree$ = this.companiesService.companiesProjectGroupTree(this.user.company).pipe(
      map((pgs) => {
        if (pgs.length === 1) {
          this.projectForm.controls.project_group.setValue(pgs[0].id);
        }

        return pgs.map((pg) => {
          pg.name = '-'.repeat(pg.level) + ' ' + pg.name;
          return pg;
        });
      }),
    );
  }

  private onFocusoutEventListener = (): void => {
    this.inputTextArea.nativeElement.removeEventListener('keydown', this.onkeydownEnterEventListener);
    this.simpleInput.nativeElement.removeEventListener('keydown', this.onkeydownEnterEventListener);
    this.onSave();
  };

  private onkeydownEnterEventListener = (event: KeyboardEvent): void => {
    event.stopPropagation();
    if (event.key === 'Enter') {
      this.inputTextArea.nativeElement.removeEventListener('focusout', this.onFocusoutEventListener);
      this.simpleInput.nativeElement.removeEventListener('focusout', this.onFocusoutEventListener);
      this.onSave();
    }
  };
}
