import { formatDate } from '@angular/common';
import { Inject, Injectable, LOCALE_ID } from '@angular/core';
import { Router } from '@angular/router';
import { WorkEnums } from '@core-project/enums';
import { ProjectEnums } from '@core-project/enums/project';
import { projectToProjectExtra } from '@core-project/helpers';
import { ProjectExtra } from '@core-project/interfaces/projects';
import { ProjectStateFormComponent } from '@core-project/modules/forms/project-state-form/project-state-form.component';
import { ProjectStateHistoryComponent } from '@core-project/modules/shared-components/project-state-history/project-state-history.component';
import { AuthenticationService } from '@core-project/services/authentication.service';
import { TranslateService } from '@ngx-translate/core';
import { Store } from '@ngxs/store';
import {
  FinanceService,
  Project,
  ProjectBudget,
  ProjectMember,
  ProjectMembersService,
  ProjectMeta,
  ProjectMetaGroup,
  ProjectMetaGroupsService,
  ProjectsService,
  ProjectState,
  ProjectStatesService,
  ProjectTypeStage,
  ProjectTypeStagesService,
  ProjectWorkPlan,
  UsersService,
  Work,
  WorkplansService,
  WorksService,
} from 'data-access/sdk/api2';
import { GlobalActions } from 'data-access/store';
import * as _ from 'lodash-es';
import { MessageService } from 'primeng/api';
import { DialogService } from 'primeng/dynamicdialog';
import { BehaviorSubject, Observable, take } from 'rxjs';
import { filter, map, switchMap } from 'rxjs/operators';
import { IProjectDetailsBudgetChart } from '../containers/project-details-container/project-details-container.component';
import WorkTypeEnum = WorkEnums.WorkTypeEnum;

@Injectable()
export class ProjectDetailsService {
  private milestones: Work[];

  metaGroupsSubject: BehaviorSubject<Array<ProjectMetaGroup>> = new BehaviorSubject([]);
  metaGroups$: Observable<Array<ProjectMetaGroup>> = this.metaGroupsSubject.asObservable();

  // todo: вынести в container
  perms: Array<string>;
  ptStages: ProjectTypeStage[];

  project: ProjectExtra;

  // todo: добавить типизацию
  events: any[];

  projectBudgetSubject: BehaviorSubject<ProjectBudget> = new BehaviorSubject(undefined);
  projectBudget$: Observable<ProjectBudget> = this.projectBudgetSubject.asObservable();

  workPlanSubject: BehaviorSubject<ProjectWorkPlan> = new BehaviorSubject(undefined);

  membersSubject: BehaviorSubject<ProjectMember[]> = new BehaviorSubject([]);

  currentStateSubject: BehaviorSubject<ProjectState> = new BehaviorSubject(null);
  currentState$: Observable<ProjectState> = this.currentStateSubject.asObservable();

  projectSubject: BehaviorSubject<ProjectExtra> = new BehaviorSubject(undefined);
  project$: Observable<ProjectExtra> = this.projectSubject.asObservable();

  constructor(
    @Inject(LOCALE_ID) private readonly locale: string,
    private readonly router: Router,
    private readonly projectsService: ProjectsService,
    private readonly projectMetaGroupsService: ProjectMetaGroupsService,
    private readonly financeService: FinanceService,
    private readonly messageService: MessageService,
    private readonly dialogService: DialogService,
    private readonly translateService: TranslateService,
    private readonly usersService: UsersService,
    private readonly authenticationService: AuthenticationService,
    private readonly worksService: WorksService,
    private readonly workplansService: WorkplansService,
    private readonly projectMembersService: ProjectMembersService,
    private readonly projectStatesService: ProjectStatesService,
    private readonly projectTypeStagesService: ProjectTypeStagesService,
    private store: Store
  ) {}

  getUsersPermissions(projectId: number): void {
    this.usersService
      .usersPermissions({
        id: this.authenticationService.user.id,
        project: projectId,
      })
      .subscribe((perms) => (this.perms = perms.permissions));
  }

  getProject(projectId: number) {
    this.projectsService
      .projectsRead({
        id: projectId,
        include: 'customer, project_type, metas, project_group',
      })
      .subscribe((project) => {
        this.project = projectToProjectExtra(project);
        this.projectSubject.next(this.project);
        this.store.dispatch(new GlobalActions.SetAppTitle({ key: this.project.name }));

        if (this.project.work_plan) {
          this.workplansService.workplansRead({ id: Number(this.project.work_plan) }).subscribe((workPlan) => {
            this.workPlanSubject.next(workPlan);
          });
        }
        this.projectMembersService
          .projectMembersList({ project: this.project.id.toString(), include: 'user, department, organization' })
          .subscribe((members) => {
            this.membersSubject.next(members);
          });

        this.getStages();
        this.getMetaGroups();
        this.getMilestones(projectId);
        this.getCurrentState(projectId);
      });
  }

  getMetaValue(metaField: ProjectMeta) {
    const mfv = this.project.metas_by_id[metaField.id];
    if (mfv === null || mfv === undefined) {
      return null;
    }
    let value = mfv['value_' + metaField.data_type];
    if (value === null || value === undefined) {
      return null;
    }

    switch (metaField.data_type) {
      case ProjectEnums.DataTypeEnum.DT:
        value = formatDate(value, 'shortDate', this.locale);
        break;
      case ProjectEnums.DataTypeEnum.MCHS:
        value = value.join(', ');
        break;
      case ProjectEnums.DataTypeEnum.FLAG:
        value = value
          ? '<i class="fa-regular fa-check-circle text-green-500"></i>'
          : '<i class="fa-regular fa-times-circle text-red-500"></i>';
        break;
    }

    value = metaField.prefix ? metaField.prefix + ' ' + value : value;
    value = metaField.postfix ? value + ' ' + metaField.postfix : value;
    return value;
  }

  editState(projectId: number, newState: boolean = false) {
    this.currentState$.pipe(take(1)).subscribe((currentState) => {
      const projectState: ProjectState = {
        ...{
          project: projectId,
          state: '',
        },
        ...currentState,
      };

      if (projectState && newState) {
        delete projectState.id;
      }

      const dialogRef = this.dialogService.open(ProjectStateFormComponent, {
        header: projectState.id
          ? this.translateService.instant('PROJECT.DETAILS.EDIT_STATE_FORM_HEADER')
          : this.translateService.instant('PROJECT.DETAILS.NEW_STATE_FORM_HEADER'),
        width: '600px',
        data: { projectState },
      });

      dialogRef.onClose.pipe(take(1)).subscribe(() => this.getCurrentState(projectId));
    });
  }

  showStateHistory(projectId: number) {
    this.dialogService.open(ProjectStateHistoryComponent, {
      header: this.translateService.instant('PROJECT.DETAILS.STATE_HISTORY'),
      width: '60%',
      data: { projectId: projectId },
    });
  }

  projectsPartialUpdate(projectId: number, editedProject: Project): Observable<Project> {
    return this.projectsService.projectsPartialUpdate({ id: projectId, data: editedProject });
  }

  pauseProject(project: Project) {
    this.projectsService.projectsPause({ id: project.id, data: project }).subscribe((currentProject) => {
      Object.assign(this.project, currentProject);
      this.projectSubject.next(this.project);
    });
  }

  cancelProject(project: Project) {
    this.projectsService.projectsCancel({ id: project.id, data: project }).subscribe((currentProject) => {
      Object.assign(this.project, currentProject);
      this.projectSubject.next(this.project);
    });
  }

  deleteProject(project: Project) {
    this.projectsService.projectsDelete(project.id).subscribe({
      complete: () => this.router.navigate(['projects']),
      error: () =>
        this.messageService.add({
          severity: 'error',
          summary: this.translateService.instant('PROJECT.DETAILS.DELETE_FAILURE'),
        }),
    });
  }

  getBudgetChart$(): Observable<IProjectDetailsBudgetChart> {
    return this.currentState$.pipe(
      filter((state) => !!state),
      switchMap((state) =>
        this.financeService.financeProjectBudgetsList({ project: state.project.toString() }).pipe(
          map((budgets) => ({
            forecast: Number(budgets[0].total_forecast),
            fact: Number(budgets[0].total_fact),
            planned: Number(budgets[0].total_planned),
          }))
        )
      )
    );
  }

  private getStages() {
    this.projectTypeStagesService
      .projectTypeStagesList({
        projectType: this.project.project_type.toString(),
        include: 'stage',
      })
      .subscribe((ptss) => (this.ptStages = ptss));
  }

  private getMetaGroups() {
    this.projectMetaGroupsService.projectMetaGroupsList('meta_fields').subscribe((mgs) => {
      mgs.forEach((mg) => {
        mg.rel_meta_fields = _.filter(mg.rel_meta_fields, (mf) => mf.all_types || mf.project_types.includes(this.project.project_type));
        mg.meta_fields = mg.rel_meta_fields.map((mf) => mf.id);
      });
      _.remove(mgs, (mg) => mg.meta_fields.length === 0);
      this.metaGroupsSubject.next(mgs);
    });
  }

  private getMilestones(projectId: number) {
    this.worksService
      .worksList({
        project: projectId.toString(),
        workType: WorkTypeEnum.MS,
        milestoneLevel: 1,
      })
      .subscribe((milestones) => {
        this.milestones = milestones;
      });
  }

  private getCurrentState(projectId: number) {
    this.projectStatesService.projectStatesCurrent(projectId.toString()).subscribe((projectState) => {
      this.currentStateSubject.next(projectState);
    });
  }
}
