import { Component, OnInit } from '@angular/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import * as moment from 'moment';
import { Observable, tap } from 'rxjs';
import { ProjectBudgetItem } from 'data-access/sdk/api2';
import { FinanceService } from 'data-access/sdk/api2';
import { switchMap } from 'rxjs/operators';

@Component({
  selector: 'app-project-budget-item',
  templateUrl: './project-budget-item.component.html',
  styleUrls: ['./project-budget-item.component.scss'],
})
export class ProjectBudgetItemComponent implements OnInit {
  item: ProjectBudgetItem;
  itemForm: FormGroup;
  currency: string;
  planDisabled: boolean;
  perms$: Observable<Array<string>>;
  overplan: number;

  constructor(
    public ref: DynamicDialogRef,
    public config: DynamicDialogConfig,
    public financeService: FinanceService,
    private formBuilder: FormBuilder
  ) {}

  get isPastMonth() {
    return moment(this.item.date).isBefore(moment(), 'month');
  }

  get currOverPlan() {
    return this.overplan + Number(this.itemForm.controls.plan.value);
  }

  ngOnInit(): void {
    this.item = this.config.data.item;
    this.currency = this.config.data.currency || 'RUB';
    this.planDisabled = this.config.data.planDisabled || false;
    this.perms$ = this.config.data.perms$;
    this.overplan = this.config.data.overplan;

    this.initItemForm(this.item);
  }

  initItemForm(item: ProjectBudgetItem) {
    this.itemForm = this.formBuilder.group({
      id: [item.id],
      plan: [item.plan, { updateOn: 'change' }],
      forecast: [item.forecast],
      project: [item.project, Validators.required],
      budget_type: [item.budget_type, Validators.required],
      date: [item.date],
    });

    this.itemForm.controls.plan.valueChanges.subscribe((val) => this.itemForm.controls.forecast.setValue(val));
  }

  submit() {
    this.ref.close(this.itemForm.value);
  }

  approve() {
    if (!this.item.id) {
      this.financeService
        .financeProjectBudgetItemsCreate(this.item)
        .pipe(
          switchMap((item) =>
            this.financeService.financeProjectBudgetItemsToggleApproval(item.id).pipe(tap(() => this.ref.close({ id: item.id })))
          )
        )
        .subscribe();
    } else {
      this.financeService.financeProjectBudgetItemsToggleApproval(this.item.id).subscribe(() => this.ref.close(this.itemForm.value));
    }
  }

  planInput($event) {
    this.itemForm.controls.plan.setValue($event.fn);
  }

  maxPlan() {
    let val = Number(this.itemForm.controls.plan.value) - this.currOverPlan;
    if (val < 0) {
      val = 0;
    }
    this.itemForm.controls.plan.setValue(val);
  }
}
