import { Component, Inject, Input, LOCALE_ID, OnInit } from '@angular/core';
import { forkJoin, Observable } from 'rxjs';
import { DialogService } from 'primeng/dynamicdialog';
import { ProjectExpenseFormComponent } from '@core-project/modules/forms/project-expense-form/project-expense-form.component';
import * as moment from 'moment';
import { TranslateService } from '@ngx-translate/core';
import { formatDate } from '@angular/common';
import { ProjectMonthInfoComponent } from '@core-project/containers/project/project-finance/project-month-info/project-month-info.component';
import { isPastMonth } from '@core-project/containers/project/project-finance/helpers';
import { ConfirmationService } from 'primeng/api';
import { Project, ProjectBudgetType, ProjectExpense } from 'data-access/sdk/api2';
import { FinanceService } from 'data-access/sdk/api2';

@Component({
  selector: 'app-project-expenses',
  templateUrl: './project-expenses.component.html',
  styleUrls: ['./project-expenses.component.scss'],
})
export class ProjectExpensesComponent implements OnInit {
  @Input() project: Project;
  @Input() projectId: number;
  @Input() perms$: Observable<Array<string>>;
  @Input() extension: string;

  public expenses: Array<ProjectExpense>;
  public loading = true;
  public projectBudgetTypes: Array<ProjectBudgetType>;
  public monthsApproved: { [key: string]: boolean };
  public monthsExpanded: { [key: string]: boolean };

  public isPastMonth = isPastMonth;

  constructor(
    @Inject(LOCALE_ID) private locale: string,
    private financeService: FinanceService,
    private dialogService: DialogService,
    private translateService: TranslateService,
    private confirmationService: ConfirmationService
  ) {}

  ngOnInit(): void {
    this.loadData();
  }

  loadData() {
    this.loading = true;
    forkJoin([
      this.financeService.financeProjectExpensesList({
        budgetItemProject: String(this.project.id),
      }),
      this.financeService.financeProjectBudgetTypesList({
        project: String(this.projectId),
      }),
    ]).subscribe((res) => {
      this.expenses = res[0];

      this.monthsApproved = {};
      this.expenses.forEach(
        (e) =>
          (this.monthsApproved[e.period] =
            (this.monthsApproved[e.period] === undefined ? true : this.monthsApproved[e.period]) && e.approved)
      );

      this.monthsExpanded = {};
      for (const key of Object.keys(this.monthsApproved)) {
        this.monthsExpanded[key] = !this.monthsApproved[key];
      }
      this.projectBudgetTypes = res[1];
      this.loading = false;
    });
  }

  editExpense(exp: ProjectExpense = null): void {
    let header = this.translateService.instant('FINANCE.EXPENSES.ADD_HEADER');
    if (exp) {
      header = this.translateService.instant('FINANCE.EXPENSES.EDIT_HEADER', { date: exp.date });
    }
    const dialogRef = this.dialogService.open(ProjectExpenseFormComponent, {
      header: header,
      width: '50%',
      data: {
        expense: exp,
        projectBudgetTypes: this.projectBudgetTypes,
        projectId: this.project.id,
      },
    });
    dialogRef.onClose.subscribe((expense) => {
      if (!expense) {
        return;
      }
      const budgetType = expense['budget_type'];
      delete expense['budget_type'];

      console.log(expense);
      debugger
      if (expense.id) {
        this.financeService
          .financeProjectExpensesPartialUpdate({ id: expense.id, data: expense })
          .toPromise()
          .then((_) => this.loadData());
      } else {
        this.financeService
          .financeProjectBudgetItemsList({
            project: String(this.project.id),
            budgetType: String(budgetType),
            date: moment(expense.date).format('YYYY-MM-DD HH:mm'),
          })
          .toPromise()
          .then((pbis) => {
            if (pbis.length === 0) {
              this.financeService
                .financeProjectBudgetItemsCreate({
                  date: moment(expense.date, 'DD.MM.YYYY').set({ date: 1 }).format('YYYY-MM-DD'),
                  project: this.project.id,
                  budget_type: budgetType,
                })
                .toPromise()
                .then((pbi) => {
                  expense.budget_item = pbi.id;
                  this.financeService
                    .financeProjectExpensesCreate(expense)
                    .toPromise()
                    .then((_) => this.loadData());
                });
            } else {
              expense.budget_item = pbis[0].id;
              this.financeService
                .financeProjectExpensesCreate(expense)
                .toPromise()
                .then((_) => this.loadData());
            }
          });
      }
    });
  }

  deleteExpense(exp: ProjectExpense): void {
    this.confirmationService.confirm({
      message: this.translateService.instant('FINANCE.EXPENSES.DELETE', {
        description: exp.description || '',
        date: moment(exp.date).format('L'),
      }),
      accept: () => {
        this.financeService
          .financeProjectExpensesDelete(exp.id)
          .toPromise()
          .then((_) => this.loadData());
      },
    });
  }

  approveMonth(period: string) {
    const ref = this.dialogService.open(ProjectMonthInfoComponent, {
      data: {
        period: period,
        projectId: this.projectId,
        perms$: this.perms$,
        currency: this.project.rel_project_budget.total_planned_currency,
      },
      header: this.translateService.instant('FINANCE.MONTH_INFO.HEADER', { period: formatDate(period, 'LLLL YYYY', this.locale) }),
      width: '60%',
    });

    ref.onClose.subscribe((res) => {
      if (res) {
        this.loadData();
      }
    });
  }
}
