import { Component, Inject, Input, LOCALE_ID, OnInit } from '@angular/core';
import { firstValueFrom, forkJoin, Observable } from 'rxjs';
import { DialogService } from 'primeng/dynamicdialog';
import { TranslateService } from '@ngx-translate/core';
import { ConfirmationService, MessageService } from 'primeng/api';
import { ProjectFinanceAddBudgetTypeComponent } from '@core-project/containers/project/project-finance/project-finance-add-budget-type/project-finance-add-budget-type.component';
import { ProjectBudgetComponent } from '@core-project/containers/project/project-finance/project-budget/project-budget.component';
import * as moment from 'moment';
import { formatNumber, formatPercent } from '@angular/common';
import { ProjectBudgetApprovalComponent } from '@core-project/containers/project/project-finance/project-budget-approval/project-budget-approval.component';
import { BudgetType, Project, ProjectBudget, ProjectBudgetType, User } from 'data-access/sdk/api2';
import { CompanyIndicatorConstraint } from 'data-access/sdk/api2';
import { AuthenticationService } from '@core-project/services/authentication.service';
import { ProjectBudgetParamsComponent } from '@core-project/containers/project/project-finance/project-budget-params/project-budget-params.component';
import { FinanceService } from 'data-access/sdk/api2';

class NumberBudgetStats {
  total_planned = 0;
  plan = 0;
  fact = 0;
  forecast = 0;
  current_fact_and_fc = 0;
  plan_fact_remaining = 0;
  current_fact_and_fc_overplan = 0;

  get fact_percent(): number {
    return !this.total_planned ? 0 : this.fact / this.total_planned;
  }

  get plan_fact_remaining_percent(): number {
    return !this.total_planned ? 0 : this.plan_fact_remaining / this.total_planned;
  }

  get current_fact_and_fc_overplan_percent(): number {
    return !this.total_planned ? 0 : Math.abs(this.current_fact_and_fc_overplan) / this.total_planned;
  }

  add(pb: ProjectBudgetType) {
    this.total_planned += Number(pb.total_planned);
    this.plan += Number(pb.stats.plan);
    this.fact += Number(pb.stats.fact);
    this.forecast += Number(pb.stats.forecast);
    this.current_fact_and_fc += Number(pb.stats.current_fact_and_fc);
    this.plan_fact_remaining += Number(pb.stats.plan_fact_remaining);
    this.current_fact_and_fc_overplan += Number(pb.stats.current_fact_and_fc_overplan);
  }
}

interface ProjectBudgetTotalStats {
  total: NumberBudgetStats;
  categories: { [key: number]: NumberBudgetStats };
}

const DISCIPLINE_RED_DELTA = 25;
const DISCIPLINE_ORANGE_DELTA = 15;
const DISCIPLINE_YELLOW_DELTA = 5;

@Component({
  selector: 'app-project-finance-general',
  templateUrl: './project-finance-general.component.html',
  styleUrls: ['./project-finance-general.component.scss'],
})
export class ProjectFinanceGeneralComponent implements OnInit {
  @Input() projectId: number;
  @Input() project: Project;
  @Input() perms$: Observable<Array<string>>;
  @Input() extension: string;
  @Input() projectBudget: ProjectBudget;

  public budgetTypes: Array<BudgetType>;
  public projectBudgetTypes: Array<ProjectBudgetType>;

  public totalStats: ProjectBudgetTotalStats;

  public dashboardData = {
    fact_percent: 0,
    fact_percent_color: 'green',
    discipline: 0,
    discipline_color: 'green',
  };
  public constr: CompanyIndicatorConstraint;
  private user: User;

  constructor(
    @Inject(LOCALE_ID) private locale: string,
    private financeService: FinanceService,
    public dialogService: DialogService,
    public translateService: TranslateService,
    private confirmationService: ConfirmationService,
    private messageService: MessageService,
    private authenticationService: AuthenticationService
  ) {}

  get improperPlan(): boolean {
    return this.totalStats.total.total_planned !== Number(this.projectBudget?.total_planned);
  }

  ngOnInit(): void {
    this.user = this.authenticationService.user;
    this.constr = this.user.rel_company.rel_indicator_constraints;

    this.loadBudgetTypes();
  }

  editBudgetType(bt: ProjectBudgetType) {
    let pbts = this.projectBudgetTypes;
    if (bt) {
      pbts = pbts.filter((pbt) => pbt.id !== bt.id);
    }

    const ref = this.dialogService.open(ProjectFinanceAddBudgetTypeComponent, {
      data: {
        bt: bt,
        budgetTypes: this.budgetTypes,
        projectBudgetTypes: pbts,
        projectId: this.projectId,
        totalBudget: this.projectBudget.total_planned,
        perms$: this.perms$,
      },
      header: bt
        ? this.translateService.instant('FINANCE.BUDGET_TYPE.HEADER_EDIT')
        : this.translateService.instant('FINANCE.BUDGET_TYPE.HEADER_ADD'),
      width: '20%',
    });

    ref.onClose.subscribe((editedBt: ProjectBudgetType) => {
      if (!editedBt) {
        return;
      }
      if (editedBt.id) {
        this.financeService
          .financeProjectBudgetTypesPartialUpdate({
            id: editedBt.id,
            data: editedBt,
          })
          .toPromise()
          .then((_) => this.loadBudgetTypes());
      } else {
        this.financeService
          .financeProjectBudgetTypesCreate(editedBt)
          .toPromise()
          .then((_) => this.loadBudgetTypes());
      }
    });
  }

  deleteBudgetType(bt: ProjectBudgetType) {
    this.confirmationService.confirm({
      message: this.translateService.instant('FINANCE.BUDGET_TYPE.DELETE', { name: bt.name }),
      accept: () => {
        this.financeService
          .financeProjectBudgetTypesDelete(bt.id)
          .toPromise()
          .then((_) => this.loadBudgetTypes());
      },
    });
  }

  editGeneralInfo() {
    const ref = this.dialogService.open(ProjectBudgetComponent, {
      data: {
        pb: this.projectBudget,
      },
      header: this.translateService.instant('FINANCE.PROJECT.EDIT'),
      width: '40%',
    });

    ref.onClose.subscribe((editedBudget: ProjectBudget) => {
      editedBudget.period_start = moment(editedBudget.period_start, 'DD.MM.YYYY').format('YYYY-MM-DD');
      editedBudget.period_end = moment(editedBudget.period_end, 'DD.MM.YYYY').format('YYYY-MM-DD');
      if (editedBudget) {
        this.financeService
          .financeProjectBudgetsPartialUpdate({
            id: this.projectBudget.id,
            data: editedBudget,
          })
          .subscribe((pb) => {
            if (pb.total_planned !== this.projectBudget.total_planned) {
              this.loadBudgetTypes();
            }
            this.projectBudget = pb;
          });
      }
    });
  }

  approve(): void {
    const errors = this.checkApproval();
    if (errors) {
      errors.forEach((e) =>
        this.messageService.add({
          severity: 'error',
          summary: this.translateService.instant('FINANCE.PROJECT.ERRORS.HEADER'),
          detail: e,
        })
      );
      return;
    }
    const ref = this.dialogService.open(ProjectBudgetApprovalComponent, {
      header: this.translateService.instant('FINANCE.PROJECT.APPROVE_HEADER'),
      width: '30%',
      data: {
        projectBudget: this.projectBudget,
        projectBudgetTypes: this.projectBudgetTypes,
      },
    });
    firstValueFrom(ref.onClose).then((res) => {
      const obs: Array<Observable<any>> = [];
      res.pb['approved'] = true;
      obs.push(
        this.financeService.financeProjectBudgetsPartialUpdate({
          id: res.pb.id,
          data: res.pb,
        })
      );

      res.pbts.forEach((pbt) =>
        obs.push(
          this.financeService.financeProjectBudgetTypesPartialUpdate({
            id: pbt.id,
            data: pbt,
          })
        )
      );

      firstValueFrom(forkJoin(obs)).then((_res) => this.loadBudgetTypes());
    });
  }

  unapprove(): void {
    firstValueFrom(this.financeService.financeProjectBudgetsToggleApproval({ id: this.projectBudget.id, data: null })).then(
      (pb) => (this.projectBudget = pb)
    );
  }

  checkApproval(): Array<string> | null {
    const errors = [];

    if (!this.projectBudget.total_planned) {
      errors.push(this.translateService.instant('FINANCE.PROJECT.ERRORS.NO_BUDGET'));
    }

    if (!this.projectBudget.period_start || !this.projectBudget.period_end) {
      errors.push(this.translateService.instant('FINANCE.PROJECT.ERRORS.NO_DATES'));
    }

    const totalPlannedDelta = Number(this.projectBudget.total_planned) - this.totalStats.total.total_planned;
    if (totalPlannedDelta) {
      errors.push(
        this.translateService.instant('FINANCE.PROJECT.ERRORS.BUDGET_NOT_FULLY_PLANNED', {
          amount: formatNumber(totalPlannedDelta, this.locale, '1.2-2'),
          percent: formatPercent(totalPlannedDelta / Number(this.projectBudget.total_planned), this.locale),
        })
      );
    }

    this.projectBudgetTypes.forEach((pbt) => {
      const btPlannedDelta = Number(pbt.total_planned) - Number(pbt.stats.plan);
      if (btPlannedDelta) {
        errors.push(
          this.translateService.instant('FINANCE.PROJECT.ERRORS.BUDGET_TYPE_NOT_FULLY_PLANNED', {
            name: pbt.name,
            amount: formatNumber(btPlannedDelta, this.locale, '1.2-2'),
            percent: formatPercent(btPlannedDelta / Number(pbt.total_planned), this.locale),
          })
        );
      }
    });

    return errors.length ? errors : null;
  }

  editBudgetParams() {
    const ref = this.dialogService.open(ProjectBudgetParamsComponent, {
      header: this.translateService.instant('FINANCE.PROJECT.EDIT_PARAMS'),
      data: {
        projectBudget: this.projectBudget,
      },
      width: '60%',
    });
    firstValueFrom(ref.onClose).then((pb) => {
      if (pb) {
        this.projectBudget = pb;
      }
    });
  }

  private loadBudgetTypes() {
    forkJoin([
      this.financeService.financeProjectBudgetsList({ project: String(this.projectId) }),
      this.financeService.financeBudgetTypesList('category'),
      this.financeService.financeProjectBudgetTypesList({
        project: String(this.projectId),
        include: 'budget_type__category',
      }),
    ])
      .toPromise()
      .then((res) => {
        this.projectBudget = res[0][0];
        this.budgetTypes = res[1];
        this.projectBudgetTypes = res[2];

        this.dashboardData.fact_percent = Math.trunc(Number(this.projectBudget.stats.fact_percent));
        if (this.dashboardData.fact_percent > 100) {
          this.dashboardData.fact_percent_color = 'red';
        }
        this.dashboardData.discipline = Math.trunc(Number(this.projectBudget.stats.discipline));
        if (Math.abs(100 - this.dashboardData.discipline) > DISCIPLINE_RED_DELTA) {
          this.dashboardData.discipline_color = 'red';
        } else if (Math.abs(100 - this.dashboardData.discipline) > DISCIPLINE_ORANGE_DELTA) {
          this.dashboardData.discipline_color = 'orange';
        } else if (Math.abs(100 - this.dashboardData.discipline) > DISCIPLINE_YELLOW_DELTA) {
          this.dashboardData.discipline_color = 'yellow';
        }
        this.totalStats = this.calculateTotals();
      });
  }

  private calculateTotals(): ProjectBudgetTotalStats {
    const res = {
      total: new NumberBudgetStats(),
      categories: {},
    };
    this.projectBudgetTypes.forEach((pbt) => {
      res.total.add(pbt);
      const catId = pbt.rel_budget_type.rel_category.id;
      if (!res.categories[catId]) {
        res.categories[catId] = new NumberBudgetStats();
      }
      res.categories[catId].add(pbt);
    });

    return res;
  }
}
