import {Component, Input, OnInit} from '@angular/core';
import * as moment from 'moment';
import {DialogService} from 'primeng/dynamicdialog';
import {ProjectBudgetItemComponent} from '@core-project/containers/project/project-finance/project-budget-item/project-budget-item.component';
import {TranslateService} from '@ngx-translate/core';
import {firstValueFrom, forkJoin, Observable} from 'rxjs';
import {generateMockData, generateTotalEmptyData, isPastMonth} from '@core-project/containers/project/project-finance/helpers';
import {PBTItemRow, YearData, YearHeader} from '@core-project/interfaces/finance';
import {ProjectMonthInfoComponent} from '@core-project/containers/project/project-finance/project-month-info/project-month-info.component';
import * as _ from 'lodash-es';
import * as xlsx from 'xlsx';
import {Dates, getMonthFromDate, getQuarterByMonth, getYearFromDate} from '@core-project/helpers';
import {FinanceService, Project, ProjectBudget, ProjectBudgetItem, ProjectBudgetType} from 'data-access/sdk/api2';

@Component({
  selector: 'app-project-finance-table',
  templateUrl: './project-finance-table.component.html',
  styleUrls: ['./project-finance-table.component.scss'],
})
export class ProjectFinanceTableComponent implements OnInit {
  @Input() project: Project;
  @Input() projectId: number;
  @Input() perms$: Observable<Array<string>>;
  @Input() extension: string;
  @Input() projectBudget: ProjectBudget;

  public projectBudgetTypes: Array<ProjectBudgetType>;
  public projectBudgetItems: Array<ProjectBudgetItem>;
  public projectBudgetItemsByType: { [key: number]: Array<ProjectBudgetItem> };
  public years: Array<number> = [];
  public yearMonths: Array<YearHeader> = [];
  public activeYear: number;
  public activeYears: Array<number>;
  public rows: Array<PBTItemRow> = [];
  public detailedStats: any;

  public viewSelectorOptions;
  public viewSelected = 'plan';
  public loading = true;
  public monthShift = 0;
  public isPastMonth = isPastMonth;
  private firstLoad = true;

  constructor(private financeService: FinanceService, public dialogService: DialogService, public translateService: TranslateService) {
  }

  get dualView(): boolean {
    return this.viewSelected === 'both';
  }

  get planView(): boolean {
    return this.viewSelected === 'plan' || this.viewSelected === 'both';
  }

  get factView(): boolean {
    return this.viewSelected === 'fact' || this.viewSelected === 'both';
  }

  get projectIdString(): string {
    return String(this.projectId);
  }

  ngOnInit() {
    this.viewSelectorOptions = [
      {name: this.translateService.instant('FINANCE.VIEW.PLAN'), value: 'plan'},
      {name: this.translateService.instant('FINANCE.VIEW.FACT'), value: 'fact'},
      {name: this.translateService.instant('FINANCE.VIEW.PLAN_FACT'), value: 'both'},
    ];

    this.loadData();
  }

  loadData(): void {
    firstValueFrom(
      forkJoin([
        this.financeService.financeProjectBudgetsList({
          project: this.projectIdString,
        }),
        this.financeService.financeProjectBudgetTypesList({project: this.projectIdString}),
        this.financeService.financeProjectBudgetsDetailedStats({id: this.project.project_budget}),
        this.financeService.financeProjectBudgetItemsList({
          project: this.projectIdString,
          include: 'expenses',
        }),
      ]),
    ).then((res) => {
      this.projectBudget = res[0][0];
      this.projectBudgetTypes = res[1];
      this.detailedStats = res[2].stats;
      this.projectBudgetItems = res[3];
      this.projectBudgetItemsByType = _.groupBy(this.projectBudgetItems, 'budget_type') || {};

      this.makeHeaders();
      this.loadItems();
      this.loading = false;
    });
  }

  updateData(): void {
    this.financeService.financeProjectBudgetTypesList({project: this.projectIdString}).subscribe((types) => {
      this.projectBudgetTypes = types;
      this.loadItems();
    });
  }

  makeHeaders(): void {
    if (this.project === undefined) {
      return;
    }

    const firstItemDate = moment(this.projectBudgetItems[0]?.date);
    const lastItemDate = moment(_.last(this.projectBudgetItems)?.date);

    const budgetStartDate = moment(this.projectBudget.period_start);
    const budgetEndDate = moment(this.projectBudget.period_end);

    const startDate = firstItemDate < budgetStartDate ? firstItemDate : budgetStartDate;
    const endDate = lastItemDate > budgetEndDate ? lastItemDate : budgetEndDate;
    endDate.add(this.monthShift, 'months');

    const startYear = startDate.year();
    const endYear = endDate.year();

    this.years = _.range(startYear, endYear + 1);
    const nowYear = moment().year();
    const nowMonth = moment().month();

    if (this.firstLoad) {
      if (this.years.includes(nowYear)) {
        this.activeYears = [nowYear];
      } else {
        this.activeYears = [this.years[0]];
      }
      if (this.projectBudget.approved) {
        this.viewSelected = 'both';
      }
      this.firstLoad = false;
    }

    this.yearMonths = [];
    this.years.forEach((year) => {
      let startMonth = 1;
      let endMonth = 12;
      if (year === startYear) {
        startMonth = startDate.month() + 1;
      }
      if (year === endYear) {
        endMonth = endDate.month() + 1;
      }

      const quarters = _.range(getQuarterByMonth(startMonth), getQuarterByMonth(endMonth) + 1);
      const months = _.range(startMonth, endMonth + 1);
      const ym: YearData = {
        year: year,
        months: months,
        quarters: quarters,
        quarterLengths: new Array(quarters.length).fill(3),
      };
      if (startMonth > 1) {
        ym.quarterLengths[0] = Dates.startQuarterMapper[startMonth % 3];
      }
      if (endMonth < 12) {
        ym.quarterLengths[quarters.length - 1] = Dates.endQuarterMapper[endMonth % 3];
      }
      if (year === nowYear && ym.months.includes(nowMonth)) {
        ym.nowMonth = nowMonth;
      }
      this.yearMonths.push(ym);
    });
  }

  loadItems() {
    this.loading = true;
    this.rows = [];
    this.projectBudgetTypes.forEach((pbt) => {
      const row: PBTItemRow = {
        pbt: pbt,
        data: [],
      };
      const items = this.projectBudgetItemsByType[pbt.id] || [];
      this.yearMonths.forEach((year) => {
        const ym: YearData = {
          year: year.year,
          months: year.months,
          quarters: year.quarters,
          quarterLengths: year.quarterLengths,
          yearData: generateTotalEmptyData(1, year.year),
          quarterData: year.quarters.map((q) => generateTotalEmptyData((q - 1) * 3 + 1, year.year)),
          monthData: [],
          nowMonth: year.nowMonth,
        };

        ym.months.forEach((m) => {
          const item = items[0];
          if (!item) {
            ym.monthData.push(generateMockData(m, year.year, pbt.id, this.projectId));
            return;
          }
          const itemDate = moment(item?.date);
          if (itemDate.year() === year.year && itemDate.month() + 1 === m) {
            ym.monthData.push(item);
            ym.yearData.plan += Number(item.plan);
            ym.yearData.forecast += Number(item.forecast);
            ym.yearData.fact += Number(item.fact);
            if (item.approved) {
              ym.yearData.factForecast += Number(item.fact);
            } else {
              ym.yearData.factForecast += Number(item.forecast);
            }
            if (isPastMonth(itemDate)) {
              ym.yearData.approved = ym.yearData.approved && item.approved;
            }

            items.shift();
          } else {
            ym.monthData.push(generateMockData(m, year.year, pbt.id, this.projectId));
            if (isPastMonth(itemDate)) {
              ym.yearData.approved = false;
            }
          }
        });

        row.data.push(ym);
      });
      this.rows.push(row);
    });
    this.loading = false;
  }

  yearChange(year: number) {
    if (this.activeYears.includes(year)) {
      _.pull(this.activeYears, year);
    } else {
      this.activeYears.push(year);
    }

    if (year !== this.activeYear) {
      this.activeYear = year;
    } else {
      this.activeYear = null;
    }
  }

  itemClick(item: ProjectBudgetItem, pbt: ProjectBudgetType) {
    const ref = this.dialogService.open(ProjectBudgetItemComponent, {
      data: {
        item: item,
        currency: this.projectBudget.total_planned_currency,
        planDisabled: this.projectBudget.approved,
        perms$: this.perms$,
        overplan: Number(pbt.stats.total_overplan) - Number(item.plan),
      },
      width: '40%',
      header:
        pbt.name +
        ' - ' +
        this.translateService.instant('MONTHS.NUMBERS.' + (getMonthFromDate(item.date) + 1)) +
        ' ' +
        getYearFromDate(item.date),
    });

    ref.onClose.subscribe((editedItem: ProjectBudgetItem) => {
      if (!editedItem) {
        return;
      }
      if (editedItem?.id) {
        this.financeService
          .financeProjectBudgetItemsPartialUpdate({
            id: editedItem.id,
            data: editedItem,
          })
          .subscribe(() => this.loadData());
      } else {
        this.financeService.financeProjectBudgetItemsCreate(editedItem).subscribe(() => this.loadData());
      }
    });
  }

  monthInfo(month: number, year: number) {
    const date = moment(+String(year) + '-' + String(month) + '-1');
    const ref = this.dialogService.open(ProjectMonthInfoComponent, {
      data: {
        period: date,
        projectId: this.projectId,
        perms$: this.perms$,
        currency: this.projectBudget.total_planned_currency,
      },
      header: this.translateService.instant('FINANCE.MONTH_INFO.HEADER', {period: date.format('MMMM YYYY')}),
      width: '60%',
    });

    ref.onClose.subscribe((res) => {
      if (res) {
        this.loadData();
      }
    });
  }

  addColumn() {
    this.monthShift += 1;
    this.loadData();
    this.activeYear = _.last(this.years);
    this.activeYears = [_.last(this.years)];
  }

  exportToXLSX() {
    const oldActiveYears = this.activeYears;
    const oldExtension = this.extension;

    this.activeYears = this.years;
    this.extension = 'none';
    setTimeout(() => {
      const tableEl = document.getElementById('fintable');

      const workbook = xlsx.utils.table_to_book(tableEl);

      const filename = this.translateService.instant('FINANCE.TABLE.EXCEL_FILENAME', {project: this.project.name});
      xlsx.writeFile(workbook, filename + '.xlsx');
      this.activeYears = oldActiveYears;
      this.extension = oldExtension;
    }, 500);
  }
}
