import { Component, OnInit } from '@angular/core';
import { forkJoin, Observable } from 'rxjs';
import { generateMockData, isPastMonth } from '@core-project/containers/project/project-finance/helpers';
import * as moment from 'moment';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { switchMap } from 'rxjs/operators';
import { ProjectBudgetItem, ProjectBudgetType } from 'data-access/sdk/api2';
import { FinanceService } from 'data-access/sdk/api2';

@Component({
  selector: 'app-project-month-info',
  templateUrl: './project-month-info.component.html',
  styleUrls: ['./project-month-info.component.scss'],
})
export class ProjectMonthInfoComponent implements OnInit {
  public period: string;
  public projectId: number;
  public projectBudgetTypes: Array<ProjectBudgetType>;
  public projectBudgetItems: Array<ProjectBudgetItem>;
  public currency: string;
  public loading = true;
  public approveLoading = false;
  public periodDate: moment.Moment;
  public pbisByBudgetId: { [n: number]: ProjectBudgetItem };
  public allApproved = true;
  public perms$: Observable<Array<string>>;

  public isPastMonth = isPastMonth;

  constructor(
    public financeService: FinanceService,
    public dynamicDialogRef: DynamicDialogRef,
    public dynamicDialogConfig: DynamicDialogConfig
  ) {}

  ngOnInit(): void {
    this.projectId = this.dynamicDialogConfig.data.projectId;
    this.perms$ = this.dynamicDialogConfig.data.perms$;
    this.currency = this.dynamicDialogConfig.data.currency;
    this.period = this.dynamicDialogConfig.data.period;
    this.periodDate = moment(this.period);
    this.loadData();
  }

  public loadData(): void {
    this.loading = true;
    forkJoin([
      this.financeService.financeProjectBudgetTypesList({
        project: String(this.projectId),
      }),
      this.financeService.financeProjectBudgetItemsList({
        project: String(this.projectId),
        date: this.periodDate.format('YYYY-MM-DD HH:mm'),
        include: 'expenses',
      }),
    ]).subscribe((res) => {
      this.projectBudgetTypes = res[0];
      this.projectBudgetItems = res[1];
      const pbtIds = this.projectBudgetTypes.map((pbt) => pbt.id);
      const pbtIdsInitems = this.projectBudgetItems.map((pbi) => pbi.budget_type);
      const pbtWOitems = pbtIds.filter((pbtId) => !pbtIdsInitems.includes(pbtId));
      pbtWOitems.forEach((p) =>
        this.projectBudgetItems.push(generateMockData(this.periodDate.month() + 1, this.periodDate.year(), p, this.projectId))
      );
      this.pbisByBudgetId = {};
      this.allApproved = true;
      this.projectBudgetItems.forEach((pbi) => {
        this.pbisByBudgetId[pbi.budget_type] = pbi;
        this.allApproved = this.allApproved && pbi.approved;
      });
      this.loading = false;
    });
  }

  public approve(): void {
    this.approveLoading = true;
    const observables: Array<Observable<ProjectBudgetItem>> = [];
    this.projectBudgetItems.forEach((pbi) => {
      if (!pbi.approved) {
        if (pbi.id) {
          observables.push(this.financeService.financeProjectBudgetItemsToggleApproval(pbi.id));
        } else {
          observables.push(
            this.financeService
              .financeProjectBudgetItemsCreate(pbi)
              .pipe(switchMap((newPbi) => this.financeService.financeProjectBudgetItemsToggleApproval(newPbi.id)))
          );
        }
      }
    });
    forkJoin(observables).subscribe((_) => this.dynamicDialogRef.close(true));
  }
}
