import {Component, OnInit} from '@angular/core';
import {ActivatedRoute} from '@angular/router';
import {Project, ProjectsService, ProjectWorkPlan, WorklinksService, WorkplansService, WorksService} from 'data-access/sdk/api2';
import {BehaviorSubject, firstValueFrom, Observable, of} from 'rxjs';
import {map, switchMap, takeUntil} from 'rxjs/operators';
import {WorkEnums} from '@core-project/enums';
import {DialogService} from 'primeng/dynamicdialog';
import {TranslateService} from '@ngx-translate/core';
import {
  WorksFromTemplateFormComponent,
} from '@core-project/modules/shared-components/works/works-from-template-form/works-from-template-form.component';
import {MessageService} from 'primeng/api';
import {GanttWork, GanttWorkLink, WorkLinkToGanttAdapter, WorkToGanttAdapter} from '@core-project/helpers/gantt-adapter';
import {Actions, ofActionSuccessful, Select, Store} from '@ngxs/store';
import {ProjectRulesSelectors, WorksActions} from 'data-access/store';
import {Unsubscriber} from 'utils/shared-services';

@Component({
  selector: 'app-project-plan',
  templateUrl: './project-plan.component.html',
  styleUrls: ['./project-plan.component.scss'],
  providers: [Unsubscriber],
})
export class ProjectPlanComponent implements OnInit {
  private readonly createObserverSub = this.actions
    .pipe(
      takeUntil(this.unsubscriber.destroy$),
      ofActionSuccessful(WorksActions.CreateWork),
    )
    .subscribe(() => {
      this.loadWorks();
    });

  @Select(ProjectRulesSelectors.canNotUpdate('projectworkplan')) canNotUpdate$: Observable<boolean>;

  projectId: number;

  projectSubject: BehaviorSubject<Project> = new BehaviorSubject(undefined);
  project$: Observable<Project> = this.projectSubject.asObservable();

  worksSubject: BehaviorSubject<GanttWork[]> = new BehaviorSubject([]);
  works$: Observable<GanttWork[]> = this.worksSubject.asObservable();

  linksSubject: BehaviorSubject<GanttWorkLink[]> = new BehaviorSubject([]);
  links$: Observable<GanttWorkLink[]> = this.linksSubject.asObservable();

  workPlanSubject: BehaviorSubject<ProjectWorkPlan> = new BehaviorSubject(undefined);
  workPlan$: Observable<ProjectWorkPlan> = this.workPlanSubject.asObservable();

  scale: WorkEnums.GanttScaleEnum = WorkEnums.GanttScaleEnum.WEEK;
  autoschedule = false;
  viewMode: WorkEnums.GanttViewModeEnum = WorkEnums.GanttViewModeEnum.CURR;

  constructor(
    private readonly activatedRoute: ActivatedRoute,
    private readonly worksService: WorksService,
    private readonly workLinksService: WorklinksService,
    private readonly workPlansService: WorkplansService,
    private readonly dialogService: DialogService,
    private readonly projectsService: ProjectsService,
    private readonly translateService: TranslateService,
    private readonly messageService: MessageService,
    private readonly store: Store,
    private readonly actions: Actions,
    private readonly unsubscriber: Unsubscriber,
  ) {
  }

  ngOnInit() {
    this.projectId = Number(this.activatedRoute.parent.snapshot.paramMap.get('projectId'));

    this.projectsService.projectsRead({id: this.projectId}).subscribe((project) => {
      this.projectSubject.next(project);
    });

    this.workPlansService
      .workplansList({project: this.projectId.toString()})
      .pipe(
        switchMap((wps) => {
          if (wps.length > 0) {
            return of(wps[0]);
          }
          return this.workPlansService.workplansCreate({project: this.projectId});
        }),
      )
      .subscribe((projectWorkPlan) => {
        this.workPlanSubject.next(projectWorkPlan);
      });

    this.loadWorks();
    this.loadLinks();
  }

  addWork() {
    this.store.dispatch(new WorksActions.ShowCreateWorkInSidebar({
      projectId: this.projectId.toString(),
    }));
  }

  toggleWorkPlanApproval(workPlan: ProjectWorkPlan) {
    this.workPlansService.workplansToggleApproval({id: workPlan.id, data: null}).subscribe((projectWorkPlan) => {
      this.workPlanSubject.next(projectWorkPlan);
      this.loadWorks();
    });
  }

  fillFromTemplate(workPlan: ProjectWorkPlan, project: Project) {
    const ref = this.dialogService.open(WorksFromTemplateFormComponent, {
      header: this.translateService.instant('WORKS.TMPL_FORM.HEADER'),
      width: '40%',
      data: {
        start_date: null,
      },
    });

    firstValueFrom(ref.onClose).then((data) => {
      if (data) {
        this.messageService.add({
          summary: this.translateService.instant('GANTT.PLAN_CREATING_HDR'),
          detail: this.translateService.instant('GANTT.PLAN_CREATING_MSG'),
          severity: 'info',
        });
        firstValueFrom(this.workPlansService.workplansPlanFromTemplate({id: workPlan.id, data: data})).then((_wp) => {
          this.loadWorks();
          this.loadLinks();
          this.messageService.add({
            summary: this.translateService.instant('GANTT.PLAN_CREATED_HDR'),
            detail: this.translateService.instant('GANTT.PLAN_CREATED_MSG'),
            severity: 'success',
          });
        });
      }
    });
  }

  private loadWorks() {
    this.worksService
      .worksList({
        project: this.projectId.toString(),
        workType: 'TS,MS,GR',
        include: 'to_user, to_department',
      })
      .pipe(map((works) => works.map((work) => WorkToGanttAdapter(work))))
      .subscribe((works) => {
        this.worksSubject.next(works);
      });
  }

  private loadLinks() {
    this.workLinksService
      .worklinksList({project: this.projectId.toString()})
      .pipe(map((links) => links.map((link) => WorkLinkToGanttAdapter(link))))
      .subscribe((links) => {
        this.linksSubject.next(links);
      });
  }
}
