import { toSignal } from '@angular/core/rxjs-interop';
import { Component, OnInit, Signal } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { Actions, ofActionSuccessful, Store } from '@ngxs/store';
import { BehaviorSubject, switchMap } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import { ProjectRulesSelectors, WorksActions } from 'data-access/store';
import { Unsubscriber } from 'utils/shared-services';
import { WorkApiService, WorkModel } from 'data-access/sdk/internal-api';
import { KanbanChangeInterface } from 'ui/kanban-board';

@Component({
  selector: 'app-project-tasks',
  templateUrl: './project-tasks.component.html',
  styleUrls: ['./project-tasks.component.scss'],
  providers: [Unsubscriber],
})
export class ProjectTasksComponent implements OnInit {
  private readonly createObserverSub = this.actions
    .pipe(
      takeUntil(this.unsubscriber.destroy$),
      ofActionSuccessful(WorksActions.CreateWork, WorksActions.UpdateWork, WorksActions.ChangeWorkStatus, WorksActions.DeleteWork)
    )
    .subscribe(() => {
      this.loadData();
    });

  projectId: number;

  readonly worksSubject: BehaviorSubject<WorkModel[]> = new BehaviorSubject([]);
  readonly works: Signal<WorkModel[]> = toSignal(this.worksSubject.asObservable());

  readonly canNotUpdateWork: Signal<boolean> = toSignal(this.store.select(ProjectRulesSelectors.canNotUpdate('work')));

  constructor(
    private readonly activatedRoute: ActivatedRoute,
    private readonly store: Store,
    private readonly actions: Actions,
    private readonly unsubscriber: Unsubscriber,
    private readonly workApi: WorkApiService
  ) {}

  ngOnInit() {
    this.projectId = Number(this.activatedRoute.parent.snapshot.paramMap.get('projectId'));
    this.loadData();
  }

  openCard(work: WorkModel) {
    this.store.dispatch(new WorksActions.ShowWorkInSidebar(work.id)).subscribe();
  }

  moveCard(change: KanbanChangeInterface) {
    const currentWorkId = change.card.rawModel.id;

    this.workApi
      .changeWorkStatusAndOrderProject(
        currentWorkId,
        change.nextCard?.rawModel.id,
        change.previousCard?.rawModel.id,
        change.card.rawModel.status
      )
      .pipe(switchMap(() => this.store.dispatch(new WorksActions.RefreshWork(currentWorkId))))
      .subscribe();
  }

  loadData() {
    this.workApi
      .readList({
        workType: 'TS,MS',
        project: this.projectId.toString(),
        include: 'to_user, to_department',
      })
      .subscribe((works: WorkModel[]) => {
        this.worksSubject.next(works);
      });
  }

  addWork() {
    return this.store
      .dispatch(
        new WorksActions.ShowCreateWorkInSidebar({
          projectId: this.projectId.toString(),
        })
      )
      .subscribe();
  }
}
