import {Component, OnInit} from '@angular/core';
import {ActivatedRoute} from '@angular/router';
import {DialogService} from 'primeng/dynamicdialog';
import {firstValueFrom, Observable} from 'rxjs';
import {ConfirmationService} from 'primeng/api';
import {TranslateService} from '@ngx-translate/core';
import {ProjectMember, ProjectMembersService, UserRole, UserRolesService} from 'data-access/sdk/api2';
import {
  UserRoleFormComponent,
  UserRoleFormComponentData,
} from '@core-project/modules/forms/role-forms/user-role-form/user-role-form.component';
import {ProjectMemberFormComponent} from '@core-project/modules/forms/project-member-form/project-member-form.component';

@Component({
  selector: 'app-project-team',
  templateUrl: './project-team.component.html',
  styleUrls: ['./project-team.component.scss'],
})
export class ProjectTeamComponent implements OnInit {

  userRoles$: Observable<UserRole[]>;
  members$: Observable<ProjectMember[]>;

  private projectId: number;

  constructor(
    private readonly activatedRoute: ActivatedRoute,
    private readonly userRolesService: UserRolesService,
    private readonly projectMembersService: ProjectMembersService,
    private readonly translateService: TranslateService,
    private readonly dialogService: DialogService,
    private readonly confirmationService: ConfirmationService,
  ) {
  }

  ngOnInit() {
    this.projectId = this.activatedRoute.parent.snapshot.params.projectId;
    this.getMembers();
    this.getUserRoles();
  }

  editUserRole(userRole?: UserRole): void {
    const header = userRole
      ? this.translateService.instant('PROJECT_TEAM.USER_ROLE_EDIT_TITLE')
      : this.translateService.instant('PROJECT_TEAM.USER_ROLE_ADD_TITLE');

    const data: UserRoleFormComponentData = {
      type: 'project',
      userRole,
      projectId: this.projectId,
      isDepartmentsResponsible: false,
    };

    const ref = this.dialogService.open(UserRoleFormComponent, {
      header: header,
      width: '50%',
      data,
    });

    firstValueFrom(ref.onClose).then(ur => {
      if (ur) {
        this.getUserRoles();
      }
    });
  }

  editMember(member?: ProjectMember): void {
    const header = member
      ? this.translateService.instant('PROJECT_TEAM.MEMBER_EDIT_TITLE')
      : this.translateService.instant('PROJECT_TEAM.MEMBER_ADD_TITLE');

    const ref = this.dialogService.open(ProjectMemberFormComponent, {
      header: header,
      width: '50%',
      data: {
        member: member,
        projectId: this.projectId,
      },
    });

    firstValueFrom(ref.onClose).then(m => {
      if (m) {
        this.getMembers();
      }
    });
  }

  deleteUserRole(userRoleId: number): void {
    this.confirmationService.confirm({
      header: this.translateService.instant('PROJECT.TEAM.DELETE_USER_HEADER'),
      message: this.translateService.instant('PROJECT.TEAM.DELETE_USER_MESSAGE'),
      accept: () => this.userRolesService.userRolesDelete(userRoleId)
        .subscribe(() => this.getUserRoles()),
    });
  }

  deleteMember(memberId: number): void {
    this.confirmationService.confirm({
      header: this.translateService.instant('PROJECT.TEAM.DELETE_MEMBER_HEADER'),
      message: this.translateService.instant('PROJECT.TEAM.DELETE_MEMBER_MESSAGE'),
      accept: () => this.projectMembersService.projectMembersDelete(memberId)
        .subscribe(() => this.getMembers()),
    });
  }

  private getUserRoles() {
    this.userRoles$ = this.userRolesService
      .userRolesList({project: this.projectId.toString(), include: 'role, user__telephones'});
  }

  private getMembers() {
    this.members$ = this.projectMembersService
      .projectMembersList({project: this.projectId.toString(), include: 'organization, department, user'});
  }
}
