import { Component, Inject, OnInit, ViewChild, ViewEncapsulation } from '@angular/core';
import { firstValueFrom, forkJoin, Observable, of } from 'rxjs';
import * as _ from 'lodash-es';
import { switchMap } from 'rxjs/operators';
import { DialogService } from 'primeng/dynamicdialog';
import { ConfirmationService, MessageService, SelectItemGroup } from 'primeng/api';
import { TranslateService } from '@ngx-translate/core';
import { Router } from '@angular/router';
import * as moment from 'moment';
import * as xlsx from 'xlsx';

import { Table, TableColResizeEvent, TableFilterEvent } from 'primeng/table';
import { Store } from '@ngxs/store';
import { DOCUMENT } from '@angular/common';
import { ProjectEnums } from '@core-project/enums/project';
import {
  BudgetType,
  FinanceService,
  Project,
  ProjectGroupsService,
  ProjectMeta,
  ProjectMetaGroup,
  ProjectMetaGroupsService,
  ProjectMetaService,
  ProjectMetaValue,
  ProjectMetaValueService,
  ProjectsService,
  ProjectTypesService,
  ProjectView,
  ProjectViewService,
  Role,
  RolesService,
  StagesService,
  User,
  UsersService,
} from 'data-access/sdk/api2';
import { ProjectColumn, ProjectExtra } from '@core-project/interfaces/projects';
import { ProjectGroupColumn } from '@core-project/interfaces/projects/project-group-column.interface';
import { YearData, YearHeader } from '@core-project/interfaces/finance';
import { ChoicefieldInterface } from '@core-project/interfaces';
import { AuthenticationService } from '@core-project/services/authentication.service';
import { TableResizeService } from '@core-project/modules/ui-kit/table-resize/table-resize.service';
import { checkEnumValue, Dates, getQuarterByMonth, projectToProjectExtra } from '@core-project/helpers';
import { ProjectStatusComponent } from '@core-project/modules/shared-components/project-status/project-status.component';
import { ProjectStateComponent } from '@core-project/modules/shared-components/project-state/project-state.component';
import { ProjectFormComponent } from '@core-project/modules/forms/project-form/project-form.component';
import { ProjectViewFormComponent } from '@core-project/modules/forms/project-view-form/project-view-form.component';
import { UsersStringDynamicComponent } from 'features/string-info-overlays';
import { isNumber } from 'lodash';
import { GlobalRulesSelectors } from 'data-access/store';

const currentViewIdLSKey = 'projectsViewId';

// TODO remove "as any"
@Component({
  selector: 'app-projects-structure',
  templateUrl: './projects-structure.component.html',
  styleUrls: ['./projects-structure.component.scss'],
  encapsulation: ViewEncapsulation.None,
})
export class ProjectsStructureComponent implements OnInit {
  @ViewChild('projectsTable') projectsTable: Table;

  lodash = _;
  dataTypes = ProjectEnums.DataTypeEnum;
  scaleTypes = ProjectEnums.BudgetPeriodScaleEnum;

  user: User;
  perms: string[]; // deprecated
  projects: Array<ProjectExtra>;
  metaFields: Array<ProjectMeta>;
  metaGroups: Array<ProjectMetaGroup>;
  projectRoles: Array<Role>;
  projectMemberTypes: Array<ProjectEnums.MemberTypeEnum> = Object.values(ProjectEnums.MemberTypeEnum);
  views: Array<ProjectView>;
  viewsChoices: Array<SelectItemGroup>;
  columns: Array<ProjectColumn>;
  groupColumns: Array<ProjectGroupColumn>;
  displaySidebar = false;

  _currentView: ProjectView | undefined;

  filterState: null | {};

  editMode = false;
  editedValue: any;
  editedColumn: ProjectColumn;
  editedProject: ProjectExtra;
  choices = {
    customers: null,
    types: null,
    groups: null,
    statuses: null,
    stages: null,
    users: null,
  };
  loading = true;
  loadingValues = [...Array(5).keys()];
  budgetCalendarHeader: Array<YearHeader>;
  budgetDetailOptions: Array<ChoicefieldInterface> = [];
  budgetOptionsOptions: Array<ChoicefieldInterface> = [];
  extension = 'auto';
  budgetTypes: Array<BudgetType>;
  columnBaseWidth = '150px';

  isPublishView$: Observable<boolean> = this.store.select(GlobalRulesSelectors.customRule('publish', 'projectview'));

  constructor(
    private readonly projectsService: ProjectsService,
    private readonly usersService: UsersService,
    private readonly authenticationService: AuthenticationService,
    private readonly projectMetaService: ProjectMetaService,
    private readonly projectMetaGroupsService: ProjectMetaGroupsService,
    private readonly projectMetaValueService: ProjectMetaValueService,
    private readonly projectTypesService: ProjectTypesService,
    private readonly projectViewService: ProjectViewService,
    private readonly projectGroupsService: ProjectGroupsService,
    private readonly dialogService: DialogService,
    private readonly messageService: MessageService,
    private readonly translateService: TranslateService,
    private readonly router: Router,
    private readonly confirmationService: ConfirmationService,
    private readonly financeService: FinanceService,
    private readonly rolesService: RolesService,
    private readonly stagesService: StagesService,
    private readonly tableResizeService: TableResizeService,
    private readonly store: Store,
    @Inject(DOCUMENT) private readonly document: Document
  ) {}

  get currentView(): ProjectView | undefined {
    return this._currentView;
  }

  get currentViewEdited() {
    if (this.currentView?.id === undefined || this.currentView?.id === null) {
      return true;
    }
    const oldView = _.find(this.views, ['id', this.currentViewId]);
    return !_.isEqual(oldView, this.currentView);
  }

  get groupedColumn(): string {
    return this.currentView.group_by;
  }

  get groupableColumns(): ProjectColumn[] {
    return this.columns.filter((c) => c.groupable);
  }

  get currentViewId(): number | undefined {
    return this.currentView?.id;
  }

  set currentView(view: ProjectView) {
    this._currentView = view;
  }

  set groupedColumn(columnDisplayKey: string) {
    _.remove(this.currentView.sorting, ['field', this.currentView.group_by]);
    if (columnDisplayKey) {
      _.remove(this.currentView.sorting, ['field', columnDisplayKey]);
      this.currentView.sorting = [{ field: columnDisplayKey, order: 1 }, ...this.currentView.sorting];
    }
    this.currentView.group_by = columnDisplayKey;
    this.saveCurrentViewLS();
  }

  set currentViewId(id: number) {
    const view: ProjectView = _.find(this.views, ['id', id]);
    this.changeView(view);
  }

  ngOnInit() {
    this.user = this.authenticationService.user;
    this.usersService.usersPermissions({ id: this.user.id, company: this.user.company }).subscribe((userPermissions) => {
      this.perms = userPermissions.permissions;
      this.initChoices();
      this.loadData();
    });
    Object.values(ProjectEnums.BudgetPeriodScaleEnum).forEach((scale) =>
      this.budgetDetailOptions.push({
        value: scale,
        label: this.translateService.instant('PROJECTS.BUDGETS.PERIODS.' + scale),
      })
    );
    Object.values(ProjectEnums.BudgetOptionsEnum).forEach((opt) =>
      this.budgetOptionsOptions.push({
        value: opt,
        label: this.translateService.instant('PROJECTS.BUDGETS.OPTIONS.' + opt),
      })
    );
  }

  loadData() {
    firstValueFrom(
      forkJoin([
        this.loadProjects(),
        this.loadMetaFields(),
        this.loadProjectViews(),
        this.loadMetaGroups(),
        this.loadBudgetTypes(),
        this.loadProjectRoles(),
      ])
    ).then((_res) => {
      this.generateColumns();
      const savedViewId = this.loadCurrentViewLS();

      const savedView: ProjectView | undefined = _.find(this.views, ['id', savedViewId]);
      const defaultView: ProjectView | undefined = _.find(this.views, 'default');

      if (savedView) {
        this.updateView(savedView);
      } else if (defaultView) {
        this.updateView(defaultView);
      } else {
        this.updateView(this.generateDefaultView());
      }

      this.filterAndSortColumns();
      this.loadBudgetCalendar();
      this.loading = false;
    });
  }

  loadProjects(): Observable<Array<ProjectExtra>> {
    return this.usersService.usersProjectsProjectsAll(this.user.id).pipe(
      switchMap((upr) => {
        this.projects = upr.projects.map((p) => projectToProjectExtra(p));
        return of(this.projects);
      })
    );
  }

  loadMetaFields(): Observable<any> {
    return this.projectMetaService.projectMetaList({}).pipe(
      switchMap((ms) => {
        this.metaFields = ms;
        return of(this.metaFields);
      })
    );
  }

  loadMetaGroups(): Observable<any> {
    return this.projectMetaGroupsService.projectMetaGroupsList('meta_fields').pipe(
      switchMap((mgs) => {
        this.metaGroups = mgs;
        return of(this.metaGroups);
      })
    );
  }

  loadProjectViews(): Observable<any> {
    return this.projectViewService.projectViewList().pipe(
      switchMap((views) => {
        this.views = views;
        this.viewsChoices = [
          {
            label: this.translateService.instant('PROJECTS.PUBLIC_VIEWS'),
            items: this.views.filter((v) => v.public).map((v) => ({ label: v.name, value: v.id })),
          },
          {
            label: this.translateService.instant('PROJECTS.PRIVATE_VIEWS'),
            items: this.views.filter((v) => !v.public).map((v) => ({ label: v.name, value: v.id })),
          },
        ];
        return of(this.views);
      })
    );
  }

  loadProjectRoles(): Observable<any> {
    return this.rolesService.rolesList({ project: 'true' }).pipe(
      switchMap((roles) => {
        this.projectRoles = roles;
        return of(this.projectRoles);
      })
    );
  }

  generateColumns() {
    const groups: Array<ProjectGroupColumn> = [
      {
        name: this.translateService.instant('PROJECTS.GROUPS.PRIMARY'),
        columns: [
          {
            id: 'name',
            name: this.translateService.instant('PROJECTS.HEADERS.NAME'),
            dataDisplayKey: 'name',
            required: true,
            type: ProjectEnums.DataTypeEnum.CH,
            editable: true,
            frozen: true,
          },
          {
            id: 'project_type',
            name: this.translateService.instant('PROJECTS.HEADERS.PROJECT_TYPE'),
            required: false,
            groupable: true,
            editable: false,
            type: ProjectEnums.DataTypeEnum.CHS,
            dataEditKey: 'project_type',
            dataDisplayKey: 'rel_project_type.name',
            choices: this.choices.types,
          },
          {
            id: 'project_group',
            name: this.translateService.instant('PROJECTS.HEADERS.PROJECT_GROUP'),
            required: false,
            groupable: true,
            type: ProjectEnums.DataTypeEnum.CHS,
            dataEditKey: 'project_group',
            editable: false,
            dataDisplayKey: 'rel_project_group.name',
            choices: this.choices.groups,
          },
          {
            id: 'status',
            name: this.translateService.instant('PROJECTS.HEADERS.STATUS'),
            dataDisplayKey: 'status',
            dataTranslate: true,
            dataTranslateKey: 'PROJECTS.STATUS',
            required: false,
            groupable: true,
            type: ProjectEnums.DataTypeEnum.CHS,
            choices: this.choices.statuses,
            editable: false,
            displayComponent: ProjectStatusComponent,
            displayComponentInput: 'status',
            displayComponentStaticInput: { textStyle: 'short' },
          },
          {
            id: 'stage',
            name: this.translateService.instant('PROJECTS.HEADERS.STAGE'),
            dataDisplayKey: 'rel_stage.name',
            dataEditKey: 'stage',
            dataTranslate: false,
            required: false,
            groupable: true,
            editable: false,
            type: ProjectEnums.DataTypeEnum.CHS,
            choices: this.choices.stages,
          },
          {
            id: 'state',
            name: this.translateService.instant('PROJECTS.HEADERS.STATE'),
            dataDisplayKey: 'id',
            dataTranslate: false,
            required: false,
            groupable: false,
            type: ProjectEnums.DataTypeEnum.CHS,
            editable: false,
            displayComponent: ProjectStateComponent,
            displayComponentInput: 'projectId',
          },
        ],
      },
      {
        name: this.translateService.instant('PROJECTS.GROUPS.ROLES'),
        columns: this.projectRoles.map((role) => ({
          id: 'role__' + role.id,
          name: role.name,
          dataDisplayKey: 'users_by_role_id.' + role.id.toString(),
          mtmDisplayKey: 'short_name',
          choices: this.choices.users,
          required: false,
          type: ProjectEnums.DataTypeEnum.MTM,
          editable: false,
          groupable: true,
          displayComponent: UsersStringDynamicComponent,
          displayComponentInput: 'users',
          displayComponentStaticInput: {
            addBr: false,
          },
        })),
      },
      {
        name: this.translateService.instant('PROJECTS.GROUPS.MEMBERS'),
        columns: this.projectMemberTypes.map((mt) => ({
          id: 'member__' + mt,
          name: this.translateService.instant('PROJECT.TEAM.TYPES.' + mt),
          dataDisplayKey: 'members_by_type.' + mt,
          // dataDisplayFn: (members) => members.join(', '),
          required: false,
          type: ProjectEnums.DataTypeEnum.MCHS,
          editable: false,
          groupable: true,
        })),
      },
      {
        name: this.translateService.instant('PROJECTS.GROUPS.BUDGET'),
        columns: [
          {
            id: 'project_budget',
            name: this.translateService.instant('PROJECTS.HEADERS.PROJECT_BUDGET'),
            dataDisplayKey: 'rel_project_budget.total_planned',
            required: false,
            groupable: false,
            type: ProjectEnums.DataTypeEnum.INT,
            editable: false,
          },
          {
            id: 'project_budget_code',
            name: this.translateService.instant('PROJECTS.HEADERS.PROJECT_BUDGET_CODE'),
            dataDisplayKey: 'rel_project_budget.code',
            required: false,
            groupable: false,
            type: ProjectEnums.DataTypeEnum.CH,
            editable: false,
          },
          {
            id: 'project_budget_approved',
            name: this.translateService.instant('PROJECTS.HEADERS.PROJECT_BUDGET_APPROVED'),
            dataDisplayKey: 'rel_project_budget.approved',
            required: false,
            groupable: false,
            type: ProjectEnums.DataTypeEnum.FLAG,
            editable: false,
          },
        ]
          .concat(
            [
              'effect_plan',
              'effect_forecast',
              'effect_fact',
              'npv_plan',
              'npv_forecast',
              'npv_fact',
              'irr_plan',
              'irr_forecast',
              'irr_fact',
            ].map((bp) => ({
              id: 'project_budget_' + bp,
              name: this.translateService.instant('PROJECTS.HEADERS.PROJECT_BUDGET_' + bp.toUpperCase()),
              dataDisplayKey: 'rel_project_budget.' + bp,
              required: false,
              groupable: false,
              type: ProjectEnums.DataTypeEnum.INT,
              editable: false,
            }))
          )
          .concat(
            [
              'plan',
              'plan_percent',
              'fact',
              'fact_percent',
              'current_fact_and_fc',
              'current_fact_and_fc_percent',
              'plan_fact_remaining',
              'plan_fact_remaining_percent',
              'current_fact_and_fc_overplan',
              'current_fact_and_fc_overplan_percent',
              'discipline',
            ].map((bs) => ({
              id: 'project_budget_stats_' + bs,
              name: this.translateService.instant('PROJECTS.HEADERS.PROJECT_BUDGET_STATS.' + bs.toUpperCase()),
              dataDisplayKey: 'rel_project_budget.stats.' + bs,
              required: false,
              groupable: false,
              type: ProjectEnums.DataTypeEnum.INT,
              editable: false,
            }))
          ),
      },
    ];

    this.metaGroups.forEach((mg) => {
      const gr: ProjectGroupColumn = {
        name: mg.name,
        columns: [],
      };
      mg.rel_meta_fields.forEach((mf) => {
        const col: ProjectColumn = {
          id: 'meta__' + mf.id,
          name: mf.name,
          meta: mf,
          dataDisplayKey: 'metas_by_id.' + mf.id.toString() + '.value_' + mf.data_type,
          required: false,
          type: checkEnumValue(ProjectEnums.DataTypeEnum, mf.data_type),
          choices: of(mf.choices?.map((ch) => ({ value: ch, label: ch }))),
          editable: true,
          groupable: mf.data_type === ProjectEnums.DataTypeEnum.CHS || mf.data_type === ProjectEnums.DataTypeEnum.MCHS,
        };
        gr.columns.push(col);
      });
      groups.push(gr);
    });

    this.groupColumns = groups;
  }

  filterAndSortColumns() {
    let cols: ProjectColumn[] = [];

    if (this.currentView.fields && this.currentView.fields.length > 0) {
      this.groupColumns.forEach((group) => {
        const filteredGroupCols = group.columns.filter((column) => this.currentView.fields.includes(column.id));

        cols.push(...filteredGroupCols);
      });

      cols = _.sortBy(cols, (column) => this.currentView.fields.indexOf(column.id));
    } else {
      cols = JSON.parse(JSON.stringify(this.groupColumns[0].columns));
    }

    if (!this.currentView.fields_width) {
      this.currentView.fields_width = {};
    }

    cols = cols.map((column) => {
      column.width = this.currentView.fields_width[column.id];
      return column;
    });

    this.tableResizeService.resetCelsWidth(this.projectsTable);

    this.columns = cols;
  }

  createColumnId(column: ProjectColumn) {
    return `col-${column.id}`;
  }

  getIdFromColumn(columnStr: string) {
    return columnStr.slice(4);
  }

  columnsResize(event: TableColResizeEvent) {
    const element = event.element;
    const colId = this.getIdFromColumn(element.id);

    const column: ProjectColumn | undefined = this.columns.find((col) => col.id === colId);
    if (column) {
      this.projectsTable.setResizeTableWidth(this.tableResizeService.calculateTableWidth(this.projectsTable) + 'px');

      // need timeout for render
      setTimeout(() => {
        column.width = element.offsetWidth + 'px';
        const view: ProjectView = JSON.parse(JSON.stringify(this.currentView));
        view.fields_width[column.id] = column.width;
        this.updateView(view);
      }, 1000);
    }
  }

  projectAdd() {
    const ref = this.dialogService.open(ProjectFormComponent, {
      header: this.translateService.instant('PROJECTS.ADD'),
      width: '50%',
      data: {},
    });

    firstValueFrom(
      ref.onClose.pipe(
        switchMap((editedProject) => {
          if (editedProject) {
            return this.projectsService.projectsCreate(editedProject);
          }
        })
      )
    ).then((p) => this.openProject(p));
  }

  getColumnRawDisplayValue(project: ProjectExtra, column: ProjectColumn): any {
    const res: string = _.get(project, column.dataDisplayKey);
    if (res === null || res === undefined || res === '') {
      return null;
    }
    return res;
  }

  getColumnDisplayValue(project: ProjectExtra, column: ProjectColumn): string {
    const raw = this.getColumnRawDisplayValue(project, column);
    if (raw === null || raw === undefined) {
      return null;
    }

    if (column.dataDisplayFn) {
      return column.dataDisplayFn(raw);
    }

    let value: string;

    switch (column.type) {
      case ProjectEnums.DataTypeEnum.MCHS:
        value = raw.join(', ');
        break;
      case ProjectEnums.DataTypeEnum.MTM:
        value = _.flatMap(raw, column.mtmDisplayKey).join(', ');
        break;
      case ProjectEnums.DataTypeEnum.FLAG:
        return raw
          ? '<i class="fa-regular fa-check-circle text-green-500"></i>'
          : '<i class="fa-regular fa-times-circle text-red-500"></i>';
      default:
        value = raw;
    }

    if (value && column.dataTranslate) {
      value = this.translateService.instant(column.dataTranslateKey + '.' + value);
    }

    if (column.meta) {
      value = column.meta.prefix ? column.meta.prefix + ' ' + value : value;
      value = column.meta.postfix ? value + ' ' + column.meta.postfix : value;
    }

    return value;
  }

  columnsReorder(_event) {
    this.currentView.fields = _.map(this.columns, 'id');
    this.saveCurrentViewLS();
  }

  generateDefaultView(): ProjectView {
    return {
      name: this.translateService.instant('PROJECTS.VIEWS.DEFAULT'),
      fields: this.groupColumns[0].columns.map((c) => c.id),
      sorting: [],
      group_by: null,
      filter: null,
      company: this.user.company,
      owner: this.user.id,
      budget: false,
      budget_options: ProjectEnums.BudgetOptionsEnum.FTFC,
      budget_period_scale: ProjectEnums.BudgetPeriodScaleEnum.QTR,
      budget_period_start: moment.utc().startOf('year').format('YYYY-MM-DD'),
      budget_period_end: moment.utc().endOf('year').format('YYYY-MM-DD'),
      budget_types: this.budgetTypes.map((bt) => bt.id),
    };
  }

  changeView(view: ProjectView) {
    if (this.currentViewEdited) {
      this.confirmationService.confirm({
        header: this.translateService.instant('PROJECTS.UNDO_VIEW.HEADER'),
        message: this.translateService.instant('PROJECTS.UNDO_VIEW.MESSAGE'),
        icon: 'pi pi-exclamation-triangle',
        accept: () => {
          this.switchView(view);
        },
      });
    } else {
      this.switchView(view);
    }
  }

  toggleColumn(column: ProjectColumn) {
    if (this.currentView.fields.includes(column.id)) {
      if (!column.required) {
        if (this.currentView.group_by === column.dataDisplayKey) {
          this.groupedColumn = null;
        }
        _.remove(this.currentView.fields, (f) => f === column.id);
      } else {
        this.messageService.add({
          severity: 'error',
          summary: this.translateService.instant('ERRORS.ERROR'),
          detail: this.translateService.instant('PROJECTS.REQUIRED_COLUMN_REMOVE_ERROR'),
        });
      }
    } else {
      this.currentView.fields.push(column.id);
    }
    this.filterAndSortColumns();
    this.saveCurrentViewLS();
  }

  toggleSidebar() {
    this.displaySidebar = !this.displaySidebar;
  }

  sortClick($event) {
    this.currentView.sorting = $event.multisortmeta;
    _.remove(this.currentView.sorting, (f: any) => f.field === undefined || typeof f.field !== 'string');
    this.saveCurrentViewLS();
  }

  toggleSort(column) {
    if (!this.currentView.sorting) {
      this.currentView.sorting = [];
    }
    const curSort = _.find(this.currentView.sorting as any[], { field: column.dataDisplayKey });
    if (!curSort) {
      this.currentView.sorting.push({ field: column.dataDisplayKey, order: 1 });
    } else {
      if (curSort.order === 1) {
        curSort.order = -1;
      } else {
        if (curSort.field !== this.currentView.group_by) {
          _.remove(this.currentView.sorting, { field: curSort.field });
        } else {
          curSort.order = 1;
        }
      }
    }
    if (this.currentView.sorting.length === 0) {
      this.currentView.sorting = [];
    } else {
      this.currentView.sorting = [...this.currentView.sorting];
    }
    this.saveCurrentViewLS();
  }

  openProject(project: Project) {
    if (!this.editMode) {
      this.router.navigate(['/project', project.id, 'details']);
    }
  }

  editInit($event) {
    this.editedValue = this.getColumnRawDisplayValue($event.data, $event.field);
    this.editedColumn = _.cloneDeep($event.field);
    this.editedProject = _.cloneDeep($event.data);
  }

  editComplete() {
    const editedValue = this.editedValue;
    const editedColumn: ProjectColumn = _.cloneDeep(this.editedColumn);
    const project: ProjectExtra = _.cloneDeep(this.editedProject);
    const projectIndex = _.findIndex(this.projects, ['id', project.id]);
    if (editedColumn.meta) {
      let metaValue: ProjectMetaValue = project.metas_by_id[editedColumn.meta.id];
      if (metaValue) {
        metaValue['value_' + editedColumn.type] = editedValue;
        firstValueFrom(this.projectMetaValueService.projectMetaValuePartialUpdate({ id: metaValue.id, data: metaValue })).then((nmv) => {
          this.projects[projectIndex].metas_by_id[editedColumn.meta.id] = nmv;
          this.projects = [...this.projects];
        });
      } else {
        metaValue = {
          meta: editedColumn.meta.id,
          project: project.id,
        };
        metaValue['value_' + editedColumn.type] = editedValue;
        firstValueFrom(this.projectMetaValueService.projectMetaValueCreate(metaValue)).then((nmv) => {
          this.projects[projectIndex].metas.push(nmv.id);
          this.projects[projectIndex].metas_by_id[editedColumn.meta.id] = nmv;
          this.projects = [...this.projects];
        });
      }
    } else {
      const editedParam = editedColumn.dataEditKey || editedColumn.dataDisplayKey;
      project[editedParam] = editedValue;
      firstValueFrom(this.projectsService.projectsPartialUpdate({ id: project.id, data: project })).then((np) => {
        this.projects[projectIndex][editedParam] = np[editedParam];
        this.projects = [...this.projects];
      });
    }
  }

  editCancel() {}

  toggleEditMode() {
    this.editMode = !this.editMode;
  }

  initChoices() {
    this.choices.groups = this.projectGroupsService
      .projectGroupsList({})
      .pipe(switchMap((groups) => of(groups.map((gr) => ({ value: gr.name, label: gr.name })))));

    this.choices.types = this.projectTypesService
      .projectTypesList()
      .pipe(switchMap((types) => of(types.map((tp) => ({ value: tp.name, label: tp.name })))));

    this.choices.statuses = of(
      Object.keys(ProjectEnums.StatusEnum).map((st) => ({ value: st, label: this.translateService.instant('PROJECTS.STATUS.' + st) }))
    );

    this.choices.stages = this.stagesService
      .stagesList()
      .pipe(switchMap((stages) => of(stages.map((s) => ({ value: s.name, label: s.name })))));

    this.choices.users = this.usersService
      .usersList({})
      .pipe(switchMap((users) => of(users.map((u) => ({ value: u.id, label: u.full_name })))));
  }

  saveCurrentViewLS() {
    if (this.currentView?.id) {
      localStorage.setItem(currentViewIdLSKey, this.currentView.id.toString());
    }
  }

  loadCurrentViewLS(): number {
    return Number(localStorage.getItem(currentViewIdLSKey));
  }

  editMetaField() {
    this.router.navigate(['/knowledgebase/methodology/project-metas']);
  }

  saveView(view: ProjectView) {
    firstValueFrom(this.projectViewService.projectViewPartialUpdate({ id: view.id, data: this.currentView })).then((npv) => {
      this.updateView(npv);
      firstValueFrom(this.loadProjectViews()).then();
    });
  }

  editView(view?: ProjectView) {
    const canBePublic = this.store.selectSnapshot(GlobalRulesSelectors.customRule('publish', 'projectview'));
    const ref = this.dialogService.open(ProjectViewFormComponent, {
      header: this.translateService.instant('PROJECTS.VIEW.FORM.HEADER'),
      width: '40%',
      data: {
        view: view,
        canBePublic,
      },
    });

    firstValueFrom(ref.onClose).then((newView) => {
      if (newView) {
        const viewToSave = JSON.parse(JSON.stringify(this.currentView));
        viewToSave.name = newView.name;
        viewToSave.public = newView.public;
        const saveObs$ = view?.id
          ? this.projectViewService.projectViewPartialUpdate({ id: view.id, data: viewToSave })
          : this.projectViewService.projectViewCreate(viewToSave);

        firstValueFrom(saveObs$).then((npv) => {
          this.updateView(npv);
          this.saveCurrentViewLS();
          firstValueFrom(this.loadProjectViews()).then();
        });
      }
    });
  }

  deleteView(view: ProjectView) {
    this.confirmationService.confirm({
      header: this.translateService.instant('PROJECTS.VIEW_DELETE_HEADER'),
      message: this.translateService.instant('PROJECTS.VIEW_DELETE_MSG'),
      accept: () => {
        this.projectViewService.projectViewDelete(view.id).subscribe((_r) => {
          this.loadProjectViews().subscribe((_v) => {
            const defaultView = _.find(this.views, 'default');
            const firstView = this.views[0];

            if (defaultView) {
              this.updateView(defaultView);
            } else if (firstView) {
              this.updateView(firstView);
            } else {
              this.updateView(this.generateDefaultView());
            }

            this.filterAndSortColumns();
            this.saveCurrentViewLS();
          });
        });
      },
    });
  }

  getColumnByDisplayKey(key: string): ProjectColumn {
    if (!key) {
      return null;
    }
    return _.find(this.columns, ['dataDisplayKey', key]);
  }

  filterChange($event: TableFilterEvent) {
    this.resizeColumnAfterFilterChange($event);
    this.currentView.filter = _.cloneDeep($event.filters);
    this.saveCurrentViewLS();
  }

  resizeColumnAfterFilterChange(event: TableFilterEvent): void {
    if (Boolean(this.filterState)) {
      Object.keys(event.filters).forEach((key) => {
        // проверка на наличие измененных фильтров
        const isChanged = !_.isEqual(event.filters[key][0], this.filterState[key][0]);
        // поиск измененного фильтра по ключу
        const checkChangedFilterFromEvent = Boolean(event.filters[key][0].fn);
        // только если есть один добавленный новый фильтр.

        // для фильтра с множественной фильтрацией
        const checkFirstChangedValueFromEvent: boolean = event.filters[key][0].fn?.length === 1 || isNumber(event.filters[key][0].fn);

        // исключает кейс, когда произошло переключение с двух выбранных фильтров на один
        const checkHasNotPrevValueFromState: boolean = this.filterState[key][0].fn?.length !== 2;

        if (isChanged && checkChangedFilterFromEvent && checkFirstChangedValueFromEvent && checkHasNotPrevValueFromState) {
          this.recalcColumnWidth(key);
        }
      });
    } else {
      Object.keys(event.filters).forEach((key) => {
        if (Boolean(event.filters[key][0])) {
          this.recalcColumnWidth(key);
        }
      });
    }

    this.filterState = _.cloneDeep(event.filters);
  }

  loadBudgetTypes(): Observable<any> {
    return this.financeService.financeBudgetTypesList().pipe(
      switchMap((bts) => {
        this.budgetTypes = bts;
        return of(this.budgetTypes);
      })
    );
  }

  loadBudgetCalendar() {
    this.makeBudgetCalendarHeaders();
    let bts = null;
    if (this.currentView.budget_types && this.currentView.budget_types.length > 0) {
      bts = this.currentView.budget_types.join(',');
    }
    this.projects.forEach(
      (p) =>
        (p['stats'] = p.project_budget
          ? this.financeService.financeProjectBudgetsDetailedStats({ id: p.project_budget, budgetTypes: bts })
          : of({}))
    );
  }

  makeBudgetCalendarHeaders() {
    const startDate = moment(this.currentView.budget_period_start);
    const endDate = moment(this.currentView.budget_period_end);

    const startYear = startDate.year();
    const endYear = endDate.year();

    const years = _.range(startYear, endYear + 1);

    const nowYear = moment().year();
    const nowMonth = moment().month();

    const res = [];

    years.forEach((year) => {
      let startMonth = 1;
      let endMonth = 12;
      if (year === startYear) {
        startMonth = startDate.month() + 1;
      }
      if (year === endYear) {
        endMonth = endDate.month() + 1;
      }

      const quarters = _.range(getQuarterByMonth(startMonth), getQuarterByMonth(endMonth) + 1);
      const months = _.range(startMonth, endMonth + 1);
      const ym: YearData = {
        year: year,
        months: months,
        quarters: quarters,
        quarterLengths: new Array(quarters.length).fill(3),
      };
      if (startMonth > 1) {
        ym.quarterLengths[0] = Dates.startQuarterMapper[startMonth % 3];
      }
      if (endMonth < 12) {
        ym.quarterLengths[quarters.length - 1] = Dates.endQuarterMapper[endMonth % 3];
      }
      if (year === nowYear && ym.months.includes(nowMonth)) {
        ym.nowMonth = nowMonth;
      }
      res.push(ym);
    });

    this.budgetCalendarHeader = res;
  }

  getStatValue(stat: Object, option: ProjectView['budget_options'], year: number, quarter: number = null, month: number = null) {
    if (stat === null || stat === undefined) {
      return 0;
    }

    let res = stat[year];
    if (res === null || res === undefined) {
      return 0;
    }
    if (quarter) {
      res = res['quarters'][quarter];
    } else if (month) {
      res = res['months'][month];
    } else {
      res = res['total'];
    }
    if (res === null || res === undefined) {
      return 0;
    }

    switch (option) {
      case ProjectEnums.BudgetOptionsEnum.FACT:
        return res['fact'];
      case ProjectEnums.BudgetOptionsEnum.PLAN:
        return res['plan'];
      case ProjectEnums.BudgetOptionsEnum.FTFC:
        return res['current_fact_and_fc'];
      default:
        return 0;
    }
  }

  toggleBudgetType(bt: BudgetType) {
    if (this.currentView.budget_types === null) {
      this.currentView.budget_types = [bt.id];
    } else if (this.currentView.budget_types.includes(bt.id)) {
      _.pull(this.currentView.budget_types, bt.id);
    } else {
      this.currentView.budget_types.push(bt.id);
    }
    this.saveCurrentViewLS();
    this.loadBudgetCalendar();
  }

  toggleBudgetView() {
    this.currentView.budget = !this.currentView.budget;
    this.saveCurrentViewLS();
  }

  isColumnForThisProject(column: ProjectColumn, project: Project): boolean {
    if (column.meta) {
      return column.meta.all_types || column.meta.project_types.includes(project.project_type);
    }
    return true;
  }

  activeColumnsInGroup(group: ProjectGroupColumn) {
    return (
      _.intersection(
        group.columns.map((c) => c.id),
        this.currentView.fields
      ).length.toString() +
      ' / ' +
      group.columns.length.toString()
    );
  }

  exportToXLSX() {
    const tableEl = document.getElementById('projectsTable');
    const workbook = xlsx.utils.table_to_book(tableEl);

    const filename = this.currentView.name + '_' + moment().format('L');
    xlsx.writeFile(workbook, filename + '.xlsx');
  }

  getGroupBy(): string | Function {
    if (!this.currentView.group_by) {
      return null;
    }
    const groupByColumn: ProjectColumn = this.getColumnByDisplayKey(this.currentView.group_by);
    if (!groupByColumn) {
      return null;
    }
    if (groupByColumn.type === ProjectEnums.DataTypeEnum.MCHS || groupByColumn.type === ProjectEnums.DataTypeEnum.MTM) {
      return (rowData) => this.getColumnDisplayValue(rowData, groupByColumn);
    }
    return groupByColumn.dataDisplayKey;
  }

  getComponentInputObject(project: ProjectExtra, column: ProjectColumn) {
    if (!column.displayComponentInput) {
      return null;
    }
    const res = {};
    res[column.displayComponentInput] = this.getColumnRawDisplayValue(project, column);
    Object.assign(res, column.displayComponentStaticInput);
    return res;
  }

  onFilterClick(id: string): void {
    this.recalcColumnWidth(id);
  }

  private switchView(view: ProjectView) {
    this.updateView(view);
    this.filterAndSortColumns();
    this.saveCurrentViewLS();
  }

  private updateView(view: ProjectView) {
    this.currentView = JSON.parse(JSON.stringify(view));
    this.filterState = _.cloneDeep(this.currentView.filter);
  }

  private recalcColumnWidth(id: string): void {
    const offsetColumnPadding = 24;
    const column: ProjectColumn | undefined = this.columns.find((col) => col.dataDisplayKey === id);
    const columnContainer: Element = this.document.getElementById('col-' + column.id);
    const textContainer: Element = columnContainer.getElementsByClassName('p-sortable-custom-title')[0];
    const controlsContainer: Element = columnContainer.getElementsByClassName('p-sortable-custom-filters')[0];

    const textContainerWidth = textContainer.clientWidth;
    const controlsContainerWidth = controlsContainer.clientWidth;

    if (textContainerWidth + controlsContainerWidth > columnContainer.clientWidth) {
      column.width = textContainerWidth + controlsContainerWidth + offsetColumnPadding + 'px';
      this.tableResizeService.resetCelsWidth(this.projectsTable);
    }
  }
}
