import * as moment from 'moment';
import { Department, Project, ProjectGroup, Work, WorkLink, WorkLinkTemplate, WorkTemplate } from 'data-access/sdk/api2';
import { WorkEnums } from '@core-project/enums';
import { checkEnumValue, dbStringToJsDate, dbStringToMoment, jsDateToDbString, momentToDbString } from '@core-project/helpers/index';
import { User } from 'data-access/sdk/api2';
import {
  getGanttIdFromDepartmentId,
  getGanttIdFromProjectGroupId,
  getGanttIdFromProjectId,
  getGanttIdFromUserId,
} from '@core-project/helpers/gantt-common';

export const WorkToGanttTypeMapping = {
  TS: 'task',
  GR: 'project',
  MS: 'milestone',
  AS: 'assignment',
};

export const GanttToWorkTypeMapping = {
  task: 'TS',
  project: 'GR',
  milestone: 'MS',
  assignment: 'AS',
};

export interface GanttWork {
  id: number | string;
  text: string;
  start_date?: Date;
  end_date?: Date;
  start_plan?: Date;
  end_plan?: Date;
  duration?: number;
  unscheduled?: boolean;
  type?: string;
  objType: WorkEnums.GanttNodeEnum;
  open?: boolean;
  order?: number;
  progress?: number;
  parent?: number | string;
  render?: string;
  readonly: boolean;
  editable: boolean;
  toUser?: number;
  work?: Work;
  workTemplate?: WorkTemplate;

  [key: string]: any;
}

export function WorkToGanttAdapter(work: Work): GanttWork {
  const res: GanttWork = {
    id: work.id,
    text: work.name,
    start_date: work.start_eff ? dbStringToJsDate(work.start_eff) : null,
    end_date: work.end_eff ? dbStringToMoment(work.end_eff).add(1, 'd').toDate() : null,
    start_plan: work.start_base ? dbStringToJsDate(work.start_base) : null,
    end_plan: work.end_base ? moment(work.end_base, 'YYYY-MM-DD').toDate() : null,
    type: WorkToGanttTypeMapping[work.work_type],
    open: true,
    order: work.order,
    parent: work.group || 0,
    progress: work.end_fact ? 100 : 0,
    readonly: false,
    editable: true,
    work: work,
    toUser: work.to_user,
    objType: WorkEnums.GanttNodeEnum.WORK,
  };
  res.unscheduled = res.start_date === null || res.end_date === null;
  return res;
}

export function GanttToWorkAdapter(ganttWork: GanttWork): Work {
  const res: Work = ganttWork.work || { name: '' };
  Object.assign(res, {
    name: ganttWork.text,
    work_type: GanttToWorkTypeMapping[ganttWork.type],
    group: ganttWork.parent === 0 ? null : Number(ganttWork.parent),
    order: ganttWork.$local_index,
  });

  if (ganttWork.start_date && !res.start_fact) {
    res.start_base = jsDateToDbString(ganttWork.start_date);
  }

  if (ganttWork.end_date && !res.end_fact) {
    const endDate =
      ganttWork.type === 'milestone' ? jsDateToDbString(ganttWork.end_date) : momentToDbString(moment(ganttWork.end_date).subtract(1, 'd'));

    res.end_base = endDate;
  }
  return res;
}

export function WorkTemplateToGanttAdapter(wt: WorkTemplate): GanttWork {
  return {
    id: wt.id,
    text: wt.name,
    start_date: wt.start_date ? dbStringToJsDate(wt.start_date) : null,
    end_date: wt.end_date ? dbStringToMoment(wt.end_date).add(1, 'day').toDate() : dbStringToMoment(wt.start_date).add(2, 'day').toDate(),
    type: WorkToGanttTypeMapping[wt.work_type],
    open: true,
    order: wt.order,
    parent: wt.group || 0,
    progress: 0,
    readonly: false,
    editable: true,
    workTemplate: wt,
    objType: WorkEnums.GanttNodeEnum.TEMPLATE,
    unscheduled: !wt.start_date,
  };
}

export function GanttToWorkTemplateAdapter(ganttWork: GanttWork): WorkTemplate {
  const res: WorkTemplate = ganttWork.workTemplate || { name: '', project_type: null };
  Object.assign(res, {
    name: ganttWork.text,
    work_type: GanttToWorkTypeMapping[ganttWork.type],
    group: ganttWork.parent === 0 ? null : Number(ganttWork.parent),
    start_date: jsDateToDbString(ganttWork.start_date),
    end_date:
      ganttWork.type === 'milestone' ? jsDateToDbString(ganttWork.end_date) : momentToDbString(moment(ganttWork.end_date).subtract(1, 'd')),
    order: ganttWork.$local_index,
  });

  return res;
}

export function UWorkToGanttAdapter(work: Work): GanttWork {
  const res = WorkToGanttAdapter(work);
  res.parent = work.to_user
    ? getGanttIdFromUserId(work.to_user)
    : work.to_department
    ? getGanttIdFromDepartmentId(work.to_department)
    : null;
  return res;
}

export function GanttToUWorkAdapter(ganttWork: GanttWork): Work {
  const res = GanttToWorkAdapter(ganttWork);
  delete res.group;
  if (typeof ganttWork.parent === 'string') {
    const to = ganttWork.parent.split('_');
    if (to[0] === 'user') {
      res.to_user = Number(to[1]);
    }
    if (to[0] === 'dep') {
      res.to_user = null;
      res.to_department = Number(to[1]);
    }
  }

  return res;
}

export function PWorkToGanttAdapter(work: Work): GanttWork {
  const res = WorkToGanttAdapter(work);
  res.parent = getGanttIdFromProjectId(work.project);
  return res;
}

export function GanttToPWorkAdapter(ganttWork: GanttWork): Work {
  const res = GanttToWorkAdapter(ganttWork);
  res.group = ganttWork.work.group;
  return res;
}

export function UserToGanttAdapter(user: User): GanttWork {
  return {
    id: getGanttIdFromUserId(user.id),
    text: user.short_name,
    type: 'project',
    render: 'split',
    parent: user.department ? getGanttIdFromDepartmentId(user.department) : 0,
    readonly: true,
    editable: false,
    objType: WorkEnums.GanttNodeEnum.USER,
  };
}

export function DepartmentToGanttAdapter(department: Department): GanttWork {
  return {
    id: getGanttIdFromDepartmentId(department.id),
    text: department.name,
    type: 'project',
    parent: department.parent_department ? getGanttIdFromDepartmentId(department.parent_department) : 0,
    readonly: true,
    editable: false,
    open: !department.parent_department,
    objType: WorkEnums.GanttNodeEnum.DEPARTMENT,
  };
}

export function ProjectToGanttAdapter(project: Project): GanttWork {
  return {
    id: getGanttIdFromProjectId(project.id),
    text: project.name,
    type: 'project',
    render: 'split',
    parent: getGanttIdFromProjectGroupId(project.project_group),
    readonly: true,
    editable: false,
    objType: WorkEnums.GanttNodeEnum.PROJECT,
    // start_date: dbStringToJsDate(project.start_date),
    // end_date: dbStringToJsDate(project.end_date),
    //
  };
}

export function ProjectGroupToGanttAdapter(projectGroup: ProjectGroup): GanttWork {
  return {
    id: getGanttIdFromProjectGroupId(projectGroup.id),
    text: projectGroup.name,
    type: 'project',
    parent: projectGroup.parent_group ? getGanttIdFromProjectGroupId(projectGroup.parent_group) : 0,
    readonly: true,
    editable: false,
    open: true,
    objType: WorkEnums.GanttNodeEnum.PROJECT_GROUP,
  };
}

export interface GanttWorkLink {
  id: number;
  source: number;
  target: number;
  type: WorkEnums.LinkTypeEnum;
  lag?: number;
  readonly?: boolean;
  editable?: boolean;
}

export function WorkLinkToGanttAdapter(link: WorkLink): GanttWorkLink {
  return {
    id: link.id,
    source: link.source,
    target: link.target,
    type: checkEnumValue(WorkEnums.LinkTypeEnum, link.link_type),
    lag: link.lag,
  };
}

export function GanttToWorkLinkAdapter(ganttLink: GanttWorkLink): WorkLink {
  return {
    id: ganttLink.id,
    source: ganttLink.source,
    target: ganttLink.target,
    link_type: ganttLink.type,
    lag: ganttLink.lag,
  };
}

export function WorkLinkTemplateToGanttAdapter(link: WorkLinkTemplate): GanttWorkLink {
  return {
    id: link.id,
    source: link.source,
    target: link.target,
    type: checkEnumValue(WorkEnums.LinkTypeEnum, link.link_type),
    lag: link.lag,
  };
}

export function GanttToWorkLinkTemplateAdapter(ganttLink: GanttWorkLink): WorkLinkTemplate {
  return {
    id: ganttLink.id,
    source: ganttLink.source,
    target: ganttLink.target,
    link_type: ganttLink.type,
    lag: ganttLink.lag,
  };
}
