import { Component, OnInit } from '@angular/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { FileHandle } from '@core-project/directives/file-drop.directive';
import { firstValueFrom, forkJoin, Observable } from 'rxjs';
import * as moment from 'moment';
import { Document, DocumentMetaField, DocumentType } from 'data-access/sdk/api2';
import { DocumentsService } from 'data-access/sdk/api2';

interface MetaForm {
  metaField: DocumentMetaField;
  form: FormGroup;
}

@Component({
  selector: 'app-document-form',
  templateUrl: './document-form.component.html',
  styleUrls: ['./document-form.component.scss'],
})
export class DocumentFormComponent implements OnInit {
  document: Document;
  docTypeChoices: Array<DocumentType>;
  documentForm: FormGroup;
  documentFileForm: FormGroup;
  metaForms: Array<MetaForm>;
  selectedFile;
  private projectId: number;
  private selectedDocType: number;

  constructor(
    private readonly config: DynamicDialogConfig,
    private readonly formBuilder: FormBuilder,
    private readonly documentsService: DocumentsService,
    private readonly dynamicDialogRef: DynamicDialogRef
  ) {}

  ngOnInit(): void {
    this.projectId = this.config.data.projectId;
    this.selectedDocType = this.config.data.selectedDocType;
    this.document = this.config.data.document;

    if (this.config.data.files && this.config.data.files.length) {
      this.selectedFile = this.config.data.files[0].file;
    }

    firstValueFrom(
      this.documentsService.documentsTypesList({
        parentType: null,
        uncategorized: 'false',
        include: 'meta_fields',
      })
    ).then((docTypes) => {
      this.docTypeChoices = docTypes;
      this.initForm(this.document);
    });
  }

  filesDropped(files: FileHandle[]): void {
    this.selectedFile = files[0]?.file;
  }

  selectFile(event, document, fileUpload?) {
    this.selectedFile = event.currentFiles[0];
    fileUpload.clear();
  }

  selectDocumentType(event) {
    let metaFields: Array<DocumentMetaField> = null;
    const selectedType: DocumentType = this.docTypeChoices.find((dt) => dt.id === event.fn);
    if (selectedType) {
      metaFields = selectedType.rel_meta_fields;
    }

    if (metaFields && metaFields.length > 0) {
      const fields: Array<MetaForm> = [];
      metaFields.forEach((mf) => {
        const form: FormGroup = this.formBuilder.group({
          id: [null],
          meta: [mf.id],
          document: [this.document?.id],
          content: [null, mf.required ? Validators.required : null],
          object_id: [null],
        });

        if (this.document) {
          const exMeta = this.document.rel_metas.find((m) => m.meta === mf.id);
          if (exMeta) {
            form.patchValue(exMeta);
          }
        }

        fields.push({ form: form, metaField: mf });
      });
      this.metaForms = fields;
    } else {
      this.metaForms = null;
    }
  }

  save() {
    if (this.document) {
      const docSave$ = this.documentsService.documentsDocumentsPartialUpdate({
        id: this.document.id,
        data: this.documentForm.value,
      });
      const metasSave: Array<Observable<any>> =
        this.metaForms && this.metaForms.length > 0
          ? this.metaForms.map((mf) => {
              const formValue = mf.form.value;
              if (formValue.id) {
                return this.documentsService.documentsMetaValuesPartialUpdate({ id: formValue.id, data: formValue });
              } else {
                return this.documentsService.documentsMetaValuesCreate(formValue);
              }
            })
          : [];
      firstValueFrom(forkJoin([docSave$, ...metasSave])).then((res) => this.dynamicDialogRef.close(res[0]));
    } else {
      firstValueFrom(this.documentsService.documentsDocumentsCreate(this.documentForm.value)).then((newDoc) => {
        const saves: Array<Observable<any>> = [];
        if (this.metaForms && this.metaForms.length > 0) {
          this.metaForms.forEach((mf) => {
            const formValue = mf.form.value;
            formValue.document = newDoc.id;
            saves.push(this.documentsService.documentsMetaValuesCreate(formValue));
          });
        }

        if (this.selectedFile) {
          saves.push(
            this.documentsService.documentsFilesCreate({
              document: newDoc.id,
              comment: this.documentFileForm.value.comment,
              docFile: this.selectedFile,
            })
          );
        }
        firstValueFrom(forkJoin(saves)).then((_) => this.dynamicDialogRef.close(newDoc));
      });
    }
  }

  isValid() {
    let res = true;
    res = res && this.documentForm?.valid;
    if (this.documentFileForm) {
      res = res && this.documentFileForm.valid;
    }
    if (this.metaForms) {
      this.metaForms.forEach((mf) => (res = res && mf.form.valid));
    }
    return res;
  }

  private initForm(d: Document) {
    this.documentForm = this.formBuilder.group({
      id: [d?.id],
      name: [d?.name, Validators.required],
      document_type: [d?.document_type, Validators.required],
      number: [d?.number],
      date: [d && d.date ? moment.utc(d.date).format('DD.MM.YYYY') : null],
      description: [d?.description],
      projects: [d?.projects || (this.projectId ? [this.projectId] : [])],
    });
    if (!d) {
      this.documentFileForm = this.formBuilder.group({
        document: [null],
        comment: [''],
      });
    }

    if (d) {
      this.selectDocumentType({ value: this.document.document_type });
    } else if (this.selectedDocType) {
      this.documentForm.controls.document_type.setValue(this.selectedDocType);
      this.selectDocumentType({ value: this.selectedDocType });
    }
  }
}
