import { Component, OnInit, ViewEncapsulation } from '@angular/core';
import { DocumentType } from 'data-access/sdk/api2';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { DocumentsService } from 'data-access/sdk/api2';
import { Observable } from 'rxjs';
import { TranslateService } from '@ngx-translate/core';
import { DocumentMetaField } from 'data-access/sdk/api2';
import * as _ from 'lodash-es';
import { MessageService } from 'primeng/api';
import { map } from 'rxjs/operators';

@Component({
  selector: 'app-document-type-form',
  templateUrl: './document-type-form.component.html',
  styleUrls: ['./document-type-form.component.scss'],
  encapsulation: ViewEncapsulation.None,
})
export class DocumentTypeFormComponent implements OnInit {
  type: DocumentType;
  category: boolean;
  form: FormGroup;
  categoryOptions: Array<DocumentType>;
  categoryOptions$: Observable<Array<DocumentType>>;
  metaOptions$: Observable<Array<DocumentMetaField>>;
  groupMetaOptions$: Observable<Array<DocumentMetaField>>;

  constructor(
    private config: DynamicDialogConfig,
    private formBuilder: FormBuilder,
    private ref: DynamicDialogRef,
    private documentsService: DocumentsService,
    private translateService: TranslateService,
    private messageService: MessageService
  ) {}

  ngOnInit(): void {
    this.type = this.config.data.type;
    this.category = this.config.data.category;
    this.categoryOptions$ = this.documentsService.documentsTypesList({ uncategorized: 'true' }).pipe(
      map((cats) => {
        this.categoryOptions = cats;
        return cats;
      })
    );
    this.metaOptions$ = this.documentsService.documentsMetaFieldsList();

    const groupTypes = ['FK', 'MTM', 'CHS', 'MCHS'];
    this.groupMetaOptions$ = this.documentsService
      .documentsMetaFieldsList()
      .pipe(map((metas) => metas.filter((m) => groupTypes.includes(m.meta_type))));
    this.initForm();
  }

  initForm() {
    this.form = this.formBuilder.group({
      name: [this.type?.name, Validators.required],
      name_for_file: [this.type?.name_for_file || this.type?.name || '', Validators.required],
      parent_type: [this.type?.parent_type || null],
      meta_fields: [this.type?.meta_fields || []],
      group_by: [this.type?.group_by],
    });
  }

  save() {
    this.ref.close(this.form.value);
  }

  categoryChange($event) {
    if ($event.fn) {
      const cat = _.find(this.categoryOptions, ['id', $event.fn]);
      const currentMetas = this.form.controls.meta_fields.value;
      if (cat.group_by && !currentMetas.includes(cat.group_by)) {
        currentMetas.push(cat.group_by);
        this.form.controls.meta_fields.setValue(currentMetas);
      }
    }
  }

  metaFieldsChange() {
    const catId = this.form.controls.parent_type.value;
    if (catId) {
      const cat = _.find(this.categoryOptions, ['id', catId]);
      const currentMetas = this.form.controls.meta_fields.value;
      if (cat.group_by && !currentMetas.includes(cat.group_by)) {
        this.messageService.add({
          severity: 'error',
          summary: this.translateService.instant('ERRORS.DELETE_ERROR'),
          detail: this.translateService.instant('KB.METHODOLOGY.DOCUMENTS.TYPE_FORM.META_DELETE_ERROR'),
        });
        currentMetas.push(cat.group_by);
        this.form.controls.meta_fields.setValue(currentMetas);
      }
    }
  }

  nameChange($event) {
    if (this.category || (!this.type && this.form.controls.name_for_file.untouched)) {
      this.form.controls.name_for_file.setValue(this.form.controls.name.value);
    }
  }
}
