import {Component, OnInit} from '@angular/core';
import {DynamicDialogConfig, DynamicDialogRef} from 'primeng/dynamicdialog';
import {FormArray, FormBuilder, FormGroup, Validators} from '@angular/forms';
import {ConfirmationService, MessageService} from 'primeng/api';
import {AuthenticationService} from '@core-project/services/authentication.service';
import {DynamicDialogService} from '@core-project/services/dynamic-dialog.service';
import {environment} from '../../../../environments/environment';
import {Contact, ContactsService, Email, EmailService, OrganizationService, Telephone, TelephoneService, User} from 'data-access/sdk/api2';

@Component({
  selector: 'app-organization-form',
  templateUrl: './organization-form.component.html',
  styleUrls: ['./organization-form.component.scss'],
})
export class OrganizationFormComponent implements OnInit {
  organization: any;
  organizationForm: FormGroup;
  telephoneForm: FormArray = this.formBuilder.array([]);
  emailForm: FormArray = this.formBuilder.array([]);
  contactTelephoneForm: FormArray = this.formBuilder.array([]);
  contactEmailForm: FormArray = this.formBuilder.array([]);
  contactForm: FormArray = this.formBuilder.array([]);
  disabledForm = false;
  contactsPhoto: any = {};
  defaultPhoto = environment.API_BASE_PATH + '/upload/ava.png';
  numbers: any = {};

  private user: User;

  constructor(
    readonly dynamicDialogService: DynamicDialogService,
    private readonly dynamicDialogRef: DynamicDialogRef,
    private readonly formBuilder: FormBuilder,
    private readonly messageService: MessageService,
    private readonly organizationService: OrganizationService,
    private readonly authenticationService: AuthenticationService,
    private readonly dialogConfig: DynamicDialogConfig,
    private readonly confirmationService: ConfirmationService,
    private readonly telephoneService: TelephoneService,
    private readonly emailService: EmailService,
    private readonly contactService: ContactsService,
  ) {
  }

  ngOnInit(): void {
    this.authenticationService.user$.subscribe((user) => {
      this.user = user;
      if (user) {
        this.initForm();
      }
    });
  }

  uploadFile(event, contact, fileUpload?) {
    const contactId = contact?.id ? contact.id : contact.fn.id;
    if (contact && contactId && typeof contactId !== 'string') {
      this.contactService
        .contactsContactPhotoUpdatePhoto({
          id: contactId,
          photo: event.currentFiles[0],
        })
        .toPromise()
        .then((photo) => {
          contact.controls['photo']?.setValue(environment.API_BASE_PATH + photo.photo);
          if (fileUpload) {
            fileUpload.clear();
          }
        })
        .catch((err) => console.error('Error: ', err));
    } else {
      this.contactsPhoto[contactId] = event;
      const file = event.currentFiles[0];
      const reader = new FileReader();
      reader.readAsDataURL(file);
      reader.onload = () => {
        contact.controls['photo']?.setValue(reader.result);
        if (fileUpload) {
          fileUpload.clear();
        }
      };
    }
  }

  addTelephone(type, contact?) {
    const newTelephone = this.createTelephoneForm(contact ? {contact: contact.id} : null);
    if (type === 'contactTelephone') {
      this.contactTelephoneForm.push(newTelephone);
    }
    if (type === 'telephone') {
      this.telephoneForm.push(newTelephone);
    }
  }

  addEmail(type, contact?) {
    const newEmail = this.createEmailForm(contact ? {contact: contact.id} : null);
    if (type === 'contactEmail') {
      this.contactEmailForm.push(newEmail);
    }
    if (type === 'telephone') {
      this.emailForm.push(newEmail);
    }
  }

  addContact() {
    const newContact = this.createContactForm(null);
    this.contactForm.push(newContact);
  }

  deleteItem(type, item) {
    let textMessage;
    let deleteMessage;
    let nameItem;
    let deleteMethod;
    let form;
    if (type === 'telephone' || type === 'contactTelephone') {
      deleteMethod = item.id ? this.telephoneService.telephoneDelete(item.id) : null;
      nameItem = item.number ? item.number : '';
      form = type === 'telephone' ? this.telephoneForm : this.contactTelephoneForm;
      textMessage = 'номер телефона';
      deleteMessage = 'Номер телефона успешно удален';
    } else if (type === 'email' || type === 'contactEmail') {
      deleteMethod = item.id ? this.emailService.emailDelete(item.id) : null;
      nameItem = item.email ? item.email : '';
      form = type === 'email' ? this.emailForm : this.contactEmailForm;
      textMessage = 'эл. почту';
      deleteMessage = 'Эл. почта успешно удалена';
    } else if (type === 'contact') {
      deleteMethod = item.id ? this.contactService.contactsDelete(item.id) : null;
      nameItem =
        `${item?.last_name ? item?.last_name : ''} ` +
        `${item?.first_name ? item?.first_name : ''} ${item?.middle_name ? item?.middle_name : ''}`;
      form = this.contactForm;
      textMessage = 'контакт';
      deleteMessage = 'Контакт успешно удален';
    } else {
      return;
    }
    this.confirmationService.confirm({
      message: `Вы действительно хотите удалить ${textMessage} ${nameItem}?`,
      header: 'Пожалуйста, подтвердите ваши действия',
      accept: () => {
        if (item.id) {
          deleteMethod.toPromise().then(() => {
            const deleteIndex = form.fn.findIndex((tel) => tel.id === item.id);
            form.removeAt(deleteIndex);
          });
        } else {
          const deleteIndex = form.fn.findIndex((tel) => tel.id === item.id);
          form.removeAt(deleteIndex);
        }
        this.messageService.add({
          severity: 'success',
          detail: deleteMessage,
        });
      },
      reject: () => {
      },
    });
  }

  editActive() {
    this.telephoneForm.enable();
    this.organizationForm.enable();
    this.emailForm.enable();
    this.contactForm.enable();
    this.contactTelephoneForm.enable();
    this.contactEmailForm.enable();
    this.disabledForm = false;
  }

  deleteOrganization() {
    if (this.organization) {
      this.confirmationService.confirm({
        message: `Вы действительно хотите удалить компанию ${this.organization?.name}?`,
        header: 'Пожалуйста, подтвердите ваши действия',
        accept: () => {
          this.organizationService
            .organizationDelete(this.organization.id)
            .toPromise()
            .then(() => {
              this.dynamicDialogRef.close();

              return this.messageService.add({
                severity: 'success',
                detail: 'Компания успешно удалена',
              });
            });
        },
        reject: () => {
        },
      });
    }
  }

  clickMessenger(type, phone, value) {
    if (this.disabledForm) {
      return;
    }
    if (type === 'telephone') {
      const index = this.telephoneForm.value.findIndex((tel) => tel.id === phone.fn.id);
      phone.get(value).fn = !phone.get(value).fn;
      this.telephoneForm.value[index][value] = phone.get(value).fn;
    }
    if (type === 'contactTelephone') {
      const index = this.contactTelephoneForm.value.findIndex((tel) => tel.id === phone.fn.id);
      phone.get(value).fn = !phone.get(value).fn;
      this.contactTelephoneForm.value[index][value] = phone.get(value).fn;
    }
    return;
  }

  async save() {
    if (!this.organizationForm.valid && !this.telephoneForm.valid) {
      return;
    }

    this.organizationForm.markAsPending();

    await this.addOrEditOrganization();

    // Редактирование телефонов
    if (this.telephoneForm.value && this.telephoneForm.controls.length) {
      for (const telephone of this.telephoneForm.controls) {
        await this.addOrEditTelephone(telephone.value, 'organization');
      }
    }

    // Редактирование эл. почты
    if (this.emailForm.value && this.emailForm.controls.length) {
      for (const email of this.emailForm.controls) {
        await this.addOrEditEmail(email.value, 'organization');
      }
    }

    // Редактирование контактов
    if (this.contactForm.value && this.contactForm.controls.length) {
      for (const contact of this.contactForm.controls) {
        let oldId;
        const telephonesForContact = this.contactTelephoneForm.value.filter((phone) => phone.contactId === contact.value.id);
        const emailsForContact = this.contactEmailForm.value.filter((phone) => phone.contactId === contact.value.id);
        if (typeof contact.value.id === 'string') {
          oldId = contact.value.id;
          contact.value.id = null;
        }
        contact.value.photo = contact.value?.photo?.startsWith('data') ? null : contact.value.photo;
        const contactFromServer = await this.addOrEditContact(contact.value);

        if (this.contactsPhoto[oldId]) {
          await this.uploadFile(this.contactsPhoto[oldId], contactFromServer);
        }

        for (const phone of telephonesForContact) {
          phone.contact = contactFromServer.id;
          await this.addOrEditTelephone(phone);
        }
        for (const mail of emailsForContact) {
          mail.contact = contactFromServer.id;
          await this.addOrEditEmail(mail);
        }
      }
    }

    this.organizationForm.setErrors({});
    this.organizationForm.markAsPristine();

    this.messageService.add({
      severity: 'success',
      detail: this.organizationForm.value.id ? 'Компания успешно отредактирована' : 'Компания успешно добавлена',
    });

    this.dynamicDialogRef.close();
  }

  private initForm() {
    this.organizationForm = this.formBuilder.group({
      id: [undefined],
      company: [undefined],
      name: ['', Validators.required],
      full_name: [''],
      postal_address: [''],
      legal_address: [''],
      detail: [''],
    });
    if (this.dialogConfig.data && this.dialogConfig.data.organizationId) {
      this.disabledForm = true;
      this.organizationForm.disable();
      this.organization = this.dialogConfig.data.organizationId;
      this.organizationForm.patchValue(this.organization);
    }

    if (this.organization && this.organization.rel_telephones && this.organization.rel_telephones.length > 0) {
      for (const phone of this.organization.rel_telephones) {
        this.telephoneForm.push(this.createTelephoneForm(phone));
      }
    }

    if (this.organization && this.organization.rel_emails && this.organization.rel_emails.length > 0) {
      for (const email of this.organization.rel_emails) {
        this.emailForm.push(this.createEmailForm(email));
      }
    }

    if (this.organization && this.organization.rel_contacts && this.organization.rel_contacts.length > 0) {
      for (const contact of this.organization.rel_contacts) {
        if (contact.rel_telephones && contact.rel_telephones.length > 0) {
          for (const telephone of contact.rel_telephones) {
            this.contactTelephoneForm.push(this.createTelephoneForm(telephone));
          }
        }
        if (contact.rel_emails && contact.rel_emails.length > 0) {
          for (const email of contact.rel_emails) {
            this.contactEmailForm.push(this.createEmailForm(email));
          }
        }
        this.contactForm.push(this.createContactForm(contact));
      }
    }
  }

  private createTelephoneForm(telephone: Telephone | null | any) {
    const telephoneGroup = this.formBuilder.group({
      id: [telephone ? telephone.id : null],
      number: [telephone ? telephone.number : null, Validators.required],
      primary: [telephone ? telephone.primary : false],
      whatsapp: [telephone ? telephone.whatsapp : false],
      telegram: [telephone ? telephone.telegram : false],
      comment: [telephone ? telephone.comment : ''],
      organization: [telephone ? telephone.organizationId : null],
      contact: [telephone ? telephone.contactId : null],
    });
    if (telephone && telephone.id) {
      telephoneGroup.disable();
    }
    return telephoneGroup;
  }

  private createEmailForm(email: Email | null | any) {
    const emailGroup = this.formBuilder.group({
      id: [email ? email.id : null],
      email: [email ? email.email : null, Validators.required],
      primary: [email ? email.primary : false],
      comment: [email ? email.comment : ''],
      organization: [email ? email.organizationId : null],
      contact: [email ? email.contactId : null],
    });
    if (email && email.id) {
      emailGroup.disable();
    }
    return emailGroup;
  }

  private createContactForm(contact: Contact | null) {
    const idxContact = 'new_' + String(this.contactForm.value.length + 1);
    const contactGroup = this.formBuilder.group({
      id: [contact ? contact.id : idxContact],
      first_name: [contact ? contact.first_name : null, Validators.required],
      middle_name: [contact ? contact.middle_name : ''],
      last_name: [contact ? contact.last_name : null, Validators.required],
      position: [contact ? contact.position : ''],
      organization: [contact ? contact.organization : null],
      comment: [contact ? contact.comment : ''],
      photo: [contact ? contact.photo : null],
    });
    if (contact && contact.id) {
      contactGroup.disable();
    }
    return contactGroup;
  }

  private addOrEditTelephone(telephone, type?): Promise<void> {
    telephone.organization = type === 'organization' ? this.organization.id : null;
    telephone.comment = telephone.comment ? telephone.comment : '';
    telephone.primary = telephone.primary ? telephone.primary : false;
    telephone.telegram = telephone.telegram ? telephone.telegram : false;
    telephone.whatsapp = telephone.whatsapp ? telephone.whatsapp : false;
    if (String(telephone.number).startsWith('7')) {
      telephone.number = '+' + telephone.number;
    }
    if (telephone.id) {
      return this.telephoneService
        .telephoneUpdate({
          id: telephone.id,
          data: telephone,
        })
        .toPromise()
        .then(() => Promise.resolve());
    } else {
      return this.telephoneService
        .telephoneCreate(telephone)
        .toPromise()
        .then(() => Promise.resolve());
    }
  }

  private addOrEditEmail(email, type?): Promise<void> {
    email.organization = type === 'organization' ? this.organization.id : null;
    email.comment = email.comment ? email.comment : '';
    email.primary = email.primary ? email.primary : false;
    if (email.id) {
      return this.emailService
        .emailUpdate({id: email.id, data: email})
        .toPromise()
        .then(() => Promise.resolve());
    } else {
      return this.emailService
        .emailCreate(email)
        .toPromise()
        .then(() => Promise.resolve());
    }
  }

  private addOrEditContact(contact): Promise<Contact> {
    delete contact.photo;
    contact.organization = this.organization.id;
    if (contact.id) {
      return this.contactService
        .contactsUpdate({id: contact.id, data: contact})
        .toPromise()
        .then((cont) => Promise.resolve(cont));
    } else {
      return this.contactService
        .contactsCreate(contact)
        .toPromise()
        .then((cont) => Promise.resolve(cont));
    }
  }

  private addOrEditOrganization(): Promise<void> {
    if (!this.organizationForm.value.companyId) {
      this.organizationForm.value.companyId = this.user?.company;
    }
    if (this.organizationForm.value.id) {
      return this.organizationService
        .organizationUpdate({
          id: this.organizationForm.value.id,
          data: this.organizationForm.value,
        })
        .toPromise()
        .then((organization) => Promise.resolve());
    } else {
      return this.organizationService
        .organizationCreate(this.organizationForm.value)
        .toPromise()
        .then((organization) => {
          this.organization = organization;
          return Promise.resolve();
        });
    }
  }
}
