import { Component, OnInit } from '@angular/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { FormArray, FormBuilder, FormGroup, Validators } from '@angular/forms';
import { MessageService } from 'primeng/api';

import { firstValueFrom, forkJoin, Observable } from 'rxjs';
import { TranslateService } from '@ngx-translate/core';
import { TelephoneService, UsersService } from 'data-access/sdk/api2';
import { Telephone, User } from 'data-access/sdk/api2';

interface UserWithAvatar extends User {
  avatar: string;
}

@Component({
  selector: 'app-profile-form',
  templateUrl: './profile-form.component.html',
  styleUrls: ['./profile-form.component.scss'],
})
export class ProfileFormComponent implements OnInit {
  public user: UserWithAvatar;
  public telephones: Telephone[];
  public selectedFile: File;

  public telephoneForm: FormArray<FormGroup> = new FormArray<FormGroup>([]);
  public defaultAvatar = 'assets/images/default_photo.png';
  public currentAvatar: string;

  constructor(
    private config: DynamicDialogConfig,
    private formBuilder: FormBuilder,
    private userService: UsersService,
    private dynamicDialogRef: DynamicDialogRef,
    private messageService: MessageService,
    private telephoneService: TelephoneService,
    private translateService: TranslateService
  ) {}

  ngOnInit(): void {
    this.user = this.config.data.user;
    this.currentAvatar = this.user.avatar || this.defaultAvatar;
    this.telephones = this.config.data.telephones || [];
    this.initForm();
  }

  public deleteTelephoneForm(telephone) {
    const deleteIndex = this.telephoneForm.value.findIndex((tel) => tel.id === telephone.fn.id);
    this.telephoneForm.removeAt(deleteIndex);
  }

  public selectFile(event, fileUpload?) {
    this.selectedFile = event.currentFiles[0];
    this.currentAvatar = this.selectedFile['objectURL'];
    fileUpload.clear();
  }

  public uploadFile() {
    return this.userService.usersUpdateAvatar({
      id: this.user.id,
      avatar: this.selectedFile,
    });
  }

  public clickMessenger(phone: FormGroup, field: string) {
    phone.controls[field].setValue(!phone.controls[field].value);
  }

  public addTelephoneForm() {
    const newTelephone = this.createTelephoneForm(null);
    this.telephoneForm.push(newTelephone);
  }

  public save() {
    const res$: Observable<any>[] = [];
    this.telephoneForm.controls.forEach((telephone) => res$.push(this.addOrEditTelephone(telephone.value)));

    const curIds = this.telephoneForm.controls.map((t) => t.value.id);
    const deletedIds = this.telephones.filter((t) => !curIds.includes(t.id)).map((t) => t.id);
    deletedIds.forEach((id) => res$.push(this.deleteTelephone(id)));

    if (this.selectedFile) {
      res$.push(this.uploadFile());
    }

    firstValueFrom(forkJoin(res$)).then((_r) => {
      this.messageService.add({
        severity: 'success',
        detail: this.translateService.instant('PROFILE.SAVE_MSG'),
      });
      this.dynamicDialogRef.close();
    });
  }

  private initForm() {
    for (const phone of this.telephones) {
      this.telephoneForm.push(this.createTelephoneForm(phone));
    }
  }

  private createTelephoneForm(telephone?: Telephone) {
    return this.formBuilder.group({
      id: [telephone?.id],
      number: [telephone?.number, Validators.required],
      primary: [telephone?.primary || false],
      whatsapp: [telephone?.whatsapp || false],
      telegram: [telephone?.telegram || false],
      comment: [telephone?.comment || ''],
      user: [this.user.id],
    });
  }

  private addOrEditTelephone(telephone): Observable<Telephone> {
    if (telephone.id) {
      return this.telephoneService.telephonePartialUpdate({
        id: telephone.id,
        data: telephone,
      });
    } else {
      return this.telephoneService.telephoneCreate(telephone);
    }
  }

  private deleteTelephone(telId) {
    return this.telephoneService.telephoneDelete(telId);
  }
}
