import {Component, OnInit} from '@angular/core';
import {DynamicDialogConfig, DynamicDialogRef} from 'primeng/dynamicdialog';
import {FormBuilder, FormGroup, Validators} from '@angular/forms';
import * as moment from 'moment';
import {map} from 'rxjs/operators';
import {Observable, of} from 'rxjs';
import {FinanceService, ProjectBudgetType, ProjectExpense} from 'data-access/sdk/api2';

@Component({
  selector: 'app-project-expense-form',
  templateUrl: './project-expense-form.component.html',
  styleUrls: ['./project-expense-form.component.scss'],
})
export class ProjectExpenseFormComponent implements OnInit {
  expense: ProjectExpense;
  expenseForm: FormGroup;
  projectBudgetTypes: Array<ProjectBudgetType>;
  budgetTypeChoices: Array<ProjectBudgetType>;
  currency: string;
  projectId: number;

  constructor(
    public ref: DynamicDialogRef,
    public config: DynamicDialogConfig,
    public financeService: FinanceService,
    private formBuilder: FormBuilder,
  ) {
  }

  ngOnInit(): void {
    this.expense = this.config.data.expense;
    this.projectId = this.config.data.projectId;
    this.projectBudgetTypes = this.config.data.projectBudgetTypes;
    this.currency = this.config.data.currency || 'RUB';
    this.initForm(this.expense);
  }

  initForm(expense: ProjectExpense) {
    let date = null;
    if (expense?.date) {
      date = moment(expense.date).toDate();
    }
    this.expenseForm = this.formBuilder.group({
      id: [expense?.id],
      date: [date, Validators.required],
      budget_type: [expense?.budget_type, Validators.required],
      value: [expense?.value, Validators.required],
      description: [expense?.description],
    });
    this.expenseForm.controls.date.valueChanges.subscribe((_) =>
      this.updateBudgetTypeChoices().subscribe((btis) => (this.budgetTypeChoices = btis)),
    );
    this.updateBudgetTypeChoices()
      .toPromise()
      .then((btis) => (this.budgetTypeChoices = btis));
  }

  updateBudgetTypeChoices(): Observable<Array<ProjectBudgetType>> {
    if (!this.expenseForm.controls.date.value || !this.expenseForm.controls.date.valid) {
      return of([]);
    }
    return this.financeService
      .financeProjectBudgetItemsList({
        project: String(this.projectId),
        budgetType: null,
        date: moment(this.expenseForm.controls.date.value).format('YYYY-MM-DD HH:mm'),
        approved: 'true',
      })
      .pipe(
        map((pbtis) => {
          const approvedIds = pbtis.map((pbti) => pbti.budget_type);
          return this.projectBudgetTypes.filter((pbt) => !approvedIds.includes(pbt.id));
        }),
      );
  }

  save() {
    this.ref.close(this.expenseForm.value);
  }
}
