import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { AuthenticationService } from '@core-project/services/authentication.service';
import { Observable } from 'rxjs';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import * as moment from 'moment';
import { map } from 'rxjs/operators';
import { ProjectEnums } from '@core-project/enums/project';
import StageEnum = ProjectEnums.StageEnum;
import { Project } from 'data-access/sdk/api2';
import { User } from 'data-access/sdk/api2';
import { ProjectGroup } from 'data-access/sdk/api2';
import { ProjectType } from 'data-access/sdk/api2';
import { ProjectsService } from 'data-access/sdk/api2';
import { UsersService } from 'data-access/sdk/api2';
import { OrganizationService } from 'data-access/sdk/api2';
import { ProjectTypesService } from 'data-access/sdk/api2';
import { ProjectGroupsService } from 'data-access/sdk/api2';
import { CompaniesService } from 'data-access/sdk/api2';
import { ProjectTypeStage } from 'data-access/sdk/api2';
import { ProjectTypeStagesService } from 'data-access/sdk/api2';

@Component({
  selector: 'app-project-form',
  templateUrl: 'project-form.component.html',
  styleUrls: ['project-form.component.scss'],
})
export class ProjectFormComponent implements OnInit {
  user: User;
  project: Project;
  projectForm: FormGroup;
  projectGroups$: Observable<Array<ProjectGroup>>;
  projectGroupsTree$: Observable<Array<ProjectGroup>>;
  projectTypes$: Observable<Array<ProjectType>>;

  ptStages$: Observable<ProjectTypeStage[]>;

  maxAbbrLen = 5;

  constructor(
    private authenticationService: AuthenticationService,
    private projectsService: ProjectsService,
    private usersService: UsersService,
    private formBuilder: FormBuilder,
    private organizationService: OrganizationService,
    private projectTypesService: ProjectTypesService,
    private projectGroupsService: ProjectGroupsService,
    private companiesService: CompaniesService,
    private projectTypeStagesService: ProjectTypeStagesService,
    private ref: DynamicDialogRef,
    private config: DynamicDialogConfig
  ) {}

  ngOnInit() {
    this.user = this.authenticationService.user;
    this.project = this.config.data.projectId;

    this.initForm();
    this.getOptions();

    this.projectForm.controls.name.valueChanges.subscribe((name: string) => {
      if (this.projectForm.controls.abbr.pristine || this.projectForm.controls.abbr.value === '') {
        let abbr: string;
        const words = name.split(' ');
        if (words.length > 1) {
          let abbrArr: any[];
          if (words.length >= this.maxAbbrLen) {
            abbrArr = words.map((w) => w[0]);
          } else {
            const minLetters = Math.floor(this.maxAbbrLen / words.length);
            const addLetterInd = this.maxAbbrLen - minLetters;
            abbrArr = words.map((w, i) => {
              let subInd = minLetters;
              if (i < addLetterInd) {
                subInd += 1;
              }
              return w.substring(0, subInd);
            });
          }
          abbr = abbrArr.join('').substring(0, this.maxAbbrLen);
        } else {
          abbr = name.substring(0, this.maxAbbrLen);
        }

        this.projectForm.controls.abbr.setValue(abbr.toUpperCase());
      }
    });

    this.projectForm.controls.project_type.valueChanges.subscribe((pt) => {
      if (pt) {
        this.ptStages$ = this.projectTypeStagesService
          .projectTypeStagesList({
            projectType: pt,
            include: 'stage',
          })
          .pipe(
            map((ptStages) => {
              this.projectForm.controls.stage.setValue(ptStages[0].rel_stage.id);
              return ptStages;
            })
          );
      }
    });
  }

  getOptions(): void {
    this.projectTypes$ = this.projectTypesService.projectTypesList().pipe(
      map((pts) => {
        if (pts.length === 1) {
          this.projectForm.controls.project_type.setValue(pts[0].id);
        }
        return pts;
      })
    );
    this.projectGroups$ = this.projectGroupsService.projectGroupsList({}).pipe(
      map((pgs) => {
        if (pgs.length === 1) {
          this.projectForm.controls.project_group.setValue(pgs[0].id);
        }
        return pgs;
      })
    );

    this.projectGroupsTree$ = this.companiesService.companiesProjectGroupTree(this.user.company).pipe(
      map((pgs) => {
        if (pgs.length === 1) {
          this.projectForm.controls.project_group.setValue(pgs[0].id);
        }

        return pgs.map((pg) => {
          pg.name = '-'.repeat(pg.level) + ' ' + pg.name;
          return pg;
        });
      })
    );
  }

  submit() {
    this.ref.close(this.projectForm.value);
  }

  initForm() {
    // TODO: dates validation - end > start

    const p = this.project;
    this.projectForm = this.formBuilder.group({
      id: [p?.id],
      project_group: [p?.project_group, Validators.required],
      project_type: [p?.project_type, Validators.required],
      name: [p?.name, Validators.required],
      abbr: [p?.abbr, [Validators.required, Validators.maxLength(5)]],
      stage: [p?.stage, Validators.required],
    });
  }
}
