import { Component, OnInit } from '@angular/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { UsersService } from 'data-access/sdk/api2';
import { DepartmentsService } from 'data-access/sdk/api2';
import { OrganizationService } from 'data-access/sdk/api2';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ProjectMember } from 'data-access/sdk/api2';
import * as _ from 'lodash-es';
import { firstValueFrom, forkJoin, Observable, of } from 'rxjs';
import { ProjectMembersService } from 'data-access/sdk/api2';
import { SelectItem, SelectItemGroup } from 'primeng/api';
import { switchMap } from 'rxjs/operators';
import { TranslateService } from '@ngx-translate/core';
import { ProjectEnums } from '@core-project/enums/project';
import MemberTypeEnum = ProjectEnums.MemberTypeEnum;

@Component({
  selector: 'app-project-member-form',
  templateUrl: './project-member-form.component.html',
  styleUrls: ['./project-member-form.component.scss'],
})
export class ProjectMemberFormComponent implements OnInit {
  form: FormGroup;
  objChoices: Observable<SelectItemGroup[]>;
  memberTypeChoices: SelectItem[];

  private member: ProjectMember;
  private projectId: number;

  constructor(
    private dialogConfig: DynamicDialogConfig,
    private dialogRef: DynamicDialogRef,
    private usersService: UsersService,
    private departmentsService: DepartmentsService,
    private organizationsService: OrganizationService,
    private formBuilder: FormBuilder,
    private projectMembersService: ProjectMembersService,
    private translateService: TranslateService
  ) {}

  ngOnInit() {
    this.member = this.dialogConfig.data.member;
    this.projectId = this.dialogConfig.data.projectId;
    this.memberTypeChoices = Object.values(MemberTypeEnum).map((mt) => ({
      label: this.translateService.instant('PROJECT.TEAM.TYPES.' + mt),
      value: mt,
    }));
    this.initForm();
  }

  save() {
    const val = this.form.value;
    Object.assign(val, this.convertObjId(val.object));
    delete val.object;

    const obs$: Observable<ProjectMember> = this.member?.id
      ? this.projectMembersService.projectMembersPartialUpdate({ id: this.member.id, data: val })
      : this.projectMembersService.projectMembersCreate(val);
    firstValueFrom(obs$).then((m) => this.dialogRef.close(m));
  }

  createObjectChoices() {
    const users$: Observable<SelectItemGroup> = this.usersService.usersList({}).pipe(
      switchMap((users) =>
        of({
          label: this.translateService.instant('SELECTITEMGROUP.USERS'),
          items: users.map((u) => ({ label: u.full_name, value: this.getObjId(u.id, 'user') })),
        })
      )
    );
    const departments$: Observable<SelectItemGroup> = this.departmentsService.departmentsList({}).pipe(
      switchMap((deps) =>
        of({
          label: this.translateService.instant('SELECTITEMGROUP.DEPARTMENTS'),
          items: deps.map((d) => ({ label: d.name, value: this.getObjId(d.id, 'department') })),
        })
      )
    );
    const orgs$: Observable<SelectItemGroup> = this.organizationsService.organizationList({}).pipe(
      switchMap((orgs) =>
        of({
          label: this.translateService.instant('SELECTITEMGROUP.ORGANIZATIONS'),
          items: orgs.map((o) => ({ label: o.name, value: this.getObjId(o.id, 'organization') })),
        })
      )
    );

    if ([MemberTypeEnum.GENERAL_CUSTOMER, MemberTypeEnum.CUSTOMER].includes(this.form.controls.member_type.value)) {
      this.objChoices = forkJoin([orgs$, departments$, users$]);
    } else if (
      [MemberTypeEnum.CO_EXECUTOR, MemberTypeEnum.CONTRACTOR, MemberTypeEnum.SUPPLIER].includes(this.form.controls.member_type.value)
    ) {
      this.objChoices = forkJoin([orgs$]);
    } else {
      this.objChoices = of([]);
    }
  }

  private initForm() {
    let objType = null;
    if (this.member?.user) {
      objType = 'user';
    } else if (this.member?.department) {
      objType = 'department';
    } else if (this.member?.organization) {
      objType = 'organization';
    }

    this.form = this.formBuilder.group({
      member_type: [this.member?.member_type, Validators.required],
      project: [this.member?.project || this.projectId, Validators.required],
      comment: [this.member?.comment],
      object: [this.getObjId(this.member?.user || this.member?.department || this.member?.organization, objType), Validators.required],
    });

    if (this.member?.id) {
      this.form.controls.member_type.disable();
    }

    if (this.member?.member_type) {
      this.createObjectChoices();
    }
  }

  private getObjId(objId: number, objType: 'user' | 'department' | 'organization'): string {
    switch (objType) {
      case 'user':
        return 'usr_' + objId.toString();
      case 'department':
        return 'dep_' + objId.toString();
      case 'organization':
        return 'org_' + objId.toString();
    }
    return null;
  }

  private convertObjId(objId: string): Object {
    const res = { user: null, department: null, organization: null };
    const sp = _.split(objId, '_', 2);
    switch (sp[0]) {
      case 'usr':
        res.user = Number(sp[1]);
        break;
      case 'dep':
        res.department = Number(sp[1]);
        break;
      case 'org':
        res.organization = Number(sp[1]);
        break;
    }
    return res;
  }
}
