import { Component, OnInit, Signal, WritableSignal, signal } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { TranslateService } from '@ngx-translate/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';

import { ProjectMeta, ProjectMetaGroup, ProjectType } from 'data-access/sdk/api2';
import { ProjectMetaGroupsService, ProjectTypesService } from 'data-access/sdk/api2';

import { ChoicefieldInterface } from '@core-project/interfaces';
import { ProjectEnums } from '@core-project/enums/project';
import { forkJoin } from 'rxjs';

@Component({
  selector: 'app-project-meta-form',
  templateUrl: './project-meta-form.component.html',
  styleUrls: ['./project-meta-form.component.scss'],
})
export class ProjectMetaFormComponent implements OnInit {
  private readonly _projectMetaDataTypeListChoice: WritableSignal<ChoicefieldInterface<ProjectEnums.DataTypeEnum | ''>[]> = signal([]);
  private readonly _projectTypes: WritableSignal<ProjectType[]> = signal([]);
  private readonly _groupChoices: WritableSignal<ProjectMetaGroup[]> = signal([]);

  readonly meta: ProjectMeta;

  readonly form: FormGroup = new FormGroup({
    name: new FormControl('', [Validators.required]),
    data_type: new FormControl(null, [Validators.required]),
    project_types: new FormControl([]),
    prefix: new FormControl(''),
    postfix: new FormControl(''),
    all_types: new FormControl(false),
    choices: new FormControl(null),
    unit: new FormControl(''),
    description: new FormControl(''),
    group: new FormControl(null, [Validators.required]),
  });

  readonly projectMetaDataTypeListChoice: Signal<ChoicefieldInterface<ProjectEnums.DataTypeEnum | ''>[]> =
    this._projectMetaDataTypeListChoice.asReadonly();
  readonly projectTypes: Signal<ProjectType[]> = this._projectTypes.asReadonly();
  readonly groupChoices: Signal<ProjectMetaGroup[]> = this._groupChoices.asReadonly();

  readonly allowedDataTypes = [
    ProjectEnums.DataTypeEnum.CH,
    ProjectEnums.DataTypeEnum.INT,
    ProjectEnums.DataTypeEnum.DT,
    ProjectEnums.DataTypeEnum.CHS,
    ProjectEnums.DataTypeEnum.MCHS,
    ProjectEnums.DataTypeEnum.FLAG,
  ];

  showChoices = false;

  constructor(
    private readonly projectTypesService: ProjectTypesService,
    private readonly projectMetaGroupsService: ProjectMetaGroupsService,
    private readonly dynamicDialogRef: DynamicDialogRef,
    private readonly translateService: TranslateService,
    dialogConfig: DynamicDialogConfig
  ) {
    this.meta = dialogConfig.data.meta;
  }

  ngOnInit() {
    forkJoin([this.projectTypesService.projectTypesList(), this.projectMetaGroupsService.projectMetaGroupsList()]).subscribe(
      ([projectTypes, groupChoices]) => {
        this._projectTypes.set(projectTypes);
        this._groupChoices.set(groupChoices);
        this.initForm();
      }
    );
  }

  onSubmit() {
    this.dynamicDialogRef.close(this.form.value);
  }

  showChoicesToggle(form: FormGroup) {
    const dataType = form.controls.data_type.value;

    if (dataType) {
      this.showChoices = form.controls.data_type.value === 'CHS' || form.controls.data_type.value === 'MCHS';
      return;
    }

    this.showChoices = false;
  }

  showTypesToggle(form: FormGroup) {
    if (form.controls.all_types.value) {
      form.controls.project_types.disable();
    } else {
      form.controls.project_types.enable();
    }
  }

  private initForm(meta: ProjectMeta = this.meta) {
    this.createDataTypeOptions();

    if (meta) {
      // primeng problem (?) need to wait render dropdown elements
      setTimeout(() => {
        this.form.controls.name.setValue(meta.name ?? '');
        this.form.controls.project_types.setValue(meta.project_types ?? []);
        this.form.controls.prefix.setValue(meta.prefix ?? '');
        this.form.controls.postfix.setValue(meta.postfix ?? '');
        this.form.controls.all_types.setValue(meta.all_types || false);
        this.form.controls.choices.setValue(meta.choices || null);
        this.form.controls.unit.setValue(meta.unit ?? '');
        this.form.controls.description.setValue(meta.description ?? '');
        this.form.controls.group.setValue(meta.group || null);

        this.form.controls.data_type.disable();

        if (meta.data_type) {
          this.form.controls.data_type.setValue(meta.data_type);
        }

        this.showTypesToggle(this.form);
        this.showChoicesToggle(this.form);
      });
    }
  }

  private createDataTypeOptions() {
    const allowedDataTypes = this.allowedDataTypes.map((adt) => ({
      value: adt,
      label: this.translateService.instant('KB.METHODOLOGY.PROJECT_METAS.META_TYPES.' + adt),
    }));
    this._projectMetaDataTypeListChoice.set(allowedDataTypes);
  }
}
