import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ProjectMetaValueService } from 'data-access/sdk/api2';
import { ProjectMeta, ProjectMetaValue } from 'data-access/sdk/api2';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import * as _ from 'lodash-es';
import { firstValueFrom, forkJoin, Observable } from 'rxjs';
import { ProjectEnums } from '@core-project/enums/project';

interface MetaValueFormField {
  meta: ProjectMeta;
  form: FormGroup;
}

@Component({
  selector: 'app-project-meta-values-form',
  templateUrl: './project-meta-values-form.component.html',
  styleUrls: ['./project-meta-values-form.component.scss'],
})
export class ProjectMetaValuesFormComponent implements OnInit {
  metaFields: Array<ProjectMeta>;
  metaValues: Array<ProjectMetaValue>;
  forms: Array<MetaValueFormField>;

  projectId: number;

  dataTypes: typeof ProjectEnums.DataTypeEnum = ProjectEnums.DataTypeEnum;

  saving: boolean;

  constructor(
    private formBuilder: FormBuilder,
    private projectMetaValueService: ProjectMetaValueService,
    private config: DynamicDialogConfig,
    private ref: DynamicDialogRef
  ) {}

  ngOnInit(): void {
    this.metaFields = this.config.data.metaFields;
    this.metaValues = this.config.data.metaValues;
    this.projectId = this.config.data.projectId;

    const forms = [];
    this.metaFields.forEach((mf) => forms.push({ meta: mf, form: this.buildFormGroup(mf, this.findValueForMeta(mf)) }));
    this.forms = forms;
  }

  buildFormGroup(metaField: ProjectMeta, metaValue?: ProjectMetaValue): FormGroup {
    return this.formBuilder.group({
      id: [metaValue?.id],
      meta: [metaField.id, Validators.required],
      project: [this.projectId, Validators.required],
      value_CH: [metaValue?.value_CH],
      value_CHS: [metaValue?.value_CHS],
      value_DT: [metaValue?.value_DT],
      value_FLAG: [metaValue?.value_FLAG],
      value_FLT: [metaValue?.value_FLT],
      value_INT: [metaValue?.value_INT],
      value_MCHS: [metaValue?.value_MCHS],
      value_TXT: [metaValue?.value_TXT],
    });
  }

  findValueForMeta(metaField: ProjectMeta): ProjectMetaValue {
    return _.find(this.metaValues, ['meta', metaField.id]);
  }

  allFormsValid(): boolean {
    return _.every(this.forms, ['form.valid', true]);
  }

  save() {
    this.saving = true;
    const saveObs$: Array<Observable<any>> = [];
    this.forms.forEach((f) => {
      const val = f.form.value;
      saveObs$.push(
        val.id
          ? this.projectMetaValueService.projectMetaValuePartialUpdate({ id: val.id, data: val })
          : this.projectMetaValueService.projectMetaValueCreate(val)
      );
    });
    firstValueFrom(forkJoin(saveObs$)).then((_r) => this.ref.close());
  }
}
