import { Component, OnInit } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { BehaviorSubject, firstValueFrom, Observable, of } from 'rxjs';
import { map, switchMap, takeUntil, tap } from 'rxjs/operators';
import { TranslateService } from '@ngx-translate/core';
import { Actions, ofActionSuccessful, Store } from '@ngxs/store';
import { MenuItem } from 'primeng/api';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';

import { IssueEnums } from '@core-project/enums';
import { AuthenticationService } from '@core-project/services/authentication.service';

import { IssuesActions, WorksActions } from 'data-access/store';
import {
  CompanyPreferences,
  FinanceService,
  Issue,
  IssuesService,
  ProjectBudget,
  ProjectsService,
  ProjectState,
  ProjectStatesService,
  ProjectStatusReport,
  Work,
  WorksService,
} from 'data-access/sdk/api2';
import { CustomDatePipe } from 'utils/shared-pipes';
import { Unsubscriber } from 'utils/shared-services';
import { ProjectEnums } from '@core-project/enums/project';

@Component({
  selector: 'app-project-state-form',
  templateUrl: './project-state-form.component.html',
  providers: [CustomDatePipe, Unsubscriber],
})
export class ProjectStateFormComponent implements OnInit {
  private milestonesSubject: BehaviorSubject<Work[]> = new BehaviorSubject([]);
  private projectBudgetSubject: BehaviorSubject<ProjectBudget> = new BehaviorSubject(undefined);
  private issuesSubject: BehaviorSubject<Issue[]> = new BehaviorSubject([]);
  private projectStatusReportSubject: BehaviorSubject<ProjectStatusReport> = new BehaviorSubject(undefined);

  private readonly workObserverSub = this.actions
    .pipe(
      takeUntil(this.unsubscriber.destroy$),
      ofActionSuccessful(WorksActions.CreateWork, WorksActions.UpdateWork, WorksActions.ChangeWorkStatus, WorksActions.DeleteWork)
    )
    .subscribe(() => {
      this.loadMilestones$().subscribe();
      this.loadStatusReport$().subscribe();
    });

  private readonly issueObserverSub = this.actions
    .pipe(
      takeUntil(this.unsubscriber.destroy$),
      ofActionSuccessful(IssuesActions.CreateIssue, IssuesActions.UpdateIssue, IssuesActions.DeleteIssue)
    )
    .subscribe(() => {
      this.loadIssues$().subscribe();
    });

  milestones$: Observable<Work[]> = this.milestonesSubject.asObservable();
  projectBudget$: Observable<ProjectBudget | undefined> = this.projectBudgetSubject.asObservable();
  issues$: Observable<Issue[]> = this.issuesSubject.asObservable();
  projectStatusReport$: Observable<ProjectStatusReport | undefined> = this.projectStatusReportSubject.asObservable();

  readonly form: FormGroup = new FormGroup({
    budget_comment: new FormControl(),
    budget_fact: new FormControl(),
    budget_forecast: new FormControl(),
    budget_indicator: new FormControl(null, [Validators.required]),
    done: new FormControl(),
    indicators_comment: new FormControl(),
    issues_comment: new FormControl(),
    issues_indicator: new FormControl(null, [Validators.required]),
    procurements_comment: new FormControl(),
    procurements_indicator: new FormControl(null, [Validators.required]),
    progress: new FormControl(),
    project: new FormControl(null, [Validators.required]),
    state: new FormControl(null, [Validators.required]),
    works_comment: new FormControl(),
    works_indicator: new FormControl(null, [Validators.required]),
  });
  projectState: ProjectState;

  items: MenuItem[] = [];
  activeItemId?: string;
  preferences?: CompanyPreferences;

  constructor(
    private readonly projectStatesService: ProjectStatesService,
    private readonly worksService: WorksService,
    private readonly financeService: FinanceService,
    private readonly issuesService: IssuesService,
    private readonly projectsService: ProjectsService,
    private readonly dialogConfig: DynamicDialogConfig,
    private readonly dialogRef: DynamicDialogRef,
    private readonly translateService: TranslateService,
    private readonly authenticationService: AuthenticationService,
    private readonly store: Store,
    private readonly actions: Actions,
    private readonly unsubscriber: Unsubscriber
  ) {}

  get formControls() {
    return this.form.controls;
  }

  ngOnInit() {
    this.items = [
      { id: 'general', label: this.translateService.instant('PROJECT.STATE.MENU_GENERAL'), icon: 'fa-regular fa-chart-line' },
      { id: 'indicators', label: this.translateService.instant('PROJECT.STATE.MENU_INDICATORS'), icon: 'fa-regular fa-traffic-light-stop' },
    ];
    this.activeItemId = this.items[0].id;
    this.projectState = this.dialogConfig.data.projectState;

    this.loadIssues$().subscribe();
    this.loadMilestones$().subscribe();
    this.loadStatusReport$().subscribe();

    this.loadFinanceProjectBudgetsList$().subscribe((budget) => {
      this.initForm(this.form, this.projectState, budget);
    });

    this.preferences = this.authenticationService.user.rel_company.rel_preferences;
  }

  getDaysBetweenDates(date1: string, date2: string): string | undefined {
    if (!date1 || !date2) {
      return undefined;
    }

    const diffTime = new Date(date2).getTime() - new Date(date1).getTime();
    const days = Math.ceil(diffTime / (1000 * 60 * 60 * 24));

    return `${days > 0 ? '+' + days : '' + days} ${this.translateService.instant('DIFFS.DAYS')}`;
  }

  save() {
    let obs$: Observable<ProjectState>;
    if (this.projectState && this.projectState.id) {
      obs$ = this.projectStatesService.projectStatesPartialUpdate({ id: this.projectState.id, data: this.form.value });
    } else {
      obs$ = this.projectStatesService.projectStatesCreate(this.form.value);
    }

    firstValueFrom(obs$).then((ps) => this.dialogRef.close(ps));
  }

  itemChanged(activeItem: MenuItem) {
    this.activeItemId = activeItem.id;
  }

  openMilestone(work: Work) {
    this.store.dispatch(new WorksActions.ShowWorkInSidebar(work.id)).subscribe();
  }

  openIssue(issue: Issue) {
    this.store.dispatch(new IssuesActions.ShowIssueInSidebar(issue.id)).subscribe();
  }

  createIssue() {
    this.store.dispatch(new IssuesActions.ShowCreateIssueInSidebar(this.projectState.project)).subscribe();
  }

  private initForm(form: FormGroup, projectState: ProjectState, projectBudget?: ProjectBudget) {
    form.controls['project'].setValue(projectState.project);
    form.controls['works_indicator'].setValue(projectState?.works_indicator ?? ProjectEnums.IndicatorEnum.GREEN);
    form.controls['issues_indicator'].setValue(projectState?.issues_indicator ?? ProjectEnums.IndicatorEnum.GREEN);
    form.controls['budget_indicator'].setValue(projectState?.budget_indicator ?? ProjectEnums.IndicatorEnum.GREEN);
    form.controls['procurements_indicator'].setValue(projectState?.procurements_indicator ?? ProjectEnums.IndicatorEnum.NA);
    form.controls['works_comment'].setValue(projectState?.works_comment);
    form.controls['issues_comment'].setValue(projectState?.issues_comment);
    form.controls['budget_comment'].setValue(projectState?.budget_comment);
    form.controls['procurements_comment'].setValue(projectState?.procurements_comment);
    form.controls['indicators_comment'].setValue(projectState?.indicators_comment);
    form.controls['state'].setValue(projectState?.state);
    form.controls['done'].setValue(projectState?.done);
    form.controls['progress'].setValue(projectState?.progress);
    form.controls['budget_forecast'].setValue(projectState?.budget_forecast ?? projectBudget?.total_planned);
    form.controls['budget_fact'].setValue(projectState?.budget_fact);
  }

  private loadFinanceProjectBudgetsList$(projectId: string = this.projectState.project.toString()) {
    return this.financeService.financeProjectBudgetsList({ project: projectId }).pipe(
      switchMap((budgets) => of(budgets[0])),
      tap((budget) => this.projectBudgetSubject.next(budget))
    );
  }

  private loadMilestones$(projectId: string = this.projectState.project.toString()) {
    return this.worksService
      .worksList({ project: projectId, milestoneLevel: 1 })
      .pipe(tap((milestones) => this.milestonesSubject.next(milestones)));
  }

  private loadIssues$(projectId: string = this.projectState.project.toString()) {
    return this.issuesService
      .issuesIssuesList({
        project: projectId,
        status: IssueEnums.StatusEnum.ACT,
      })
      .pipe(tap((issues) => this.issuesSubject.next(issues)));
  }

  private loadStatusReport$(projectId: number = this.projectState.project) {
    return this.projectsService.projectsStatusReport(projectId).pipe(
      map((statusReport) => {
        ['works_indicator', 'budget_indicator', 'procurements_indicator', 'issues_indicator'].forEach((indicator) => {
          const prefName = 'auto_' + indicator;
          if (this.preferences[prefName]) {
            this.formControls[indicator].setValue(statusReport[indicator]);
          }
        });
        return statusReport;
      }),
      tap((statusReport) => this.projectStatusReportSubject.next(statusReport))
    );
  }
}
