import {Component, OnInit} from '@angular/core';
import {DynamicDialogConfig, DynamicDialogRef} from 'primeng/dynamicdialog';
import {FormBuilder, FormGroup, Validators} from '@angular/forms';
import {Permission, PermissionsService, Role} from 'data-access/sdk/api2';
import * as _ from 'lodash-es';
import {TranslateService} from '@ngx-translate/core';
import {CheckboxChangeEvent} from 'primeng/checkbox';

interface Perm {
  id: number;
  action: string;
  checked: boolean;
  label?: string;
}

interface PermissionModel {
  model: string;
  label: string;
  add: Perm;
  change: Perm;
  delete: Perm;
  view: Perm;
  otherPerms: Array<Perm>;
}

interface PermissionApp {
  app: string;
  label: string;
  models: Array<PermissionModel>;
}

interface ExcludeModel {
  model: string;
  perms?: Array<string>;
}

interface ExcludeApp {
  app: string;
  models?: Array<ExcludeModel>;
}

interface ExcludeItems {
  company: Array<ExcludeApp>;
  project_group: Array<ExcludeApp>;
  project: Array<ExcludeApp>;
}

@Component({
  selector: 'app-role-form',
  templateUrl: './role-form.component.html',
  styleUrls: ['./role-form.component.scss'],
})
export class RoleFormComponent implements OnInit {
  role: Role;
  form: FormGroup;
  permissions: Array<Permission>;
  permByApps: Array<PermissionApp>;
  allPermByApps: Array<PermissionApp>;
  exclude: ExcludeItems = {
    company: [
      {app: 'companies', models: [{model: 'company', perms: ['add', 'delete']}]},
      {app: 'documents', models: [{model: 'document', perms: ['can_undelete']}]},
    ],
    project_group: [
      {app: 'companies'},
      {app: 'contacts'},
      {
        app: 'documents',
        models: [{model: 'document', perms: ['can_undelete']}, {model: 'documentmetafield'}, {model: 'documenttype'}],
      },
      {app: 'finance', models: [{model: 'budgetcategory'}, {model: 'budgettype'}]},
      {app: 'issues', models: [{model: 'issuecategory'}, {model: 'issuedomain'}]},
      {app: 'projects', models: [{model: 'projectcategory'}, {model: 'projecttype'}]},
      {app: 'roles', models: [{model: 'role'}]},
      {app: 'users'},
    ],
    project: [
      {app: 'companies'},
      {app: 'contacts'},
      {
        app: 'documents',
        models: [{model: 'document', perms: ['can_undelete']}, {model: 'documentmetafield'}, {model: 'documenttype'}],
      },
      {app: 'finance', models: [{model: 'budgetcategory'}, {model: 'budgettype'}]},
      {app: 'issues', models: [{model: 'issuecategory'}, {model: 'issuedomain'}]},
      {
        app: 'projects',
        models: [
          {model: 'projectcategory'},
          {model: 'projectgroup'},
          {model: 'projectmeta'},
          {model: 'projecttype'},
          {model: 'projectview'},
          {model: 'project', perms: ['add']},
        ],
      },
      {app: 'roles', models: [{model: 'role'}]},
      {app: 'users'},
    ],
  };

  objectFn = Object;

  constructor(
    private dialogConfig: DynamicDialogConfig,
    private dialogRef: DynamicDialogRef,
    private formBuilder: FormBuilder,
    private permissionsService: PermissionsService,
    private translateService: TranslateService,
  ) {
  }

  get controls() {
    return this.form.controls;
  }

  static checkAppValidator(group: FormGroup) {
    return [group.controls.company.value, group.controls.project_group.value, group.controls.project.value].some((i) => i)
      ? null
      : {application: true};
  }

  ngOnInit(): void {
    this.role = this.dialogConfig.data?.role;
    this.initForm();
    this.permissionsService.permissionsList()
      .subscribe((perms) => {
        // construct permissions object
        const res = [];
        const appPerms = _.groupBy(perms, 'content_type.app_label');
        Object.keys(appPerms).forEach((app) => {
          appPerms[app] = _.groupBy(appPerms[app], 'content_type.model') as any;
          const pa: PermissionApp = {
            app: app,
            label: this.translateService.instant('APPS.' + app.toUpperCase() + '.NAME'),
            models: [],
          };
          Object.keys(appPerms[app]).forEach((model) => {
            const pm: PermissionModel = {
              model: model,
              label: this.translateService.instant('APPS.' + pa.app.toUpperCase() + '.MODELS.' + model.toUpperCase()),
              add: null,
              change: null,
              delete: null,
              view: null,
              otherPerms: [],
            };
            appPerms[app][model].forEach((perm) => {
              const p: Perm = {
                id: perm.id,
                checked: this.role?.permissions.includes(perm.id) || false,
                action: _.replace(perm.codename, '_' + model, ''),
              };

              switch (p.action) {
                case 'add':
                  pm.add = p;
                  break;
                case 'change':
                  pm.change = p;
                  break;
                case 'delete':
                  pm.delete = p;
                  break;
                case 'view':
                  pm.view = p;
                  break;
                default: {
                  p.label = this.translateService.instant(
                    'PERMS.' + pa.app.toUpperCase() + '.' + pm.model.toUpperCase() + '.' + p.action.toUpperCase(),
                  );
                  pm.otherPerms.push(p);
                }
              }
            });
            pa.models.push(pm);
          });
          res.push(pa);
        });

        this.allPermByApps = res;
        if (this.role) {
          if (this.role.company) {
            this.excludePerms('company');
          }
          if (this.role.project_group) {
            this.excludePerms('project_group');
          }
          if (this.role.project) {
            this.excludePerms('project');
          }
        } else {
          this.permByApps = JSON.parse(JSON.stringify(res));
        }

        this.permissions = perms;
      });
  }

  initForm() {
    this.form = this.formBuilder.group(
      {
        name: [this.role?.name, Validators.required],
        company: [this.role?.company || false],
        project_group: [this.role?.project_group || false],
        project: [this.role?.project || false],
        permissions: [this.role?.permissions || []],
        all_permissions: [this.role?.all_permissions || false],
        important: [this.role?.important || false],
      },
      {validators: RoleFormComponent.checkAppValidator},
    );

    if (this.role?.id) {
      this.form.controls.project.disable();
      this.form.controls.project_group.disable();
      this.form.controls.company.disable();
    }
  }

  save() {
    this.dialogRef.close(this.form.value);
  }

  permClicked($event: CheckboxChangeEvent, perm: Perm) {
    const perms = this.form.controls.permissions.value;
    if ($event.checked) {
      perms.push(perm.id);
    } else {
      _.pull(perms, perm.id);
    }
    this.form.controls.permissions.setValue(perms);
  }

  appClicked($event: CheckboxChangeEvent, domain: string) {
    if ($event.checked) {
      const domainsToUncheck = _.pull(['company', 'project_group', 'project'], domain);
      domainsToUncheck.forEach((d) => this.form.controls[d].setValue(false));
      this.excludePerms(domain);
    }
  }

  excludePerms(domain: string) {
    const inPerms = ['add', 'change', 'delete', 'view'];
    const res = _.cloneDeep(this.allPermByApps);
    this.exclude[domain].forEach((ex) => {
      if (ex.models) {
        const app = _.find(res, ['app', ex.app]);
        ex.models.forEach((exm) => {
          if (exm.perms) {
            const model = _.find(app.models, ['model', exm.model]);
            exm.perms.forEach((exp) => {
              if (inPerms.includes(exp)) {
                model[exp] = null;
              } else {
                _.remove(model.otherPerms, ['action', exp]);
              }
            });
          } else {
            _.remove(app.models, function (pm) {
              return pm.model === exm.model;
            });
          }
        });
      } else {
        _.remove(res, function (pa) {
          return pa.app === ex.app;
        });
      }
    });

    this.permByApps = res;
  }

  massChange(option: string) {
    const allToAdd = [];
    const allToDelete = [];
    this.permByApps.forEach((pa) =>
      pa.models.forEach((pm) => {
        if (pm[option]) {
          if (pm[option].checked) {
            allToDelete.push(pm[option].id);
          } else {
            allToAdd.push(pm[option].id);
          }
          pm[option].checked = !pm[option].checked;
        }
      }),
    );
    const perms = this.form.controls.permissions.value;
    _.pullAll(perms, allToDelete);
    perms.push(...allToAdd);
    this.form.controls.permissions.setValue(perms);
  }
}
