import {Component, OnInit} from '@angular/core';
import {DynamicDialogConfig, DynamicDialogRef} from 'primeng/dynamicdialog';
import {FormControl, FormGroup, Validators} from '@angular/forms';
import {BehaviorSubject, finalize, forkJoin, Observable, of} from 'rxjs';
import {SelectItemGroup} from 'primeng/api';
import {SelectItem} from 'primeng/api/selectitem';
import {map} from 'rxjs/operators';
import {TranslateService} from '@ngx-translate/core';
import {DepartmentsService, Role, RolesService, UserRole, UserRolesService, UsersService} from 'data-access/sdk/api2';
import {
  ResponsiblePrefixEnum,
  ResponsiblePrefixService,
} from '@core-project/modules/forms/role-forms/user-role-form/responsible-prefix.service';

export interface UserRoleFormComponentData {
  type: 'company' | 'group' | 'project';
  userRole: UserRole;
  isDepartmentsResponsible: boolean;
  companyId?: number;
  projectGroupId?: number;
  projectId?: number;
}

interface IRoleFG {
  roleId: FormControl<number>;
  responsible: FormControl<string>;
  customName: FormControl<string>;
  projectId: FormControl<number>;
  projectGroupId: FormControl<number>;
  companyId: FormControl<number>;
}

@Component({
  selector: 'app-user-role-form',
  templateUrl: './user-role-form.component.html',
  styleUrls: ['./user-role-form.component.scss'],
})
export class UserRoleFormComponent implements OnInit {
  formGroup: FormGroup<IRoleFG>;

  readonly responsibleDropdownGroup: boolean;

  readonly saveLoading: BehaviorSubject<boolean> = new BehaviorSubject(false);

  readonly roleListSubject: BehaviorSubject<Role[]> = new BehaviorSubject<Role[]>([]);
  readonly roleList$: Observable<Role[]> = this.roleListSubject.asObservable();

  readonly responsibleListSubject: BehaviorSubject<SelectItemGroup[] | SelectItem[]> =
    new BehaviorSubject<SelectItemGroup[] | SelectItem[]>([]);
  readonly responsibleList$: Observable<SelectItemGroup[] | SelectItem[]> =
    this.responsibleListSubject.asObservable();

  private readonly isDepartmentsResponsible: boolean;
  private readonly type: string | undefined;
  private readonly userRole: UserRole | undefined;

  constructor(
    private readonly dynamicDialogRef: DynamicDialogRef,
    private readonly dialogConfig: DynamicDialogConfig<UserRoleFormComponentData>,
    private readonly roleService: RolesService,
    private readonly userService: UsersService,
    private readonly departmentService: DepartmentsService,
    private readonly userRolesService: UserRolesService,
    private readonly responsiblePrefixService: ResponsiblePrefixService,
    private readonly translateService: TranslateService,
  ) {
    this.type = this.dialogConfig.data.type;
    this.userRole = this.dialogConfig.data.userRole;
    this.responsibleDropdownGroup = this.dialogConfig.data.isDepartmentsResponsible;
    this.isDepartmentsResponsible = this.dialogConfig.data.isDepartmentsResponsible;
  }

  get responsibleLabel() {
    return this.translateService.instant('PROJECT_TEAM.DIALOG_ADD_USER')
      + (this.isDepartmentsResponsible
          ? ` / ${this.translateService.instant('PROJECT_TEAM.DIALOG_ADD_RESPONSIBLE').toLowerCase()}`
          : ''
      );
  }

  ngOnInit(): void {
    this.getFormOptions()
      .subscribe(([roles, responsibleList]) => {
        this.roleListSubject.next(roles);
        this.responsibleListSubject.next(responsibleList);
      });
    this.initForm(this.dialogConfig.data);
  }

  save() {
    this.saveLoading.next(true);

    const obs$ = this.userRole
      ? this.userRolesService.userRolesPartialUpdate({id: this.userRole.id, data: this.toUserRoleDTO(this.formGroup)})
      : this.userRolesService.userRolesCreate(this.toUserRoleDTO(this.formGroup));

    obs$
      .pipe(
        finalize(() => this.saveLoading.next(false)),
      )
      .subscribe({
        next: userRole => {
          this.dynamicDialogRef.close(userRole);
        },
        error: err => {
          console.error(err);
        },
      });
  }

  private getFormOptions() {
    return forkJoin([
      this.getRolesOptions(),
      this.getResponsibleOptions(this.isDepartmentsResponsible),
    ]);
  }

  private getResponsibleOptions(isDepartmentsResponsible: boolean) {
    return forkJoin({
      users: this.userService.usersList({}),
      departments: isDepartmentsResponsible ? this.departmentService.departmentsList({}) : of([]),
    })
      .pipe(
        map(({users, departments}) => {
            const departmentsItems = departments.map(department => ({
                label: department.name,
                value: this.responsiblePrefixService.addPrefix(department.id, ResponsiblePrefixEnum.Department),
              }),
            );

            const usersItems = users.map(
              user => ({
                label: user.full_name,
                value: this.responsiblePrefixService.addPrefix(user.id, ResponsiblePrefixEnum.User),
              }),
            );

            // map labels
            const list: SelectItemGroup[] = [
              {
                label: this.translateService.instant('PROJECT_TEAM.DROPDOWN_GROUP_DEPARTMENTS'),
                items: departmentsItems,
              },
              {
                label: this.translateService.instant('PROJECT_TEAM.DROPDOWN_GROUP_USERS'),
                items: usersItems,
              },
            ];

            // if there is only one item in the list, return it as SelectItem, else return SelectItemGroup
            const filteredList = list.filter(item => item.items.length > 0);
            if (filteredList.length === 1) {
              return filteredList[0].items.map(item => ({
                label: item.label,
                value: item.value,
              }));
            } else {
              return filteredList;
            }
          },
        ),
      );
  }

  private getRolesOptions() {
    const roleParams: RolesService.RolesListParams = {};
    switch (this.type) {
      case 'project':
        roleParams.project = 'true';
        break;
      case 'group':
        roleParams.projectGroup = 'true';
        break;
      case 'company':
        roleParams.company = 'true';
        break;
    }

    return this.roleService.rolesList(roleParams);
  }

  private toUserRoleDTO(formGroup: FormGroup<IRoleFG>): UserRole {
    const data: UserRole = {
      role: formGroup.controls.roleId.value,
      custom_name: formGroup.controls.customName.value,
      project: formGroup.controls.projectId.value,
      project_group: formGroup.controls.projectGroupId.value,
      company: formGroup.controls.companyId.value,
      user: undefined,
      department: undefined,
    };
    const responsibleId = this.responsiblePrefixService.deletePrefix(formGroup.controls.responsible.value);
    const responsibleType = this.responsiblePrefixService.getPrefix(formGroup.controls.responsible.value);

    switch (responsibleType) {
      case ResponsiblePrefixEnum.User:
        data.user = responsibleId;
        break;
      case ResponsiblePrefixEnum.Department:
        data.department = responsibleId;
        break;
      default:
        console.error(`Unknown responsible type: ${responsibleType}`);
    }

    return data;
  }

  private initForm({userRole, projectId, projectGroupId, companyId}: UserRoleFormComponentData) {
    this.formGroup = new FormGroup<IRoleFG>({
      roleId: new FormControl(userRole?.role, [Validators.required]),
      responsible: new FormControl(this.responsiblePrefixService.mapResponsible(userRole), [Validators.required]),
      customName: new FormControl(userRole?.custom_name ?? '', []),
      projectId: new FormControl(projectId, []),
      projectGroupId: new FormControl(projectGroupId, []),
      companyId: new FormControl(companyId, []),
    });

    if (this.userRole?.id) {
      this.formGroup.controls.roleId.disable();
      this.formGroup.controls.responsible.disable();
    }
  }
}
