import { Component, OnInit } from '@angular/core';
import { Department, Telephone, User } from 'data-access/sdk/api2';
import { defaultIfEmpty, firstValueFrom, forkJoin, Observable } from 'rxjs';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { FormArray, FormBuilder, FormGroup, Validators } from '@angular/forms';
import { AuthService, CompaniesService, TelephoneService, UsersService } from 'data-access/sdk/api2';
import { AuthenticationService } from '@core-project/services/authentication.service';
import { map, switchMap } from 'rxjs/operators';
import { MessageService } from 'primeng/api';
import { TranslateService } from '@ngx-translate/core';

@Component({
  selector: 'app-user-form',
  templateUrl: './user-form.component.html',
  styleUrls: ['./user-form.component.scss'],
})
export class UserFormComponent implements OnInit {
  public departmentChoices$: Observable<Array<Department>>;
  public form: FormGroup;
  public defaultAvatar = 'assets/images/default_photo.png';
  public currentAvatar: string;
  public selectedFile: File;
  public telephoneForms: FormArray<FormGroup> = new FormArray<FormGroup>([]);
  public saveLoading = false;
  private user: User;
  private telephones: Telephone[];

  constructor(
    private config: DynamicDialogConfig,
    private dynamicDialogRef: DynamicDialogRef,
    private formBuilder: FormBuilder,
    private companiesService: CompaniesService,
    private authenticationService: AuthenticationService,
    private telephoneService: TelephoneService,
    private userService: UsersService,
    private authService: AuthService,
    private messageService: MessageService,
    private translateService: TranslateService
  ) {}

  ngOnInit(): void {
    this.departmentChoices$ = this.companiesService.companiesDepartmentTree(this.authenticationService.user.company).pipe(
      map((deps) =>
        deps.map((d) => {
          d.name = '-'.repeat(d.level) + ' ' + d.name;
          return d;
        })
      )
    );
    this.user = this.config.data.user;
    this.telephones = this.config.data.telephones || [];
    this.currentAvatar = this.user?.avatar || this.defaultAvatar;
    this.initForm();
  }

  initForm() {
    this.form = this.formBuilder.group({
      email: ['', Validators.compose([Validators.required, Validators.email])],
      first_name: ['', Validators.required],
      middle_name: [''],
      last_name: ['', Validators.required],
      birthday: [null],
      place: [''],
      department: [null],
      department_manager: [false],
      position: [''],
    });

    if (this.user) {
      this.form.patchValue(this.user);
    }

    for (const phone of this.telephones) {
      this.telephoneForms.push(this.createTelephoneForm(phone));
    }
  }

  save() {
    this.saveLoading = true;
    let userObs$: Observable<User>;
    const newUser = !this.user;
    if (this.user) {
      userObs$ = this.authService.authUsersPartialUpdate({ id: this.user.id, data: this.form.value });
    } else {
      userObs$ = this.authService
        .authUsersCreate(this.form.value)
        .pipe(switchMap((u) => this.authService.authUsersPartialUpdate({ id: u.id, data: this.form.value })));
    }

    firstValueFrom(userObs$).then((u) => {
      this.user = u;

      const res$: Observable<any>[] = [];
      this.telephoneForms.controls.forEach((telephone) => res$.push(this.addOrEditTelephone(telephone.value)));

      const curIds = this.telephoneForms.controls.map((t) => t.value.id);
      const deletedIds = this.telephones.filter((t) => !curIds.includes(t.id)).map((t) => t.id);
      deletedIds.forEach((id) => res$.push(this.deleteTelephone(id)));

      if (this.selectedFile) {
        res$.push(this.uploadFile());
      }

      firstValueFrom(forkJoin(res$).pipe(defaultIfEmpty(null))).then((_r) => {
        if (newUser) {
          this.messageService.add({
            severity: 'success',
            summary: this.translateService.instant('KB.STRUCTURE.USER_CREATED_HEADER'),
            detail: this.translateService.instant('KB.STRUCTURE.USER_CREATED_MSG', { name: this.user.full_name }),
          });
        } else {
          this.messageService.add({
            severity: 'success',
            summary: this.translateService.instant('KB.STRUCTURE.USER_EDITED_HEADER'),
            detail: this.translateService.instant('KB.STRUCTURE.USER_EDITED_MSG', { name: this.user.full_name }),
          });
        }
        this.dynamicDialogRef.close(this.user);
      });
    });
  }

  public selectFile(event, fileUpload?) {
    this.selectedFile = event.currentFiles[0];
    this.currentAvatar = this.selectedFile['objectURL'];
    fileUpload.clear();
  }

  public addTelephoneForm() {
    const newTelephone = this.createTelephoneForm(null);
    this.telephoneForms.push(newTelephone);
  }

  public deleteTelephoneForm(telephone) {
    const deleteIndex = this.telephoneForms.value.findIndex((tel) => tel.id === telephone.fn.id);
    this.telephoneForms.removeAt(deleteIndex);
  }

  public clickMessenger(phone: FormGroup, field: string) {
    phone.controls[field].setValue(!phone.controls[field].value);
  }

  public uploadFile() {
    return this.userService.usersUpdateAvatar({
      id: this.user.id,
      avatar: this.selectedFile,
    });
  }

  private addOrEditTelephone(telephone): Observable<Telephone> {
    telephone.user = this.user.id;
    if (telephone.id) {
      return this.telephoneService.telephonePartialUpdate({
        id: telephone.id,
        data: telephone,
      });
    } else {
      return this.telephoneService.telephoneCreate(telephone);
    }
  }

  private createTelephoneForm(telephone?: Telephone) {
    return this.formBuilder.group({
      id: [telephone?.id],
      number: [telephone?.number, Validators.required],
      primary: [telephone?.primary || false],
      whatsapp: [telephone?.whatsapp || false],
      telegram: [telephone?.telegram || false],
      comment: [telephone?.comment || ''],
      user: [this.user?.id],
    });
  }

  private deleteTelephone(telId) {
    return this.telephoneService.telephoneDelete(telId);
  }
}
