import { Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges } from '@angular/core';
import { firstValueFrom } from 'rxjs';
import { DocumentFormComponent } from '@core-project/modules/forms/document-forms/document-form/document-form.component';
import { DocumentFileFormComponent } from '@core-project/modules/forms/document-forms/document-file-form/document-file-form.component';
import { ConfirmationService, MessageService } from 'primeng/api';
import { FileHandle } from '@core-project/directives/file-drop.directive';
import { DialogService } from 'primeng/dynamicdialog';
import { TranslateService } from '@ngx-translate/core';
import { Document } from 'data-access/sdk/api2';
import { DocumentsService } from 'data-access/sdk/api2';

@Component({
  selector: 'app-document-info',
  templateUrl: './document-info.component.html',
  styleUrls: ['./document-info.component.scss'],
})
export class DocumentInfoComponent implements OnInit, OnChanges {
  @Input() documentId: number;
  @Input() projectId: number;
  @Input() canCreate = true;
  @Input() canDelete = true;
  @Input() canUpdate = true;

  @Output() closeInfo: EventEmitter<any> = new EventEmitter<any>();
  @Output() updateDocumentList: EventEmitter<any> = new EventEmitter<any>();

  document: Document;

  constructor(
    private confirmationService: ConfirmationService,
    private messageService: MessageService,
    private dialogService: DialogService,
    private documentsService: DocumentsService,
    private translateService: TranslateService
  ) {}

  ngOnInit(): void {
    this.getDocument();
  }

  ngOnChanges(changes: SimpleChanges) {
    if (changes.documentId.currentValue !== changes.documentId.previousValue) {
      this.getDocument();
    }
  }

  getDocument() {
    firstValueFrom(
      this.documentsService.documentsDocumentsRead({
        id: this.documentId,
        include: 'files__author, metas__meta, projects, document_type',
      })
    ).then((doc) => (this.document = doc));
  }

  filesDropped(files: FileHandle[]): void {
    this.documentFileCreateOrEdit(null, files);
  }

  closeDocumentInfo(onDelete?) {
    this.closeInfo.emit({ documentId: this.documentId, onDelete: onDelete });
  }

  documentEdit() {
    const ref = this.dialogService.open(DocumentFormComponent, {
      header: this.translateService.instant('DOCUMENTS.FORM.HEADER_EDIT'),
      width: '60%',
      data: {
        projectId: this.projectId,
        document: this.document,
      },
    });

    firstValueFrom(ref.onClose).then(() => {
      this.getDocument();
      this.updateDocumentList.emit();
    });
  }

  documentFileCreateOrEdit(documentFile?, droppedFiles?) {
    const header = documentFile ? 'DOCUMENTS.FORM.HEADER_FILE_EDIT' : 'DOCUMENTS.FORM.HEADER_FILE_ADD';
    const ref = this.dialogService.open(DocumentFileFormComponent, {
      header: this.translateService.instant(header),
      width: '50%',
      data: {
        documentId: this.documentId,
        docFile: documentFile,
        files: droppedFiles,
      },
    });

    firstValueFrom(ref.onClose).then((_) => this.getDocument());
  }

  documentDelete() {
    this.confirmationService.confirm({
      message: `Вы действительно хотите удалить документ "${this.document.name}"?`,
      header: 'Пожалуйста, подтвердите ваши действия',
      accept: () => {
        this.documentsService
          .documentsDocumentsDelete(this.document.id)
          .toPromise()
          .then(() => {
            this.closeDocumentInfo(true);
            this.messageService.add({
              severity: 'success',
              detail: `Документ ${this.document.name} успешно удален`,
            });
          });
      },
      reject: () => {},
    });
  }
}
