import { Component, Input, OnInit } from '@angular/core';
import { firstValueFrom, forkJoin, Observable, of } from 'rxjs';
import { DocumentFormComponent } from '@core-project/modules/forms/document-forms/document-form/document-form.component';
import { FileHandle } from '@core-project/directives/file-drop.directive';
import { DialogService } from 'primeng/dynamicdialog';
import { TranslateService } from '@ngx-translate/core';
import * as _ from 'lodash-es';
import { switchMap } from 'rxjs/operators';
import { Document, DocumentMetaField, DocumentType } from 'data-access/sdk/api2';
import { DocumentsService } from 'data-access/sdk/api2';

interface Folder {
  value: number | string;
  name: string;
}

interface Folders {
  metaId: number;
  folders: Array<Folder>;
}

@Component({
  selector: 'app-documents-structure',
  templateUrl: './documents-structure.component.html',
  styleUrls: ['./documents-structure.component.scss'],
})
export class DocumentsStructureComponent implements OnInit {
  @Input() projectId: number = null;
  @Input() canCreate = true;
  @Input() canDelete = true;
  @Input() canUpdate = true;

  catSelected: number;
  typeSelected: number;
  docSelected: number;
  catList: Array<DocumentType>;
  docList: Array<Document>;

  folders: Folders;
  folderSelected: Folder;

  metaFields: Array<DocumentMetaField>;

  constructor(
    private documentsService: DocumentsService,
    private dialogService: DialogService,
    private translateService: TranslateService
  ) {}

  ngOnInit() {
    this.getDocumentTypes();
  }

  filesDropped(files: FileHandle[]): void {
    this.documentCreate(null, files);
  }

  getDocuments(metaId: number, value: number | string): Observable<Array<Document>> {
    let opts: DocumentsService.DocumentsDocumentsListParams;
    if (metaId) {
      opts = {
        documentTypeParentType: this.catSelected.toString(),
        metasMeta: metaId.toString(),
        metasContent: value.toString(),
        include: 'metas, projects, files',
      };
    } else {
      opts = {
        documentType: value.toString(),
        include: 'metas, projects, files',
      };
    }
    if (this.projectId) {
      return forkJoin([
        this.documentsService.documentsDocumentsList({
          projectsIsnull: 'true',
          ...opts,
        }),
        this.documentsService.documentsDocumentsList({
          projects: String(this.projectId),
          ...opts,
        }),
      ]).pipe(switchMap((res) => of(res[0].concat(res[1]))));
    } else {
      return this.documentsService.documentsDocumentsList(opts);
    }
  }

  selectParentType(category: DocumentType) {
    this.docList = null;
    this.catSelected = category.id;

    const folders: Folders = { metaId: null, folders: [] };

    if (category.group_by && category.rel_group_by.meta_type === 'CHS') {
      folders.metaId = category.group_by;
      category.rel_group_by.choices.forEach((ch) => folders.folders.push({ value: ch, name: ch }));
    } else {
      _.sortBy(category.rel_subtypes, 'order').forEach((st) => folders.folders.push({ value: st.id, name: st.name }));
    }

    this.folders = folders;
    this.folderSelected = null;
    this.typeSelected = null;
    if (folders.folders.length === 1) {
      this.selectChildrenType(folders.folders[0]);
    }
  }

  selectChildrenType(folder: Folder) {
    this.folderSelected = folder;
    if (!this.folders.metaId) {
      this.typeSelected = Number(folder.value);
      this.metaFields = this.catList
        .find((c) => c.id === this.catSelected)
        .rel_subtypes.find((st) => st.id === Number(folder.value)).rel_meta_fields;
    } else {
      this.typeSelected = null;

      const mfs = [];
      this.catList
        .find((c) => c.id === this.catSelected)
        .rel_subtypes.forEach((st) => {
          st.rel_meta_fields?.forEach((mf) => mfs.push(mf));
        });
      this.metaFields = _.uniqBy(mfs, 'id');
    }
    this.loadDocuments();
  }

  loadDocuments() {
    this.getDocuments(this.folders.metaId, this.folderSelected.value).subscribe((docs) => (this.docList = docs));
  }

  documentInfo(documentId) {
    this.docSelected = documentId;
  }

  async closedInfo(data) {
    if (data?.onDelete) {
      _.remove(this.docList, { id: data.documentId });
    }
    this.docSelected = null;
  }

  documentCreate(document?: Document, droppedFiles?) {
    const header = document ? 'DOCUMENTS.FORM.HEADER_EDIT' : 'DOCUMENTS.FORM.HEADER_ADD';

    const ref = this.dialogService.open(DocumentFormComponent, {
      header: this.translateService.instant(header),
      width: '80%',
      data: {
        projectId: this.projectId,
        selectedDocType: this.typeSelected,
        files: droppedFiles,
      },
    });

    firstValueFrom(ref.onClose).then((_r) => {
      this.loadDocuments();
    });
  }

  getMetaValue(document, metaId) {
    const metaValue = document.rel_metas.find((value) => value.meta === metaId);
    return metaValue?.content;
  }

  private getDocumentTypes() {
    this.documentsService
      .documentsTypesList({
        parentType: null,
        uncategorized: 'true',
        include: 'group_by, subtypes__meta_fields',
      })
      .subscribe((dts) => (this.catList = _.sortBy(dts, ['order'])));
  }
}
