// noinspection AngularMissingOrInvalidDeclarationInModule

import { Component, OnInit } from '@angular/core';
import { DialogService, DynamicDialogConfig } from 'primeng/dynamicdialog';
import { Issue, IssueDomain, User } from 'data-access/sdk/api2';
import { firstValueFrom, Observable, of } from 'rxjs';
import { HistoryService, IssuesService, UsersService } from 'data-access/sdk/api2';
import { IssueEnums, WorkEnums } from '@core-project/enums';
import { switchMap } from 'rxjs/operators';
import * as _ from 'lodash-es';
import { ActionRecord } from '@core-project/interfaces';
import { generateActions } from '@core-project/helpers';
import {TranslateModule, TranslateService} from '@ngx-translate/core';
import { MessageService } from 'primeng/api';
import { WorkFormComponent } from '@core-project/modules/shared-components/works/work-form/work-form.component';
import {SharedComponentsModule} from '../../shared-components.module';
import {FormsModule} from '@angular/forms';
import {InputTextModule} from 'primeng/inputtext';
import {CardModule} from 'primeng/card';

/** @deprecated */
@Component({
  selector: 'app-issue-card',
  templateUrl: './issue-card.component.html',
  styleUrls: ['./issue-card.component.scss'],
})
export class IssueCardComponent implements OnInit {
  private projectId: number;

  issue: Issue;
  issueDomains$: Observable<IssueDomain[]>;
  users$: Observable<User[]>;
  saveLoading = false;
  statusEnum = IssueEnums.StatusEnum;
  addUserForm = false;
  selectedUser: number;
  actions: ActionRecord[];
  newComment: string;
  actionsLoading = true;


  constructor(
    private config: DynamicDialogConfig,
    private issuesService: IssuesService,
    private usersService: UsersService,
    private dialogService: DialogService,
    private historyService: HistoryService,
    private translateService: TranslateService,
    private messageService: MessageService
  ) {}

  ngOnInit() {
    this.issue = this.config.data.parentIssueId;
    this.projectId = this.config.data.projectId;
    this.issueDomains$ = this.issuesService.issuesDomainsList();
    if (this.issue) {
      this.loadIssue(this.issue.id);
    } else {
      this.issue = {
        name: null,
        project: this.projectId,
        severity: 0,
        status: IssueEnums.StatusEnum.ACT,
        company: null,
        author: null,
      };
      this.users$ = this.usersService.usersList({});
    }
  }

  loadIssue(id: number) {
    firstValueFrom(
      this.issuesService.issuesIssuesRead({ id: id, include: 'domains, works__to_user, comments__author, issue_users__user' })
    ).then((issue) => {
      this.issue = issue;
      this.loadIssueActions();
      this.getUsersList();
    });
  }

  loadIssueActions() {
    this.actionsLoading = true;
    firstValueFrom(this.historyService.historyList({ contentTypeModel: 'issue', objectId: this.issue.id.toString() })).then((hs) => {
      this.actions = generateActions(this.issue.rel_comments, hs);
      this.actionsLoading = false;
    });
  }

  save() {
    if (!this.issue.name || this.issue.name === '') {
      this.messageService.add({
        severity: 'error',
        summary: this.translateService.instant('ERRORS.ERROR'),
        detail: this.translateService.instant('ISSUES.NO_NAME_ERROR_MSG'),
      });
      return;
    }
    this.saveLoading = true;
    let saveObs$: Observable<Issue>;
    if (this.issue.id) {
      saveObs$ = this.issuesService.issuesIssuesPartialUpdate({ id: this.issue.id, data: this.issue });
    } else {
      saveObs$ = this.issuesService.issuesIssuesCreate(this.issue);
    }
    firstValueFrom(saveObs$).then((newIssue) => {
      this.loadIssue(newIssue.id);
      this.saveLoading = false;
    });
  }

  changeStatus(status: IssueEnums.StatusEnum) {
    this.issue.status = status;
    this.save();
  }

  addWork() {
    const workRef = this.dialogService.open(WorkFormComponent, {
      header: this.translateService.instant('WORKS.ADD_ASSIGNMENT'),
      width: '50%',
      data: {
        work: null,
        projectId: this.issue.project,
        allowedTypes: [WorkEnums.WorkTypeEnum.AS],
      },
    });

    workRef.onClose.subscribe((newWork) => {
      if (!newWork) {
        return;
      }
      this.issue.works.push(newWork.id);
      this.issuesService.issuesIssuesPartialUpdate({ id: this.issue.id, data: this.issue }).subscribe((i) => this.loadIssue(i.id));
    });
  }

  addUser() {
    firstValueFrom(this.issuesService.issuesUsersCreate({ issue: this.issue.id, user: this.selectedUser })).then((iu) => {
      this.selectedUser = null;
      this.addUserForm = false;
      this.loadIssue(this.issue.id);
    });
  }

  getUsersList() {
    this.users$ = this.usersService.usersList({}).pipe(
      switchMap((uis) => {
        const addedUsers = _.flatMap(this.issue.rel_issue_users, (iu) => iu.user);
        return of(uis.filter((ui) => !addedUsers.includes(ui.id)));
      })
    );
  }

  closeAddUser() {
    this.selectedUser = null;
    this.addUserForm = false;
  }

  addComment() {
    if (this.newComment) {
      firstValueFrom(
        this.issuesService.issuesIssuesAddComment({
          id: this.issue.id,
          data: { text: this.newComment },
        })
      ).then((_c) => {
        this.loadIssue(this.issue.id);
      });
    }
  }
}
