import {ChangeDetectorRef, Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges} from '@angular/core';
import {Issue, IssueDomain, IssuesService, Project} from 'data-access/sdk/api2';
import {Observable} from 'rxjs';
import {IssueEnums} from '@core-project/enums';
import * as _ from 'lodash-es';
import {Actions, ofActionSuccessful, Store} from '@ngxs/store';
import {IssuesActions, IssuesSelectors} from 'data-access/store';
import {Unsubscriber} from 'utils/shared-services';
import {takeUntil} from 'rxjs/operators';

@Component({
  selector: 'app-issues-table',
  templateUrl: './issues-table.component.html',
  styleUrls: ['./issues-table.component.scss'],
  providers: [Unsubscriber],
})
export class IssuesTableComponent implements OnInit, OnChanges {
  @Input() issues: Issue[];
  @Input() project: Project;
  @Input() showProject = false;
  @Input() onlyActive: boolean;

  @Output() onIssueChange: EventEmitter<Issue> = new EventEmitter<Issue>();

  updateObserverSub = this.actions
    .pipe(
      takeUntil(this.unsubscriber.destroy$),
      ofActionSuccessful(IssuesActions.UpdateIssue, IssuesActions.CreateIssue),
    )
    .subscribe(() => {
      this.reloadIssue(this.store.selectSnapshot(IssuesSelectors.issue));
    });

  deleteObserverSub = this.actions
    .pipe(
      takeUntil(this.unsubscriber.destroy$),
      ofActionSuccessful(IssuesActions.DeleteIssue),
    )
    .subscribe(({id}) => {
      this.issues = this.issues.filter((issueResponse) => issueResponse.id !== id);
    });

  issueDomains$: Observable<IssueDomain[]>;
  issueStatus = IssueEnums.StatusEnum;
  projects: Project[];

  constructor(
    private readonly issuesService: IssuesService,
    private readonly store: Store,
    private readonly actions: Actions,
    private readonly unsubscriber: Unsubscriber,
    private readonly changeDetectorRef: ChangeDetectorRef,
  ) {
  }

  ngOnInit() {
    this.issueDomains$ = this.issuesService.issuesDomainsList();
  }

  ngOnChanges(changes: SimpleChanges) {
    const res: Project[] = _.flatMap(this.issues, (i) => i.rel_project);
    this.projects = _.uniqBy(res, 'id');
  }

  openIssue(issue: Issue) {
    this.store.dispatch(new IssuesActions.ShowIssueInSidebar(issue.id)).subscribe();
  }

  private reloadIssue(issue) {
    this.issuesService.issuesIssuesRead({id: issue.id, include: 'domains, works__to_user'})
      .subscribe((i) => {
        const index = this.issues.findIndex((issueResponse) => issueResponse.id === issue.id);
        this.issues[index] = i;
        this.changeDetectorRef.markForCheck();
      });
  }
}
