import { Component, OnInit, Input } from '@angular/core';
import { NewsService } from 'data-access/sdk/api2';
import { NewsEntry } from 'data-access/sdk/api2';
import { NewsFormComponent } from '@core-project/modules/forms/news-form/news-form.component';
import { ConfirmationService } from 'primeng/api';
import { TranslateService } from '@ngx-translate/core';
import { DialogService } from 'primeng/dynamicdialog';
import { UsersService } from 'data-access/sdk/api2';
import { AuthenticationService } from '@core-project/services/authentication.service';
import { User } from 'data-access/sdk/api2';

@Component({
  selector: 'app-newsfeed',
  templateUrl: './newsfeed.component.html',
  styleUrls: ['./newsfeed.component.scss'],
})
export class NewsfeedComponent implements OnInit {
  @Input() projectId: number;
  @Input() departmentId: number;

  newsList: NewsEntry[];
  perms: string[];
  user: User;

  constructor(
    private newsService: NewsService,
    private confirmationService: ConfirmationService,
    private translateService: TranslateService,
    private dialogService: DialogService,
    private userService: UsersService,
    private authenticationService: AuthenticationService
  ) {}

  ngOnInit(): void {
    this.loadNews();
    this.user = this.authenticationService.user;
    this.getPerms();
  }

  loadNews() {
    const newsParams: NewsService.NewsListParams = {
      include: 'author, project, department',
    };

    if (this.projectId) {
      newsParams.project = this.projectId.toString();
    }

    if (this.departmentId) {
      newsParams.department = this.departmentId.toString();
    }

    this.newsService.newsList(newsParams).subscribe((newslist) => {
      this.newsList = newslist.results;
    });
  }

  getPerms() {
    if (this.departmentId) {
      if (this.user.department === this.departmentId && this.user.department_manager) {
        this.perms = ['add_newsentry', 'change_newsentry', 'delete_newsentry', 'view_newsentry'];
      } else {
        this.perms = ['view_newsentry'];
      }
      return;
    }

    const permsParams: UsersService.UsersPermissionsParams = {
      id: this.authenticationService.user.id,
    };

    if (this.projectId) {
      permsParams.project = this.projectId;
    } else {
      permsParams.company = this.authenticationService.user.company;
    }
    this.userService.usersPermissions(permsParams).subscribe((userPermissions) => {
      this.perms = userPermissions.permissions;
    });
  }

  deleteNews(news: NewsEntry) {
    this.confirmationService.confirm({
      message: this.translateService.instant('NEWS.DELETE_HEADER'),
      header: this.translateService.instant('NEWS.DELETE_MSG'),
      accept: () =>
        this.newsService.newsDelete(news.id).subscribe(() => {
          this.newsList = this.newsList.filter((element) => element.id !== news.id);
        }),
    });
  }

  editNews(newsEntry?: NewsEntry) {
    const ref = this.dialogService.open(NewsFormComponent, {
      header: newsEntry ? this.translateService.instant('NEWS.FORM.EDIT_HEADER') : this.translateService.instant('NEWS.FORM.ADD_HEADER'),
      width: '40%',
      data: {
        newsEntry: newsEntry,
        projectId: this.projectId,
        departmentId: this.departmentId,
      },
    });

    ref.onClose.subscribe(() => this.loadNews());
  }
}
