import { Component, Input, OnInit, Signal, WritableSignal, computed, signal } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { Router } from '@angular/router';
import { Store } from '@ngxs/store';
import { forkJoin } from 'rxjs';

import { RightSidebarActions } from 'data-access/store';
import { Issue, IssuesService, Project, ProjectsService, ProjectStatusReport } from 'data-access/sdk/api2';
import { ProjectStageModel, ProjectsApiService } from 'data-access/sdk/internal-api';

import { IssueEnums } from '@core-project/enums';

@Component({
  selector: 'app-project-status-report',
  templateUrl: './project-status-report.component.html',
})
export class ProjectStatusReportComponent implements OnInit {
  private readonly _projectTimelineStages: WritableSignal<ProjectStageModel[]> = signal([]);
  private readonly _project: WritableSignal<Project | undefined> = signal(undefined);
  private readonly _issues: WritableSignal<Issue[]> = signal([]);

  @Input() projectStatusReport: ProjectStatusReport;

  readonly projectTimelineStages: Signal<ProjectStageModel[]> = this._projectTimelineStages.asReadonly();
  readonly project: Signal<Project> = this._project.asReadonly();
  readonly projectStageName: Signal<string | undefined> = computed(() => this.project()?.rel_stage?.name || undefined);
  readonly issues: Signal<Issue[]> = this._issues.asReadonly();

  constructor(
    private readonly projectsService: ProjectsService,
    private readonly issuesService: IssuesService,
    private readonly translateService: TranslateService,
    private readonly router: Router,
    private readonly store: Store,
    private readonly projectsApi: ProjectsApiService
  ) {}

  ngOnInit() {
    this.setHeader(this.projectStatusReport.name, this.translateService.instant('REPORTS.STATUS.OPEN_PROJECT'));
    this.readProjectInfo(this.projectStatusReport.id);
    this.readIssues(this.projectStatusReport.id.toString(), IssueEnums.StatusEnum.ACT);
  }

  private openProject() {
    const url = this.router.serializeUrl(this.router.createUrlTree(['/project', this.projectStatusReport.id, 'details']));
    window.open(url, '_blank');
  }

  private readIssues(projectId: string, status: string) {
    this.issuesService
      .issuesIssuesList({
        project: projectId,
        status,
      })
      .subscribe((issues) => {
        this._issues.set(issues);
      });
  }

  private readProjectInfo(projectId: number) {
    forkJoin([
      this.projectsService.projectsRead({ id: projectId, include: 'stage' }),
      this.projectsApi.readProjectTimeline(projectId),
    ]).subscribe(([project, projectStages]) => {
      this._project.set(project);
      this._projectTimelineStages.set(projectStages);
    });
  }

  private setHeader(projectName: string, tooltip: string) {
    this.store.dispatch(
      new RightSidebarActions.SetHeader(projectName, [
        {
          icon: 'fa-light fa-arrow-up-right-from-square',
          tooltip,
          action: () => this.openProject(),
        },
      ])
    );
  }
}
