import {Component, EventEmitter, OnInit, Output} from '@angular/core';
import {ControlValueAccessor, NG_VALUE_ACCESSOR} from '@angular/forms';
import {ProjectBase, ProjectGroup, ProjectGroupsService} from 'data-access/sdk/api2';
import {Observable, of} from 'rxjs';
import {TreeNode, TreeTableNode} from 'primeng/api';
import {switchMap} from 'rxjs/operators';
import {treeNodeFindByKey} from '@core-project/helpers/tree';
import {isArray} from 'lodash';

@Component({
  selector: 'app-project-tree-select',
  templateUrl: './project-tree-select.component.html',
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      multi: true,
      useExisting: ProjectTreeSelectComponent,
    },
  ],
})
export class ProjectTreeSelectComponent implements ControlValueAccessor, OnInit {
  @Output() onChange: EventEmitter<Number[]> = new EventEmitter<Number[]>();

  touched = false;
  disabled = false;

  selectedIds: Number[] = [];

  treeItems: Observable<TreeNode[]>;
  selectedNodes: TreeTableNode | TreeTableNode[] = [];

  constructor(private projectGroupsService: ProjectGroupsService) {
  }

  ngOnInit() {
    this.treeItems = this.projectGroupsService.projectGroupsList({include: 'projects__stage'}).pipe(
      switchMap((pgs) => {
        const items: TreeNode[] = [];
        pgs.forEach((pg) => {
          const pgNode = this.pgToNode(pg);
          if (pg.level === 0) {
            pgNode.expanded = true;
            items.push(pgNode);
          } else {
            const parentNode = treeNodeFindByKey(items, this.pgKey(pg.parent_group));
            if (parentNode) {
              parentNode.children.unshift(pgNode);
            }
          }
        });
        return of(items);
      }),
    );
  }

  onChangeFn = (indicator) => {
  };
  onTouchedFn = () => {
  };

  writeValue(ids: Number[]) {
    this.selectedIds = ids;
  }

  registerOnChange(onChange: any) {
    this.onChangeFn = onChange;
  }

  registerOnTouched(onTouched: any) {
    this.onTouchedFn = onTouched;
  }

  markAsTouched() {
    if (!this.touched) {
      this.onTouchedFn();
      this.touched = true;
    }
  }

  setDisabledState(disabled: boolean) {
    this.disabled = disabled;
  }

  nodeSelect() {
    this.selectedIds = isArray(this.selectedNodes)
      ? this.selectedNodes.filter((node) => !node.leaf).map((node) => Number(node.key))
      : [Number(this.selectedNodes.key)];
    this.onChangeFn(this.selectedIds);
    this.onChange.emit(this.selectedIds);
  }

  private pgToNode(pg: ProjectGroup): TreeNode {
    return {
      key: this.pgKey(pg.id),
      label: pg.name,
      data: pg,
      children: pg.rel_projects.map((p) => this.projectToNode(p)),
      leaf: true,
    };
  }

  private projectToNode(p: ProjectBase): TreeNode {
    const projectNode: TreeNode = {
      key: p.id.toString(),
      label: p.name,
      data: p,
    };
    if (this.selectedIds && this.selectedIds.includes(p.id)) {
      if (isArray(this.selectedNodes)) {
        this.selectedNodes.push(projectNode);
      } else {
        this.selectedNodes = [projectNode];
      }
    }
    return projectNode;
  }

  private pgKey(pgId: number) {
    return 'pg_' + pgId.toString();
  }
}
