import {
  Component,
  ElementRef,
  Inject,
  Input,
  LOCALE_ID,
  OnChanges,
  OnDestroy,
  OnInit,
  Renderer2,
  SimpleChanges,
  ViewChild,
  ViewContainerRef,
  ViewEncapsulation,
} from '@angular/core';
import {switchMap, takeUntil} from 'rxjs/operators';
import {defaultIfEmpty, firstValueFrom, forkJoin} from 'rxjs';
import {TranslateService} from '@ngx-translate/core';
import {DialogService} from 'primeng/dynamicdialog';
import {MessageService} from 'primeng/api';
import {formatDate} from '@angular/common';

import * as moment from 'moment';
import {Gantt, GanttStatic} from '../../../../../assets/dhtmlxgantt/dhtmlxgantt';
import {
  GanttToPWorkAdapter,
  GanttToUWorkAdapter,
  GanttToWorkAdapter,
  GanttToWorkLinkAdapter,
  GanttWork,
  GanttWorkLink,
  PWorkToGanttAdapter,
  UWorkToGanttAdapter,
  WorkLinkToGanttAdapter,
  WorkToGanttAdapter,
} from '@core-project/helpers/gantt-adapter';
import {defaultScales, getGanttTreeIcon} from '@core-project/helpers/gantt-common';

import {Project, Work, WorklinksService, WorksService} from 'data-access/sdk/api2';
import {dbStringToJsDate, dbStringToMoment} from '@core-project/helpers';
import {DialogEnums, WorkEnums} from '@core-project/enums';
import {WorklinkInfoComponent} from '@core-project/modules/shared-components/works/worklink-info/worklink-info.component';
import {WorkFormComponent} from '@core-project/modules/shared-components/works/work-form/work-form.component';
import {WorkStatusComponent} from '@core-project/modules/shared-components/works/work-status/work-status.component';
import {Actions, ofActionSuccessful, Store} from '@ngxs/store';
import {WorksActions} from 'data-access/store';
import {Unsubscriber} from 'utils/shared-services';

@Component({
  selector: 'app-gantt-chart',
  templateUrl: './gantt-chart.component.html',
  styleUrls: ['../../../../../assets/dhtmlxgantt/dhtmlxgantt.css', './gantt-chart.component.scss'],
  encapsulation: ViewEncapsulation.None,
  providers: [Unsubscriber],
})
export class GanttChartComponent implements OnInit, OnChanges, OnDestroy {
  private scales = defaultScales;
  private gi: GanttStatic;

  private readonly deleteObserverSub = this.actions
    .pipe(
      takeUntil(this.unsubscriber.destroy$),
      ofActionSuccessful(WorksActions.DeleteWork),
    )
    .subscribe(({id}) => {
       this.gi.deleteTask(id);
    });

  private readonly updateObserverSub = this.actions
    .pipe(
      takeUntil(this.unsubscriber.destroy$),
      ofActionSuccessful(WorksActions.UpdateWork),
    )
    .subscribe(({work}) => {
       firstValueFrom(this.readWork(work.id)).then((w) => this.updateGanttWork(w));
    });

  @ViewChild('ganttChart') ganttContainer: ElementRef;

  @Input() ganttWorks: GanttWork[];
  @Input() ganttLinks: GanttWorkLink[];
  @Input() project: Project;
  @Input() readonly = false;
  @Input() canCreateLinks = true;
  @Input() viewMode: WorkEnums.GanttViewModeEnum = WorkEnums.GanttViewModeEnum.CURR;
  @Input() scale: WorkEnums.GanttScaleEnum = WorkEnums.GanttScaleEnum.WEEK;
  @Input() autoschedule = false;
  @Input() criticalPath = false;
  @Input() mode: WorkEnums.GanttModeEnum = WorkEnums.GanttModeEnum.PROJECT;
  @Input() chartStart: string;
  @Input() chartEnd: string;


  taskTopMargin = '0px';
  arrowTopMargin = '-3px';
  projectDisplay = 'block';

  constructor(
    @Inject(LOCALE_ID) private locale: string,
    private translateService: TranslateService,
    private dialogService: DialogService,
    private worksService: WorksService,
    private workLinksService: WorklinksService,
    private messageService: MessageService,
    private renderer: Renderer2,
    private viewContainerRef: ViewContainerRef,
    private readonly store: Store,
    private readonly actions: Actions,
    private readonly unsubscriber: Unsubscriber,
  ) {
  }

  private get wtgAdapter() {
    switch (this.mode) {
      case WorkEnums.GanttModeEnum.PROJECT:
        return WorkToGanttAdapter;
      case WorkEnums.GanttModeEnum.DEPARTMENT:
        return UWorkToGanttAdapter;
      case WorkEnums.GanttModeEnum.ROADMAP:
        return PWorkToGanttAdapter;
      default:
        return WorkToGanttAdapter;
    }
  }

  private get gtwAdapter() {
    switch (this.mode) {
      case WorkEnums.GanttModeEnum.PROJECT:
        return GanttToWorkAdapter;
      case WorkEnums.GanttModeEnum.DEPARTMENT:
        return GanttToUWorkAdapter;
      case WorkEnums.GanttModeEnum.ROADMAP:
        return GanttToPWorkAdapter;
      default:
        return GanttToWorkAdapter;
    }
  }

  ngOnInit() {
    this.gi = Gantt.getGanttInstance({
      config: {
        date_format: '%Y-%m-%d %H:%i',
        auto_scheduling: this.autoschedule,
        auto_types: true,
        autoscroll: true,
        autosize: true,
        drag_links: !this.readonly && this.canCreateLinks,
        drag_move: !this.readonly,
        drag_mode: !this.readonly ? {resize: 'resize', progress: 'progress', move: 'move', ignore: 'ignore'} : 'ignore',
        drag_progress: false,
        highlight_critical_path: this.criticalPath,
        open_split_tasks: true,
        order_branch: !this.readonly,
        order_branch_free: !this.readonly,
        placeholder_task: false,
        preserve_scroll: true,
        readonly: this.readonly,
        show_markers: true,
        show_tasks_outside_timescale: true,
        scales: this.scales[this.scale],
        scale_height: 54,
        tooltip_timeout: 200,

        external_render: {
          isElement: (element) => !!element,
          renderElement: (element, container) => {
            this.renderer.appendChild(container, element);
          },
        },
      },
      plugins: {
        auto_scheduling: true,
        critical_path: true,
        marker: true,
        drag_timeline: true,
      },
      templates: {
        grid_date_format: (date, _column) => formatDate(date, 'shortDate', this.locale),

        tooltip_date_format: (date) => formatDate(date, 'shortDate', this.locale),

        task_end_date: (date) => this.gi.templates.task_date(new Date(date.valueOf() - 1)),

        grid_folder: (item) => {
          switch (item.objType) {
            case WorkEnums.GanttNodeEnum.DEPARTMENT:
              return getGanttTreeIcon('fa-regular fa-users');
            case WorkEnums.GanttNodeEnum.USER:
              return getGanttTreeIcon('fa-regular fa-user');
            case WorkEnums.GanttNodeEnum.PROJECT:
              return getGanttTreeIcon('fa-regular fa-rocket-launch');
            case WorkEnums.GanttNodeEnum.PROJECT_GROUP:
              return getGanttTreeIcon('fa-regular fa-diagram-project');
          }
          return getGanttTreeIcon('fa-regular fa-folder' + (item.$open ? '-open' : ''));
        },

        grid_file: (item) => {
          switch (item.objType) {
            case WorkEnums.GanttNodeEnum.DEPARTMENT:
              return getGanttTreeIcon('fa-regular fa-users');
            case WorkEnums.GanttNodeEnum.USER:
              return getGanttTreeIcon('fa-regular fa-user');
            case WorkEnums.GanttNodeEnum.PROJECT:
              return getGanttTreeIcon('fa-regular fa-rocket-launch');
            case WorkEnums.GanttNodeEnum.PROJECT_GROUP:
              return getGanttTreeIcon('fa-regular fa-diagram-project');
          }
          if (item.type === 'milestone') {
            return getGanttTreeIcon('fa-regular fa-flag-swallowtail');
          }
          return getGanttTreeIcon('fa-regular fa-file');
        },
      },
    });
    this.configureGantt();

    if (this.ganttWorks && this.ganttLinks) {
      this.loadData();
    }
  }

  ngOnChanges(changes: SimpleChanges) {
    if (this.gi && (changes.ganttWorks || changes.ganttLinks)) {
      if (this.ganttWorks && this.ganttLinks) {
        this.loadData();
      }
    }

    if (changes.scale && this.gi) {
      this.setScale();
    }

    if (changes.viewMode && this.gi) {
      this.setViewMode();
    }

    if (changes.autoschedule && this.gi) {
      this.setAutoschedule();
    }

    if ((changes.chartStart || changes.startEnd) && this.gi) {
      this.changeRange();
    }
  }

  ngOnDestroy() {
    this.gi.destructor();
  }

  private loadData() {
    this.gi.clearAll();
    this.gi.parse({tasks: this.ganttWorks, links: this.ganttLinks});
    this.initGantt();
    this.initMarkers();
  }

  private configureGantt() {
    this.gi.config.columns = this.getColumns();
    this.gi.i18n.setLocale(this.locale);

    this.changeRange();

    if (this.viewMode === 'base') {
      this.gi.config.bar_height = 16;
      this.gi.config.row_height = 40;
    }

    if (!this.readonly) {
      this.gi.attachEvent(
        'onTaskClick',
        (id, _e) => {
          if (this.gi.getTask(id).type !== 'project') {
            this.openWork(id);
          }
          return false;
        },
        null,
      );

      this.gi.attachEvent(
        'onBeforeLightbox',
        (id) => {
          this.openWork(id);
          return false;
        },
        null,
      );

      this.gi.attachEvent(
        'onBeforeTaskDrag',
        (id, mode, _e) => {
          const w = this.gi.getTask(id).work;
          if (!w) {
            return false;
          }

          if (mode === 'move') {
            if (w.status === WorkEnums.StatusEnum.CR || w.status === WorkEnums.StatusEnum.BL) {
              return true;
            }
            this.messageService.add({
              severity: 'error',
              summary: this.translateService.instant('ERRORS.ERROR'),
              detail: this.translateService.instant('GANTT.ERR_MOVE_STARTED_TASK_MSG'),
            });
            return false;
          }

          if (mode === 'resize') {
            if (w.status === WorkEnums.StatusEnum.CR || w.status === WorkEnums.StatusEnum.WP || w.status === WorkEnums.StatusEnum.BL) {
              return true;
            }
            this.messageService.add({
              severity: 'error',
              summary: this.translateService.instant('ERRORS.ERROR'),
              detail: this.translateService.instant('GANTT.ERR_RESIZE_ENDED_TASK_MSG'),
            });
            return false;
          }

          return true;
        },
        null,
      );

      this.gi.attachEvent(
        'onTaskDrag',
        (id, mode, task, original) => {
          if (mode !== 'resize') {
            return true;
          }
          if (task.start_date !== original.start_date && task.work.start_fact) {
            task.start_date = original.start_date;
          }
        },
        null,
      );

      this.gi.attachEvent(
        'onAfterTaskUpdate',
        (id, task) => {
          this.updateWork(task);
        },
        null,
      );

      this.gi.attachEvent(
        'onRowDragEnd',
        (id, target) => {
          const task = this.gi.getTask(id);
          this.updateWork(task);
          if (this.mode === WorkEnums.GanttModeEnum.PROJECT) {
            const children = this.gi.getChildren(task.parentWorkId);
            const updates$ = [];
            children.forEach((c) => {
              if (c === id) {
                return;
              }
              const tsk = this.gi.getTask(c);
              const index = tsk.$local_index;
              if (index !== tsk.order) {
                // @ts-ignore
                updates$.push(this.worksService.worksPartialUpdate({id: tsk.id, data: {order: index}}));
              }
            });
            firstValueFrom(forkJoin(updates$).pipe(defaultIfEmpty(null))).then();
          }
        },
        null,
      );

      this.gi.attachEvent(
        'onAfterLinkAdd',
        (id, item) => {
          firstValueFrom(this.workLinksService.worklinksCreate(GanttToWorkLinkAdapter(item))).then((wl) => this.gi.changeLinkId(id, wl.id));
        },
        null,
      );

      this.gi.attachEvent(
        'onAfterLinkUpdate',
        (id, item) => {
          firstValueFrom(this.workLinksService.worklinksPartialUpdate({id: id, data: GanttToWorkLinkAdapter(item)})).then();
        },
        null,
      );

      this.gi.attachEvent(
        'onLinkClick',
        (linkId, _e) => {
          const ref = this.dialogService.open(WorklinkInfoComponent, {
            header: this.translateService.instant('GANTT.EDIT_LINK'),
            width: '40%',
            data: {
              linkId: linkId,
            },
          });

          firstValueFrom(ref.onClose).then((action) => {
            if (action === DialogEnums.ActionEnum.DEL) {
              this.gi.deleteLink(linkId);
            } else {
              firstValueFrom(this.workLinksService.worklinksRead({id: linkId})).then((wl) => {
                const ganttLink = this.gi.getLink(linkId);
                Object.assign(ganttLink, WorkLinkToGanttAdapter(wl));
                this.gi.refreshLink(linkId);
              });
            }
          });
          return false;
        },
        null,
      );

      this.gi.attachEvent(
        'onBeforeTaskAutoSchedule',
        (task, _start, _link, _predecessor) => task.work?.status === WorkEnums.StatusEnum.CR,
        null,
      );

      if (this.mode === WorkEnums.GanttModeEnum.DEPARTMENT) {
        this.gi.attachEvent('onBeforeTaskMove', (id, parent, _tindex) => typeof parent === 'string', null);

        this.gi.templates.grid_row_class = (start, end, task) => {
          if (task.type !== 'project') {
            return 'no_add';
          }
          return '';
        };

        this.gi.templates.task_class = (start, end, task) => {
          if (!task.work?.approved) {
            return 'task-unapproved';
          }
          return '';
        };

        this.gi.config.auto_types = false;

        this.projectDisplay = 'none';
      }

      if (this.mode === WorkEnums.GanttModeEnum.ROADMAP) {
        this.gi.templates.task_class = (start, end, task) => {
          if (!task.work?.approved) {
            return 'task-unapproved';
          }
          return '';
        };
        this.gi.config.auto_types = false;
        this.projectDisplay = 'none';
      }
    } else {
      // force empty event
      this.gi.attachEvent(
        'onTaskClick',
        (id, _e) => {
          if (this.gi.getTask(id).type !== 'project') {
            this.openWork(id);
          }
          return false;
        },
        null,
      );
    }
  }

  private openWork(workId: string) {
    const work = this.gi.getTask(workId).work;
    if (!work) {
      this.addWork(workId, false);
      return;
    }
    this.gi.selectTask(workId);
    this.store.dispatch(new WorksActions.ShowWorkInSidebar(work.id)).subscribe();
  }

  private addWork(workId: string, fromRow = false) {
    const ganttWork = this.gi.getTask(workId);
    const work = this.gtwAdapter(ganttWork);
    work.work_type = this.mode === WorkEnums.GanttModeEnum.DEPARTMENT ? WorkEnums.WorkTypeEnum.AS : WorkEnums.WorkTypeEnum.TS;

    const ref = this.dialogService.open(WorkFormComponent, {
      header: this.translateService.instant('WORKS.ADD_WORK'),
      width: '50%',
      data: {
        work: work,
        projectId: this.project?.id,
        allowedTypes:
          this.mode === WorkEnums.GanttModeEnum.DEPARTMENT
            ? [WorkEnums.WorkTypeEnum.AS]
            : [WorkEnums.WorkTypeEnum.MS, WorkEnums.WorkTypeEnum.TS],
      },
    });

    firstValueFrom(ref.onClose).then((w) => {
      if (w) {
        this.gi.changeTaskId(workId, w.id);
        firstValueFrom(this.readWork(w.id)).then((ww) => this.updateGanttWork(ww, ganttWork));
      } else {
        if (!fromRow) {
          this.gi.deleteTask(workId);
        }
      }
      this.gi.setSizes();
    });
  }

  private updateGanttWork(work: Work, ganttWork: GanttWork = null) {
    if (!ganttWork) {
      ganttWork = this.gi.getTask(work.id.toString());
    }

    if (!ganttWork) {
      return;
    }
    Object.assign(ganttWork, this.wtgAdapter(work));
    ganttWork.work = work;
    this.gi.refreshTask(ganttWork.id);
    if (work.group) {
      const gr = this.gi.getTask(work.group.toString());
      this.updateGroupDates(gr);
    }
  }

  private updateGroupDates(group: GanttWork) {
    this.gi.resetProjectDates(group);
    this.gi.refreshTask(group.id);
    if (group.parent) {
      this.updateGroupDates(this.gi.getTask(group.parent.toString()));
    }
  }

  private setScale() {
    this.gi.config.scales = this.scales[this.scale];
    this.initGantt();
  }

  private setViewMode() {
    if (this.viewMode === WorkEnums.GanttViewModeEnum.BASE) {
      this.gi.config.bar_height = 16;
      this.gi.config.row_height = 40;
      this.taskTopMargin = '-9px';
      this.arrowTopMargin = '-12px';
    } else {
      this.gi.config.bar_height = 'full';
      this.gi.config.row_height = 35;
      this.taskTopMargin = '0px';
      this.arrowTopMargin = '-3px';
    }

    this.initGantt();
  }

  private initGantt() {
    if (this.ganttContainer) {
      this.gi.init(this.ganttContainer.nativeElement);
      // this.gi.renderMarkers();

      this.gi.showDate(new Date());

      if (this.viewMode === WorkEnums.GanttViewModeEnum.BASE) {
        this.gi.addTaskLayer((task) => {
          if (task.type === 'project') {
            return false;
          }

          if (task.type === 'milestone' && task.start_plan) {
            const sizes = this.gi.getTaskPosition(task, task.start_plan, task.end_plan);
            const el = document.createElement('div');
            el.className = 'baseline-milestone';
            el.style.left = sizes.left + 'px';
            el.style.top = sizes.top + this.gi.config.bar_height + 13 + 'px';
            return el;
          }

          if (task.start_plan && task.end_plan) {
            const sizes = this.gi.getTaskPosition(task, task.start_plan, task.end_plan);
            const el = document.createElement('div');
            el.className = 'baseline';
            el.style.left = sizes.left + 'px';
            el.style.width = sizes.width + 'px';
            el.style.top = sizes.top + this.gi.config.bar_height + 13 + 'px';
            return el;
          }
          return false;
        });
      }
    }
  }

  private setAutoschedule() {
    this.gi.config.auto_scheduling = this.autoschedule;
    if (this.autoschedule) {
      this.gi.autoSchedule();
    }
  }

  private updateWork(ganttWork: GanttWork) {
    firstValueFrom(
      this.worksService
        .worksPartialUpdate({
          id: Number(ganttWork.id),
          data: this.gtwAdapter(ganttWork),
        })
        .pipe(switchMap((w) => this.readWork(w.id))),
    ).then((w) => this.updateGanttWork(w, ganttWork));
  }

  private readWork(id: number) {
    return this.worksService.worksRead({id: id, include: 'to_user, to_department'});
  }

  private initMarkers() {
    const now = moment();
    this.gi.addMarker({
      start_date: now.toDate(),
      css: 'today-marker',
      text: this.translateService.instant('GANTT.MARKERS.TODAY'),
      title: 'GANTT.MARKERS.TODAY ' + now.format('L'),
    });
    // if (this.mode === WorkEnums.GanttModeEnum.PROJECT) {
    //   if (this.project?.start_date) {
    //     const st = moment(this.project.start_date, 'YYYY-MM-DD');
    //     this.gi.addMarker({
    //       start_date: st.toDate(),
    //       css: 'project-start-marker',
    //       text: this.ts.instant('GANTT.MARKERS.PROJECT_START'),
    //       title: 'GANTT.MARKERS.PROJECT_START ' + st.format('L'),
    //     });
    //   }
    //   if (this.project?.end_date) {
    //     const en = moment(this.project.end_date, 'YYYY-MM-DD');
    //     this.gi.addMarker({
    //       start_date: en.toDate(),
    //       css: 'project-end-marker',
    //       text: this.ts.instant('GANTT.MARKERS.PROJECT_END'),
    //       title: 'GANTT.MARKERS.PROJECT_END ' + en.format('L'),
    //     });
    //   }
    // }
  }

  private getColumns() {
    const textEditor = {type: 'text', map_to: 'text'};
    const dateEditor = {type: 'date', map_to: 'start_date'};
    const durationEditor = {type: 'number', map_to: 'duration', min: 0};
    const columns: any[] = [];

    switch (this.mode) {
      case WorkEnums.GanttModeEnum.DEPARTMENT:
        columns.push(
          ...[
            {name: 'wbs', label: '#', width: 60, align: 'center', template: this.gi.getWBSCode},
            {
              name: 'status',
              label: '',
              width: 30,
              align: 'left',
              resize: false,
              onrender: (task, _node) => {
                if (task.type === 'project') {
                  return null;
                }
                const statusRef = this.viewContainerRef.createComponent<WorkStatusComponent>(WorkStatusComponent);
                statusRef.instance.status = task.work?.status;
                statusRef.instance.showLabel = false;
                return statusRef.instance.elementRef.nativeElement;
              },
            },
            {name: 'text', label: this.translateService.instant('GANTT.COLUMNS.TASK'), tree: true, width: 300, resize: true},
            {
              name: 'start_date',
              label: this.translateService.instant('GANTT.COLUMNS.START'),
              width: 80,
              align: 'center',
              resize: true,
              template: (w) => {
                if (w.objType === WorkEnums.GanttNodeEnum.WORK) {
                  return w.start_date;
                }
                return null;
              },
            },
            {
              name: 'end_date',
              label: this.translateService.instant('GANTT.COLUMNS.END'),
              width: 80,
              align: 'center',
              resize: true,
              template: (w) => {
                if (w.objType === WorkEnums.GanttNodeEnum.WORK) {
                  return w.end_date;
                }
                return null;
              },
            },
          ],
        );

        if (!this.readonly) {
          columns.push({name: 'add'});
        } else {
          columns.forEach((column) => {
            if (column.editor) {
              delete column.editor;
            }
          });
        }

        return columns;
      case WorkEnums.GanttModeEnum.PROJECT:
        columns.push(
          ...[
            {name: 'wbs', label: '#', width: 60, align: 'center', template: this.gi.getWBSCode},
            {
              name: 'status',
              label: '',
              width: 30,
              align: 'left',
              resize: false,
              onrender: (task, _node) => {
                if (task.type === 'project') {
                  return null;
                }
                const statusRef = this.viewContainerRef.createComponent<WorkStatusComponent>(WorkStatusComponent);
                statusRef.instance.status = task.work?.status;
                statusRef.instance.showLabel = false;
                return statusRef.instance.elementRef.nativeElement;
              },
            },
            {
              name: 'text',
              label: this.translateService.instant('GANTT.COLUMNS.TASK'),
              tree: true,
              width: 200,
              resize: true,
              editor: textEditor,
            },
            {
              name: 'to_user',
              label: this.translateService.instant('GANTT.COLUMNS.RESPONSIBLE'),
              width: 80,
              resize: true,
              template: (w) => {
                if (w.type === 'project') {
                  return null;
                }
                return w.work?.rel_to_user?.short_name || w.work?.rel_to_department?.name;
              },
            },
            {
              name: 'start_date',
              label: this.translateService.instant('GANTT.COLUMNS.START'),
              width: 80,
              align: 'center',
              resize: true,
              editor: dateEditor,
            },
            {
              name: 'duration',
              label: this.translateService.instant('GANTT.COLUMNS.DURATION'),
              width: 80,
              align: 'center',
              resize: true,
              editor: durationEditor,
            },
          ],
        );

        if (!this.readonly) {
          columns.push({name: 'add'});
        } else {
          columns.forEach((column) => {
            if (column.editor) {
              delete column.editor;
            }
          });
        }

        return columns;
      case WorkEnums.GanttModeEnum.ROADMAP:
        return [
          {name: 'wbs', label: '#', width: 60, align: 'center', template: this.gi.getWBSCode},
          {name: 'text', label: this.translateService.instant('GANTT.COLUMNS.TASK'), tree: true, width: 300, resize: true},
          {
            name: 'start_plan',
            label: this.translateService.instant('GANTT.COLUMNS.DATE_PLAN'),
            width: 80,
            align: 'center',
            resize: true,
            template: (w) => {
              if (w.objType === WorkEnums.GanttNodeEnum.WORK) {
                return w.start_plan;
              }
              return null;
            },
          },
          {
            name: 'start_date',
            label: this.translateService.instant('GANTT.COLUMNS.DATE_FORECAST'),
            width: 80,
            align: 'center',
            resize: true,
            template: (w) => {
              if (w.objType === WorkEnums.GanttNodeEnum.WORK) {
                return w.start_date;
              }
              return null;
            },
          },
        ];
    }
  }

  private changeRange() {
    if (this.chartStart && this.chartEnd) {
      this.gi.config.start_date = dbStringToJsDate(this.chartStart);
      this.gi.config.end_date = dbStringToMoment(this.chartEnd).add(1, 'day').toDate();
    }
  }
}
