import {Component, EventEmitter, Input, OnChanges, Output, SimpleChanges} from '@angular/core';
import {DialogService} from 'primeng/dynamicdialog';
import {TranslateService} from '@ngx-translate/core';
import {WorkEnums} from '@core-project/enums';
import {Work, WorkBase, WorksService} from 'data-access/sdk/api2';
import {firstValueFrom} from 'rxjs';
import * as _ from 'lodash-es';
import {checkEnumValue} from '@core-project/helpers';
import {Store} from '@ngxs/store';
import {WorksActions} from 'data-access/store';
import StatusEnum = WorkEnums.StatusEnum;

@Component({
  selector: 'app-kanban-board',
  templateUrl: './kanban-board.component.html',
  styleUrls: ['./kanban-board.component.scss'],
})
export class KanbanBoardComponent implements OnChanges {
  @Input() works: Array<WorkBase>;
  @Input() projectId: number;
  @Input() dropDisabled = false;

  @Output() update: EventEmitter<any> = new EventEmitter<any>();

  swimlaneNames: Array<StatusEnum> = [StatusEnum.CR, StatusEnum.WP, StatusEnum.DN, StatusEnum.AC];
  swimlaneExpanded = {CR: true, WP: true, DN: true, AC: true};

  swimlanes: { [s: string]: Array<Work> };
  loading = true;

  draggedWork: Work;
  draggedWorkColumn: StatusEnum;
  columnsToDrop: Array<string> = [];
  draggedOverColumn: string;

  constructor(
    private worksService: WorksService,
    private dialogService: DialogService,
    private translateService: TranslateService,
    private readonly store: Store,
  ) {
  }

  ngOnChanges(changes: SimpleChanges) {
    if (this.works) {
      if (this.swimlanes) {
        this.separateToLanes(this.works);
      } else {
        this.loading = true;
        this.separateToLanes(this.works);
        this.loading = false;
      }
    }
  }

  toggleExpand(column) {
    this.swimlaneExpanded[column] = !this.swimlaneExpanded[column];
  }

  dragStart(work: Work, column: StatusEnum) {
    this.draggedWork = work;
    this.draggedWorkColumn = column;
    this.columnsToDrop = ['INPROGRESS', 'INREVIEW'];
  }

  dragEnd() {
    this.draggedWork = null;
    this.draggedWorkColumn = null;
    this.columnsToDrop = [];
    this.draggedOverColumn = null;
  }

  dropWork(column) {
    if (column === this.draggedWorkColumn) {
      return;
    }

    const oldStatus = this.draggedWork.status;
    const oldColumn = this.draggedWorkColumn;

    firstValueFrom(this.worksService.worksChangeStatus({id: this.draggedWork.id, data: {status: column}})).then((newWork) => {
      if (oldStatus !== newWork.status) {
        this.changeColumn(newWork, column, oldColumn);
      }
    });
  }

  changeColumn(work: Work, newStatus: StatusEnum, oldStatus: StatusEnum) {
    if (this.swimlaneNames.includes(newStatus)) {
      this.swimlanes[newStatus].push(work);
      this.swimlanes[newStatus] = [...this.swimlanes[newStatus]];
    }

    this.swimlanes[oldStatus] = this.swimlanes[oldStatus].filter((w) => w.id !== work.id);
  }

  dragEnter(event, column) {
    this.draggedOverColumn = column;
  }

  dragLeave(event, column) {
    // console.log('DRAG LEAVE', event, column);
    // this.draggedOverColumn = null;
  }

  workInfo(work: Work) {
    this.store.dispatch(new WorksActions.ShowWorkInSidebar(work.id)).subscribe();
    // if (r === DialogEnums.ActionEnum.DEL) {
    //   this.removeWork(work);
    // } else {
    //   this.updateWork(work);
    // }
  }

  updateWork(work: Work) {
    firstValueFrom(this.worksService.worksRead({id: work.id, include: 'to_user'})).then((firstWork) => {
      if (work.status !== firstWork.status) {
        this.changeColumn(work, checkEnumValue(WorkEnums.StatusEnum, firstWork.status), checkEnumValue(WorkEnums.StatusEnum, work.status));
      }
      Object.assign(work, firstWork);
    });
  }

  removeWork(work: Work) {
    _.remove(this.swimlanes[work.status], ['id', work.id]);
  }

  private separateToLanes(works: Array<WorkBase>) {
    this.swimlanes = _.groupBy(works, 'status') as any;
    this.swimlaneNames.forEach((sln) => {
      if (this.swimlanes[sln] === undefined) {
        this.swimlanes[sln] = [];
      }
    });
  }
}
