import { Component, OnInit } from '@angular/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { TranslateService } from '@ngx-translate/core';
import * as moment from 'moment';
import { AuthenticationService } from '@core-project/services/authentication.service';
import { firstValueFrom, forkJoin, Observable } from 'rxjs';
import { Department, ProjectTypeStage, User, Work } from 'data-access/sdk/api2';
import { WorkEnums } from '@core-project/enums';
import { DepartmentsService, ProjectTypeStagesService, UsersService, WorksService } from 'data-access/sdk/api2';
import * as _ from 'lodash-es';
import { SelectItem, SelectItemGroup } from 'primeng/api';
import { ProjectEnums } from '@core-project/enums/project';
import StageEnum = ProjectEnums.StageEnum;

/** @deprecated */
@Component({
  selector: 'app-work-form',
  templateUrl: './work-form.component.html',
  styleUrls: ['./work-form.component.scss'],
})
export class WorkFormComponent implements OnInit {
  work: Work;
  workForm: FormGroup;
  workTypeChoices = [];
  showOptionalValues = false;
  user: User;
  userList$: Observable<Array<User>>;
  departmentList$: Observable<Array<Department>>;
  objChoices: SelectItemGroup[];
  workTypes = WorkEnums.WorkTypeEnum;
  ptStages$: Observable<ProjectTypeStage[]>;

  private projectId: number;
  private allowedTypes: Array<WorkEnums.WorkTypeEnum>;
  private userSourceMode: string;

  constructor(
    private readonly dialogRef: DynamicDialogRef,
    private readonly dialogConfig: DynamicDialogConfig,
    private readonly formBuilder: FormBuilder,
    private readonly translateService: TranslateService,
    private readonly authenticationService: AuthenticationService,
    private readonly usersService: UsersService,
    private readonly departmentsService: DepartmentsService,
    private readonly worksService: WorksService,
    private readonly projectTypeStagesService: ProjectTypeStagesService
  ) {}

  get workType() {
    return this.workForm.controls.work_type.value;
  }

  get hasProject() {
    return !!this.projectId;
  }

  get formControls() {
    return this.workForm.controls;
  }

  static checkDatesValidator(group: FormGroup) {
    const error = moment(group.controls.start_base.value, 'DD.MM.YYYY') > moment(group.controls.end_base.value, 'DD.MM.YYYY');
    if (error) {
      group.controls.start_base.setErrors({ period: true });
      group.controls.end_base.setErrors({ period: true });
    } else {
      group.controls.start_base.setErrors(null);
      group.controls.end_base.setErrors(null);
    }
    return error ? { period: true } : null;
  }

  ngOnInit(): void {
    this.work = this.dialogConfig.data.work;
    this.projectId = this.dialogConfig.data.projectId;
    this.user = this.authenticationService.user;

    this.allowedTypes = this.dialogConfig.data.allowedTypes;
    if (!this.allowedTypes) {
      this.allowedTypes = Object.values(WorkEnums.WorkTypeEnum);
    }
    this.allowedTypes.forEach((wt) =>
      this.workTypeChoices.push({
        name: this.translateService.instant('WORKS.TYPE.' + wt),
        value: wt,
      })
    );

    if (this.projectId) {
      this.ptStages$ = this.projectTypeStagesService.projectTypeStagesList({
        projectTypeProjects: this.projectId.toString(),
        include: 'stage',
      });
    }

    this.userSourceMode = this.dialogConfig.data.userSourceMode;
    if (!this.userSourceMode) {
      if (this.projectId) {
        this.userSourceMode = 'project';
      } else {
        this.userSourceMode = 'department';
      }
    }

    if (this.userSourceMode === 'project') {
      this.userList$ = this.usersService.usersList({});
      this.departmentList$ = this.departmentsService.departmentsList({});
    } else {
      this.userList$ = this.departmentsService.departmentsAllUsers(this.user.department);
      this.departmentList$ = this.departmentsService.departmentsSubdepartments(this.user.department);
    }

    firstValueFrom(forkJoin([this.departmentList$, this.userList$])).then((res) => {
      this.objChoices = [
        {
          label: this.translateService.instant('WORK.FORM.DEPARTMENTS'),
          items: res[0].map((d) => ({ label: d.name, value: 'dep_' + d.id.toString() })),
        },
        {
          label: this.translateService.instant('WORK.FORM.USERS'),
          items: res[1].map((u) => ({ label: u.full_name, value: 'user_' + u.id.toString() })),
        },
      ];
    });

    this.initForm(this.work);
    this.workForm.controls.work_type.valueChanges.subscribe((wt) => {
      if (!wt) {
        this.showOptionalValues = false;
        return;
      }
      this.showOptionalValues = true;
      switch (wt) {
        case WorkEnums.WorkTypeEnum.AS: {
          if (!this.workForm.controls.start_base.value) {
            this.workForm.controls.start_base.setValue(moment().toISOString());
          }
          break;
        }
        case WorkEnums.WorkTypeEnum.MS: {
          this.workForm.controls.end_base.setValue(this.workForm.controls.start_base.value);
          break;
        }
        case WorkEnums.WorkTypeEnum.TS: {
          break;
        }
      }
    });
    this.workForm.controls.start_base.valueChanges.subscribe((sb) => {
      if (this.workForm.controls.work_type.value === WorkEnums.WorkTypeEnum.MS) {
        this.workForm.controls.end_base.setValue(sb);
      }
    });

    if (this.workTypeChoices.length === 1) {
      this.workForm.controls.work_type.setValue(this.allowedTypes[0]);
    }

    if (this.work && this.work.work_type) {
      this.workForm.controls.work_type.setValue(this.work.work_type);
    }
  }

  initForm(work: Work) {
    this.workForm = this.formBuilder.group(
      {
        id: [work?.id],
        name: [work?.name, Validators.required],
        description: [work?.description],
        result: [work?.result],
        work_type: [work?.work_type, Validators.required],
        project: [work?.project || this.projectId],
        parent: [work?.parent],
        group: [work?.group],
        start_base: [work?.start_base ? moment.utc(work.start_base).toDate() : null],
        end_base: [work?.end_base ? moment.utc(work.end_base).toDate() : null],
        to_obj: [this.getToObjId(work?.to_user, work?.to_department)],
        val_user: [work?.val_user || this.user.id],
        milestone_level: [work?.milestone_level || 3],
        milestone_stage: [work?.milestone_stage],
      },
      { validators: WorkFormComponent.checkDatesValidator }
    );
  }

  save() {
    const val = this.workForm.value;
    Object.assign(val, this.convertToObjId(val.to_obj));
    delete val.to_obj;

    const obs$: Observable<Work> = this.work?.id
      ? this.worksService.worksPartialUpdate({ id: this.work.id, data: val })
      : this.worksService.worksCreate(val);
    firstValueFrom(obs$).then((w) => this.dialogRef.close(w));
  }

  private getToObjId(to_user: number, to_department: number): string {
    if (to_user) {
      return 'user_' + to_user.toString();
    }
    if (to_department) {
      return 'dep_' + to_department.toString();
    }
    return null;
  }

  private convertToObjId(objId: string): Object {
    const res = { to_user: null };
    const sp = _.split(objId, '_', 2);
    if (sp.length === 2) {
      if (sp[0] === 'user') {
        res.to_user = Number(sp[1]);
      }
      if (sp[0] === 'dep') {
        res['to_department'] = Number(sp[1]);
        res.to_user = null;
      }
    }
    return res;
  }
}
