import {Component, OnInit} from '@angular/core';
import {DialogService, DynamicDialogConfig, DynamicDialogRef} from 'primeng/dynamicdialog';
import {firstValueFrom, Observable} from 'rxjs';
import {TranslateService} from '@ngx-translate/core';
import {AuthenticationService} from '@core-project/services/authentication.service';
import * as moment from 'moment/moment';
import {SelectItem} from 'primeng/api';
import {
  HistoryService,
  Issue,
  IssuesService,
  ProjectTypeStage,
  ProjectTypeStagesService,
  User,
  UsersService,
  Work,
  WorksService,
} from 'data-access/sdk/api2';

import {DialogEnums, WorkEnums} from '@core-project/enums';
import {ActionRecord} from '@core-project/interfaces';
import {generateActions} from '@core-project/helpers';
import {WorkFormComponent} from '@core-project/modules/shared-components/works/work-form/work-form.component';
import {ProjectEnums} from '@core-project/enums/project';
import {Store} from '@ngxs/store';
import {IssuesActions, WorksActions} from 'data-access/store';
import StageEnum = ProjectEnums.StageEnum;

// noinspection AngularMissingOrInvalidDeclarationInModule
/** @deprecated */
@Component({
  selector: 'app-work-info',
  templateUrl: './work-info.component.html',
  styleUrls: ['./work-info.component.scss'],
})
export class WorkInfoComponent implements OnInit {
  work: Work;
  parentWork: Work;
  issue$: Observable<Issue>;
  currentUser: User;
  perms: Array<string>;
  subWorks: Array<Work>;
  users$: Observable<User[]>;
  newComment: string;
  actions: ActionRecord[];
  actionsLoading = true;
  statusChoices: SelectItem[] = [];
  workTypes = WorkEnums.WorkTypeEnum;
  stagesChoices: SelectItem[] = [];
  ptStages$: Observable<ProjectTypeStage[]>;

  constructor(
    private dialogRef: DynamicDialogRef,
    private dialogConfig: DynamicDialogConfig,
    private historyService: HistoryService,
    private dialogService: DialogService,
    private translateService: TranslateService,
    private worksService: WorksService,
    private authenticationService: AuthenticationService,
    private usersService: UsersService,
    private issuesService: IssuesService,
    private projectTypeStagesService: ProjectTypeStagesService,
    private readonly store: Store,
  ) {
  }

  ngOnInit(): void {
    this.work = this.dialogConfig.data.work;
    this.currentUser = this.authenticationService.user;
    this.users$ = this.usersService.usersList({});
    Object.keys(WorkEnums.StatusEnum).forEach((st) =>
      this.statusChoices.push({label: this.translateService.instant('WORKS.STATUS.' + st), value: st}),
    );
    Object.values(StageEnum).forEach((stage) =>
      this.stagesChoices.push({label: this.translateService.instant('PROJECTS.STAGE.' + stage), value: stage}),
    );
    if (this.work.project) {
      this.ptStages$ = this.projectTypeStagesService.projectTypeStagesList({
        projectTypeProjects: this.work.project.toString(),
        include: 'stage',
      });
    }

    this.loadWork();
    this.loadSubworks();
    this.loadParentWork();
    this.loadIssues();
  }

  save() {
    firstValueFrom(this.worksService.worksPartialUpdate({id: this.work.id, data: this.work})).then((_w) => this.loadWork());
  }

  addComment() {
    if (this.newComment) {
      firstValueFrom(this.worksService.worksAddComment({id: this.work.id, data: {text: this.newComment}})).then((_c) =>
        this.loadWork(),
      );
    }
  }

  loadSubworks() {
    firstValueFrom(
      this.worksService.worksList({
        parent: String(this.work.id),
        include: 'to_user',
      }),
    ).then((works) => (this.subWorks = works));
  }

  loadParentWork() {
    if (this.work.parent) {
      firstValueFrom(this.worksService.worksRead({id: this.work.parent, include: 'to_user'})).then((w) => (this.parentWork = w));
    }
  }

  split() {
    const newWork = {
      name: this.work.name,
      project: this.work.project,
      description: this.work.description,
      val_user: this.work.to_user,
      start_base: this.work.start_base,
      end_base: this.work.end_base,
      parent: this.work.id,
    };
    const workDialog = this.dialogService.open(WorkFormComponent, {
      header: this.translateService.instant('WORKS.NEW_WORK'),
      data: {
        work: newWork,
        projectId: this.work.project,
        allowedTypes: [WorkEnums.WorkTypeEnum.AS],
        userSourceMode: 'department',
      },
    });
    firstValueFrom(workDialog.onClose).then((_w) => this.loadSubworks());
  }

  workInfo(work: Work) {
    this.store.dispatch(new WorksActions.ShowWorkInSidebar(work.id)).subscribe();
    // this.loadSubworks()
  }

  changeStatus() {
    firstValueFrom(this.worksService.worksChangeStatus({id: this.work.id, data: {status: this.work.status}})).then((_w) =>
      this.loadWork(),
    );
  }

  startBaseChanged() {
    if (this.work.work_type === WorkEnums.WorkTypeEnum.MS) {
      this.work.end_base = this.work.start_base;
    }
  }

  openIssue(issue: Issue) {
    this.store.dispatch(new IssuesActions.ShowIssueInSidebar(issue.id)).subscribe();
  }

  delete() {
    firstValueFrom(this.worksService.worksDelete(this.work.id)).then((_w) => this.dialogRef.close(DialogEnums.ActionEnum.DEL));
  }

  private loadWork() {
    this.worksService
      .worksRead({
        id: this.work.id,
        include: 'to_user, to_department, val_user, created_by, val_by, comments__author, project, milestone_stage',
      })
      .subscribe((w) => {
        this.work = w;
        this.loadWorkActions();
      });
  }

  private loadWorkActions() {
    firstValueFrom(this.historyService.historyList({contentTypeModel: 'work', objectId: String(this.work.id)})).then((hs) => {
      this.actions = generateActions(this.work.rel_comments, hs);
      this.actionsLoading = false;
    });
  }

  private loadIssues() {
    if (this.work.issues && this.work.issues.length > 0) {
      this.issue$ = this.issuesService.issuesIssuesRead({id: this.work.issues[0]});
    }
  }
}
