import {Injectable} from '@angular/core';
import {BehaviorSubject, Observable, of} from 'rxjs';
import {catchError, map, switchMap} from 'rxjs/operators';
import {AuthService, CompaniesService, User} from 'data-access/sdk/api2';
import {TranslateService} from '@ngx-translate/core';
import {Store} from '@ngxs/store';
import {GlobalActions} from 'data-access/store';
import {UserAdapterService} from '../../../../../libs/data-access/sdk/internal-api/src/lib/users/services/user-adapter.service';
import {UserModel} from 'data-access/sdk/internal-api';

@Injectable({providedIn: 'root'})
export class AuthenticationService {
  private readonly tokenSubject = new BehaviorSubject<string | undefined>(localStorage.getItem('token'));
  private readonly isImpersonatedSubject = new BehaviorSubject<boolean>(false);
  private userBeforeImpersonation: User;
  private tokenBeforeImpersonation: string;
  readonly userSubject: BehaviorSubject<User | undefined> = new BehaviorSubject<User | undefined>(undefined);
  readonly user$: Observable<User | undefined> = this.userSubject.asObservable();

  constructor(
    private readonly authService: AuthService,
    private readonly companiesService: CompaniesService,
    private readonly translateService: TranslateService,
    private readonly store: Store,
    private readonly userAdapterService: UserAdapterService,
  ) {
    this.user$
      .pipe(
        switchMap((user: User | undefined) => {
          if (!user) {
            return of(undefined);
          }

          return this.userAdapterService.convert(user);
        })
      )
      .subscribe((user: UserModel | undefined) => {
        if (user) {
          this.store.dispatch(new GlobalActions.SetUser(user));
        } else {
          this.store.dispatch(new GlobalActions.ResetUser());
        }
      });

    this.tokenSubject
      .subscribe(token => {
        // null token means logout
        if (!token) {
          localStorage.setItem('token', '');
        }
      });
  }

  get user(): User | undefined {
    return this.userSubject.value;
  }

  get isImpersonated(): boolean {
    return this.isImpersonatedSubject.value;
  }

  get token(): string {
    return this.tokenSubject.value;
  }

  get isLoggedIn(): boolean {
    return Boolean(this.tokenSubject.value);
  }

  loginByToken(token: string, remember: boolean = true): Observable<any> {
    this.tokenSubject.next(token);

    return this.authService.authUsersMeRead().pipe(
      map((user) => {
        if (user) {
          if (remember) {
            localStorage.setItem('token', token);
          }
          this.userSubject.next(user);
          return user;
        } else {
          this.userSubject.next(undefined);
          this.tokenSubject.next('');
          return;
        }
      }),
      switchMap((user) => {
        if (!user) {
          return;
        }

        return this.companiesService.companiesTranslations({id: user.company, lang: this.translateService.currentLang});
      }),
      map((trans) => {
        if (trans) {
          this.translateService.setTranslation(this.translateService.currentLang, trans.translations, true);
        }
      }),
    );
  }

  login(email: string, password: string, remember: boolean): Observable<any> {
    const request = {
      email: email,
      password: password,
    };

    return this.authService.authTokenLoginCreate(request).pipe(
      switchMap((data) => {
        const token = data['auth_token'];
        return this.loginByToken(token, remember);
      }),
    );
  }

  updateCurrentUser() {
    this.authService.authUsersMeRead().subscribe((user) => this.userSubject.next(user));
  }

  logout() {
    this.tokenSubject.next('');
    this.userSubject.next(undefined);
  }

  impersonate(userId: number): Observable<User> {
    return this.authService.authUsersImpersonate({user_id: userId}).pipe(
      switchMap((token) => {
        this.isImpersonatedSubject.next(true);
        this.userBeforeImpersonation = this.userSubject.value;
        this.tokenBeforeImpersonation = this.tokenSubject.value;
        return this.loginByToken(token.auth_token, false);
      }),
      catchError((_val) => of(null)),
    );
  }

  stopImpersonation() {
    this.userSubject.next(this.userBeforeImpersonation);
    this.tokenSubject.next(this.tokenBeforeImpersonation);
    this.isImpersonatedSubject.next(false);
    this.userBeforeImpersonation = null;
    this.tokenBeforeImpersonation = null;
  }
}
