import {Injectable} from '@angular/core';
import {BehaviorSubject} from 'rxjs';
import {AuthenticationService} from './authentication.service';
import {TranslateService} from '@ngx-translate/core';
import {User} from 'data-access/sdk/api2';
import {environment} from '../../environments/environment';
import {CustomMenuItemInterface} from '@core-project/interfaces/custom-menu-item.interface';

@Injectable()
export class SidemenuService {
  user: User;

  itemsBSubject = new BehaviorSubject<CustomMenuItemInterface[]>([]);
  items$ = this.itemsBSubject.asObservable();

  constructor(
    private authenticationService: AuthenticationService,
    private translateService: TranslateService,
  ) {
    this.authenticationService.user$.subscribe((user) => {
      if (!user) {
        this.setDefaultNoUserItems();
      }
    });
  }

  setActiveItemByFirstSegment(segment: string) {
    const items = this.itemsBSubject.value;
    const currentItemId: string = items.find((item) => item.routerLink.split('/')[1] === this.routeRewriteHack(segment))?.id;

    if (currentItemId) {
      this.setActiveItemById(currentItemId);
    } else {
      console.error(`No item found for segment: ${segment}`);
    }
  }

  setActiveItemById(itemId: string) {
    const items = this.itemsBSubject.value;
    items.forEach((_item) => {
      _item.customActiveStatus = _item.id === itemId;
    });
    this.itemsBSubject.next(items);
  }

  setItems(items: CustomMenuItemInterface[]) {
    this.itemsBSubject.next(items);
  }

  setDefaultNoUserItems() {
    const items: CustomMenuItemInterface[] = [
      {routerLink: '/login', icon: 'fa-regular fa-sign-in-alt', label: this.translateService.instant('SIDEMENU.LOGIN')},
    ];

    if (environment['registrationEnabled']) {
      items.push({
        routerLink: '/registration',
        icon: 'fa-regular fa-check',
        label: this.translateService.instant('SIDEMENU.REGISTRATION'),
      });
    }

    this.setItems(items);
  }

  setDefaultItems() {
    this.user = this.authenticationService.user;

    const items: CustomMenuItemInterface[] = [
      {
        id: 'main',
        label: this.translateService.instant('SIDEMENU.MAIN'),
        routerLink: '/profile',
        icon: 'home',
      },
      {
        id: 'dashboards',
        label: this.translateService.instant('SIDEMENU.DASHBOARD'),
        routerLink: '/dashboards',
        icon: 'dashboard',
      },
      {
        id: 'pr_list',
        label: this.translateService.instant('SIDEMENU.PROJECTS_LIST'),
        routerLink: '/projects/structure',
        icon: 'list',
      },
      {
        id: 'risks',
        label: this.translateService.instant('SIDEMENU.RISKS'),
        routerLink: '/risks',
        icon: 'problems',
      },
      {
        id: 'docs',
        label: this.translateService.instant('SIDEMENU.DOCUMENTS'),
        icon: 'documents',
        routerLink: '/documents',
      },
      {id: 'crm', label: this.translateService.instant('SIDEMENU.CRM'), icon: 'contacts', routerLink: '/crm'},
      {
        id: 'structure',
        label: this.translateService.instant('KB.MENU.STRUCTURE'),
        icon: 'structure',
        routerLink: '/company-structure',
      },
      {
        id: 'settings',
        label: this.translateService.instant('KB.MENU.SETTINGS'),
        icon: 'settings',
        routerLink: '/settings/project-cats',
      },
    ];

    if (this.user && this.user.department) {
      items.splice(4, 0, {
        id: 'dep',
        label: this.translateService.instant('SIDEMENU.DEPARTMENT'),
        icon: 'department',
        routerLink: `/department/${this.user.department}/tasks`,
      });
    }

    this.setItems(items);
  }

  private routeRewriteHack(routerLink: string) {
    // route hack for projects without routes refactor
    if (routerLink === 'project') {
      return 'projects';
    }
    return routerLink;
  }
}
