import { Injectable } from '@angular/core';
import * as moment from 'moment';
import { map, switchMap } from 'rxjs/operators';
import { BehaviorSubject, Observable, of } from 'rxjs';
import { AuthenticationService } from '@core-project/services/authentication.service';
import { UsersService } from 'data-access/sdk/api2';
import { User, UserPermissions } from 'data-access/sdk/api2';

/** @deprecated */
interface ObjectPermissions {
  updatedAt: moment.Moment;
  perms: BehaviorSubject<Array<string>>;
}

/** @deprecated */
export enum PermissionObjectType {
  company = 'COM',
  projectGroup = 'PRG',
  project = 'PRJ',
}

/** @deprecated */
@Injectable({
  providedIn: 'root',
})
export class UserPermissionsService {
  userPermissions: ObjectPermissions;
  permStorage: { [key: string]: { [key: number]: ObjectPermissions } } = {};
  updatePeriodMs = 60000;

  constructor(private authenticationService: AuthenticationService, private userService: UsersService) {
    Object.values(PermissionObjectType).forEach((pot) => (this.permStorage[pot] = {}));
    this.initUserPermissionList();
  }

  public getUserObjectPermissions(objectType: PermissionObjectType = null, objectId: number = null): Observable<Array<string>> {
    if (!objectType) {
      return this.fetchOrUpdateUserPermissions();
    } else {
      return this.fetchOrUpdateObjectPermissions(objectType, objectId);
    }
  }

  public hasPermission(permissionKey: string, objectType: PermissionObjectType = null, objectId: number = null): Observable<boolean> {
    return this.getUserObjectPermissions(objectType, objectId).pipe(
      map((permissions: Array<string>) => permissions.includes(permissionKey))
    );
  }

  private initUserPermissionList(): void {
    this.authenticationService.user$
      .pipe(
        switchMap((user: User) => {
          if (!user) {
            return of({
              permissions: [],
            });
          }
          return this.userService.usersPermissions({ id: user.id });
        })
      )
      .subscribe((userPermissions: UserPermissions) => {
        this.userPermissions = {
          updatedAt: moment(),
          perms: new BehaviorSubject([]),
        };
        this.userPermissions.perms.next(userPermissions.permissions);
      });
  }

  private fetchOrUpdateUserPermissions(): Observable<Array<string>> {
    return this.authenticationService.user$.pipe(
      switchMap((user: User) => {
        if (!user) {
          return of([]);
        }

        if (!this.userPermissions || moment().diff(this.userPermissions?.updatedAt) > this.updatePeriodMs) {
          this.userService.usersPermissions({ id: user.id }).subscribe((perms) => {
            this.userPermissions.perms.next(perms.permissions);
            this.userPermissions.updatedAt = moment();
          });
        }
        return this.userPermissions.perms.asObservable();
      })
    );
  }

  private fetchOrUpdateObjectPermissions(objectType: PermissionObjectType, objectId: number): Observable<Array<string>> {
    return this.authenticationService.user$.pipe(
      switchMap((user: User) => {
        if (!user) {
          return of([]);
        }
        if (!this.permStorage[objectType][objectId]) {
          this.permStorage[objectType][objectId] = {
            perms: new BehaviorSubject([]),
            updatedAt: null,
          };
        }
        if (
          !this.permStorage[objectType][objectId].updatedAt ||
          moment().diff(this.permStorage[objectType][objectId].updatedAt) > this.updatePeriodMs
        ) {
          this.permStorage[objectType][objectId].updatedAt = moment();
          const params: UsersService.UsersPermissionsParams = {
            id: user.id,
          };

          switch (objectType) {
            case PermissionObjectType.company:
              params.company = objectId;
              break;
            case PermissionObjectType.projectGroup:
              params.projectGroup = objectId;
              break;
            case PermissionObjectType.project:
              params.project = objectId;
              break;
          }

          this.userService.usersPermissions(params).subscribe((perms) => {
            this.permStorage[objectType][objectId].perms.next(perms.permissions);
          });
        }
        return this.permStorage[objectType][objectId].perms.asObservable();
      })
    );
  }
}
