import {Injectable} from '@angular/core';
import {Observable, withLatestFrom} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {UserBaseModel, UserModel} from '../../users';
import {IssueDomainModel, IssueModel, IssueStatusEnum, NewIssueModel} from '../../issues';
import {IssueDTO} from '../models/issue.dto';
import {UserAdapterService} from '../../users/services/user-adapter.service';
import {CommentAdapterService} from '../../comments/services/comment-adapter.service';
import {CommentModel} from '../../comments';
import {IssueDomainAdapterService} from './issue-domain-adapter.model';
import {WorkAdapterService} from '../../works/services/work-adapter.service';
import {ProjectAdapterService} from '../../projects/services/project-adapter';
import {WorkModel} from '../../works';
import {ProjectModel} from '../../projects';
import {UserBaseAdapterService} from '../../users/services/user-base-adapter.service';
import {IssueBaseAdapterService} from './issue-base-adapter.service';

@Injectable({
  providedIn: 'root',
})
export class IssueAdapterService extends BaseAdapter<IssueModel, IssueDTO> {
  constructor(
    private readonly userAdapterService: UserAdapterService,
    private readonly commentAdapterService: CommentAdapterService,
    private readonly issueDomainAdapterService: IssueDomainAdapterService,
    private readonly workAdapterService: WorkAdapterService,
    private readonly projectAdapterService: ProjectAdapterService,
    private readonly userBaseAdapterService: UserBaseAdapterService,
    private readonly issueBaseAdapterService: IssueBaseAdapterService,
  ) {
    super();
  }

  convert(dto: IssueDTO): Observable<IssueModel> {
    const author$: Observable<UserModel | undefined> = this.createModelObs(dto.rel_author, this.userAdapterService);
    const comments$: Observable<CommentModel[]> = this.createModelArrObs(dto.rel_comments, this.commentAdapterService);
    const domains$: Observable<IssueDomainModel[]> = this.createModelArrObs(dto.rel_domains, this.issueDomainAdapterService);
    const users$: Observable<UserBaseModel[]> = this.createModelArrObs(dto.rel_users, this.userBaseAdapterService);
    const parentWork$: Observable<WorkModel | undefined> = this.createModelObs(dto.rel_parent_work, this.workAdapterService);
    const project$: Observable<ProjectModel | undefined> = this.createModelObs(dto.rel_project, this.projectAdapterService);
    const responsible$: Observable<UserModel | undefined> = this.createModelObs(dto.rel_responsible, this.userAdapterService);
    const works$: Observable<WorkModel[]> = this.createModelArrObs(dto.rel_works, this.workAdapterService);

    return this.issueBaseAdapterService.convert(dto)
      .pipe(
        withLatestFrom(
          author$,
          comments$,
          domains$,
          users$,
          parentWork$,
          project$,
          responsible$,
          works$,
        ),
        switchMap(([
                     issueBase,
                     author,
                     comments,
                     domains,
                     users,
                     parentWork,
                     project,
                     responsible,
                     works,
                   ]) => {
          const model: IssueModel = {
            ...issueBase,

            // relation ids
            authorId: dto.author ?? undefined,
            commentIds: dto.comments ?? [],
            domainIds: dto.domains ?? [],
            parentWorkId: dto.parent_work ?? undefined,
            projectId: dto.project ?? undefined,
            responsibleId: dto.responsible ?? undefined,
            userIds: dto.users ?? [],
            workIds: dto.works ?? [],

            // relation models
            author,
            comments,
            domains,
            users,
            parentWork,
            project,
            responsible,
            works,
          };

          return this.createConvertObservable(model);
        }),
      );
  }

  reverse(model: IssueModel): IssueDTO {
    return {
      description: model.description ?? null,
      name: model.name,
      order: model.order,
      resolve_date: model.resolveDate ?? null,
      severity: model.severity,
      status: model.status,

      // relation ids
      author: model.authorId ?? null,
      domains: model.domainIds ?? null,
      users: model.userIds ?? null,
      parent_work: model.parentWorkId ?? null,
      project: model.projectId ?? null,
      responsible: model.responsibleId ?? null,
    };
  }

  reverseToNewModel(model: NewIssueModel): IssueDTO {
    return {
      description: model.description,
      name: model.name,
      resolve_date: model.resolveDate,
      severity: model.severity,
      status: model.status,

      // relation ids
      author: model.authorId,
      domains: model.domainIds,
      users: model.userIds,
      parent_work: model.parentWorkId,
      project: model.projectId,
      responsible: model.responsibleId,
    };
  }
}
