import {Injectable} from '@angular/core';
import {catchError, map, switchMap} from 'rxjs/operators';
import {forkJoin, Observable, throwError} from 'rxjs';
import * as moment from 'moment';

import {Comment, Issue, IssuesService, ProjectsService} from 'data-access/sdk/api2';
import {CommentModel, IssueModel, NewCommentModel, NewIssueModel} from 'data-access/sdk/internal-api';

import {IssueAdapterService} from './issue-adapter.service';
import {IssueDomainAdapterService} from './issue-domain-adapter.model';
import {CommentAdapterService} from '../../comments/services/comment-adapter.service';

@Injectable({
  providedIn: 'root',
})
export class IssueApiService {
  constructor(
    private readonly issuesService: IssuesService,
    private readonly issueAdapterService: IssueAdapterService,
    private readonly issueDomainAdapterService: IssueDomainAdapterService,
    private readonly commentAdapterService: CommentAdapterService,
    private readonly projectsService: ProjectsService,
  ) {
  }

  updateIssue(issue: IssueModel) {
    const id = issue.id;
    const data: Issue = this.issueAdapterService.reverse(issue);
    return this.issuesService.issuesIssuesPartialUpdate({id, data})
      .pipe(
        catchError((error) => {
          console.error('Error while updating issue', error);
          // TODO #errorService
          return throwError(() => error);
        }),
        switchMap((issueResponse) => this.issueAdapterService.convert(issueResponse)),
      );
  }

  createComment(issueId: IssueModel['id'], comment: NewCommentModel) {
    const data: Comment = this.commentAdapterService.reverseToNewModel(comment);

    return this.issuesService.issuesIssuesAddComment({id: issueId, data})
      .pipe(
        catchError((error) => {
          console.error('Error while creating comment', error);
          // TODO #errorService
          return throwError(() => error);
        }),
        switchMap((commentResponse) => this.commentAdapterService.convert(commentResponse)),
      );
  }

  createIssue(issue: NewIssueModel) {
    const data: Issue = this.issueAdapterService.reverseToNewModel(issue);

    return this.issuesService.issuesIssuesCreate(data)
      .pipe(
        catchError((error) => {
          console.error('Error while creating issue', error);
          // TODO #errorService
          return throwError(() => error);
        }),
        switchMap((issueResponse) => this.issueAdapterService.convert(issueResponse)),
      );
  }

  readIssuePermissions(issueId: number): Observable<string[]> {
    return this.issuesService.issuesIssuesPermissions(issueId)
      .pipe(
        catchError((error) => {
          console.error('Error while getting permissions', error);
          // TODO #errorService
          return throwError(() => error);
        }),
        map((permissions) => (permissions.permissions?.filter((p) => !!p) || []) as string[]),
      );
  }

  readProjectPermissions(projectId: number): Observable<string[]> {
    return this.projectsService.projectsPermissions(projectId)
      .pipe(
        catchError((error) => {
          console.error('Error while getting permissions', error);
          // TODO #errorService
          return throwError(() => error);
        }),
        map((permissions) => (permissions.permissions?.filter((p) => !!p) || []) as string[]),
      );
  }

  readIssue(
    id: IssueModel['id'],
    include: string = 'domains, responsible, works__to_user, comments__author, issue_users__user, project, parent_work',
  ) {
    return this.issuesService.issuesIssuesRead({id, include})
      .pipe(
        catchError((error) => {
          console.error('Error while getting issue', error);
          // TODO #errorService
          return throwError(() => error);
        }),
        switchMap((issue) => this.issueAdapterService.convert(issue)),
        map((issue) => ({
          ...issue,
          comments: this.sortComments(issue.comments),
        })),
      );
  }

  readIssueDomainsList() {
    return this.issuesService.issuesDomainsList()
      .pipe(
        catchError((error) => {
          console.error('Error while getting issue domains list', error);
          // TODO #errorService
          return throwError(() => error);
        }),
        switchMap((domains) => forkJoin(
          domains.map((domain) => this.issueDomainAdapterService.convert(domain)),
        )),
      );
  }

  deleteIssue(id: IssueModel['id']) {
    return this.issuesService.issuesIssuesDelete(id)
      .pipe(
        catchError((error) => {
          console.error('Error while deleting issue', error);
          // TODO #errorService
          return throwError(() => error);
        }),
      );
  }

  private sortComments(comments: CommentModel[]) {
    return comments.sort((a, b) => {
      const dateA = moment(a.created);
      const dateB = moment(b.created);

      if (dateB.isBefore(dateA)) {
        return 1;
      }

      if (dateB.isAfter(dateA)) {
        return -1;
      }

      return 0;
    });
  }
}
