import {Injectable} from '@angular/core';
import {Observable} from 'rxjs';
import * as moment from 'moment';

import {Notification} from 'data-access/sdk/api2';

import {BaseAdapter} from 'utils/shared-services';

import {NotificationDTO} from '../models/notification.dto';
import {NewNotificationModel, NotificationModel} from '../models/notification.model';
import {NotificationTypeEnum} from '../models/notification-type.enum';
import {NotificationSeverityEnum} from '../models/notification-severity.enum';

@Injectable({
  providedIn: 'root',
})
export class NotificationAdapterService extends BaseAdapter<NotificationModel, NotificationDTO> {

  convert(dto: NotificationDTO): Observable<NotificationModel> {
    const model: NotificationModel = {
      id: dto.id ?? this.convertError('Convert error: id is not defined'),
      isUnread: dto.unread ?? false,
      timestamp: dto.timestamp ? moment(dto.timestamp).valueOf() : this.convertError('Convert error: timestamp is not defined'),
      severity: this.convertLevelToSeverity(dto.level),
      nameTemplate: dto.name_template ?? this.convertError('Convert error: name_template is not defined'),
      nameParams: dto.name_params ?? undefined,
      descriptionTemplate: dto.description_template ?? this.convertError('Convert error: description_template is not defined'),
      descriptionParams: dto.description_params ?? undefined,
      urlType: this.convertUrlType(dto.url_type),
      urlParams: dto.url_params ?? undefined,

      // relation ids
      recipientId: dto.recipient ?? this.convertError('Convert error: recipient is not defined'),
    };

    return this.createConvertObservable(model);
  }

  reverse(model: NewNotificationModel): NotificationDTO {
    return {
      recipient: model.recipientId,
      level: this.reverseLevelToSeverity(model.severity),
      name_template: model.nameTemplate,
      name_params: model.nameParams,
      description_template: model.descriptionTemplate,
      description_params: model.descriptionParams,
      url_type: this.reverseUrlType(model.urlType),
      url_params: model.urlParams,
    };
  }

  private convertUrlType(urlType: Notification['url_type']): NotificationTypeEnum | undefined {
    switch (urlType) {
      case 'PROJECT_INFO':
        return NotificationTypeEnum.ProjectInfo;
      case 'PROJECT_PLAN':
        return NotificationTypeEnum.ProjectPlan;
      case 'WORK':
        return NotificationTypeEnum.Work;
      case 'ISSUE':
        return NotificationTypeEnum.Issue;
      default:
        return undefined;
    }
  }

  private reverseUrlType(urlType: NotificationTypeEnum | undefined): NotificationDTO['url_type'] {
    switch (urlType) {
      case NotificationTypeEnum.ProjectInfo:
        return 'PROJECT_INFO';
      case NotificationTypeEnum.ProjectPlan:
        return 'PROJECT_PLAN';
      case NotificationTypeEnum.Work:
        return 'WORK';
      case NotificationTypeEnum.Issue:
        return 'ISSUE';
      default:
        return null;
    }
  }

  private convertLevelToSeverity(level: Notification['level']): NotificationSeverityEnum {
    switch (level) {
      case 'DEBUG':
        return NotificationSeverityEnum.Warn;
      case 'SUCCESS':
        return NotificationSeverityEnum.Success;
      case 'INFO':
        return NotificationSeverityEnum.Info;
      case 'WARNING':
        return NotificationSeverityEnum.Warn;
      case 'ERROR':
        return NotificationSeverityEnum.Error;
      default:
        return NotificationSeverityEnum.Info;
    }
  }

  private reverseLevelToSeverity(level: NotificationSeverityEnum): NotificationDTO['level'] {
    switch (level) {
      case NotificationSeverityEnum.Info:
        return 'INFO';
      case NotificationSeverityEnum.Success:
        return 'SUCCESS';
      case NotificationSeverityEnum.Warn:
        return 'WARNING';
      case NotificationSeverityEnum.Error:
        return 'ERROR';
      default:
        return 'DEBUG';
    }
  }
}
