import {Injectable} from '@angular/core';
import {forkJoin, Observable} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {ProjectModel} from '../models/project.model';
import {ProjectDTO} from '../models/project.dto';
import {ProjectMemberModel} from '../models/project-member.model';
import {ProjectMetaValueModel} from '../models/project-meta-value.model';
import {ProjectBudgetModel} from '../../finances/models/project-budget.model';
import {ProjectBudgetTypeModel} from '../../finances/models/project-budget-type.model';
import {ProjectGroupBaseModel} from '../models/project-group-base.model';
import {ProjectMemberAdapterService} from './project-member-adapter.service';
import {ProjectMetaValueAdapterService} from './project-meta-value-adapter.service';
import {ProjectBudgetAdapterService} from '../../issues/services/project-budget-adapter.service';
import {ProjectBudgetTypeAdapterService} from '../../finances/services/project-budget-type-adapter.service';
import {ProjectGroupBaseAdapterService} from './project-group-base-adapter.service';
import {ProjectTypeBaseAdapterService} from './project-type-base-adapter.service';
import {UserRoleAdapterService} from '../../roles/services/user-role-adapter.service';
import {StageModel} from '../../stages';
import {ProjectStatusEnum, ProjectTypeBaseModel} from '../../projects';
import {UserRoleModel} from '../../roles';
import {StageAdapterService} from '../../stages/services/stage-adapter.service';

@Injectable({
  providedIn: 'root',
})
export class ProjectAdapterService extends BaseAdapter<ProjectModel, ProjectDTO> {
  constructor(
    private readonly projectMemberAdapterService: ProjectMemberAdapterService,
    private readonly projectMetaValueAdapterService: ProjectMetaValueAdapterService,
    private readonly projectBudgetAdapterService: ProjectBudgetAdapterService,
    private readonly projectBudgetTypeAdapterService: ProjectBudgetTypeAdapterService,
    private readonly projectGroupBaseAdapterService: ProjectGroupBaseAdapterService,
    private readonly projectTypeBaseAdapterService: ProjectTypeBaseAdapterService,
    private readonly stageAdapterService: StageAdapterService,
    private readonly userRoleAdapterService: UserRoleAdapterService,
  ) {
    super();
  }

  convert(dto: ProjectDTO): Observable<ProjectModel> {
    const members$: Observable<ProjectMemberModel[]> = this.createModelArrObs(dto.rel_members, this.projectMemberAdapterService);
    const metas$: Observable<ProjectMetaValueModel[]> = this.createModelArrObs(dto.rel_metas, this.projectMetaValueAdapterService);
    const projectBudget$: Observable<ProjectBudgetModel | undefined> = this.createModelObs(
      dto.rel_project_budget,
      this.projectBudgetAdapterService,
    );
    const projectBudgetTypes$: Observable<ProjectBudgetTypeModel[]> = this.createModelArrObs(
      dto.rel_project_budget_types,
      this.projectBudgetTypeAdapterService,
    );
    const projectGroup$: Observable<ProjectGroupBaseModel | undefined> = this.createModelObs(
      dto.rel_project_group,
      this.projectGroupBaseAdapterService,
    );
    const projectType$: Observable<ProjectTypeBaseModel | undefined> = this.createModelObs(
      dto.rel_project_type,
      this.projectTypeBaseAdapterService,
    );
    const stage$: Observable<StageModel | undefined> = this.createModelObs(dto.rel_stage, this.stageAdapterService);
    const usersRoles$: Observable<UserRoleModel[]> = this.createModelArrObs(dto.rel_users_roles, this.userRoleAdapterService);

    return forkJoin([
      members$,
      metas$,
      projectBudget$,
      projectBudgetTypes$,
      projectGroup$,
      projectType$,
      stage$,
      usersRoles$,
    ])
      .pipe(
        switchMap(([
                     members,
                     metas,
                     projectBudget,
                     projectBudgetTypes,
                     projectGroup,
                     projectType,
                     stage,
                     usersRoles,
                   ]) => {
          const model: ProjectModel = {
            id: dto.id ?? this.convertError('Convert error: id is not defined'),
            abbr: dto.abbr ?? this.convertError('Convert error: abbr is not defined'),
            name: dto.name ?? this.convertError('Convert error: name is not defined'),
            status: this.convertStatus(dto.status),
            workPlan: dto.work_plan ?? undefined,

            // ids
            memberIds: dto.members ?? [],
            metaIds: dto.metas ?? [],
            projectBudgetId: dto.project_budget ?? undefined,
            projectBudgetTypeIds: dto.project_budget_types ?? [],
            projectGroupId: dto.project_group ?? undefined,
            projectTypeId: dto.project_type ?? undefined,
            stageId: dto.stage ?? undefined,
            usersRolesIds: dto.users_roles ?? [],

            // models
            members,
            metas,
            projectBudget,
            projectBudgetTypes,
            projectGroup,
            projectType,
            stage,
            usersRoles,
          };

          return this.createConvertObservable(model);
        }),
      );
  }

  reverse(model: ProjectModel): ProjectDTO {
    throw new Error('Method not implemented.');
  }

  private convertStatus(status: ProjectDTO['status']): ProjectModel['status'] {
    switch (status) {
      case 'FUT':
        return ProjectStatusEnum.FUT;
      case 'ACT':
        return ProjectStatusEnum.ACT;
      case 'CNC':
        return ProjectStatusEnum.CNC;
      case 'CLS':
        return ProjectStatusEnum.CLS;
      case 'PSP':
        return ProjectStatusEnum.PSP;
      default:
        return undefined;
    }
  }
}
