import {Injectable} from '@angular/core';
import {forkJoin, Observable} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {ProjectMemberModel} from '../models/project-member.model';
import {ProjectMemberDTO} from '../models/project-member.dto';
import {DepartmentAdapterService} from '../../departments/services/department-adapter.service';
import {PartnerAdapterService} from '../../organisations/services/partner-adapter.service';
import {UserAdapterService} from '../../users/services/user-adapter.service';
import {MemberTypeEnum} from '../models/member-type.enum';

@Injectable({
  providedIn: 'root',
})
export class ProjectMemberAdapterService extends BaseAdapter<ProjectMemberModel, ProjectMemberDTO> {
  constructor(
    private readonly departmentAdapterService: DepartmentAdapterService,
    private readonly partnerAdapterService: PartnerAdapterService,
    private readonly userAdapterService: UserAdapterService,
  ) {
    super();
  }

  convert(dto: ProjectMemberDTO): Observable<ProjectMemberModel> {
    const department$ = this.createModelObs(dto.rel_department, this.departmentAdapterService);
    const organization$ = this.createModelObs(dto.rel_organization, this.partnerAdapterService);
    const user$ = this.createModelObs(dto.rel_user, this.userAdapterService);

    return forkJoin([
      department$,
      organization$,
      user$,
    ])
      .pipe(
        switchMap(([department, organization, user]) => {
          const model: ProjectMemberModel = {
            id: dto.id ?? this.convertError('Convert error: id is not defined'),
            comment: dto.comment ?? undefined,
            createdAt: dto.created_at ?? undefined,
            memberType: this.convertMemberType(dto.member_type),
            objectName: dto.object_name ?? undefined,
            order: dto.order ?? 0,
            updatedAt: dto.updated_at ?? undefined,

            // relation ids
            departmentId: dto.department ?? undefined,
            organizationId: dto.organization ?? undefined,
            projectId: dto.project ?? this.convertError('Convert error: project is not defined'),
            userId: dto.user ?? undefined,

            // relation models
            department,
            organization,
            user,
          };

          return this.createConvertObservable(model);
        }),
      );
  }

  reverse(model: ProjectMemberModel): ProjectMemberDTO {
    throw new Error('Method not implemented.');
  }

  private convertMemberType(memberType: ProjectMemberDTO['member_type']): ProjectMemberModel['memberType'] {
    switch (memberType) {
      case 'GENERAL_CUSTOMER':
        return MemberTypeEnum.GENERAL_CUSTOMER;
      case 'CUSTOMER':
        return MemberTypeEnum.CUSTOMER;
      case 'CO_EXECUTOR':
        return MemberTypeEnum.CO_EXECUTOR;
      case 'CONTRACTOR':
        return MemberTypeEnum.CONTRACTOR;
      case 'SUPPLIER':
        return MemberTypeEnum.SUPPLIER;
      default:
        throw new Error('Unknown member type');
    }
  }
}
