import {Injectable} from '@angular/core';
import {forkJoin, Observable} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {ProjectMetaModel} from '../models/project-meta.model';
import {ProjectMetaDto} from '../models/project-meta.dto';
import {ProjectMetaDataTypeEnum} from '../models/project-meta-data-type.enum';
import {UserAdapterService} from '../../users/services/user-adapter.service';
import {UserModel} from '../../users';
import {ProjectTypeAdapterService} from './project-type-adapter.service';
import {ProjectTypeModel} from '../models/project-type.model';

@Injectable({
  providedIn: 'root',
})
export class ProjectMetaAdapterService extends BaseAdapter<ProjectMetaModel, ProjectMetaDto> {
  constructor(
    private readonly userAdapterService: UserAdapterService,
    private readonly projectTypeAdapterService: ProjectTypeAdapterService,
  ) {
    super();
  }

  convert(dto: ProjectMetaDto): Observable<ProjectMetaModel> {
    const owner$: Observable<UserModel | undefined> = this.createModelObs(dto.rel_owner, this.userAdapterService);
    const projectTypes$: Observable<ProjectTypeModel[]> = this.createModelArrObs(
      dto.rel_project_types,
      this.projectTypeAdapterService,
    );

    return forkJoin([
      owner$,
      projectTypes$,
    ])
      .pipe(
        switchMap(([owner, projectTypes]) => {
          const model: ProjectMetaModel = {
            id: dto.id ?? this.convertError('Convert error: id is not defined'),
            isAllTypes: dto.all_types ?? false,
            choices: dto.choices ?? undefined,
            created: dto.created ?? undefined,
            dataType: this.convertDataType(dto.data_type),
            description: dto.description ?? undefined,
            name: dto.name ?? this.convertError('Convert error: name is not defined'),
            order: dto.order ?? 0,
            postfix: dto.postfix ?? undefined,
            prefix: dto.prefix ?? undefined,
            isPublic: dto.public ?? false,
            unit: dto.unit ?? undefined,

            // ids
            companyId: dto.company ?? undefined,
            groupId: dto.group ?? undefined,
            ownerId: dto.owner ?? undefined,
            projectTypesIds: dto.project_types ?? [],

            // models
            owner,
            projectTypes,
          };

          return this.createConvertObservable(model);
        }),
      );
  }

  reverse(model: ProjectMetaModel): ProjectMetaDto {
    throw new Error('Method not implemented.');
  }

  private convertDataType(dataType: ProjectMetaDto['data_type']): ProjectMetaModel['dataType'] {
    switch (dataType) {
      case 'DT':
        return ProjectMetaDataTypeEnum.DT;
      case 'CH':
        return ProjectMetaDataTypeEnum.CH;
      case 'TXT':
        return ProjectMetaDataTypeEnum.TXT;
      case 'INT':
        return ProjectMetaDataTypeEnum.INT;
      case 'FLT':
        return ProjectMetaDataTypeEnum.FLT;
      case 'FK':
        return ProjectMetaDataTypeEnum.FK;
      case 'MTM':
        return ProjectMetaDataTypeEnum.MTM;
      case 'CHS':
        return ProjectMetaDataTypeEnum.CHS;
      case 'MCHS':
        return ProjectMetaDataTypeEnum.MCHS;
      case 'FLAG':
        return ProjectMetaDataTypeEnum.FLAG;
      case 'URL':
        return ProjectMetaDataTypeEnum.URL;
      case 'MONEY':
        return ProjectMetaDataTypeEnum.MONEY;
      default:
        return undefined;
    }
  }
}
