import {Injectable} from '@angular/core';
import {webSocket, WebSocketSubject} from 'rxjs/webSocket';
import {first, Observable, of, tap, throwError} from 'rxjs';
import {catchError, map, switchMap} from 'rxjs/operators';

import {WebsocketsNotificationModel, WebsocketsResponseType} from '../models/websockets-response.model';
import {WebsocketsTypesEnum} from '../models/websockets-types.enum';
import {NotificationAdapterService} from '../../notifications/services/notification-adapter.service';
import {WebsocketsResponseDTOType} from '../models/websockets-response.dto';

@Injectable()
export class WebsocketsApiService {
  private websocketSubject: WebSocketSubject<WebsocketsResponseDTOType> | undefined;
  private websocket$: Observable<WebsocketsResponseType> | undefined;
  private firstConnection = true;

  constructor(
    private readonly notificationAdapterService: NotificationAdapterService,
  ) {
  }

  connect(url: string, token: string): Observable<WebsocketsResponseType> {
    if (!this.checkUrl(url)) {
      return throwError(() => new Error('Url is not valid!'));
    }

    if (!this.firstConnection && this.websocketSubject) {
      console.warn('Websocket is already initialized!');
      console.warn('Connection will be destroyed and recreated!');
      this.destroyWsConnection();
    }

    this.websocketSubject = this.createWsConnection(url);
    this.websocket$ = this.websocketSubject
      .pipe(
        switchMap((response) => {
          if (response.type === WebsocketsTypesEnum.NOTIFICATION) {
            return this.notificationAdapterService.convert(response.data)
              .pipe(
                map((notification) => ({
                  type: WebsocketsTypesEnum.NOTIFICATION,
                  data: notification,
                } as WebsocketsNotificationModel)),
              );
          }

          return of({
            type: response.type,
            data: response.data,
          });
        }),
        catchError((error) => {
          console.error(error);
          // TODO message
          return throwError(() => error);
        }),
      );

    this.websocket$.pipe(
      first(),
      tap(() => {
        this.send(token);
      }),
    ).subscribe();

    return this.websocket$
      .pipe(
        tap(() => {
          if (this.firstConnection) {
            this.firstConnection = false;
          }
        }),
      );
  }

  disconnect() {
    this.destroyWsConnection();
  }

  /**
   * Отправка данных на сервер.
   *
   */
  send(data: string | { [key: string]: unknown }) {
    if (this.websocketSubject) {
      this.websocketSubject.next(data as any);
    } else {
      console.error('Websocket is not initialized! Cannot send data!');
    }
  }

  private createWsConnection(url: string): WebSocketSubject<WebsocketsResponseDTOType> {
    return webSocket({
      url,
      deserializer: (event) => JSON.parse(event.data),
      serializer: (event: unknown) => typeof event === 'string' ? event : JSON.stringify(event),
    });
  }

  private destroyWsConnection() {
    if (this.websocketSubject) {
      this.websocketSubject.complete();
      this.websocketSubject = undefined;
      this.websocket$ = undefined;
    } else {
      console.error('Cant destroy websocket connection! Websockets is not initialized!');
    }
  }

  private checkUrl(url: string): boolean {
    if (!url || url.length === 0) {
      console.error('Url is empty!');
      return false;
    }

    if (url.indexOf('ws:') < 0 && url.indexOf('wss:') < 0 && url.indexOf('http:') < 0 && url.indexOf('https:') < 0) {
      console.error('Url is not valid! Add protocol!');
      return false;
    }

    if (url[url.length - 1] === '/') {
      console.error('Url is not valid! Delete last slash!');
      return false;
    }

    return true;
  }
}
