import {Injectable} from '@angular/core';
import {Observable, withLatestFrom} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {NewWorkModel, WorkModel} from '../models/work.model';
import {WorkDTO} from '../models/work.dto';
import {CommentAdapterService} from '../../comments/services/comment-adapter.service';
import {UserBaseAdapterService} from '../../users/services/user-base-adapter.service';
import {CommentModel} from '../../comments';
import {UserBaseModel} from '../../users';
import {StageModel} from '../../stages';
import {ProjectAdapterService} from '../../projects/services/project-adapter';
import {ProjectModel} from '../../projects';
import {StageAdapterService} from '../../stages/services/stage-adapter.service';
import {WorkBaseAdapterService} from './work-base-adapter.service';
import {IssueBaseModel} from '../../issues';
import {WorkBaseModel} from '../models/work-base.model';
import {IssueBaseAdapterService} from '../../issues/services/issue-base-adapter.service';

@Injectable({
  providedIn: 'root',
})
export class WorkAdapterService extends BaseAdapter<WorkModel, WorkDTO> {
  constructor(
    private readonly commentAdapterService: CommentAdapterService,
    private readonly userBaseAdapterService: UserBaseAdapterService,
    private readonly stageAdapterService: StageAdapterService,
    private readonly projectAdapterService: ProjectAdapterService,
    private readonly workBaseAdapterService: WorkBaseAdapterService,
    private readonly issueBaseAdapterService: IssueBaseAdapterService,
  ) {
    super();
  }

  convert(dto: WorkDTO): Observable<WorkModel> {
    const comments$: Observable<CommentModel[]> = this.createModelArrObs(dto.rel_comments, this.commentAdapterService);
    const createdBy$: Observable<UserBaseModel | undefined> = this.createModelObs(dto.rel_created_by, this.userBaseAdapterService);
    const doneBy$: Observable<UserBaseModel | undefined> = this.createModelObs(dto.rel_done_by, this.userBaseAdapterService);
    const group$: Observable<WorkBaseModel | undefined> = this.createModelObs(dto.rel_group, this.workBaseAdapterService);
    const issues$: Observable<IssueBaseModel[]> = this.createModelArrObs(dto.rel_issues, this.issueBaseAdapterService);
    const milestoneStage$: Observable<StageModel | undefined> = this.createModelObs(dto.rel_milestone_stage, this.stageAdapterService);
    const parentWork$: Observable<WorkBaseModel | undefined> = this.createModelObs(dto.rel_parent, this.workBaseAdapterService);
    const parentIssue$: Observable<IssueBaseModel | undefined> = this.createModelObs(dto.rel_parent_issue, this.issueBaseAdapterService);
    const project$: Observable<ProjectModel | undefined> = this.createModelObs(dto.rel_project, this.projectAdapterService);
    const subworks$: Observable<WorkBaseModel[]> = this.createModelArrObs(dto.rel_subworks, this.workBaseAdapterService);
    const valBy$: Observable<UserBaseModel | undefined> = this.createModelObs(dto.rel_val_by, this.userBaseAdapterService);
    const valUser$: Observable<UserBaseModel | undefined> = this.createModelObs(dto.rel_val_user, this.userBaseAdapterService);

    return this.workBaseAdapterService.convert(dto)
      .pipe(
        withLatestFrom(
          comments$,
          createdBy$,
          doneBy$,
          group$,
          issues$,
          milestoneStage$,
          parentWork$,
          parentIssue$,
          project$,
          subworks$,
          valBy$,
          valUser$,
        ),
        switchMap(([
                     workBase,
                     comments,
                     createdBy,
                     doneBy,
                     group,
                     issues,
                     milestoneStage,
                     parentWork,
                     parentIssue,
                     project,
                     subworks,
                     valBy,
                     valUser,
                   ]) => {
          const model: WorkModel = {
            ...workBase,
            isApproved: dto.approved ?? false,
            order: dto.order ?? 0,

            // relation ids
            commentIds: dto.comments ?? [],
            companyId: dto.company ?? undefined,
            createdById: dto.created_by ?? undefined,
            doneById: dto.done_by ?? undefined,
            groupId: dto.group ?? undefined,
            issuesIds: dto.issues ?? [],
            milestoneStageId: dto.milestone_stage ?? undefined,
            parentWorkId: dto.parent ?? undefined,
            parentIssueId: dto.parent_issue ?? undefined,
            projectId: dto.project ?? undefined,
            subWorkIds: dto.subworks ?? [], // child works
            templateWorkId: dto.template_work ?? undefined,
            valById: dto.val_by ?? undefined,
            valUserId: dto.val_user ?? undefined,
            workIds: dto.works ?? [], // not child works! back link from work

            // relation models
            comments,
            createdBy,
            doneBy,
            group,
            issues,
            milestoneStage,
            parentWork,
            parentIssue,
            project,
            subworks,
            valBy,
            valUser,
          };

          return this.createConvertObservable(model);
        }),
      );
  }

  reverse(model: WorkModel): WorkDTO {
    return {
      id: model.id,
      deadline: model.deadline ?? null,
      description: model.description ?? null,
      end_base: model.endBase ?? null,
      name: model.name,
      priority: model.priority,
      start_base: model.startBase ?? null,
      status: model.status,
      work_type: model.workType,

      // relation ids
      to_department: model.toDepartmentId ?? null,
      to_user: model.toUserId ?? null,
      val_user: model.valUserId ?? null,
    };
  }

  reverseToNewModel(model: NewWorkModel): WorkDTO {
    return {
      deadline: model.deadline,
      description: model.description,
      end_base: model.endBase,
      name: model.name,
      priority: model.priority,
      start_base: model.startBase,
      status: model.status,
      
      // relation ids
      parent: model.parentWorkId,
      parent_issue: model.parentIssueId,
      project: model.projectId,
      to_department: model.toDepartmentId,
      to_user: model.toUserId,
      val_user: model.valUserId,
    };
  }
}
