import { Injectable } from '@angular/core';
import { map, switchMap } from 'rxjs/operators';
import { catchError, forkJoin, Observable, throwError } from 'rxjs';

import { Comment, Work, WorkChangeStatusAndOrder, WorksService } from 'data-access/sdk/api2';

import { WorkAdapterService } from './work-adapter.service';
import { NewWorkModel, WorkModel } from '../models/work.model';
import { CommentModel, NewCommentModel } from '../../comments';
import { CommentAdapterService } from '../../comments/services/comment-adapter.service';
import * as moment from 'moment/moment';
import { WorkStatusEnum } from '../models/work-status.enum';
import { WorkBaseAdapterService } from './work-base-adapter.service';

@Injectable({
  providedIn: 'root',
})
export class WorkApiService {
  constructor(
    private readonly worksService: WorksService,
    private readonly workAdapterService: WorkAdapterService,
    private readonly workBaseAdapterService: WorkBaseAdapterService,
    private readonly commentAdapterService: CommentAdapterService
  ) {}

  createWork(newWorkModel: NewWorkModel) {
    const data: Work = this.workAdapterService.reverseToNewModel(newWorkModel);
    return this.worksService.worksCreate(data).pipe(
      catchError((error) => {
        console.error('Error while updating work', error);
        // TODO #errorService
        return throwError(() => error);
      }),
      switchMap((workResponse) => this.workAdapterService.convert(workResponse))
    );
  }

  deleteWork(id: WorkModel['id']) {
    return this.worksService.worksDelete(id).pipe(
      catchError((error) => {
        console.error('Error while deleting work', error);
        // TODO #errorService
        return throwError(() => error);
      })
    );
  }

  readWork(id: WorkModel['id']) {
    return this.worksService.worksRead({ id, include: 'comments__author, subworks, issues, group, project, parent, parent_issue' }).pipe(
      catchError((error) => {
        console.error('Error while getting work', error);
        // TODO #errorService
        return throwError(() => error);
      }),
      switchMap((work) => this.workAdapterService.convert(work)),
      map((work) => ({
        ...work,
        comments: this.sortComments(work.comments),
      }))
    );
  }

  readList(params: WorksService.WorksListParams = {}) {
    return this.worksService.worksList(params).pipe(
      catchError((error) => {
        console.error('Error while getting works', error);
        // TODO #errorService
        return throwError(() => error);
      }),
      switchMap((works) => forkJoin(works.map((work) => this.workAdapterService.convert(work))))
    );
  }

  changeWorkStatus(workId: WorkModel['id'], status: WorkStatusEnum) {
    const data = { status: this.workBaseAdapterService.reverseStatus(status) };
    return this.worksService.worksChangeStatus({ id: workId, data }).pipe(
      catchError((error) => {
        console.error('Error while updating work', error);
        // TODO #errorService
        return throwError(() => error);
      }),
      switchMap((workResponse) => this.workAdapterService.convert(workResponse))
    );
  }

  changeWorkStatusAndOrderDepartment(id: number, nextWork: number, previusWork: number, status: WorkModel['status']) {
    const data: WorkChangeStatusAndOrder = { next_work: nextWork, previous_work: previusWork, status, order_type: 'DEPARTMENT' };
    return this.worksService.worksChangeStatusAndOrder({ id, data });
  }

  changeWorkStatusAndOrderProject(
    id: WorkModel['id'],
    nextWork: WorkChangeStatusAndOrder['next_work'],
    previusWork: WorkChangeStatusAndOrder['previous_work'],
    status: WorkChangeStatusAndOrder['status']
  ) {
    const data: WorkChangeStatusAndOrder = { next_work: nextWork, previous_work: previusWork, status, order_type: 'PROJECT' };
    return this.worksService.worksChangeStatusAndOrder({ id, data });
  }

  updateWork(work: WorkModel) {
    const id = work.id;
    const data: Work = this.workAdapterService.reverse(work);
    return this.worksService.worksPartialUpdate({ id, data }).pipe(
      catchError((error) => {
        console.error('Error while updating work', error);
        // TODO #errorService
        return throwError(() => error);
      }),
      switchMap((workResponse) => this.workAdapterService.convert(workResponse))
    );
  }

  readWorkPermissions(workId: number): Observable<string[]> {
    return this.worksService.worksPermissions(workId).pipe(
      catchError((error) => {
        console.error('Error while getting permissions', error);
        // TODO #errorService
        return throwError(() => error);
      }),
      map((permissions) => (permissions.permissions?.filter((p) => !!p) || []) as string[])
    );
  }

  createComment(workId: WorkModel['id'], comment: NewCommentModel) {
    const data: Comment = this.commentAdapterService.reverseToNewModel(comment);

    return this.worksService.worksAddComment({ id: workId, data }).pipe(
      catchError((error) => {
        console.error('Error while creating comment', error);
        // TODO #errorService
        return throwError(() => error);
      }),
      switchMap((commentResponse) => this.commentAdapterService.convert(commentResponse))
    );
  }

  private sortComments(comments: CommentModel[]) {
    return comments.sort((a, b) => {
      const dateA = moment(a.created);
      const dateB = moment(b.created);

      if (dateB.isBefore(dateA)) {
        return 1;
      }

      if (dateB.isAfter(dateA)) {
        return -1;
      }

      return 0;
    });
  }
}
