import { Injectable } from '@angular/core';
import { forkJoin, Observable } from 'rxjs';
import { switchMap } from 'rxjs/operators';

import { BaseAdapter } from 'utils/shared-services';

import { NewWorkModel } from '../models/work.model';
import { WorkDTO } from '../models/work.dto';
import { WorkTypeEnum } from '../models/work-type.enum';
import { UserBaseAdapterService } from '../../users/services/user-base-adapter.service';
import { UserBaseModel } from '../../users';
import { DepartmentBaseModel } from '../../departments';
import { WorkStatusEnum } from '../models/work-status.enum';
import { WorkBaseDTO } from '../models/work-base.dto';
import { WorkBaseModel } from '../models/work-base.model';
import { DepartmentBaseAdapterService } from '../../departments/services/department-base-adapter.service';

@Injectable({
  providedIn: 'root',
})
export class WorkBaseAdapterService extends BaseAdapter<WorkBaseModel, WorkBaseDTO> {
  constructor(
    private readonly userBaseAdapterService: UserBaseAdapterService,
    private readonly departmentBaseAdapterService: DepartmentBaseAdapterService
  ) {
    super();
  }

  convert(dto: WorkBaseDTO): Observable<WorkBaseModel> {
    const toDepartment$: Observable<DepartmentBaseModel | undefined> = this.createModelObs(
      dto.rel_to_department,
      this.departmentBaseAdapterService
    );
    const toUser$: Observable<UserBaseModel | undefined> = this.createModelObs(dto.rel_to_user, this.userBaseAdapterService);

    return forkJoin([toDepartment$, toUser$]).pipe(
      switchMap(([toDepartment, toUser]) => {
        const model: WorkBaseModel = {
          id: dto.id ?? this.convertError('Convert error: id is not defined'),
          createdAt: dto.created_at ?? undefined,
          deadline: dto.deadline ?? undefined,
          description: dto.description ?? undefined,
          endBase: dto.end_base ?? undefined,
          endDiff: dto.end_diff ?? undefined,
          endEff: dto.end_eff ?? undefined,
          endFact: dto.end_fact ?? undefined,
          endVal: dto.end_val ?? undefined,
          milestoneLevel: dto.milestone_level ?? undefined,
          name: dto.name ?? this.convertError('Convert error: name is not defined'),
          overdue: dto.overdue ?? false,
          priority: dto.priority ?? undefined,
          projectName: dto.project_name ?? undefined,
          result: dto.result ?? undefined,
          startBase: dto.start_base ?? undefined,
          startDiff: dto.start_diff ?? undefined,
          startFact: dto.start_fact ?? undefined,
          startEff: dto.start_eff ?? undefined,
          status: this.convertStatus(dto.status) ?? this.convertError('Convert error: status is not defined'),
          workType: this.convertWorkType(dto.work_type),

          // relation ids
          toDepartmentId: dto.to_department ?? undefined,
          toUserId: dto.to_user ?? undefined,

          // relation models
          toDepartment,
          toUser,
        };

        return this.createConvertObservable(model);
      })
    );
  }

  reverse(model: WorkBaseModel): WorkBaseDTO {
    throw new Error('Method not implemented.');
  }

  reverseToNewModel(model: NewWorkModel): WorkBaseDTO {
    return {
      name: model.name,
      end_base: model.endBase,

      // relation ids
      to_department: model.toDepartmentId,
      to_user: model.toUserId,
    };
  }

  reverseStatus(status: WorkStatusEnum): WorkDTO['status'] {
    switch (status) {
      case WorkStatusEnum.BL:
        return 'BL';
      case WorkStatusEnum.CR:
        return 'CR';
      case WorkStatusEnum.WP:
        return 'WP';
      case WorkStatusEnum.DN:
        return 'DN';
      case WorkStatusEnum.AC:
        return 'AC';
      case WorkStatusEnum.CN:
        return 'CN';
      case WorkStatusEnum.PS:
        return 'PS';
      default:
        throw new Error('Unknown status', status);
    }
  }

  private convertStatus(status: WorkBaseDTO['status']): WorkStatusEnum | undefined {
    switch (status) {
      case 'BL':
        return WorkStatusEnum.BL;
      case 'CR':
        return WorkStatusEnum.CR;
      case 'WP':
        return WorkStatusEnum.WP;
      case 'DN':
        return WorkStatusEnum.DN;
      case 'AC':
        return WorkStatusEnum.AC;
      case 'CN':
        return WorkStatusEnum.CN;
      case 'PS':
        return WorkStatusEnum.PS;
      default:
        return undefined;
    }
  }

  private convertWorkType(workType: WorkBaseDTO['work_type']): WorkTypeEnum | undefined {
    switch (workType) {
      case 'TS':
        return WorkTypeEnum.TS;
      case 'GR':
        return WorkTypeEnum.GR;
      case 'MS':
        return WorkTypeEnum.MS;
      case 'AS':
        return WorkTypeEnum.AS;
      default:
        return undefined;
    }
  }
}
