import { Action, State, StateContext, Store } from '@ngxs/store';
import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { finalize, forkJoin, of, tap, throwError, withLatestFrom } from 'rxjs';
import { switchMap } from 'rxjs/operators';
import { MessageService } from 'primeng/api';

import {
  HistoryApiService,
  IssueApiService,
  IssueModel,
  LinksAdapterService,
  NewCommentModel,
  ProjectsApiService,
  UserApiService,
  WorkModel,
} from 'data-access/sdk/internal-api';
import { getIssueCreateRouterLink, getIssueCreateRouterLinkQueryParams, getIssueRouterLink } from 'utils/core-project-routing-utils';

import { IssuesStateModel } from './issues-state.model';
import { IssuesActions } from './issues.actions';
import { WorksActions } from '../works';
import { GlobalSelectors } from '../app-global';
import { RightSidebarActions } from '../right-sidebar';
import { GlobalRulesSelectors } from '../app-global-rules';
import { Permission } from '../app-global-rules/models/permission.model';
import { IssuesSelectors } from './issues.selectors';
import { ISSUES_STATE_TOKEN } from './issue-state.token.ts';

const getDefaultState: () => IssuesStateModel = () => ({
  canDelete: true,
  canUpdate: true,
  canCreateComment: true,
  canCreateWork: true,
  issueHistory: [],
  issue: undefined,
  isIssueLoading: false,
  isCommentsLoading: false,
  isOptionsLoading: false,
  issueDomainsOptions: [],
  links: [],
  usersOptions: [],
  projectsOptions: [],
});

@State({
  name: ISSUES_STATE_TOKEN,
  defaults: getDefaultState(),
})
@Injectable()
export class IssuesState {
  linksObsSub = this.store
    .select(IssuesSelectors.issue)
    .pipe(
      switchMap((issue) => {
        if (!issue) {
          return of(undefined);
        }

        return this.store.dispatch(new IssuesActions.UpdateLinksView(issue));
      })
    )
    .subscribe();

  blockUISub = this.store
    .select(IssuesSelectors.isIssueLoading)
    .pipe(switchMap((isLoading) => this.store.dispatch(new RightSidebarActions.UpdateBlockUI(isLoading))))
    .subscribe();

  constructor(
    private readonly historyApiService: HistoryApiService,
    private readonly issueApiService: IssueApiService,
    private readonly messageService: MessageService,
    private readonly projectApiService: ProjectsApiService,
    private readonly router: Router,
    private readonly store: Store,
    private readonly translateService: TranslateService,
    private readonly userApiService: UserApiService,
    private readonly linksAdapterService: LinksAdapterService
  ) {}

  @Action(IssuesActions.ShowIssueInSidebar)
  showIssueInSidebar(ctx: StateContext<IssuesStateModel>, { id }: IssuesActions.ShowIssueInSidebar) {
    this.router.navigate(getIssueRouterLink(String(id)));
  }

  @Action(IssuesActions.ReadIssue)
  readIssue(ctx: StateContext<IssuesStateModel>, { id }: IssuesActions.ReadIssue) {
    ctx.patchState({ isIssueLoading: true });
    ctx.dispatch(new IssuesActions.ReadIssueHistory(id));

    return forkJoin([
      this.issueApiService.readIssue(id),
      ctx.dispatch(new IssuesActions.ReadPermissions(id)),
      ctx.dispatch(new IssuesActions.ReadFieldsOptions()),
    ]).pipe(
      finalize(() => ctx.patchState({ isIssueLoading: false })),
      tap(([issue]) => {
        ctx.patchState({
          issue,
        });
      })
    );
  }

  @Action(IssuesActions.RefreshIssue)
  refreshIssue(ctx: StateContext<IssuesStateModel>, { id }: IssuesActions.RefreshIssue) {
    if (ctx.getState().issue?.id === id) {
      return ctx.dispatch(new IssuesActions.ReadIssue(id));
    }

    return of(undefined);
  }

  @Action(IssuesActions.OpenWork)
  openWork(ctx: StateContext<IssuesStateModel>, { workId }: IssuesActions.OpenWork) {
    return ctx.dispatch(new WorksActions.ShowWorkInSidebar(workId));
  }

  @Action(IssuesActions.UpdateLinksView)
  updateLinksView(ctx: StateContext<IssuesStateModel>, { issue }: IssuesActions.UpdateLinksView) {
    const works: WorkModel[] = JSON.parse(JSON.stringify(issue.works));
    const parentWork$ = issue.parentWork ? this.linksAdapterService.convertFromWork(issue.parentWork, true) : of(undefined);

    return (works && works.length > 0 ? forkJoin(works.map((work) => this.linksAdapterService.convertFromWork(work))) : of([])).pipe(
      withLatestFrom(parentWork$),
      tap(([links, parentWork]) => {
        ctx.patchState({
          links: [...(parentWork ? [parentWork] : []), ...links],
        });
      })
    );
  }

  @Action(IssuesActions.ReadPermissions)
  readPermissions(ctx: StateContext<IssuesStateModel>, { issueId }: IssuesActions.ReadPermissions) {
    return this.issueApiService.readIssuePermissions(issueId).pipe(
      tap((permissions) => {
        const issuePermission = Permission.createPermissions(permissions).find((p) => p.modelName === 'issue');

        ctx.patchState({
          canDelete: issuePermission?.canDelete || false,
          canUpdate: issuePermission?.canUpdate || false,
          canCreateComment: this.store.selectSnapshot(GlobalRulesSelectors.canCreate('comment')),
          canCreateWork: this.store.selectSnapshot(GlobalRulesSelectors.canCreate('work')),
        });
      })
    );
  }

  @Action(IssuesActions.ReadIssueHistory)
  readIssueHistory(ctx: StateContext<IssuesStateModel>, { id }: IssuesActions.ReadIssueHistory) {
    return this.historyApiService.readIssueHistory(id).pipe(
      tap((historyActions) => {
        ctx.patchState({
          issueHistory: historyActions,
        });
      })
    );
  }

  @Action(IssuesActions.ReadFieldsOptions)
  readFieldsOptions(ctx: StateContext<IssuesStateModel>) {
    ctx.patchState({ isOptionsLoading: true });

    return forkJoin([
      this.issueApiService.readIssueDomainsList(),
      this.userApiService.readUsers(),
      this.projectApiService.readProjects(),
    ]).pipe(
      finalize(() => ctx.patchState({ isOptionsLoading: false })),
      tap(([issueDomainsOptions, usersOptions, projectsOptions]) => {
        ctx.patchState({
          issueDomainsOptions,
          usersOptions,
          projectsOptions,
        });
      })
    );
  }

  @Action(IssuesActions.CreateIssue)
  createIssue(ctx: StateContext<IssuesStateModel>, { issue }: IssuesActions.CreateIssue) {
    ctx.patchState({ isIssueLoading: true });

    const projectId = issue.projectId;

    if (!projectId) {
      const canCreate = this.store.selectSnapshot(GlobalRulesSelectors.canCreate('issue'));

      if (!canCreate) {
        this.messageService.add({
          severity: 'error',
          summary: this.translateService.instant('ERRORS.COMMON_ERROR_NO_RULES'),
          detail: this.translateService.instant('ERRORS.COMMON_ERROR_NO_RULES_MSG'),
        });

        return throwError(() => new Error('You dont have permission to create issue (1).'));
      }
    }

    const request$ = projectId
      ? this.issueApiService.readProjectPermissions(projectId).pipe(
          finalize(() => ctx.patchState({ isIssueLoading: false })),
          switchMap((permissions) => {
            const canCreate = Permission.createPermissions(permissions).find((p) => p.modelName === 'issue')?.canCreate || false;

            if (!canCreate) {
              this.messageService.add({
                severity: 'error',
                summary: this.translateService.instant('ERRORS.COMMON_ERROR_NO_RULES'),
                detail: this.translateService.instant('ERRORS.COMMON_ERROR_NO_RULES_MSG'),
              });

              return throwError(() => new Error('You dont have permission to create issue (2).'));
            }

            return of(undefined);
          }),
          switchMap(() => this.issueApiService.createIssue(issue))
        )
      : this.issueApiService.createIssue(issue);

    return request$.pipe(
      finalize(() => ctx.patchState({ isIssueLoading: false })),
      tap((issueResponse) => {
        ctx.patchState({
          issue: issueResponse,
        });
      }),
      tap((issueResponse) => ctx.dispatch(new IssuesActions.ShowIssueInSidebar(issueResponse.id)))
    );
  }

  @Action(IssuesActions.OpenCreateWorkModal)
  openCreateWorkModal(ctx: StateContext<IssuesStateModel>) {
    if (!ctx.getState().issue) {
      throw new Error('DEV_ERROR: Issue cant be undefined here');
    }

    // issue cant be undefined here
    const issue: IssueModel = JSON.parse(JSON.stringify(ctx.getState().issue));
    const callback = (work: WorkModel) => {
      ctx.patchState({
        issue: {
          ...issue,
          works: [...issue.works, work],
          workIds: [...issue.workIds, work.id],
        },
      });
    };

    return ctx.dispatch(new WorksActions.OpenWorkCreateModalShared({ parentIssueId: issue.id }, callback));
  }

  @Action(IssuesActions.ChangeStatus)
  onChangeStatus(ctx: StateContext<IssuesStateModel>, { status }: IssuesActions.ChangeStatus) {
    // issue cant be undefined here
    const issue: IssueModel = JSON.parse(JSON.stringify(ctx.getState().issue));

    const issueWithNewStatus: IssueModel = {
      ...issue,
      status,
    };

    return this.issueApiService.updateIssue(issueWithNewStatus).pipe(
      tap((issueResponse) => {
        ctx.patchState({
          issue: {
            ...issueWithNewStatus,
            status: issueResponse.status,
          },
        });
      }),
      finalize(() => {
        ctx.dispatch(new IssuesActions.ReadIssueHistory(issue.id));
      })
    );
  }

  @Action(IssuesActions.DeleteIssue)
  deleteIssue(ctx: StateContext<IssuesStateModel>, { id }: IssuesActions.DeleteIssue) {
    ctx.patchState({ isIssueLoading: true });
    return this.issueApiService.deleteIssue(id).pipe(
      finalize(() => ctx.patchState({ isIssueLoading: false })),
      tap(() => {
        // TODO add toast
        ctx.dispatch(new RightSidebarActions.Close());
      })
    );
  }

  @Action(IssuesActions.CreateComment)
  createComment(ctx: StateContext<IssuesStateModel>, { text, replyToId }: IssuesActions.CreateComment) {
    // ctx.patchState({isCommentsLoading: true});

    // issue can't be undefined here
    const issue: IssueModel = JSON.parse(JSON.stringify(ctx.getState().issue));

    const comment: NewCommentModel = {
      text,
      authorId: this.store.selectSnapshot(GlobalSelectors.userId),
      replyToId,
    };
    return this.issueApiService.createComment(issue.id, comment).pipe(
      // finalize(() => ctx.patchState({isCommentsLoading: false})),
      tap((commentResponse) => {
        // add author bcs we don't have it in response (no include)
        if (!commentResponse.author) {
          const usersOptions = ctx.getState().usersOptions;
          commentResponse.author = usersOptions.find((user) => user.id === commentResponse.authorId);
        }

        ctx.patchState({
          issue: {
            ...issue,
            comments: [...issue.comments, commentResponse],
          },
        });
      })
    );
  }

  @Action(IssuesActions.UpdateIssue)
  updateIssue(ctx: StateContext<IssuesStateModel>, { issue }: IssuesActions.UpdateIssue) {
    return this.issueApiService.updateIssue(issue).pipe(switchMap(() => ctx.dispatch(new IssuesActions.ReadIssue(issue.id))));
  }

  @Action(IssuesActions.ShowCreateIssueInSidebar)
  showCreateIssueInSidebar(
    ctx: StateContext<IssuesStateModel>,
    { projectId, authorId, parentWorkId }: IssuesActions.ShowCreateIssueInSidebar
  ) {
    const userId = this.store.selectSnapshot(GlobalSelectors.userId);
    this.router.navigate(getIssueCreateRouterLink(), {
      queryParams: getIssueCreateRouterLinkQueryParams(projectId, authorId ?? userId, parentWorkId),
    });
  }

  @Action(IssuesActions.DestroyState)
  destroyIssueState(ctx: StateContext<IssuesStateModel>) {
    ctx.setState(getDefaultState());
  }
}
