import {Injectable} from '@angular/core';
import {Router} from '@angular/router';
import {Action, State, StateContext, Store} from '@ngxs/store';
import {TranslateService} from '@ngx-translate/core';
import {debounceTime, filter, map, of, tap, throwError} from 'rxjs';
import {catchError, switchMap} from 'rxjs/operators';
import {chunk, isArray} from 'lodash';

import {DialogService} from 'primeng/dynamicdialog';
import {MessageService} from 'primeng/api';

import {IssuesService, WorksService} from 'data-access/sdk/api2';
import {NotificationApiService, NotificationModel, NotificationTypeEnum} from 'data-access/sdk/internal-api';
import {INotificationMessage} from 'features/notifications'; // TODO delete dependency

import {NotificationsStateModel, stateToken} from './notifications-state.model';
import {NotificationsActions} from './notifications.actions';
import {NotificationsSelectors} from './notifications.selectors';
import {GlobalSelectors} from '../app-global';
import {NOTIFICATION_PAGE_SIZE} from './notifications.constants';
import {IssuesActions} from '../issues';
import {WorksActions} from '../works';


@State({
  name: stateToken,
  defaults: {
    queue: [],
    notifications: [],
    notificationsPageSize: NOTIFICATION_PAGE_SIZE,
    unreadNotificationsCount: 0,
  },
})
@Injectable()
export class NotificationsState {
  readonly notificationDebounce = 1000;
  readonly notificationLife = 7000;

  queueSub = this.store.select(NotificationsSelectors.notificationsQueue)
    .pipe(
      filter(queue => queue && queue.length > 0),
      debounceTime(this.notificationDebounce),
      map((notifications) => {
        const messages: INotificationMessage[] = [];

        notifications.forEach((notification) => {
          messages.push({
            key: 'ws-notification-toast',
            id: notification.id,
            summary: this.translateService.instant(notification.nameTemplate, notification.nameParams),
            detail: this.translateService.instant(notification.descriptionTemplate, notification.descriptionParams),
            severity: notification.severity,
            life: this.notificationLife,
            data: notification,
            closable: false,
          });
        });

        return messages;
      }),
      tap((messages) => {
        this.store.dispatch(new NotificationsActions.ClearQueue());

        // create chunks of 5 messages
        if (messages.length > 5) {
          chunk(messages, 5).forEach((messagesChunk: INotificationMessage[], index: number) => {
            if (index !== 0) {
              setTimeout(() => {
                this.messageService.addAll(messagesChunk);
              }, index * this.notificationLife);
            } else {
              this.messageService.addAll(messagesChunk);
            }
          });
        } else {
          this.messageService.addAll(messages);
        }
      }),
    ).subscribe();

  notificationsSub = this.store.select(GlobalSelectors.userId)
    .pipe(
      filter(userId => !!userId && userId > 0),
      switchMap(() => this.store.dispatch(new NotificationsActions.GetNotifications())),
    ).subscribe();

  constructor(
    private readonly apiService: NotificationApiService,
    private readonly store: Store,
    private readonly messageService: MessageService,
    private readonly translateService: TranslateService,
    private readonly router: Router,
    private readonly dialogService: DialogService,
    private readonly worksService: WorksService, // TODO delete link to SDK
    private readonly issuesService: IssuesService, // TODO delete link to SDK
  ) {
  }

  @Action(NotificationsActions.AddToQueue)
  addToQueue(ctx: StateContext<NotificationsStateModel>, {payload}: NotificationsActions.AddToQueue) {
    ctx.patchState({
      queue: [...ctx.getState().queue, ...(isArray(payload) ? payload : [payload])],
      // add notifications without new request
      notifications: this.sortNotifications([...ctx.getState().notifications, ...(isArray(payload) ? payload : [payload])]),
    });

    return ctx.dispatch(new NotificationsActions.GetNewNotificationsCount());
  }

  @Action(NotificationsActions.ClearQueue)
  clearQueue(ctx: StateContext<NotificationsStateModel>) {
    try {
      ctx.patchState({
        queue: [],
      });
    } catch (error) {
      // TODO #errorService
      console.error('Error while clearing queue', error);
    }
  }

  @Action(NotificationsActions.MarkAllAsRead)
  markAllAsRead(ctx: StateContext<NotificationsStateModel>) {
    return this.apiService.markAllAsRead()
      .pipe(
        catchError((error) => {
          console.error('Error while marking all notifications as read', error);
          // TODO #errorService
          return throwError(() => error);
        }),
        tap(() => {
          const notifications: NotificationModel[] = JSON.parse(JSON.stringify(ctx.getState().notifications));

          notifications.forEach(notification => {
            notification.isUnread = false;
          });

          ctx.patchState({
            notifications: this.sortNotifications(notifications),
          });
        }),
        switchMap(() => ctx.dispatch(new NotificationsActions.GetNewNotificationsCount())),
      );
  }

  @Action(NotificationsActions.SendDebugNotification)
  sendDebugNotification(ctx: StateContext<NotificationsStateModel>, {notification}: NotificationsActions.SendDebugNotification) {
    this.apiService.sendTestNotification(notification)
      .pipe(
        catchError((error) => {
          console.error('Error while sending debug notification', error);
          // TODO #errorService
          return throwError(() => error);
        }),
      )
      .subscribe();
  }

  @Action(NotificationsActions.GetNotifications)
  getNotifications(ctx: StateContext<NotificationsStateModel>, {pageIndex, pageSize}: NotificationsActions.GetNotifications) {
    return this.apiService.getNotifications(pageIndex, pageSize)
      .pipe(
        catchError((error) => {
          console.error('Error while getting notifications', error);
          // TODO #errorService
          return of([]);
        }),
        tap(newNotifications => {
          ctx.patchState({
            notifications: this.sortNotifications(newNotifications),
          });
        }),
        switchMap(() => ctx.dispatch(new NotificationsActions.GetNewNotificationsCount())),
      );
  }

  @Action(NotificationsActions.GetNewNotificationsCount)
  getNewNotificationsCount(ctx: StateContext<NotificationsStateModel>) {
    return this.apiService.getNewNotificationsCount()
      .pipe(
        catchError((error) => {
          console.error('Error while getting new notifications count', error);
          // TODO #errorService
          return of(0);
        }),
        tap((count) => {
          ctx.patchState({
            unreadNotificationsCount: count,
          });
        }),
      );
  }

  @Action(NotificationsActions.MarkAsRead)
  markAsRead(ctx: StateContext<NotificationsStateModel>, {id}: NotificationsActions.MarkAsRead) {
    const notifications: NotificationModel[] = JSON.parse(JSON.stringify(ctx.getState().notifications));
    const notificationIndex = notifications.findIndex(notification => notification.id === id);

    if (!notifications[notificationIndex].isUnread) {
      console.warn('Notification to mark already marked as read');
      return of(undefined);
    }

    return this.apiService.markAsRead(id)
      .pipe(
        tap(() => {
          if (notifications[notificationIndex]) {
            notifications[notificationIndex].isUnread = false;

            ctx.patchState({
              notifications: this.sortNotifications(notifications),
            });
          } else {
            // TODO #errorService
            console.error('Notification to mark not found in store.');
          }
        }),
        switchMap(() => ctx.dispatch(new NotificationsActions.GetNewNotificationsCount())),
      );
  }

  @Action(NotificationsActions.ShowEntity)
  showEntity(ctx: StateContext<NotificationsStateModel>, {notification}: NotificationsActions.ShowEntity) {
    switch (notification.urlType) {
      case NotificationTypeEnum.ProjectInfo: {
        const projectId: number | undefined = notification.urlParams?.['id'] ? Number(notification.urlParams?.['id']) : undefined;

        if (projectId) {
          this.router.navigate(['/project', projectId, 'details'])
            .catch((error) => {
              // TODO #errorService
              console.error('Error while navigating to project details', error);
            });
        } else {
          // TODO #errorService
          console.error('Project id is not defined in notification:', notification);
        }
        return of(undefined);
      }
      case NotificationTypeEnum.ProjectPlan: {
        const projectId: number | undefined = notification.urlParams?.['id'] ? Number(notification.urlParams?.['id']) : undefined;

        if (projectId) {
          this.router.navigate(['/project', projectId, 'plan'])
            .catch((error) => {
              // TODO #errorService
              console.error('Error while navigating to project plan', error);
            });
        } else {
          // TODO #errorService
          console.error('Project id is not defined in notification:', notification);
        }
        return of(undefined);
      }
      case NotificationTypeEnum.Work: {
        const workId: number | undefined = notification.urlParams?.['id'] ? Number(notification.urlParams?.['id']) : undefined;

        if (!workId) {
          // TODO #errorService
          console.error('Work id is not defined in notification:', notification);
          return of(undefined);
        }

        return ctx.dispatch(new WorksActions.ShowWorkInSidebar(workId));
      }
      case NotificationTypeEnum.Issue: {
        const issueId: number | undefined = notification.urlParams?.['id'] ? Number(notification.urlParams?.['id']) : undefined;

        if (!issueId) {
          // TODO #errorService
          console.error('Issue id is not defined in notification:', notification);
          return of(undefined);
        }

        return this.store.dispatch(new IssuesActions.ShowIssueInSidebar(issueId));
      }
      default: {
        // TODO #errorService
        console.error('Unknown notification url type:', notification.urlType);
        return of(undefined);
      }
    }
  }

  private sortNotifications(notifications: NotificationModel[]) {
    return notifications
      // sort by time
      .sort((notificationA, notificationB) => {
        if (notificationA.timestamp > notificationB.timestamp) {
          return -1;
        } else if (notificationA.timestamp < notificationB.timestamp) {
          return 1;
        }

        return 0;
      })
      .sort((notificationA, notificationB) => {
        if (notificationA.isUnread && !notificationB.isUnread) {
          return -1;
        } else if (!notificationA.isUnread && notificationB.isUnread) {
          return 1;
        }

        return 0;
      });
  }
}
