import {Injectable} from '@angular/core';
import {Action, State, StateContext, Store} from '@ngxs/store';
import {finalize, forkJoin, of, tap} from 'rxjs';
import {catchError, map, switchMap} from 'rxjs/operators';
import {TranslateService} from '@ngx-translate/core';
import {DialogService, DynamicDialogRef} from 'primeng/dynamicdialog';

import {PluginsLoaderActions, PluginsLoaderSelectors} from './index';
import {GlobalRulesSelectors} from 'data-access/store';
import {PluginModel, PluginsApiService, PluginsOrderService} from 'data-access/sdk/internal-api';

import {
  EditPluginModalComponent,
  EditPluginModalData,
} from '../../../../../features/plugins-loader/src/lib/components/plugin-edit-modal/edit-plugin-modal.component';

export interface PluginsLoaderStateModel {
  plugins: PluginModel[];
  isListLoading: boolean;
  isCreateFormLoading: boolean;
  isEditModalLoading: boolean;
}

@State<PluginsLoaderStateModel>({
  name: 'plugins',
  defaults: {
    plugins: [],
    isListLoading: false,
    isCreateFormLoading: false,
    isEditModalLoading: false,
  },
})
@Injectable()
export class PluginsLoaderState {
  modalRef: DynamicDialogRef | undefined;

  constructor(
    private readonly apiService: PluginsApiService,
    private readonly orderService: PluginsOrderService,
    private readonly dialogService: DialogService,
    private readonly translateService: TranslateService,
    private readonly store: Store,
  ) {
  }

  @Action(PluginsLoaderActions.ReadList)
  readList(ctx: StateContext<PluginsLoaderStateModel>) {
    this.setListLoading(true, ctx);

    return this.apiService.readList()
      .pipe(
        finalize(() => this.setListLoading(false, ctx)),
        map((plugins) => this.orderService.reIndexOrder(this.orderService.sort(plugins))),
        tap((plugins) => ctx.patchState({plugins})),
      );
  }

  @Action(PluginsLoaderActions.UpdateList)
  updateList(ctx: StateContext<PluginsLoaderStateModel>, {plugins}: PluginsLoaderActions.UpdateList) {
    this.setListLoading(true, ctx);

    return forkJoin(this.orderService.reIndexOrder(plugins)
      .map((plugin) => this.apiService.update(plugin)))
      .pipe(
        finalize(() => this.setListLoading(false, ctx)),
        catchError((err) => {
          console.error(err);
          // TODO message
          return ctx.dispatch(new PluginsLoaderActions.ReadList());
        }),
        switchMap(() => ctx.dispatch(new PluginsLoaderActions.ReadList())),
      );
  }

  @Action(PluginsLoaderActions.Create)
  create(ctx: StateContext<PluginsLoaderStateModel>, {plugin}: PluginsLoaderActions.Create) {
    this.setCreateFormLoading(true, ctx);

    return this.apiService.create(plugin).pipe(
      finalize(() => this.setCreateFormLoading(false, ctx)),
      catchError((err) => {
        console.error(err);
        // TODO message
        return ctx.dispatch(new PluginsLoaderActions.ReadList());
      }),
      switchMap(() => ctx.dispatch(new PluginsLoaderActions.ReadList())),
      tap(() => this.refreshPage()),
    );
  }

  @Action(PluginsLoaderActions.Delete)
  delete(ctx: StateContext<PluginsLoaderStateModel>, {id}: PluginsLoaderActions.Delete) {
    this.setEditModalLoading(true, ctx);
    return this.apiService.delete(id).pipe(
      finalize(() => this.setEditModalLoading(false, ctx)),
      tap(() => ctx.dispatch(new PluginsLoaderActions.CloseEditModal())),
    );
  }

  @Action(PluginsLoaderActions.Update)
  update(ctx: StateContext<PluginsLoaderStateModel>, {plugin}: PluginsLoaderActions.Update) {
    this.setEditModalLoading(true, ctx);
    return this.apiService.update(plugin).pipe(
      finalize(() => this.setEditModalLoading(false, ctx)),
      switchMap(() => ctx.dispatch(new PluginsLoaderActions.CloseEditModal())),
      tap(() => {
        if (!!plugin.file) {
          this.refreshPage();
        }
      }),
    );
  }

  @Action(PluginsLoaderActions.ChangePluginStatus)
  changePluginStatus(ctx: StateContext<PluginsLoaderStateModel>, {plugin}: PluginsLoaderActions.ChangePluginStatus) {
    this.setListLoading(true, ctx);
    return this.apiService.changePluginStatus(plugin).pipe(
      finalize(() => this.setListLoading(false, ctx)),
    );
  }

  @Action(PluginsLoaderActions.OpenEditModal)
  openEditModal(ctx: StateContext<PluginsLoaderStateModel>, {id}: PluginsLoaderActions.OpenEditModal) {
    const canUpdate = this.store.selectSnapshot(GlobalRulesSelectors.canUpdate('dashboardplugin'));
    if (!canUpdate) {
      console.warn('You do not have permission to edit plugins.');
      return of(undefined);
    }

    return this.apiService.read(id).pipe(
      tap((plugin) => {
        const canDelete = this.store.selectSnapshot(GlobalRulesSelectors.canDelete('dashboardplugin'));

        const deleteFn = canDelete
          ? (pluginId: number) => ctx.dispatch(new PluginsLoaderActions.Delete(pluginId))
          : undefined;

        const updateFn = (_plugin: PluginModel) => ctx.dispatch(new PluginsLoaderActions.Update(_plugin));

        const data: EditPluginModalData = {
          plugin,
          deleteFn,
          updateFn,
          loading$: this.store.select(PluginsLoaderSelectors.editModalLoading),
        };

        if (this.modalRef) {
          this.modalRef.close();
          this.modalRef = undefined;
        }

        this.modalRef = this.dialogService.open(EditPluginModalComponent, {
          data,
          header: this.translateService.instant(
            'LIBS.FEATURES.PLUGINS.EDIT_PLUGIN_MODAL.TITLE',
            {pluginName: plugin.name},
          ),
          width: '350px',
          height: 'auto',
        });
      }),
    );
  }

  @Action(PluginsLoaderActions.CloseEditModal)
  closeEditModal(ctx: StateContext<PluginsLoaderStateModel>) {
    if (this.modalRef) {
      this.modalRef.close();
    } else {
      throw new Error('Modal is not opened');
    }

    this.modalRef = undefined;

    return ctx.dispatch(new PluginsLoaderActions.ReadList());
  }

  private setListLoading(loading: boolean, ctx: StateContext<PluginsLoaderStateModel>) {
    ctx.patchState({isListLoading: loading});
  }

  private setCreateFormLoading(loading: boolean, ctx: StateContext<PluginsLoaderStateModel>) {
    ctx.patchState({isCreateFormLoading: loading});
  }

  private setEditModalLoading(loading: boolean, ctx: StateContext<PluginsLoaderStateModel>) {
    ctx.patchState({isEditModalLoading: loading});
  }

  private refreshPage() {
    window.location.reload();
  }
}
