import {Injectable} from '@angular/core';
import {Action, State, StateContext} from '@ngxs/store';
import {switchMap} from 'rxjs/operators';
import {finalize, forkJoin, of, tap} from 'rxjs';
import {
  ProjectTypeStageModel,
  ProjectStagesApiService,
  StageModel,
  StageOrderService,
  StagesApiService,
} from 'data-access/sdk/internal-api';
import {ProjectStagesAction} from './stages.actions';


export interface ProjectStagesStateModel {
  loadingStages: boolean;
  deleteProjectStageLoading: boolean;
  addProjectStageLoading: boolean;
  activeProjectTypeId: number | null;
  projectStages: ProjectTypeStageModel[];
  stages: StageModel[];
  stagesForAdd: StageModel[];
}

@State<ProjectStagesStateModel>({
  name: 'projectStages',
  defaults: {
    loadingStages: false,
    deleteProjectStageLoading: false,
    addProjectStageLoading: false,
    activeProjectTypeId: null,
    projectStages: [],
    stages: [],
    stagesForAdd: [],
  },
})
@Injectable()
export class ProjectStagesState {
  constructor(
    private readonly projectStagesApiService: ProjectStagesApiService,
    private readonly stagesApiService: StagesApiService,
    private readonly orderService: StageOrderService,
  ) {
  }

  @Action(ProjectStagesAction.UpdateProjectStages)
  updateProjectStages(ctx: StateContext<ProjectStagesStateModel>, {stages}: ProjectStagesAction.UpdateProjectStages) {
    this.setProjectStagesLoading(true, ctx);

    const projectStages = stages.map((stage) => {
      const projectStage = ctx.getState().projectStages
        .find((_projectStage) => _projectStage.stageId === stage.id);

      const unfrozenProjectStage: ProjectTypeStageModel = JSON.parse(JSON.stringify(projectStage));

      unfrozenProjectStage.order = stage.order;
      return unfrozenProjectStage;
    });

    const reIndexedProjectStages = this.orderService.reIndexOrderProjectStages(projectStages);
    return this.projectStagesApiService.updateProjectStages(reIndexedProjectStages)
      .pipe(
        finalize(() => this.setProjectStagesLoading(false, ctx)),
        switchMap(() => {
          const activeProjectTypeId = ctx.getState().activeProjectTypeId;
          if (activeProjectTypeId) {
            return ctx.dispatch(new ProjectStagesAction.ReadProjectStages(activeProjectTypeId));
          }

          console.error('No active project type');
          return of(undefined);
        }),
      );
  }

  @Action(ProjectStagesAction.ReadProjectStages)
  readProjectStages(ctx: StateContext<ProjectStagesStateModel>, {projectTypeId}: ProjectStagesAction.ReadProjectStages) {
    this.setProjectStagesLoading(true, ctx);

    return forkJoin([
      this.projectStagesApiService.readProjectStages(projectTypeId),
      this.stagesApiService.readStages(),
    ]).pipe(
      finalize(() => this.setProjectStagesLoading(false, ctx)),
      tap(([projectStages, allStages]) => {
        const reIndexedProjectStages = this.orderService
          .reIndexOrderProjectStages(this.orderService.sortProjectStages(projectStages));

        const stages: StageModel[] = reIndexedProjectStages
          .map((projectStage) => projectStage.stage)
          .filter((stage) => !!stage) as StageModel[];
        const stagesForAdd = allStages
          .filter((stage) => !stages
            .some((relProjectStage) => relProjectStage.id === stage.id));

        ctx.patchState({
          activeProjectTypeId: projectTypeId,
          projectStages: reIndexedProjectStages,
          stages,
          stagesForAdd: stagesForAdd,
        });
      }),
    );
  }

  @Action(ProjectStagesAction.DeleteProjectStage)
  deleteProjectStage(ctx: StateContext<ProjectStagesStateModel>, {id, replaceWithStage}: ProjectStagesAction.DeleteProjectStage) {
    this.setDeleteProjectStageLoading(true, ctx);
    const projectStageId = ctx.getState().projectStages
      .find((_projectStage) => _projectStage.stageId === id)
      ?.id;
    const replaceWithStageId = ctx.getState().projectStages
      .find((_projectStage) => _projectStage.stageId === replaceWithStage)
      ?.id;

    if (!projectStageId || !replaceWithStageId) {
      console.error('No project stage or replaceWithStage found!');
      return of(undefined);
    }

    return this.projectStagesApiService.deleteProjectStage(projectStageId, replaceWithStageId).pipe(
      finalize(() => this.setDeleteProjectStageLoading(false, ctx)),
      switchMap(() => {
        const activeProjectTypeId = ctx.getState().activeProjectTypeId;
        if (activeProjectTypeId) {
          return ctx.dispatch(new ProjectStagesAction.ReadProjectStages(activeProjectTypeId));
        }

        console.error('No active project type');
        return of(undefined);
      }),
    );
  }

  @Action(ProjectStagesAction.CreateProjectStage)
  createProjectStage(
    ctx: StateContext<ProjectStagesStateModel>,
    {projectStageId, projectTypeId}: ProjectStagesAction.CreateProjectStage,
  ) {
    return this.projectStagesApiService
      .createProjectStage(projectTypeId, projectStageId)
      .pipe(
        switchMap(() => {
          const activeProjectTypeId = ctx.getState().activeProjectTypeId;
          if (activeProjectTypeId) {
            return ctx.dispatch(new ProjectStagesAction.ReadProjectStages(activeProjectTypeId));
          }

          console.error('No active project type');
          return of(undefined);
        }),
      );
  }

  private setProjectStagesLoading(loading: boolean, ctx: StateContext<ProjectStagesStateModel>) {
    ctx.patchState({loadingStages: loading});
  }

  private setDeleteProjectStageLoading(loading: boolean, ctx: StateContext<ProjectStagesStateModel>) {
    ctx.patchState({deleteProjectStageLoading: loading});
  }
}
