import { Injectable } from '@angular/core';
import { Router, RouterStateSnapshot } from '@angular/router';
import { Action, Actions, ofActionSuccessful, State, StateContext, StateToken, Store } from '@ngxs/store';
import { RouterNavigation } from '@ngxs/router-plugin';
import { switchMap } from 'rxjs/operators';
import { of } from 'rxjs';

import { RightSidebarStateModel } from './right-sidebar-state.model';
import { RightSidebarActions } from './right-sidebar.actions';

import { OutletsEnum } from 'utils/core-project-routing-utils';

export const RIGHT_SIDEBAR_STATE_TOKEN = new StateToken<RightSidebarStateModel>('rightSidebar');

@State({
  name: RIGHT_SIDEBAR_STATE_TOKEN,
  defaults: {
    isBlocked: false,
    rightSidebarOpened: false,
    header: '',
    customHeaderStatus: false,
    sidebarHeaderActions: [],
    component: undefined,
    canDeactivateFn: undefined,
  },
})
@Injectable()
export class RightSidebarState {
  outletObserver = this.actions$
    .pipe(
      ofActionSuccessful(RouterNavigation<RouterStateSnapshot>),
      switchMap((action: RouterNavigation) => this.store.dispatch(new RightSidebarActions.CheckOutlet(action)))
    )
    .subscribe();

  constructor(private readonly actions$: Actions, private readonly router: Router, private readonly store: Store) {}

  @Action(RightSidebarActions.CheckOutlet)
  checkOutlet(ctx: StateContext<RightSidebarStateModel>, { action }: RightSidebarActions.CheckOutlet) {
    const regExp = new RegExp(`.*\\(${OutletsEnum.RIGHT_SIDEBAR}.*`);
    const hasRightSidebarOutlet = regExp.test(action.routerState.url);
    const isSidebarOpen = ctx.getState().rightSidebarOpened;

    if (hasRightSidebarOutlet && !isSidebarOpen) {
      return ctx.dispatch(new RightSidebarActions.Open());
    }

    if (!hasRightSidebarOutlet && isSidebarOpen) {
      return ctx.dispatch(new RightSidebarActions.Close());
    }

    return of(action);
  }

  @Action(RightSidebarActions.Open)
  openRightSidepanel(ctx: StateContext<RightSidebarStateModel>, { component, canDeactivateFn }: RightSidebarActions.Open) {
    if (component) {
      this.clearOutlet();
    }

    ctx.patchState({
      rightSidebarOpened: true,
      header: '',
      sidebarHeaderActions: [],
      component,
      canDeactivateFn,
    });
  }

  @Action(RightSidebarActions.Close)
  closeRightSidepanel(ctx: StateContext<RightSidebarStateModel>, { manualHide }: RightSidebarActions.Close) {
    const canDeactivateFn = ctx.getState().canDeactivateFn;

    const clear = () => {
      ctx.patchState({
        rightSidebarOpened: false,
        header: '',
        sidebarHeaderActions: [],
        component: undefined,
        canDeactivateFn: undefined,
        customHeaderStatus: false,
      });

      this.clearOutlet();

      return of(true);
    };

    if (canDeactivateFn && !manualHide) {
      return canDeactivateFn().pipe(
        switchMap((result: boolean | undefined) => {
          if (result === true) {
            return clear();
          }

          return of(false);
        })
      );
    }

    return clear();
  }

  @Action(RightSidebarActions.SetHeader)
  setHeader(
    ctx: StateContext<RightSidebarStateModel>,
    { title, actions, canDeactivateFn, customHeaderStatus }: RightSidebarActions.SetHeader
  ) {
    ctx.patchState({
      header: title,
      sidebarHeaderActions: actions,
      canDeactivateFn,
      customHeaderStatus,
    });
  }

  @Action(RightSidebarActions.UpdateBlockUI)
  updateBlockUI(ctx: StateContext<RightSidebarStateModel>, { isBlocked }: RightSidebarActions.UpdateBlockUI) {
    ctx.patchState({ isBlocked });
  }

  private clearOutlet() {
    this.router.navigate([{ outlets: { [OutletsEnum.RIGHT_SIDEBAR]: null } }], { replaceUrl: true });
  }
}
