import {Injectable} from '@angular/core';
import {Action, State, StateContext, Store} from '@ngxs/store';
import {TranslateService} from '@ngx-translate/core';
import {finalize, forkJoin, of, tap} from 'rxjs';
import {catchError, map, switchMap} from 'rxjs/operators';
import {DialogService, DynamicDialogRef} from 'primeng/dynamicdialog';

import {ProjectEnums} from '@core-project/enums/project';
import {
  EditStageModalComponent,
  EditStageModalData,
} from '@core-project/containers/knowledgebase/methodology/stages/components/edit-stage-modal/edit-stage-modal.component';

import {StagesStateModel} from './stages-state-model';
import {StageModel, StageOrderService, StagesApiService} from 'data-access/sdk/internal-api';
import {StagesAction} from './stages.actions';
import {GlobalRulesSelectors} from '../app-global-rules';

@State<StagesStateModel>({
  name: 'stages',
  defaults: {
    isListLoading: false,
    isCreateFormLoading: false,
    isEditModalLoading: false,
    isDeleteListLoading: false,
    list: [],
  },
})
@Injectable()
export class StagesState {
  modalRef: DynamicDialogRef | undefined;

  constructor(
    private readonly apiService: StagesApiService,
    private readonly dialogService: DialogService,
    private readonly orderService: StageOrderService,
    private readonly translateService: TranslateService,
    private readonly store: Store,
  ) {
  }

  @Action(StagesAction.ReadStagesList)
  readStagesList(ctx: StateContext<StagesStateModel>) {
    this.setListLoading(true, ctx);

    return this.apiService.readStages().pipe(
      finalize(() => this.setListLoading(false, ctx)),
      map((stages) => this.orderService.reIndexOrderStages(this.orderService.sortStages(stages))),
      tap((stages: StageModel[]) => this.patchStagesList(stages, ctx)),
    );
  }

  @Action(StagesAction.UpdateStagesList)
  updateStagesList(ctx: StateContext<StagesStateModel>, {stages}: StagesAction.UpdateStagesList) {
    this.setListLoading(true, ctx);

    return forkJoin(
      this.orderService.reIndexOrderStages(stages)
        .map((stage: StageModel) => this.apiService.updateStage(stage)),
    ).pipe(
      finalize(() => this.setListLoading(false, ctx)),
      catchError((err) => {
        console.error(err);
        // TODO message
        return ctx.dispatch(new StagesAction.ReadStagesList());
      }),
      switchMap(() => ctx.dispatch(new StagesAction.ReadStagesList())),
    );
  }

  @Action(StagesAction.CreateStage)
  createStage(ctx: StateContext<StagesStateModel>, {stage}: StagesAction.CreateStage) {
    this.setCreateFormLoading(true, ctx);

    return this.apiService.createStage(stage).pipe(
      finalize(() => this.setCreateFormLoading(false, ctx)),
      catchError((err) => {
        console.error(err);
        // TODO message
        return ctx.dispatch(new StagesAction.ReadStagesList());
      }),
      switchMap(() => ctx.dispatch(new StagesAction.ReadStagesList())),
    );
  }

  @Action(StagesAction.DeleteStage)
  deleteStage(ctx: StateContext<StagesStateModel>, {id, replaceWithStage}: StagesAction.DeleteStage) {
    ctx.patchState({isEditModalLoading: true});
    return this.apiService.deleteStage(id, replaceWithStage).pipe(
      finalize(() => ctx.patchState({isEditModalLoading: false})),
      tap(() => ctx.dispatch(new StagesAction.CloseEditModal())),
    );
  }

  @Action(StagesAction.UpdateStage)
  updateStage(ctx: StateContext<StagesStateModel>, {stage}: StagesAction.UpdateStage) {
    ctx.patchState({isEditModalLoading: true});
    return this.apiService.updateStage(stage).pipe(
      finalize(() => ctx.patchState({isEditModalLoading: false})),
      switchMap(() => ctx.dispatch(new StagesAction.CloseEditModal())),
    );
  }

  @Action(StagesAction.OpenEditModal)
  openEditModal(ctx: StateContext<StagesStateModel>, {id}: StagesAction.OpenEditModal) {
    const canUpdate = this.store.selectSnapshot(GlobalRulesSelectors.canUpdate('stage'));
    if (!canUpdate) {
      console.warn('You do not have permission to edit stages.');
      return of(undefined);
    }

    return this.apiService.readStage(id)
      .pipe(
        tap((stage: StageModel) => {
          const stages: StageModel[] = JSON.parse(JSON.stringify(ctx.getState().list));
          const stagesForReplace = stages.filter((s) => s.id !== stage.id);
          const canDelete = this.store.selectSnapshot(GlobalRulesSelectors.canDelete('stage'));

          const deleteFn = canDelete
            ? (stageId: number, replaceWithStageId: number) => ctx.dispatch(new StagesAction.DeleteStage(stageId, replaceWithStageId))
            : undefined;

          const updateFn = (_stage: StageModel) => ctx.dispatch(new StagesAction.UpdateStage(_stage));

          const data: EditStageModalData = {
            stage,
            deleteFn,
            updateFn,
            stagesForReplace,
          };

          if (this.modalRef) {
            this.modalRef.close();
            this.modalRef = undefined;
          }

          this.modalRef = this.dialogService.open(EditStageModalComponent, {
            data,
            header: this.translateService.instant('PROJECT_STAGES.EDIT_STAGE_MODAL.TITLE', {stageName: stage.name}),
            width: '350px',
            height: 'auto',
          });
        }),
      );
  }

  @Action(StagesAction.CloseEditModal)
  closeEditModal(ctx: StateContext<StagesStateModel>) {
    if (this.modalRef) {
      this.modalRef.close();
    }

    this.modalRef = undefined;

    return ctx.dispatch(new StagesAction.ReadStagesList());
  }

  private setListLoading(loading: boolean, ctx: StateContext<StagesStateModel>) {
    ctx.patchState({isListLoading: loading});
  }

  private setCreateFormLoading(loading: boolean, ctx: StateContext<StagesStateModel>) {
    ctx.patchState({isCreateFormLoading: loading});
  }

  private setEditModalLoading(loading: boolean, ctx: StateContext<StagesStateModel>) {
    ctx.patchState({isEditModalLoading: loading});
  }

  private patchStagesList(stages: StageModel[], ctx: StateContext<StagesStateModel>) {
    const filteredStages = stages.filter(
      (stage) =>
        // these statuses not used in US
        stage.status !== ProjectEnums.StatusEnum.PSP && stage.status !== ProjectEnums.StatusEnum.CNC,
    );
    ctx.patchState({list: filteredStages});
  }
}
