import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { Action, State, StateContext, Store } from '@ngxs/store';
import { switchMap } from 'rxjs/operators';
import { finalize, forkJoin, of, tap, throwError } from 'rxjs';
import { DialogService } from 'primeng/dynamicdialog';

import { CreateWorkModalComponent } from 'features/works';
import {
  DepartmentsApiService,
  HistoryApiService,
  IssueBaseModel,
  LinksAdapterService,
  NewCommentModel,
  NewWorkModel,
  UserApiService,
  WorkApiService,
  WorkBaseModel,
  WorkModel,
} from 'data-access/sdk/internal-api';
import { DepartmentItemsClass, UserItemsClass } from 'ui/forms-ui';
import { getWorkCreateRouterLink, getWorkCreateRouterLinkQueryParams, getWorkRouterLink } from 'utils/core-project-routing-utils';

import { WorksStateModel } from './works-state.model';
import { WorksActions } from './works.actions';
import { WORKS_STATE_TOKEN } from './works-state.token';
import { Permission } from '../app-global-rules/models/permission.model';
import { GlobalRulesSelectors } from '../app-global-rules';
import { RightSidebarActions } from '../right-sidebar';
import { GlobalSelectors } from '../app-global';
import { WorksSelectors } from './works.selectors';
import { IssuesActions } from '../issues';

const getDefaultState: () => WorksStateModel = () => ({
  canCreateComment: true,
  canCreateIssue: true,
  canCreateWork: true,
  canDeleteWork: true,
  canUpdateWork: true,
  departmentsOptions: [],
  history: [],
  isCommentsLoading: false,
  isOptionsLoading: false,
  isWorkLoading: false,
  links: [],
  usersOptions: [],
  work: undefined,
});

@State({
  name: WORKS_STATE_TOKEN,
  defaults: getDefaultState(),
})
@Injectable()
export class WorksState {
  linksObsSub = this.store
    .select(WorksSelectors.work)
    .pipe(
      switchMap((work) => {
        if (!work) {
          return of(undefined);
        }

        return this.store.dispatch(new WorksActions.UpdateLinksView(work));
      })
    )
    .subscribe();

  blockUISub = this.store
    .select(WorksSelectors.isWorkLoading)
    .pipe(switchMap((isLoading) => this.store.dispatch(new RightSidebarActions.UpdateBlockUI(isLoading))))
    .subscribe();

  constructor(
    private readonly historyApiService: HistoryApiService,
    private readonly dialogService: DialogService,
    private readonly workApiService: WorkApiService,
    private readonly userApiService: UserApiService,
    private readonly departmentsApiService: DepartmentsApiService,
    private readonly store: Store,
    private readonly router: Router,
    private readonly linksAdapterService: LinksAdapterService
  ) {}

  @Action(WorksActions.ShowCreateWorkInSidebar)
  showCreateWorkInSidebar(ctx: StateContext<WorksStateModel>, { params }: WorksActions.ShowCreateWorkInSidebar) {
    this.router.navigate(getWorkCreateRouterLink(), { queryParams: getWorkCreateRouterLinkQueryParams(params) });
  }

  @Action(WorksActions.OpenIssueCreate)
  openIssueCreate(ctx: StateContext<WorksStateModel>) {
    const workId = ctx.getState().work?.id;
    const projectId = ctx.getState().work?.projectId;

    return ctx.dispatch(new IssuesActions.ShowCreateIssueInSidebar(projectId, workId));
  }

  @Action(WorksActions.ShowWorkInSidebar)
  showWorkInSidebar(ctx: StateContext<WorksStateModel>, { id }: WorksActions.ShowWorkInSidebar) {
    this.router.navigate(getWorkRouterLink(String(id)));
  }

  @Action(WorksActions.OpenWorkCreateModal)
  openClearWorkCreateModal(ctx: StateContext<WorksStateModel>) {
    const workState: WorkModel = JSON.parse(JSON.stringify(ctx.getState().work));
    const callback = (work: WorkModel) => {
      if (work) {
        workState.subworks.push(work);
      }

      ctx.patchState({
        work: {
          ...workState,
        },
      });
    };

    const relations: WorksActions.OpenWorkCreateModalShared['relations'] = {
      parentWorkId: workState.id,
    };

    return ctx.dispatch(new WorksActions.OpenWorkCreateModalShared(relations, callback));
  }

  @Action(WorksActions.OpenWorkCreateModalShared)
  openWorkCreateModal(ctx: StateContext<WorksStateModel>, { relations, callback }: WorksActions.OpenWorkCreateModalShared) {
    return forkJoin([this.userApiService.readUsers(), this.departmentsApiService.readDepartments()]).pipe(
      switchMap(([users, departments]) => {
        const usersItems = new UserItemsClass(users).setItemGroup([]);
        const departmentsItems = new DepartmentItemsClass(departments).setItemGroup([]);

        const modal = this.dialogService.open(CreateWorkModalComponent, {
          data: {
            responsibleOptions: [...usersItems, ...departmentsItems],
          },
        });

        // don't add to general stream (it's side effect (!) and cdr will have problems)
        modal.onClose
          .pipe(
            switchMap((work: NewWorkModel | undefined) => {
              if (work) {
                if (relations) {
                  work = {
                    ...work,
                    parentIssueId: relations.parentIssueId, // open from issue
                    parentWorkId: relations.parentWorkId, // open from work
                  };
                }

                return this.workApiService.createWork(work).pipe(
                  tap((workResponse) => {
                    callback(workResponse);
                  })
                );
              }

              return of(undefined);
            })
          )
          .subscribe();

        return of(undefined);
      })
    );
  }

  @Action(WorksActions.ChangeWorkStatus)
  changeWorkStatus(ctx: StateContext<WorksStateModel>, { status }: WorksActions.ChangeWorkStatus) {
    const state = ctx.getState();

    if (!state.work || !state.work.id) {
      return throwError(() => new Error('Work is not defined in state'));
    }

    const workId = state.work.id;

    ctx.patchState({ isWorkLoading: true });
    return this.workApiService.changeWorkStatus(workId, status).pipe(
      finalize(() => ctx.patchState({ isWorkLoading: false })),
      switchMap(() => ctx.dispatch(new WorksActions.ReadWork(workId)))
    );
  }

  @Action(WorksActions.DestroyState)
  destroyState(ctx: StateContext<WorksStateModel>) {
    ctx.setState(getDefaultState());
  }

  @Action(WorksActions.ReadWork)
  readWork(ctx: StateContext<WorksStateModel>, { id }: WorksActions.ReadWork) {
    ctx.patchState({ isWorkLoading: true });
    ctx.dispatch(new WorksActions.ReadWorkHistory(id));

    return forkJoin([
      this.workApiService.readWork(id),
      ctx.dispatch(new WorksActions.ReadPermissions(id)),
      ctx.dispatch(new WorksActions.ReadFieldsOptions()),
    ]).pipe(
      finalize(() => ctx.patchState({ isWorkLoading: false })),
      tap(([work]) => {
        ctx.patchState({
          work,
        });
      })
    );
  }

  @Action(WorksActions.UpdateLinksView)
  updateLinksView(ctx: StateContext<WorksStateModel>, { work }: WorksActions.UpdateLinksView) {
    const subworks: WorkBaseModel[] = JSON.parse(JSON.stringify(work.subworks));
    const issues: IssueBaseModel[] = JSON.parse(JSON.stringify(work.issues));

    const subworks$ =
      subworks && subworks.length > 0 ? forkJoin(subworks.map((subwork) => this.linksAdapterService.convertFromWork(subwork))) : of([]);
    const parentWork$ = work.parentWork ? this.linksAdapterService.convertFromWork(work.parentWork, true) : of(undefined);

    const issues$ =
      issues && issues.length > 0 ? forkJoin(issues.map((issue) => this.linksAdapterService.convertFromIssue(issue))) : of([]);
    const parentIssue$ = work.parentIssue ? this.linksAdapterService.convertFromIssue(work.parentIssue, true) : of(undefined);

    return forkJoin([subworks$, parentWork$, issues$, parentIssue$]).pipe(
      tap(([subworksLinks, parentWorkLink, issuesLinks, parentIssueLink]) => {
        ctx.patchState({
          links: [
            ...(parentIssueLink ? [parentIssueLink] : []),
            ...(parentWorkLink ? [parentWorkLink] : []),
            ...subworksLinks,
            ...issuesLinks,
          ],
        });
      })
    );
  }

  @Action(WorksActions.ReadPermissions)
  readPermissions(ctx: StateContext<WorksStateModel>, { workId }: WorksActions.ReadPermissions) {
    return this.workApiService.readWorkPermissions(workId).pipe(
      tap((permissions) => {
        const workPermission = Permission.createPermissions(permissions).find((p) => p.modelName === 'work');

        ctx.patchState({
          canDeleteWork: workPermission?.canDelete || false,
          canUpdateWork: workPermission?.canUpdate || false,
          canCreateComment: this.store.selectSnapshot(GlobalRulesSelectors.canCreate('comment')),
        });
      })
    );
  }

  @Action(WorksActions.ReadWorkHistory)
  readWorkHistory(ctx: StateContext<WorksStateModel>, { id }: WorksActions.ReadWorkHistory) {
    return this.historyApiService.readWorkHistory(id).pipe(
      tap((historyActions) => {
        ctx.patchState({
          history: historyActions,
        });
      })
    );
  }

  @Action(WorksActions.ReadFieldsOptions)
  readFieldsOptions(ctx: StateContext<WorksStateModel>) {
    ctx.patchState({ isOptionsLoading: true });

    return forkJoin([this.userApiService.readUsers(), this.departmentsApiService.readDepartments()]).pipe(
      finalize(() => ctx.patchState({ isOptionsLoading: false })),
      tap(([usersOptions, departmentsOptions]) => {
        ctx.patchState({
          usersOptions,
          departmentsOptions,
        });
      })
    );
  }

  @Action(WorksActions.CreateWork)
  createWork(ctx: StateContext<WorksStateModel>, { work }: WorksActions.CreateWork) {
    ctx.patchState({ isWorkLoading: true });

    return this.workApiService.createWork(work).pipe(
      finalize(() => ctx.patchState({ isWorkLoading: false })),
      tap((workResponse) => ctx.dispatch(new WorksActions.ShowWorkInSidebar(workResponse.id)))
    );
  }

  @Action(WorksActions.DeleteWork)
  deleteWork(ctx: StateContext<WorksStateModel>, { id }: WorksActions.DeleteWork) {
    ctx.patchState({ isWorkLoading: true });
    return this.workApiService.deleteWork(id).pipe(
      finalize(() => ctx.patchState({ isWorkLoading: false })),
      tap(() => {
        // TODO add toast
        ctx.dispatch(new RightSidebarActions.Close());
      })
    );
  }

  @Action(WorksActions.CreateComment)
  createComment(ctx: StateContext<WorksStateModel>, { text, replyToId }: WorksActions.CreateComment) {
    ctx.patchState({ isCommentsLoading: true });

    // work can't be undefined here
    const work: WorkModel = JSON.parse(JSON.stringify(ctx.getState().work));

    const comment: NewCommentModel = {
      text,
      authorId: this.store.selectSnapshot(GlobalSelectors.userId),
      replyToId,
    };
    return this.workApiService.createComment(work.id, comment).pipe(
      finalize(() => ctx.patchState({ isCommentsLoading: false })),
      tap((commentResponse) => {
        // add author bcs we don't have it in response (no include)
        if (!commentResponse.author) {
          const usersOptions = ctx.getState().usersOptions;
          commentResponse.author = usersOptions.find((user) => user.id === commentResponse.authorId);
        }

        ctx.patchState({
          work: {
            ...work,
            comments: [...work.comments, commentResponse],
          },
        });
      })
    );
  }

  @Action(WorksActions.UpdateWork)
  updateWork(ctx: StateContext<WorksStateModel>, { work }: WorksActions.UpdateWork) {
    return this.workApiService.updateWork(work).pipe(switchMap(() => ctx.dispatch(new WorksActions.ReadWork(work.id))));
  }

  @Action(WorksActions.RefreshWork)
  refreshWork(ctx: StateContext<WorksStateModel>, { id }: WorksActions.RefreshWork) {
    if (ctx.getState().work?.id === id) {
      return ctx.dispatch(new WorksActions.ReadWork(id));
    }

    return of(undefined);
  }
}
