import {AfterViewInit, ChangeDetectionStrategy, Component, ElementRef, OnDestroy, OnInit, ViewChild} from '@angular/core';
import {catchError, Observable, of, switchMap, throwError} from 'rxjs';
import {ActivatedRoute, Router} from '@angular/router';
import {TranslateService} from '@ngx-translate/core';
import {Select, Store} from '@ngxs/store';
import {DashboardPluginActions, DashboardPluginSelectors} from 'data-access/store';
import {DashboardPluginModel} from 'data-access/sdk/internal-api';

@Component({
  selector: 'lib-dashboard-plugin',
  templateUrl: './dashboard-plugin.component.html',
  styleUrls: ['./dashboard-plugin.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DashboardPluginComponent implements OnInit, OnDestroy, AfterViewInit {
  @ViewChild('script') script!: ElementRef<HTMLElement>;

  @Select(DashboardPluginSelectors.plugin) plugin$!: Observable<DashboardPluginModel | undefined>;

  constructor(
    private readonly store: Store,
    private readonly activatedRoute: ActivatedRoute,
    private readonly router: Router,
    private readonly translateService: TranslateService,
  ) {
  }

  ngOnInit() {
    this.activatedRoute.paramMap
      .pipe(
        switchMap((params) => {
          const id = params.get('id') ? Number(params.get('id')) : undefined;

          if (id) {
            return this.store.dispatch(new DashboardPluginActions.Read(id));
          } else {
            return throwError(() => new Error('Plugin id not found'));
          }
        }),
        catchError((err) => {
          console.error(err);
          // TODO notification
          // TODO logging
          this.router.navigate(['']);
          return of(undefined);
        }),
      )
      .subscribe();
  }

  ngAfterViewInit() {
    this.plugin$
      .subscribe((plugin) => {
        if (!plugin) {
          return;
        }
        // reset script tag
        this.script.nativeElement.childNodes.forEach((node) => {
          node.remove();
        });

        // append script tag
        const element = window.document.createElement('script');
        element.src = plugin.indexFileUrl;
        try {
          this.script.nativeElement.append(element);
        } catch (e) {
          console.error(e);
        }
      });
  }

  createTag(plugin: DashboardPluginModel) {
    const lang = this.translateService.currentLang;
    return `<${plugin.tag} pluginid="${plugin.id}" lang="${lang}"></${plugin.tag}>`;
  }

  ngOnDestroy() {
    this.store.dispatch(new DashboardPluginActions.Reset());
  }
}
