import { ChangeDetectionStrategy, ChangeDetectorRef, Component, Input, signal, WritableSignal } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { SelectItemGroup } from 'primeng/api';

import { IssueDomainModel, ProjectModel, UserModel } from 'data-access/sdk/internal-api';
import { EntityModel, IssueDomainItemsClass, ProjectItemsClass, UserItemsClass } from 'ui/forms-ui';

import { CreateIssueBaseRelationsFormModel } from '../../interfaces/create-issue-base-relations-form.model';
import { IssueFormModel } from '../../interfaces/issue-form.model';
import { CardFormBaseDirective } from 'utils/shared-directives';
import { IssueCardFormService } from '../../services/issue-card-form.service';
import { IssueFormGroupInterface } from '../../interfaces/issue-form-group.interface';

@Component({
  selector: 'lib-issue-card-form',
  templateUrl: './issue-card-form.component.html',
  styleUrls: ['./issue-card-form.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class IssueCardFormComponent extends CardFormBaseDirective<
  IssueFormGroupInterface,
  IssueFormModel,
  CreateIssueBaseRelationsFormModel
> {
  readonly responsibleOptions: WritableSignal<SelectItemGroup[]> = signal([]);
  readonly workGroupOptions: WritableSignal<SelectItemGroup[]> = signal([]);
  readonly projectOptions: WritableSignal<SelectItemGroup[]> = signal([]);
  readonly domainsOptions: WritableSignal<SelectItemGroup[]> = signal([]);

  constructor(private readonly changeDetectorRef: ChangeDetectorRef, private readonly issueCardFormService: IssueCardFormService) {
    super();
  }

  get isCreateForm(): boolean {
    return this.form.controls.id.value === null;
  }

  @Input() set users(users: UserModel[] | undefined) {
    if (users) {
      const items = new UserItemsClass(users);
      this.responsibleOptions.update((values) => items.setItemGroup(values));
      this.workGroupOptions.update((values) => items.setItemGroup(values));

      if (this.formRelationsHandler) {
        this.formRelationsHandler();
      }
    }
  }

  @Input() set projects(projects: ProjectModel[] | undefined) {
    if (projects) {
      const items = new ProjectItemsClass(projects);
      this.projectOptions.update((values) => items.setItemGroup(values));

      if (this.formRelationsHandler) {
        this.formRelationsHandler();
      }
    }
  }

  @Input() set issueDomains(issueDomains: IssueDomainModel[] | undefined) {
    if (issueDomains) {
      const items = new IssueDomainItemsClass(issueDomains);
      this.domainsOptions.update((values) => items.setItemGroup(values));

      if (this.formRelationsHandler) {
        this.formRelationsHandler();
      }
    }
  }

  initForm(issue?: IssueFormModel, createRelations?: CreateIssueBaseRelationsFormModel) {
    if (this.formRelationsHandler) {
      this.formRelationsHandler = undefined;
    }

    if (this.form) {
      this.form.reset();
    }

    this.form = this.createFormGroup(issue, createRelations);
    this.changeDetectorRef.detectChanges();
  }

  onIssueFormChanges() {
    const formValue = this.form.getRawValue();

    const model: IssueFormModel = {
      ...(formValue.id ? { id: formValue.id } : {}),

      createdAt: formValue.createdAt ?? undefined,
      description: formValue.description ?? undefined,
      name: formValue.name,
      severity: formValue.severity,
      resolveDate: formValue.resolveDate ?? undefined,

      // relation ids
      domainIds: formValue.domains.map((domain) => domain.id) ?? [],
      userIds: formValue.users.map((user) => user.id) ?? [],
      projectId: formValue.project?.id ?? undefined,
      responsibleId: formValue.responsible?.id ?? undefined,
    };

    this.formChanges.emit(model);
  }

  protected createFormGroup(
    model?: IssueFormModel,
    createRelations?: CreateIssueBaseRelationsFormModel
  ): FormGroup<IssueFormGroupInterface> {
    const formGroup: FormGroup<IssueFormGroupInterface> = new FormGroup<IssueFormGroupInterface>({
      id: new FormControl<number | null>(model?.id ?? null),

      createdAt: new FormControl<string | null>(model?.createdAt ?? null),
      description: new FormControl<string | null>(model?.description ?? null),
      name: new FormControl<string>(model?.name ?? '', { nonNullable: true, validators: [Validators.required] }),
      severity: new FormControl<number>(model?.severity ?? 0, { nonNullable: true }),
      resolveDate: new FormControl<string | null>(model?.resolveDate ?? null),

      // create relations (fill in effect!)
      domains: new FormControl<EntityModel[]>([], { nonNullable: true }),
      users: new FormControl<EntityModel[]>([], { nonNullable: true }),
      project: new FormControl<EntityModel | null>(null),
      responsible: new FormControl<EntityModel | null>(null),
    });

    this.formRelationsHandler = this.issueCardFormService.createIssueFormRelationsHandler(
      formGroup,
      this.responsibleOptions,
      this.workGroupOptions,
      this.projectOptions,
      this.domainsOptions,
      model,
      createRelations
    );
    this.formRelationsHandler();

    return formGroup;
  }
}
