import {
  AfterViewInit,
  ChangeDetectionStrategy,
  Component,
  computed,
  OnDestroy,
  signal,
  Signal,
  ViewChild,
  WritableSignal,
} from '@angular/core';
import { toSignal } from '@angular/core/rxjs-interop';
import { ActivatedRoute } from '@angular/router';
import { Select, Store } from '@ngxs/store';
import { catchError, Observable, of, skip, switchMap, takeUntil, tap } from 'rxjs';
import { MenuItem } from 'primeng/api';

import {
  HistoryActionModel,
  IssueDomainModel,
  IssueModel,
  IssueStatusEnum,
  LinkModel,
  LinkTypeEnum,
  NewIssueModel,
  ProjectModel,
  UserModel,
} from 'data-access/sdk/internal-api';
import { GlobalSelectors, IssuesActions, IssuesSelectors, RightSidebarActions, SidebarHeaderActionModel } from 'data-access/store';
import {
  BrowserHistoryService,
  FEATURE_ISSUE_AUTHOR_ID_QUERY_PARAM,
  FEATURE_ISSUE_CREATE_ROUTE,
  FEATURE_ISSUE_ID_PARAM,
  FEATURE_ISSUE_PARENT_WORK_ID_QUERY_PARAM,
  FEATURE_ISSUE_PROJECT_ID_QUERY_PARAM,
} from 'utils/core-project-routing-utils';
import { Unsubscriber } from 'utils/shared-services';

import { CreateIssueBaseRelationsFormModel } from '../../interfaces/create-issue-base-relations-form.model';
import { IssueFormModel } from '../../interfaces/issue-form.model';
import { IssueFormAdapterService } from '../../services/issue-form-adapter.service';
import { IssueCardFormComponent } from '../../components/issue-card-form/issue-card-form.component';
import { IssueModalsService } from '../../services/issue-modals.service';

@Component({
  selector: 'lib-sidebar-issue-container',
  templateUrl: './sidebar-issue-container.component.html',
  styleUrls: ['./sidebar-issue-container.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
  providers: [Unsubscriber],
})
export class SidebarIssueContainerComponent implements AfterViewInit, OnDestroy {
  @ViewChild('issueFormComponent') issueForm?: IssueCardFormComponent;

  @Select(IssuesSelectors.issue) issue$!: Observable<IssueModel | undefined>;
  @Select(IssuesSelectors.issueDomainsOptions) issueDomainsOptions$!: Observable<IssueDomainModel[]>;
  @Select(IssuesSelectors.usersOptions) usersOptions$!: Observable<UserModel[]>;
  @Select(IssuesSelectors.projectsOptions) projectsOptions$!: Observable<ProjectModel[]>;
  @Select(IssuesSelectors.comments) comments$!: Observable<IssueModel['comments']>;
  @Select(IssuesSelectors.isIssueLoading) isIssueLoading$!: Observable<boolean>;
  @Select(IssuesSelectors.isCommentsLoading) isCommentsLoading$!: Observable<boolean>;
  @Select(IssuesSelectors.issueHistory) issueHistory$!: Observable<HistoryActionModel[]>;
  @Select(IssuesSelectors.canCreateComment) canCreateComment$!: Observable<boolean>;
  @Select(IssuesSelectors.canCreateWork) canCreateWork$!: Observable<boolean>;
  @Select(IssuesSelectors.links) links$!: Observable<LinkModel[]>;
  @Select(IssuesSelectors.comments) comments!: Observable<IssueModel['comments']>;

  readonly tabItems: MenuItem[] = [
    {
      id: 'history',
      label: 'LIBS.FEATURES.ISSUES.ISSUE_CONTAINER.HISTORY_TAB_TITLE',
      icon: 'ci-history',
    },
    {
      id: 'comments',
      label: 'LIBS.FEATURES.ISSUES.ISSUE_CONTAINER.COMMENTS_TAB_TITLE',
      icon: 'ci-comments',
    },
  ];

  readonly canUpdate: Signal<boolean> = toSignal(this.store.select(IssuesSelectors.canUpdate), { initialValue: false });

  readonly selfAvatarLink: Signal<string | undefined> = toSignal(this.store.select(GlobalSelectors.selfAvatarLink));

  readonly activeTabIndex: WritableSignal<number> = signal(1);
  readonly isHistoryTabActive: Signal<boolean> = computed(() => this.activeTabIndex() === 0);
  readonly isCommentsTabActive: Signal<boolean> = computed(() => this.activeTabIndex() === 1);

  readonly issueTempModel: WritableSignal<IssueModel | NewIssueModel> = signal(this.getNewIssueModel());

  readonly isCreate: WritableSignal<boolean> = signal(false);
  readonly issueStatus: WritableSignal<IssueModel['status']> = signal(undefined);

  readonly isCreateButtonShow: Signal<boolean> = computed(() => this.isCreate());
  readonly isEdit: Signal<boolean> = computed(() => !this.isCreate());
  readonly isFooterShow: Signal<boolean> = computed(() => !this.isCreate());

  readonly isToSolvedButtonShow: Signal<boolean> = computed(
    () => this.issueStatus() === IssueStatusEnum.ACT && this.isEdit() && this.canUpdate()
  );
  readonly isToActiveButtonShow: Signal<boolean> = computed(
    () => this.issueStatus() === IssueStatusEnum.SLV && this.isEdit() && this.canUpdate()
  );

  // header
  headerInput: string | undefined = undefined;
  readonly headerPlaceholder: WritableSignal<string> = signal('LIBS.FEATURES.ISSUES.ISSUE_CONTAINER.CREATE_ISSUE_TITLE');
  readonly sidebarHeaderActions: WritableSignal<SidebarHeaderActionModel[]> = signal([]);

  constructor(
    private readonly activatedRoute: ActivatedRoute,
    private readonly browserHistoryService: BrowserHistoryService,
    private readonly issueFormAdapterService: IssueFormAdapterService,
    private readonly store: Store,
    private readonly unsubscriber: Unsubscriber,
    private readonly issueModalsService: IssueModalsService
  ) {}

  ngAfterViewInit() {
    this.activatedRoute.paramMap
      .pipe(
        takeUntil(this.unsubscriber.destroy$),
        tap(() => this.reset())
      )
      .subscribe((params) => {
        const id: string | null = params.get(FEATURE_ISSUE_ID_PARAM);

        // check route
        const isEditRoute: boolean = !!id && id !== FEATURE_ISSUE_CREATE_ROUTE;
        const isCreateRoute: boolean = id === FEATURE_ISSUE_CREATE_ROUTE;

        if (isEditRoute) {
          this.initEditIssue(Number(id)).subscribe();
        } else if (isCreateRoute) {
          this.initCreateIssue$().subscribe();
        } else {
          // TODO error toast
          this.browserHistoryService.back();
        }
      });
  }

  ngOnDestroy() {
    this.store.dispatch(new IssuesActions.DestroyState()).subscribe();
  }

  issueFormChanges(model: IssueFormModel) {
    if (this.isCreate()) {
      this.issueTempModel.update((value) => ({
        ...value,
        ...this.issueFormAdapterService.reverseToNewModel(model),
      }));
    }

    if (this.isEdit()) {
      this.issueTempModel.update((value) => ({
        ...value,
        ...this.issueFormAdapterService.reverse(model),
      }));

      // if form is invalid or pristine, do not update
      if (this.issueForm?.form.invalid || this.issueForm?.form.pristine) {
        return;
      }

      this.onEditIssue();
      this.issueForm?.form.markAsPristine();
    }
  }

  headerInputDeactivate(name: string = '') {
    if (this.issueForm) {
      const formControl = this.issueForm.form.controls.name;
      if (formControl.value !== name) {
        this.headerInput = name;
        formControl.setValue(name);
        formControl.markAsDirty();
        formControl.markAsTouched();
        this.issueForm.onIssueFormChanges();
      }
    } else {
      console.error('Form not found');
    }
  }

  onEditIssue() {
    const model = this.issueTempModel();

    if ('id' in model) {
      this.store.dispatch(new IssuesActions.UpdateIssue(model)).subscribe();
    } else {
      throw new Error('DEV_ERROR: issue has no id. This should not happen. Only create models can have no id.');
    }
  }

  createIssue() {
    this.store.dispatch(new IssuesActions.CreateIssue(this.issueTempModel())).subscribe();
  }

  setActiveStatus() {
    this.store.dispatch(new IssuesActions.ChangeStatus(IssueStatusEnum.ACT)).subscribe();
  }

  setSolvedStatus() {
    this.store.dispatch(new IssuesActions.ChangeStatus(IssueStatusEnum.SLV)).subscribe();
  }

  openLink(link: LinkModel) {
    if (link.type === LinkTypeEnum.Work) {
      this.store.dispatch(new IssuesActions.OpenWork(link.id)).subscribe();
      return;
    }

    console.error(`Unsupported scenario: link type = ${link.type} (openLink in issue container).`);
  }

  openCreateTaskModal() {
    this.store.dispatch(new IssuesActions.OpenCreateWorkModal()).subscribe();
  }

  createComment(comment: string, replyToId?: number) {
    this.store.dispatch(new IssuesActions.CreateComment(comment, replyToId)).subscribe();
  }

  closeSidebar() {
    this.store.dispatch(new RightSidebarActions.Close()).subscribe();
  }

  private initCreateIssue$() {
    return this.store.dispatch(new IssuesActions.ReadFieldsOptions()).pipe(
      tap(() => this.isCreate.set(true)),
      // set deactivate observable
      switchMap(() => this.store.dispatch(new RightSidebarActions.SetHeader('', [], () => this.createOnDeactivateObservable(), true))),
      tap(() => {
        if (this.issueForm) {
          this.issueForm.initForm(undefined, this.createRelations());
        } else {
          // if error => add setTimeout (component not rendered)
          console.error('Issue form not found');
          // TODO error toast
          this.browserHistoryService.back();
        }
      })
    );
  }

  private createOnDeactivateObservable(): Observable<boolean> {
    if (this.isCreate() && this.issueForm?.form.touched) {
      const modalRef = this.issueModalsService.createChangesSaveModal();

      return modalRef.onClose;
    } else {
      return of(true);
    }
  }

  private initEditIssue(id: number) {
    return of(undefined).pipe(
      // read issue
      switchMap(() => this.store.dispatch(new IssuesActions.ReadIssue(id))),
      switchMap(() => this.issue$),
      // destroy on params change
      takeUntil(this.activatedRoute.paramMap.pipe(skip(1))),
      // fill form and other
      switchMap((issue) => {
        if (!issue) {
          // ignore empty
          // if error on backend => throw error in api service
          return of(undefined);
        }

        this.issueTempModel.set(issue);
        this.isCreate.set(false);

        this.initComponentsIssue(issue);

        const actions: SidebarHeaderActionModel[] = [];

        if (this.store.selectSnapshot(IssuesSelectors.canDelete)) {
          actions.push({
            svgIcon: 'ci-trash',
            action: () => {
              const modalRef = this.issueModalsService.createDeleteAttentionModal();

              modalRef.onClose.subscribe((result: boolean | undefined) => {
                if (result === true) {
                  this.store.dispatch(new IssuesActions.DeleteIssue(issue.id)).subscribe();
                }
              });
            },
            tooltip: 'LIBS.FEATURES.ISSUES.ISSUE_CONTAINER.DELETE_ISSUE_TOOLTIP',
          });
        }

        this.headerInput = issue.name;
        this.sidebarHeaderActions.set(actions);

        this.store.dispatch(new RightSidebarActions.SetHeader('', [], undefined, true));

        return of(undefined);
      }),
      catchError((err) => {
        console.error(err);
        // TODO #errorService
        this.browserHistoryService.back();
        return of(undefined);
      }),
      takeUntil(this.unsubscriber.destroy$)
    );
  }

  private reset() {
    this.activeTabIndex.set(1);
    this.issueTempModel.set(this.getNewIssueModel());
    this.issueStatus.set(undefined);
    this.isCreate.set(false);
    this.headerInput = '';

    return this.store.dispatch(new IssuesActions.DestroyState());
  }

  private initComponentsIssue(issue: IssueModel) {
    this.issueFormAdapterService.convert(issue).subscribe((issueForm) => {
      // move in queue for render (here not necessary but better)
      setTimeout(() => {
        if (this.issueForm) {
          this.issueForm.initForm(issueForm, this.createRelations());
        } else {
          // TODO: #errorService
          console.error('Issue form not found');
        }
      });
    });

    this.issueStatus.set(issue.status);
  }

  private createRelations(routerSnapshot = this.activatedRoute.snapshot): CreateIssueBaseRelationsFormModel {
    const authorId = routerSnapshot.queryParamMap.get(FEATURE_ISSUE_AUTHOR_ID_QUERY_PARAM)
      ? Number(routerSnapshot.queryParamMap.get(FEATURE_ISSUE_AUTHOR_ID_QUERY_PARAM))
      : this.store.selectSnapshot(GlobalSelectors.userId);

    const projectId = routerSnapshot.queryParamMap.get(FEATURE_ISSUE_PROJECT_ID_QUERY_PARAM)
      ? Number(routerSnapshot.queryParamMap.get(FEATURE_ISSUE_PROJECT_ID_QUERY_PARAM))
      : undefined;

    const parentWorkId = routerSnapshot.queryParamMap.get(FEATURE_ISSUE_PARENT_WORK_ID_QUERY_PARAM)
      ? Number(routerSnapshot.queryParamMap.get(FEATURE_ISSUE_PARENT_WORK_ID_QUERY_PARAM))
      : undefined;

    return {
      authorId,
      projectId,
      parentWorkId,
    };
  }

  private getNewIssueModel(): NewIssueModel {
    const createRelations = this.createRelations();

    return {
      ...createRelations,
      name: '',
      severity: 0,

      commentIds: [],
      domainIds: [],
      userIds: [],
      workIds: [],
    };
  }
}
