import {ChangeDetectionStrategy, Component, OnInit, ViewChild} from '@angular/core';
import {FormControl, FormGroup, Validators} from '@angular/forms';
import {BehaviorSubject, Observable, takeUntil} from 'rxjs';
import {DynamicDialogConfig, DynamicDialogRef} from 'primeng/dynamicdialog';
import {FileSelectEvent, FileUpload} from 'primeng/fileupload';

import {NewPluginInterface, PluginModel} from 'data-access/sdk/internal-api';

interface PluginFGInterface {
  name: FormControl<PluginModel['name'] | null>;
  enabled: FormControl<PluginModel['isEnabled'] | null>;
  file: FormControl<NewPluginInterface['file'] | null>;
}

export interface EditPluginModalData {
  plugin: PluginModel;
  loading$: Observable<boolean>;
  deleteFn: ((pluginId: number) => Observable<void>) | undefined;
  updateFn: (plugin: PluginModel) => Observable<void>;
}

@Component({
  selector: 'lib-edit-plugin-modal',
  templateUrl: './edit-plugin-modal.component.html',
  styleUrls: ['./edit-plugin-modal.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class EditPluginModalComponent implements OnInit {
  @ViewChild('fileUpload') fileUpload!: FileUpload;

  formGroup!: FormGroup<PluginFGInterface>;

  loadingSubject: BehaviorSubject<boolean> = new BehaviorSubject<boolean>(false);
  loading$: Observable<boolean> = this.loadingSubject.asObservable();

  private readonly dialogData!: EditPluginModalData;

  constructor(
    private readonly dynamicDialogRef: DynamicDialogRef,
    private readonly dialogConfig: DynamicDialogConfig<EditPluginModalData>,
  ) {
    if (this.dialogConfig.data) {
      this.dialogData = this.dialogConfig.data;

      if (this.dialogConfig.data.loading$) {
        this.dialogConfig.data.loading$
          .pipe(takeUntil(this.dynamicDialogRef.onClose))
          .subscribe((value) => this.loadingSubject.next(value));
      }
    } else {
      console.error('No plugin passed to EditPluginModalComponent');
      this.close();
    }
  }

  get canDelete() {
    return !!this.dialogData.deleteFn;
  }

  ngOnInit() {
    this.formGroup = this.createFG();
  }

  uploadFile(event: FileSelectEvent) {
    for (const file of event.files) {
      this.formGroup.get('file')?.setValue(file);
      this.formGroup.get('file')?.updateValueAndValidity();
      this.formGroup.markAsDirty();
    }
  }

  createFG(plugin = this.dialogData.plugin): FormGroup<PluginFGInterface> {
    if (!this.dialogData.plugin) {
      console.error('No plugin passed to EditPluginModalComponent');
      this.close();
    }

    return new FormGroup<PluginFGInterface>({
      name: new FormControl(plugin.name, [Validators.required]),
      enabled: new FormControl(plugin.isEnabled, [Validators.required]),
      file: new FormControl(null),
    });
  }

  close() {
    this.dynamicDialogRef.close();
  }

  save() {
    this.fileUpload.upload();

    if (this.formGroup.invalid) {
      console.error('Form is invalid!', JSON.stringify(this.formGroup.value));
      return;
    }

    const plugin: PluginModel = {
      ...this.dialogData.plugin,
      name: this.formGroup.value.name as PluginModel['name'],
      isEnabled: this.formGroup.value.enabled as PluginModel['isEnabled'],
    };

    if (this.formGroup.value.file) {
      plugin.file = this.formGroup.value.file;
    }

    this.dialogData.updateFn(plugin)
      .subscribe();
  }

  delete() {
    if (this.canDelete && this.dialogData.deleteFn) {
      this.dialogData.deleteFn(this.dialogData.plugin.id)
        .subscribe();
    }
  }
}
