import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  computed, EventEmitter, Input,
  OnDestroy,
  Output,
  Signal,
  signal,
  ViewChild,
  WritableSignal
} from '@angular/core';
import { CommonModule } from '@angular/common';
import { AbstractControl, FormControl, FormGroup, ReactiveFormsModule, ValidationErrors, ValidatorFn, Validators } from '@angular/forms';
import { TranslateModule } from '@ngx-translate/core';
import { combineLatest, debounceTime, Subject, takeUntil, tap } from 'rxjs';
import * as moment from 'moment';
import { SelectItemGroup } from 'primeng/api';
import { ButtonModule } from 'primeng/button';
import { TooltipModule } from 'primeng/tooltip';

import {
  DepartmentItemsClass,
  EditorComponent,
  EntityEnum,
  EntityModel,
  InplaceCalendarInputComponent,
  InplaceComponent,
  InplaceDropdownInputComponent,
  InplaceDropdownOptionsInterface,
  InplaceStringInputComponent,
  InplaceSwitchCalendarInputComponent,
  SingleEntityInplaceInputComponent,
  UserItemsClass,
} from 'ui/forms-ui';
import { DiffPipe } from 'utils/shared-pipes';
import { CardFormBaseDirective } from 'utils/shared-directives';
import {
  getDeadlineHint,
  DepartmentModel,
  getEndEffHint,
  startBaseHint,
  UserModel,
  WorkStatusEnum,
  WorkTypeEnum,
  getDurationHint,
} from 'data-access/sdk/internal-api';

import { WorkFormModel } from '../../interfaces/work-form.model';
import { CreateWorkBaseRelationsFormModel } from '../../interfaces/create-work-base-relations-form.model';
import { WorkFormGroupInterface } from '../../interfaces/work-form-group.interface';
import { WorkCardFormService } from '../../services/work-card-form.service';

@Component({
  selector: 'lib-work-card-form',
  standalone: true,
  imports: [
    CommonModule,
    TranslateModule,
    ReactiveFormsModule,
    InplaceDropdownInputComponent,
    InplaceStringInputComponent,
    SingleEntityInplaceInputComponent,
    InplaceCalendarInputComponent,
    InplaceComponent,
    DiffPipe,
    EditorComponent,
    InplaceSwitchCalendarInputComponent,
    ButtonModule,
    TooltipModule,
  ],
  templateUrl: './work-card-form.component.html',
  styleUrls: ['./work-card-form.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class WorkCardFormComponent
  extends CardFormBaseDirective<WorkFormGroupInterface, WorkFormModel, CreateWorkBaseRelationsFormModel>
  implements OnDestroy
{
  private formDurationDestroySubject: Subject<void> = new Subject();
  private formWorkTypeDestroySubject: Subject<void> = new Subject();
  private hintsDestroySubject: Subject<void> = new Subject();

  private durationErrorName = 'error';

  @ViewChild('endBaseInputComponent') endBaseInputComponent?: InplaceCalendarInputComponent;

  @Input() showStopTaskButton = false;

  @Output() onStopButtonClick: EventEmitter<void> = new EventEmitter<void>();

  readonly inspectorOptions: WritableSignal<SelectItemGroup[]> = signal([]);
  readonly executorOptions: WritableSignal<SelectItemGroup[]> = signal([]);

  startBaseHint: string | undefined = undefined;
  startBaseColor: string | undefined = undefined;
  endBaseHint: string | undefined = undefined;
  endBaseColor: string | undefined = undefined;
  deadlineHint: string | undefined = undefined;
  deadlineColor: string | undefined = undefined;
  durationHint: string | undefined = undefined;
  durationColor: string | undefined = undefined;

  readonly workTypeOptions: InplaceDropdownOptionsInterface[] = [
    { label: 'LIBS.FEATURES.WORKS.WORK_TYPES.TS', value: WorkTypeEnum.TS, icon: 'ci-hammer' },
    { label: 'LIBS.FEATURES.WORKS.WORK_TYPES.MS', value: WorkTypeEnum.MS, icon: 'ci-flag-line' },
  ];

  readonly statusOptions: InplaceDropdownOptionsInterface[] = [
    { label: 'LIBS.FEATURES.WORKS.STATUS.BL', value: WorkStatusEnum.BL, icon: 'ci-backlog', iconColor: '#ADB1C2', bgColor: '#F4F4F6' },
    { label: 'LIBS.FEATURES.WORKS.STATUS.CR', value: WorkStatusEnum.CR, icon: 'ci-todo', iconColor: '#A871EF', bgColor: '#E5E4FF' },
    { label: 'LIBS.FEATURES.WORKS.STATUS.WP', value: WorkStatusEnum.WP, icon: 'ci-in-progress', iconColor: '#1677CC', bgColor: '#D9EDFF' },
    { label: 'LIBS.FEATURES.WORKS.STATUS.DN', value: WorkStatusEnum.DN, icon: 'ci-on-check', iconColor: '#FB9600', bgColor: '#FFE1B3' },
    { label: 'LIBS.FEATURES.WORKS.STATUS.AC', value: WorkStatusEnum.AC, icon: 'ci-completed', iconColor: '#71AF74', bgColor: '#CFF6D1' },
    { label: 'LIBS.FEATURES.WORKS.STATUS.CN', value: WorkStatusEnum.CN, icon: 'ci-cancelled', iconColor: '#DA4A4A', bgColor: '#FFC2C2' },
    { label: 'LIBS.FEATURES.WORKS.STATUS.PS', value: WorkStatusEnum.PS, icon: 'ci-paused', iconColor: '#DA4A4A', bgColor: '#FFC2C2' },
  ];

  readonly priorityOptions: InplaceDropdownOptionsInterface[] = [
    { label: 'LIBS.FEATURES.WORKS.PRIORITY.' + -1, value: -1, bgColor: '#CFF6D1', iconColor: '#71AF74' },
    { label: 'LIBS.FEATURES.WORKS.PRIORITY.' + 0, value: 0, bgColor: '#FFE1B3', iconColor: '#FB9600' },
    { label: 'LIBS.FEATURES.WORKS.PRIORITY.' + 1, value: 1, bgColor: '#FFC2C2', iconColor: '#DA4A4A' },
  ];

  readonly durationInDays: WritableSignal<number | null> = signal(null);

  readonly workTypeMode: WritableSignal<WorkTypeEnum> = signal(WorkTypeEnum.TS);
  readonly isTaskMode: Signal<boolean> = computed(() => this.workTypeMode() === WorkTypeEnum.TS);
  readonly endBaseLabelKey: Signal<string> = computed(() => {
    if (this.isTaskMode()) {
      return 'LIBS.FEATURES.WORKS.WORK_CARD_FORM.FORM_FIELDS.END_BASE_TS';
    }
    return 'LIBS.FEATURES.WORKS.WORK_CARD_FORM.FORM_FIELDS.END_BASE_MS';
  });

  constructor(
    private readonly changeDetectorRef: ChangeDetectorRef,
    private readonly workCardFormService: WorkCardFormService
  ) {
    super();
  }

  get durationHasError(): boolean {
    return !!this.form.get('endBase')?.hasError(this.durationErrorName);
  }

  get durationErrorText(): string {
    return this.form.get('endBase')?.getError(this.durationErrorName);
  }

  @Input() set users(users: UserModel[] | undefined) {
    if (users) {
      const items = new UserItemsClass(users);
      this.inspectorOptions.update((values) => items.setItemGroup(values));
      this.executorOptions.update((values) => items.setItemGroup(values));

      if (this.formRelationsHandler) {
        this.formRelationsHandler();
      }
    }
  }

  @Input() set departments(departments: DepartmentModel[] | undefined) {
    if (departments) {
      const items = new DepartmentItemsClass(departments);
      this.executorOptions.update((values) => items.setItemGroup(values));

      if (this.formRelationsHandler) {
        this.formRelationsHandler();
      }
    }
  }

  ngOnDestroy(): void {
    this.formWorkTypeDestroySubject.next();
    this.formDurationDestroySubject.next();
    this.hintsDestroySubject.next();
  }

  initForm(issue?: WorkFormModel, createRelations?: CreateWorkBaseRelationsFormModel): void {
    this.formDurationDestroySubject.next();
    this.formWorkTypeDestroySubject.next();
    this.hintsDestroySubject.next();

    if (this.formRelationsHandler) {
      this.formRelationsHandler = undefined;
    }

    if (this.form) {
      this.form.reset();
    }

    this.form = this.createFormGroup(issue, createRelations);
    this.changeDetectorRef.detectChanges();
  }

  onIssueFormChanges(): void {
    const formValue = this.form.getRawValue();

    const model: WorkFormModel = {
      ...(formValue.id ? { id: formValue.id } : {}),

      deadline: formValue.deadline ?? undefined,
      description: formValue.description,
      endBase: formValue.endBase ?? undefined,
      name: formValue.name,
      priority: formValue.priority,
      startBase: formValue.startBase ?? undefined,
      status: formValue.status,
      workType: formValue.workType,

      toDepartmentId: formValue.executor?.entity === EntityEnum.Department ? formValue.executor?.id : undefined,
      toUserId: formValue.executor?.entity === EntityEnum.User ? formValue.executor?.id : undefined,
      valUserId: formValue.inspector?.id,
    };

    this.formChanges.emit(model);
  }

  protected createFormGroup(model?: WorkFormModel, createRelations?: CreateWorkBaseRelationsFormModel): FormGroup<WorkFormGroupInterface> {
    const formGroup: FormGroup<WorkFormGroupInterface> = new FormGroup<WorkFormGroupInterface>(
      {
        id: new FormControl<number | null>(model?.id ?? null),
        deadline: new FormControl<string | null>(model?.deadline ?? null),
        description: new FormControl<string>(model?.description ?? '', { nonNullable: true }),
        endBase: new FormControl<string | null>(model?.endBase ?? null), // error reset in form validator!
        executor: new FormControl<EntityModel | null>(null),
        inspector: new FormControl<EntityModel | null>(null),
        name: new FormControl<string>(model?.name ?? '', { nonNullable: true, validators: [Validators.required] }),
        priority: new FormControl<number>(model?.priority ?? 0, { nonNullable: true }),
        startBase: new FormControl<string | null>(model?.startBase ?? null),
        status: new FormControl<WorkStatusEnum>(model?.status ?? WorkStatusEnum.CR, { nonNullable: true }),
        workType: new FormControl<WorkTypeEnum>(model?.workType ?? WorkTypeEnum.TS, { nonNullable: true }),
      },
      { validators: [this.endBaseValidatorFn()] }
    );

    this.formRelationsHandler = this.workCardFormService.createIssueFormRelationsHandler(
      formGroup,
      this.executorOptions,
      this.inspectorOptions,
      model,
      createRelations
    );
    this.formRelationsHandler();

    this.createHint$(formGroup).pipe(takeUntil(this.hintsDestroySubject)).subscribe();
    combineLatest([formGroup.controls.startBase.valueChanges, formGroup.controls.endBase.valueChanges])
      .pipe(takeUntil(this.formDurationDestroySubject))
      .subscribe(([startBase, endBase]) => {
        setTimeout(() => {
          this.calculateDuration(startBase, endBase);

          if (this.endBaseInputComponent && startBase && this.workTypeMode() === WorkTypeEnum.TS) {
            this.endBaseInputComponent.minDate = new Date(startBase);
          }
        });
      });
    formGroup.controls.workType.valueChanges.pipe(takeUntil(this.formWorkTypeDestroySubject)).subscribe((workType) => {
      this.workTypeMode.set(workType);
    });

    // trigger create subscription
    formGroup.controls.startBase.updateValueAndValidity();
    formGroup.controls.endBase.updateValueAndValidity();
    formGroup.controls.workType.updateValueAndValidity();

    return formGroup;
  }

  private createHint$(formGroup: FormGroup<WorkFormGroupInterface>) {
    return formGroup.valueChanges.pipe(
      debounceTime(100),
      tap((formValue) => this.setStartBaseControlInfo(formValue)),
      tap((formValue) => this.setEndEffControlInfo(formValue)),
      tap((formValue) => this.setDeadlineControlInfo(formValue)),
      tap(() => this.changeDetectorRef.markForCheck())
    );
  }

  private setStartBaseControlInfo(formValue: FormGroup<WorkFormGroupInterface>['value']): void {
    const start = formValue?.startBase ?? undefined;
    const status = formValue?.status;
    const info = startBaseHint(start, status);

    this.startBaseColor = info.color;
    this.startBaseHint = info.text;
  }

  private setEndEffControlInfo(formValue: FormGroup<WorkFormGroupInterface>['value']): void {
    const end = formValue?.endBase ?? undefined;
    const status = formValue?.status;
    const type = formValue?.workType;
    const info = getEndEffHint(end, status, type);

    this.endBaseColor = info.color;
    this.endBaseHint = info.text;
  }

  private setDeadlineControlInfo(formValue: FormGroup<WorkFormGroupInterface>['value']): void {
    const endBase = formValue?.endBase ?? undefined;
    const deadline = formValue?.deadline ?? undefined;
    const status = formValue?.status;
    const info = getDeadlineHint(endBase, deadline, status);

    this.deadlineColor = info.color;
    this.deadlineHint = info.text;
  }

  private endBaseValidatorFn(): ValidatorFn {
    return (abstractControl: AbstractControl): ValidationErrors | null => {
      const formGroup = abstractControl as FormGroup<WorkFormGroupInterface>;
      const errors: ValidationErrors = {};

      const startBase = formGroup?.get('startBase')?.value;
      const endBase = formGroup?.get('endBase')?.value;
      const workType = formGroup?.get('workType')?.value;

      if (workType !== WorkTypeEnum.TS || !startBase || !endBase) {
        return null;
      }

      const isDateAfterStart = getDurationHint(endBase, startBase).text;
      const durationError = this.durationErrorName;

      if (isDateAfterStart) {
        const errorDescription = 'LIBS.FEATURES.WORKS.WORK_CARD_FORM.FORM_FIELDS.DURATION_ERROR';
        formGroup.controls.endBase.setErrors({ [durationError]: errorDescription });
        errors[durationError] = errorDescription;
      } else if (formGroup.controls.endBase.hasError(durationError)) {
        formGroup.controls.endBase.setErrors(null);
      }

      if (Object.keys(errors).length > 0) {
        return errors;
      }

      return null;
    };
  }

  private calculateDuration(startBase?: string | null, endBase?: string | null): void {
    let duration = moment.duration(0);

    if (startBase && endBase) {
      duration = moment.duration(moment(endBase).diff(moment(startBase))).add(1, 'day');
    }

    this.durationInDays.set(duration.asDays());
  }
}
