import { ChangeDetectionStrategy, Component, OnInit, signal, WritableSignal } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormControl, FormGroup, ReactiveFormsModule, Validators } from '@angular/forms';
import { TranslateModule } from '@ngx-translate/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { SelectItemGroup } from 'primeng/api';
import { ButtonModule } from 'primeng/button';

import { NewWorkModel } from 'data-access/sdk/internal-api';
import {
  EntityEnum,
  EntityModel,
  InplaceCalendarInputComponent,
  InplaceStringInputComponent,
  SingleEntityInplaceInputComponent,
} from 'ui/forms-ui';
import { Unsubscriber } from 'utils/shared-services';

interface FormModel {
  name: FormControl<string | null>;
  endBase: FormControl<string | null>;
  responsible: FormControl<EntityModel | null>;
}

@Component({
  selector: 'lib-create-work-modal',
  standalone: true,
  imports: [
    CommonModule,
    ReactiveFormsModule,
    TranslateModule,
    ButtonModule,
    InplaceStringInputComponent,
    SingleEntityInplaceInputComponent,
    InplaceCalendarInputComponent,
  ],
  providers: [Unsubscriber], // возможно, нужно переданной в объекте функции, но это не точно
  templateUrl: './create-work-modal.component.html',
  styleUrls: ['./create-work-modal.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class CreateWorkModalComponent implements OnInit {
  form: FormGroup<FormModel> = this.createFormGroup();
  responsibleOptions: WritableSignal<SelectItemGroup[]> = signal([]);

  constructor(private readonly dynamicDialogRef: DynamicDialogRef, private readonly dynamicDialogConfig: DynamicDialogConfig) {}

  ngOnInit() {
    this.dynamicDialogConfig.width = '500px';
    this.dynamicDialogConfig.showHeader = false;
    this.dynamicDialogConfig.closeOnEscape = true;

    this.responsibleOptions.set(this.dynamicDialogConfig.data.responsibleOptions);
  }

  create() {
    if (this.form.invalid) {
      return;
    }

    const name = this.form.controls.name.value ?? undefined;
    const endBase = this.form.controls.endBase.value ?? undefined;
    const responsible = this.form.controls.responsible.value ?? undefined;
    const responsibleType = responsible?.entity;

    if (!name) {
      this.form.controls.name.setErrors({ required: true });
      // TODO #errorService
      return;
    }

    const data: NewWorkModel = {
      name,
      endBase,
      toUserId: responsibleType === EntityEnum.User ? responsible?.id : undefined,
      toDepartmentId: responsibleType === EntityEnum.Department ? responsible?.id : undefined,
    };

    this.dynamicDialogRef.close(data);
  }

  close() {
    this.dynamicDialogRef.close();
  }

  private createFormGroup() {
    return new FormGroup<FormModel>({
      name: new FormControl<string | null>(null, { validators: [Validators.required] }),
      endBase: new FormControl<string | null>(null),
      responsible: new FormControl<EntityModel | null>(null),
    });
  }
}
