import {
  AfterViewInit,
  ChangeDetectionStrategy,
  Component,
  computed,
  Injector,
  OnDestroy,
  signal,
  Signal,
  ViewChild,
  WritableSignal,
} from '@angular/core';
import { CommonModule } from '@angular/common';
import { ActivatedRoute } from '@angular/router';
import { toObservable, toSignal } from '@angular/core/rxjs-interop';
import { Store } from '@ngxs/store';
import { catchError, Observable, of, skip, switchMap, takeUntil, tap } from 'rxjs';
import { MenuItem } from 'primeng/api';
import { TranslateModule } from '@ngx-translate/core';

import { LinkModel, LinkTypeEnum, NewWorkModel, WorkModel, WorkStatusEnum, WorkTypeEnum } from 'data-access/sdk/internal-api';
import {
  GlobalSelectors,
  IssuesActions,
  RightSidebarActions,
  SidebarHeaderActionModel,
  WorksActions,
  WorksSelectors,
} from 'data-access/store';
import { CommentsTabComponent, HistoryTabComponent, LinksComponent, SidebarHeaderComponent, TabMenuComponent } from 'ui/common-layout-ui';
import {
  BrowserHistoryService,
  FEATURE_WORK_CREATE_ROUTE,
  FEATURE_WORK_END_BASE_QUERY_PARAM,
  FEATURE_WORK_ID_PARAM,
  FEATURE_WORK_PARENT_ISSUE_ID_QUERY_PARAM,
  FEATURE_WORK_PARENT_WORK_ID_QUERY_PARAM,
  FEATURE_WORK_PROJECT_ID_QUERY_PARAM,
  FEATURE_WORK_START_BASE_QUERY_PARAM,
  FEATURE_WORK_STATUS_QUERY_PARAM,
  FEATURE_WORK_TO_DEPARTMENT_ID_QUERY_PARAM,
  FEATURE_WORK_TO_USER_ID_QUERY_PARAM,
  FEATURE_WORK_VAL_USER_ID_QUERY_PARAM,
} from 'utils/core-project-routing-utils';
import { Unsubscriber } from 'utils/shared-services';

import { WorkCardFormComponent } from '../../components/work-card-form/work-card-form.component';
import { CreateWorkBaseRelationsFormModel } from '../../interfaces/create-work-base-relations-form.model';
import { WorkFormAdapterService } from '../../services/work-form-adapter.service';
import { WorkModalsService } from '../../services/work-modals.service';
import { WorkFormModel } from '../../interfaces/work-form.model';
import { ButtonModule } from 'primeng/button';
import { WorkCardFlowComponent } from '../../components/work-card-flow/work-card-flow.component';

@Component({
  selector: 'lib-sidebar-work-container',
  standalone: true,
  imports: [
    CommonModule,
    WorkCardFormComponent,
    LinksComponent,
    TabMenuComponent,
    CommentsTabComponent,
    HistoryTabComponent,
    TranslateModule,
    SidebarHeaderComponent,
    ButtonModule,
    WorkCardFlowComponent,
  ],
  templateUrl: './sidebar-work-container.component.html',
  styleUrls: ['./sidebar-work-container.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
  providers: [Unsubscriber],
})
export class SidebarWorkContainerComponent implements AfterViewInit, OnDestroy {
  @ViewChild('workCardFormComponent') workForm?: WorkCardFormComponent;

  // main containers info
  readonly work = toSignal(this.store.select(WorksSelectors.work));
  readonly links = toSignal(this.store.select(WorksSelectors.links), { initialValue: [] });
  readonly history = toSignal(this.store.select(WorksSelectors.history), { initialValue: [] });
  readonly comments = toSignal(this.store.select(WorksSelectors.comments), { initialValue: [] });

  // options
  readonly departmentsOptions = toSignal(this.store.select(WorksSelectors.departmentsOptions), { initialValue: [] });
  readonly usersOptions = toSignal(this.store.select(WorksSelectors.usersOptions), { initialValue: [] });

  // loadings
  readonly isCommentsLoading = toSignal(this.store.select(WorksSelectors.isCommentsLoading), { initialValue: false });
  readonly isWorkLoading = toSignal(this.store.select(WorksSelectors.isWorkLoading), { initialValue: false });

  // rules
  readonly canCreateComment = toSignal(this.store.select(WorksSelectors.canCreateComment), { initialValue: true });
  readonly canCreateWork = toSignal(this.store.select(WorksSelectors.canCreateWork), { initialValue: true });
  readonly canCreateIssue = toSignal(this.store.select(WorksSelectors.canCreateIssue), { initialValue: true });
  readonly canDeleteWork = toSignal(this.store.select(WorksSelectors.canDeleteWork), { initialValue: false });
  readonly canUpdateWork: Signal<boolean> = toSignal(this.store.select(WorksSelectors.canUpdateWork), { initialValue: false });

  // common
  readonly isCreate: WritableSignal<boolean> = signal(false);
  readonly isEdit: Signal<boolean> = computed(() => !this.isCreate());
  readonly isFooterShow: Signal<boolean> = computed(() => !this.isCreate());
  readonly isCreateButtonShow: Signal<boolean> = computed(() => this.isCreate());
  readonly isFlowShow: Signal<boolean> = computed(() => this.isEdit());
  readonly isFlowLoading: Signal<boolean> = computed(() => this.isWorkLoading());
  readonly isLinksShow: Signal<boolean> = computed(() => this.isEdit() && !this.isWorkLoading());

  // temp model
  readonly workTempModel: WritableSignal<WorkModel | NewWorkModel> = signal(this.getNewWorkModel());
  readonly workTempModelType: Signal<WorkModel['workType']> = computed(() => this.workTempModel().workType);

  // tabs
  readonly activeTabIndex: WritableSignal<number> = signal(1);
  readonly isHistoryTabActive: Signal<boolean> = computed(() => this.activeTabIndex() === 0);
  readonly isCommentsTabActive: Signal<boolean> = computed(() => this.activeTabIndex() === 1);

  // header
  headerInput: string | undefined = undefined;
  readonly headerPlaceholder: Signal<string> = computed(() => {
    switch (this.workTempModelType()) {
      case WorkTypeEnum.TS:
        return 'LIBS.FEATURES.WORKS.WORK_CONTAINER.CREATE_TS_TITLE';
      case WorkTypeEnum.MS:
        return 'LIBS.FEATURES.WORKS.WORK_CONTAINER.CREATE_MS_TITLE';
      default:
        console.error('Work type not found', this.workTempModelType());
        return 'LIBS.FEATURES.WORKS.WORK_CONTAINER.CREATE_TS_TITLE';
    }
  });
  readonly sidebarHeaderActions: WritableSignal<SidebarHeaderActionModel[]> = signal([]);
  readonly headerIcon: Signal<string> = computed(() => {
    switch (this.workTempModelType()) {
      case WorkTypeEnum.TS:
        return 'ci-hammer';
      case WorkTypeEnum.MS:
        return 'ci-flag-line';
      default:
        console.error('Work type not found', this.workTempModelType());
        return '';
    }
  });

  // other
  readonly selfAvatarLink = toSignal(this.store.select(GlobalSelectors.selfAvatarLink));
  readonly breadcrumbs = computed(() => {
    const projectName = this.work()?.project?.name;
    const groupName = this.work()?.group?.name;
    return this.createBreadcrumbs(projectName, groupName);
  });
  readonly hasInspector = computed(() => !!this.work()?.valUserId);
  readonly workStatus = computed(() => this.work()?.status);
  readonly showStopTaskButton = computed(() => this.workStatus() !== WorkStatusEnum.AC && this.workStatus() !== WorkStatusEnum.CN);

  readonly tabItems: MenuItem[] = [
    {
      id: 'history',
      label: 'LIBS.FEATURES.ISSUES.ISSUE_CONTAINER.HISTORY_TAB_TITLE',
      icon: 'ci-history',
    },
    {
      id: 'comments',
      label: 'LIBS.FEATURES.ISSUES.ISSUE_CONTAINER.COMMENTS_TAB_TITLE',
      icon: 'ci-comments',
    },
  ];

  constructor(
    private readonly activatedRoute: ActivatedRoute,
    private readonly browserHistoryService: BrowserHistoryService,
    private readonly store: Store,
    private readonly unsubscriber: Unsubscriber,
    private readonly workFormAdapterService: WorkFormAdapterService,
    private readonly workModalsService: WorkModalsService,
    private readonly injector: Injector
  ) {}

  ngAfterViewInit() {
    this.activatedRoute.paramMap
      .pipe(
        takeUntil(this.unsubscriber.destroy$),
        tap(() => this.reset())
      )
      .subscribe((params) => {
        const id: string | null = params.get(FEATURE_WORK_ID_PARAM);

        // check route
        const isEditRoute: boolean = !!id && id !== FEATURE_WORK_CREATE_ROUTE;
        const isCreateRoute: boolean = id === FEATURE_WORK_CREATE_ROUTE;

        if (isEditRoute) {
          this.initEditWork(Number(id)).subscribe();
        } else if (isCreateRoute) {
          this.initCreateWork$().subscribe();
        } else {
          // TODO error toast
          this.browserHistoryService.back();
        }
      });
  }

  ngOnDestroy() {
    this.store.dispatch(new WorksActions.DestroyState()).subscribe();
  }

  openCreateLink(type: 'issue' | 'work') {
    if (type === 'work') {
      this.store.dispatch(new WorksActions.OpenWorkCreateModal()).subscribe();
      return;
    }

    this.store.dispatch(new WorksActions.OpenIssueCreate()).subscribe();
  }

  createComment(comment: string, replyToId?: number) {
    this.store.dispatch(new WorksActions.CreateComment(comment, replyToId)).subscribe();
  }

  openLink(link: LinkModel) {
    if (link.type === LinkTypeEnum.Work) {
      this.store.dispatch(new WorksActions.ShowWorkInSidebar(link.id)).subscribe();
      return;
    }

    if (link.type === LinkTypeEnum.Issue) {
      this.store.dispatch(new IssuesActions.ShowIssueInSidebar(link.id)).subscribe();
      return;
    }

    console.error(`Unsupported scenario: link type = ${link.type} (openLink in work container).`);
  }

  workFormChanges(model: WorkFormModel) {
    if (this.isCreate()) {
      this.workTempModel.update((value) => ({
        ...value,
        ...this.workFormAdapterService.reverseToNewModel(model),
      }));
    }

    if (this.isEdit()) {
      this.workTempModel.update((value) => ({
        ...value,
        ...this.workFormAdapterService.reverse(model),
      }));

      // if form is invalid or pristine, do not update
      if (this.workForm?.form.invalid || this.workForm?.form.pristine) {
        return;
      }

      this.updateWork();
      this.workForm?.form.markAsPristine();
    }
  }

  headerInputDeactivate(name: string = '') {
    if (this.workForm) {
      const formControl = this.workForm.form.controls.name;
      if (formControl.value !== name) {
        this.headerInput = name;
        formControl.setValue(name);
        formControl.markAsDirty();
        formControl.markAsTouched();
        this.workForm.onIssueFormChanges();
      }
    } else {
      console.error('Form not found');
    }
  }

  updateWork() {
    const model = this.workTempModel();

    if ('id' in model) {
      this.store.dispatch(new WorksActions.UpdateWork(model)).subscribe();
    } else {
      throw new Error('DEV_ERROR: work has no id. This should not happen. Only create models can have no id.');
    }
  }

  createWork() {
    this.store.dispatch(new WorksActions.CreateWork(this.workTempModel())).subscribe();
  }

  closeSidebar() {
    this.store.dispatch(new RightSidebarActions.Close()).subscribe();
  }

  stopTask() {
    this.changeStatus(WorkStatusEnum.CN);
  }

  changeStatus(status: WorkStatusEnum) {
    this.store.dispatch(new WorksActions.ChangeWorkStatus(status)).subscribe();
  }

  openReturnModal() {
    const modalRef = this.workModalsService.declineWorkModal();

    modalRef.onClose.pipe(takeUntil(this.unsubscriber.destroy$)).subscribe((comment?: string) => {
      if (typeof comment === 'string') {
        this.store.dispatch(new WorksActions.CreateComment(comment)).subscribe();
        this.store.dispatch(new WorksActions.ChangeWorkStatus(WorkStatusEnum.WP)).subscribe();
      }
    });
  }

  private createBreadcrumbs(...args: (string | undefined)[]) {
    args = args.filter((value) => !!value);
    return args.join(' / ');
  }

  private reset() {
    this.activeTabIndex.set(1);
    this.workTempModel.set(this.getNewWorkModel());
    this.isCreate.set(false);
    this.headerInput = '';

    return this.store.dispatch(new WorksActions.DestroyState());
  }

  private initEditWork(id: number) {
    return of(undefined).pipe(
      // read
      switchMap(() => this.store.dispatch(new WorksActions.ReadWork(id))),
      switchMap(() => toObservable(this.work, { injector: this.injector })),
      // destroy on params change
      takeUntil(this.activatedRoute.paramMap.pipe(skip(1))),
      // fill form and other
      switchMap((work) => {
        if (!work) {
          // ignore empty
          // if error on backend => throw error in api service
          return of(undefined);
        }

        this.workTempModel.set(work);
        this.isCreate.set(false);

        this.initComponentsWork(work);

        const actions: SidebarHeaderActionModel[] = [];

        if (this.canDeleteWork()) {
          actions.push({
            svgIcon: 'ci-trash',
            action: () => {
              const modalRef = this.workModalsService.createDeleteAttentionModal();

              modalRef.onClose.subscribe((result: boolean | undefined) => {
                if (result === true) {
                  this.store.dispatch(new WorksActions.DeleteWork(work.id)).subscribe();
                }
              });
            },
            tooltip: 'LIBS.FEATURES.WORKS.WORK_CONTAINER.DELETE_WORK_TOOLTIP',
          });
        }

        this.headerInput = work.name;
        this.sidebarHeaderActions.set(actions);

        this.store.dispatch(new RightSidebarActions.SetHeader('', [], undefined, true));

        return of(undefined);
      }),
      catchError((err) => {
        console.error(err);
        // TODO #errorService
        this.browserHistoryService.back();
        return of(undefined);
      }),
      takeUntil(this.unsubscriber.destroy$)
    );
  }

  private initComponentsWork(work: WorkModel) {
    this.workFormAdapterService.convert(work).subscribe((workForm) => {
      // move in queue for render (here not necessary but better)
      setTimeout(() => {
        if (this.workForm) {
          this.workForm.initForm(workForm);
        } else {
          // TODO: #errorService
          console.error('Work form not found');
        }
      });
    });
  }

  private initCreateWork$() {
    return this.store.dispatch(new WorksActions.ReadFieldsOptions()).pipe(
      tap(() => this.isCreate.set(true)),
      // set deactivate observable
      switchMap(() => this.store.dispatch(new RightSidebarActions.SetHeader('', [], () => this.createOnDeactivateObservable(), true))),
      tap(() => {
        if (this.workForm) {
          this.workForm.initForm(undefined, this.createRelations());
        } else {
          // if error => add setTimeout (component not rendered)
          console.error('Work form not found');
          // TODO error toast
          this.browserHistoryService.back();
        }
      })
    );
  }

  private createRelations(routerSnapshot = this.activatedRoute.snapshot): CreateWorkBaseRelationsFormModel {
    const endBaseQueryParam = routerSnapshot.queryParamMap.get(FEATURE_WORK_END_BASE_QUERY_PARAM);
    const endBase = endBaseQueryParam ? endBaseQueryParam : undefined;

    const parentIssueIdQueryParam = routerSnapshot.queryParamMap.get(FEATURE_WORK_PARENT_ISSUE_ID_QUERY_PARAM);
    const parentIssueId = parentIssueIdQueryParam ? Number(parentIssueIdQueryParam) : undefined;

    const parentWorkIdQueryParam = routerSnapshot.queryParamMap.get(FEATURE_WORK_PARENT_WORK_ID_QUERY_PARAM);
    const parentWorkId = parentWorkIdQueryParam ? Number(parentWorkIdQueryParam) : undefined;

    const projectIdQueryParam = routerSnapshot.queryParamMap.get(FEATURE_WORK_PROJECT_ID_QUERY_PARAM);
    const projectId = projectIdQueryParam ? Number(projectIdQueryParam) : undefined;

    const startBaseQueryParam = routerSnapshot.queryParamMap.get(FEATURE_WORK_START_BASE_QUERY_PARAM);
    const startBase = startBaseQueryParam ? startBaseQueryParam : undefined;

    const statusQueryParam = routerSnapshot.queryParamMap.get(FEATURE_WORK_STATUS_QUERY_PARAM);
    const status = statusQueryParam && WorkStatusEnum.check(statusQueryParam) ? statusQueryParam : undefined;

    const toDepartmentIdQueryParam = routerSnapshot.queryParamMap.get(FEATURE_WORK_TO_DEPARTMENT_ID_QUERY_PARAM);
    const toDepartmentId = toDepartmentIdQueryParam ? Number(toDepartmentIdQueryParam) : undefined;

    const toUserIdQueryParam = routerSnapshot.queryParamMap.get(FEATURE_WORK_TO_USER_ID_QUERY_PARAM);
    const toUserId = toUserIdQueryParam ? Number(toUserIdQueryParam) : undefined;

    const valUserIdQueryParam = routerSnapshot.queryParamMap.get(FEATURE_WORK_VAL_USER_ID_QUERY_PARAM);
    const valUserId = valUserIdQueryParam ? Number(valUserIdQueryParam) : undefined;

    return {
      endBase,
      parentIssueId,
      parentWorkId,
      projectId,
      startBase,
      status,
      toDepartmentId,
      toUserId,
      valUserId,
    };
  }

  private createOnDeactivateObservable(): Observable<boolean> {
    if (this.isCreate() && this.workForm?.form.touched) {
      const modalRef = this.workModalsService.createChangesSaveModal();

      return modalRef.onClose;
    } else {
      return of(true);
    }
  }

  private getNewWorkModel(): NewWorkModel {
    const createRelations = this.createRelations();

    return {
      endBase: createRelations.endBase,
      name: '',
      parentIssueId: createRelations.parentIssueId,
      parentWorkId: createRelations.parentWorkId,
      projectId: createRelations.projectId,
      startBase: createRelations.startBase,
      status: createRelations.status,
      toDepartmentId: createRelations.toDepartmentId,
      toUserId: createRelations.toUserId,
      valUserId: createRelations.valUserId,
      workType: WorkTypeEnum.TS,
    };
  }
}
