import {Component, Input, OnInit, signal, WritableSignal} from '@angular/core';
import {TranslateService} from '@ngx-translate/core';
import {CommonModule} from '@angular/common';
import {TooltipModule} from 'primeng/tooltip';

import {HistoryActionModel} from 'data-access/sdk/internal-api';
import {CustomDatePipe} from 'utils/shared-pipes';

@Component({
  selector: 'lib-history-record',
  templateUrl: './history-record.component.html',
  styleUrls: ['./history-record.component.scss'],
  standalone: true,
  imports: [CommonModule, TooltipModule],
  providers: [CustomDatePipe],
})
export class HistoryRecordComponent implements OnInit {
  @Input({required: true}) history!: HistoryActionModel;

  readonly action: WritableSignal<string | undefined> = signal(undefined);

  readonly supportedEventTypes = [1, 2, 3];
  readonly supportedModels = ['work', 'issue'];

  readonly supportedFields: { [key: string]: string[] } = {
    work: ['start_fc', 'end_fc', 'status'],
    issue: ['status'],
  };

  readonly translatedFields: { [key: string]: string[] } = {
    work: ['status'],
    issue: ['status'],
  };

  constructor(
    private readonly translateService: TranslateService,
    private readonly customDatePipe: CustomDatePipe,
  ) {
  }

  ngOnInit(): void {
    if (!this.history.contentType) {
      console.error('No content type');
      // TODO #errorService
      return;
    }

    const modelName: string = this.history.contentType.model;

    if (!this.supportedEventTypes.includes(this.history.eventType) || !this.supportedModels.includes(modelName)) {
      return;
    }

    const modelKey = `HISTORY.${modelName.toUpperCase()}`;

    switch (this.history.eventType) {
      case 1: {
        this.action.set(this.getCreateModelKey(modelKey));
        break;
      }
      case 2: {
        this.action.set(this.getObjectChangeKey(modelName, modelKey, this.history.changedFields));
        break;
      }
      case 3: {
        this.action.set(this.getObjectRemoveKey(modelKey));
        break;
      }
    }
  }

  private getObjectRemoveKey(modelKey: string) {
    const action = this.translateService
      .instant('HISTORY.EVENT_TYPE.' + String(this.history.eventType)) + this.translateService.instant(`${modelKey}.NAME`);

    return this.getPerson() + ' ' + action + ' ' + this.getDate();
  }

  private getCreateModelKey(modelKey: string): string {
    const action = this.translateService.instant(`${modelKey}.CREATE_VERB`) + this.translateService.instant(`${modelKey}.NAME`);

    return `${this.getPerson()} ${action} ${this.getDate()}`;
  }

  private getPerson(user: HistoryActionModel['user'] = this.history.user): string {
    let person = user?.shortName ?? 'User with id ' + user?.id ?? 'Unknown user';
    person += user?.position ? ' (' + user?.position + ')' : '';

    return person;
  }

  private getDate(date: HistoryActionModel['datetime'] = this.history.datetime): string {
    return date ? this.customDatePipe.transform(date) : '';
  }

  private getObjectChangeKey(
    modelName: string,
    modelKey: string,
    changedFields: HistoryActionModel['changedFields'],
  ) {
    if (!changedFields) {
      console.error('No changed fields');
      // TODO #errorService
      return;
    }

    const actions: string[] = [];

    Object.keys(changedFields ?? [])
      .forEach((change: string) => {
        if (!this.supportedFields[modelName].includes(change)) {
          return;
        }

        const oldValue = changedFields[change as any][0];
        const newValue = changedFields[change as any][1];

        const newDisplayValue = this.translatedFields[modelName].includes(change)
          ? this.translateService.instant(`${modelKey}.FIELDS_TS.${change.toUpperCase()}.${newValue}`)
          : newValue;

        let action = '';

        if (oldValue === '' || oldValue === 'None') {
          action =
            this.translateService.instant('HISTORY.CHANGE_TYPE.SET') +
            this.translateService.instant(`${modelKey}.FIELDS.${change.toUpperCase()}`) +
            `"${newDisplayValue}"`;
        } else {
          action =
            this.translateService.instant('HISTORY.CHANGE_TYPE.EDIT') +
            this.translateService.instant(`${modelKey}.FIELDS.${change.toUpperCase()}`) +
            this.translateService.instant('HISTORY.CH_TO') +
            `"${newDisplayValue}"`;
        }

        action = this.getPerson() + ' ' + action + ' ' + this.getDate();
        actions.push(action);
      });

    return actions.join(', ');
  }
}
