import {
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  EventEmitter,
  Input,
  OnInit,
  Output,
  ViewChild,
} from '@angular/core';
import {ControlValueAccessor, NG_VALUE_ACCESSOR} from '@angular/forms';
import * as moment from 'moment';
import {Calendar, CalendarTypeView} from 'primeng/calendar';
import {BehaviorSubject, filter, Observable, take} from 'rxjs';

// TODO: move to forms-ui + standalone
@Component({
  selector: 'app-custom-calendar',
  templateUrl: './custom-calendar.component.html',
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      multi: true,
      useExisting: CustomCalendarComponent,
    },
  ],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class CustomCalendarComponent implements ControlValueAccessor, AfterViewInit, OnInit {
  @ViewChild('calendarComponent') calendar?: Calendar;

  @Input() label = '';
  @Input() view: CalendarTypeView = 'date';
  @Input() showClear = false;

  @Output() readonly onChange: EventEmitter<string | string[]> = new EventEmitter<string | string[]>();
  @Output() readonly onDeactivate: EventEmitter<void> = new EventEmitter();

  dates: Date | Date[] | undefined;
  touched = false;
  disabled = false;

  componentInitedSubject: BehaviorSubject<boolean> = new BehaviorSubject<boolean>(false);
  componentInited$: Observable<boolean> = this.componentInitedSubject.asObservable();

  _calendarConfig: { [key in keyof Calendar]: any } = {} as any;

  _minDate: Date | undefined;
  _maxDate: Date | undefined;


  constructor(
    private readonly changeDetectorRef: ChangeDetectorRef,
  ) {
  }

  get componentInited() {
    return this.componentInitedSubject.value;
  }

  @Input() set minDate(date: Date) {
    if (this.calendar) {
      this.calendar.minDate = date;
    }

    this._minDate = date;
  }

  @Input() set maxDate(date: Date) {
    if (this.calendar) {
      this.calendar.maxDate = date;
    }

    this._maxDate = date;
  }

  @Input() set selectionMode(selectionMode: 'single' | 'multiple' | 'range') {
    if (selectionMode) {
      this.calendarConfig = {selectionMode};
    }
  };

  @Input() set readonlyInput(readonlyInput: boolean) {
    if (readonlyInput) {
      this.calendarConfig = {readonlyInput};
    }
  };

  @Input() set dateFormat(format: string) {
    if (format) {
      this.calendarConfig = {dateFormat: format};
    }
  }

  @Input() set styleClass(styleClass: string) {
    if (styleClass) {
      this.calendarConfig = {styleClass};
    }
  };


  set calendarConfig(config: Partial<{ [key in keyof Calendar]: any }>) {
    if (this.componentInited) {
      Object.keys(config).forEach((key) => {
        // @ts-ignore
        this.calendar[key] = config[key];
      });
    } else {
      this._calendarConfig = {...this._calendarConfig, ...config};
    }
  }

  show() {
    this.calendar?.inputfieldViewChild?.nativeElement.click();
  }

  ngOnInit() {
    this.componentInited$
      .pipe(
        filter((inited) => inited),
        take(1),
      )
      .subscribe(() => {
        this.calendarConfig = this._calendarConfig;
      });
  }

  ngAfterViewInit() {
    this.componentInitedSubject.next(true);

    if (this.calendar && this._minDate) {
      this.calendar.minDate = this._minDate;
    }

    if (this.calendar && this._maxDate) {
      this.calendar.maxDate = this._maxDate;
    }
  }

  onTouched: Function = () => {
  };

  onChanged: Function = () => {
  };

  dateChanged() {
    this.markAsTouched();

    let newVal: string | string[] | undefined;

    if (Array.isArray(this.dates)) {
      newVal = this.dates
        .reduce((acc: string[], date: Date) => {
          const isoDate = this.convertDateToIso(date);
          if (isoDate) {
            acc.push(isoDate);
          }
          return acc;
        }, []);
    } else if (this.dates) {
      newVal = this.convertDateToIso(this.dates);
    }

    this.onChanged(newVal);
    this.onChange.emit(newVal);
  }

  writeValue(isodate: string | string[] | null) {
    if (!isodate) {
      this.convertIsoToDate('');
    } else if (Array.isArray(isodate)) {
      this.dates = isodate.map((isoDate) => this.convertIsoToDate(isoDate));
    } else {
      this.dates = this.convertIsoToDate(isodate);
    }

    this.changeDetectorRef.markForCheck();
  }

  registerOnChange(fn: Function): void {
    this.onChanged = fn;
  }

  registerOnTouched(fn: Function): void {
    this.onTouched = fn;
  }

  markAsTouched() {
    if (!this.touched) {
      this.onTouched();
      this.touched = true;
    }
  }

  setDisabledState(disabled: boolean) {
    this.disabled = disabled;
    this.changeDetectorRef.markForCheck();
  }

  private convertIsoToDate(isodate: string): Date {
    return moment.utc(isodate, 'YYYY-MM-DD').toDate();
  }

  private convertDateToIso(date: Date): string {
    return moment(date).format('YYYY-MM-DD');
  }
}
