import { ChangeDetectionStrategy, ChangeDetectorRef, Component, forwardRef, Input, signal, ViewChild, WritableSignal } from '@angular/core';
import { CommonModule } from '@angular/common';
import { ControlValueAccessor, FormsModule, NG_VALUE_ACCESSOR } from '@angular/forms';
import { TranslateService } from '@ngx-translate/core';
import { Dropdown, DropdownModule } from 'primeng/dropdown';

import { Unsubscriber } from 'utils/shared-services';

import { InplaceComponent } from '../inplace/inplace.component';
import { InplaceBase } from '../inplace/inplace-base.class';
import { InplaceDropdownOptionsInterface } from './inplace-dropdown-options.interface';

// export
export { InplaceDropdownOptionsInterface } from './inplace-dropdown-options.interface';

@Component({
  selector: 'lib-inplace-dropdown-input',
  standalone: true,
  imports: [CommonModule, InplaceComponent, FormsModule, DropdownModule],
  templateUrl: './inplace-dropdown-input.component.html',
  styleUrls: ['./inplace-dropdown-input.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      useExisting: forwardRef(() => InplaceDropdownInputComponent),
      multi: true,
    },
  ],
})
export class InplaceDropdownInputComponent extends InplaceBase implements ControlValueAccessor {
  @ViewChild('controlComponent') controlComponent?: Dropdown;

  @Input() showClear = false;

  value: number | string | null | undefined;

  readonly isDisabled: WritableSignal<boolean> = signal(false);
  readonly _options: WritableSignal<InplaceDropdownOptionsInterface[]> = signal([]);

  constructor(
    private readonly translateService: TranslateService,
    private readonly changeDetectorRef: ChangeDetectorRef,
    unsubscriber: Unsubscriber
  ) {
    super(unsubscriber);
  }

  get valueLabel(): string | undefined {
    return this._options().find((item) => item.value === this.value)?.label;
  }

  get valueBgColor(): string | undefined {
    return this._options().find((item) => item.value === this.value)?.bgColor;
  }

  get valueIcon(): string | undefined {
    return this._options().find((item) => item.value === this.value)?.icon;
  }

  get valueIconColor(): string | undefined {
    return this._options().find((item) => item.value === this.value)?.iconColor;
  }

  @Input({ required: true }) set options(items: InplaceDropdownOptionsInterface[]) {
    this._options.set(
      items.map((item) => ({
        ...item,
        label: this.translateService.instant(item.label),
      }))
    );
  }

  onTouched: Function = () => {};

  onChanged: Function = () => {};

  writeValue(value: number | string | null | undefined): void {
    this.value = value;
    this.changeDetectorRef.markForCheck();
  }

  changeValue(value: number | string | null | undefined) {
    if (this.onTouched) {
      this.onTouched();
    }

    this.value = value;

    this.onChanged(value);
  }

  registerOnChange(fn: Function): void {
    this.onChanged = fn;
  }

  registerOnTouched(fn: Function): void {
    this.onTouched = fn;
  }

  setDisabledState(isDisabled: boolean): void {
    this.isDisabled.set(isDisabled);
  }

  activateInput() {
    setTimeout(() => {
      this.controlComponent?.show();

      this.onActivate.emit();
    });
  }
}
