import { ChangeDetectionStrategy, Component, ElementRef, Input, Signal, WritableSignal, computed, signal } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormControl, FormGroup, ReactiveFormsModule } from '@angular/forms';
import { TranslateModule } from '@ngx-translate/core';
import { SelectItemGroup } from 'primeng/api';

import {
  DepartmentItemsClass,
  InplaceCalendarInputComponent,
  InplaceStringInputComponent,
  SingleEntityInplaceInputComponent,
  UserItemsClass,
} from 'ui/forms-ui';
import { CIconComponent } from 'ui/custom-icons';

import { KanbanCardInterface } from '../../interfaces/kanban-card.interface';
import { KanbanCardType } from '../../interfaces/kanban-card-type.enum';
import { DepartmentBaseModel, UserBaseModel } from 'data-access/sdk/internal-api';
import { KanbanCardStatusEnum } from '../../interfaces/kanban-card-status.enum';
import { KanbanStatusColorsService } from '../../services/kanban-status-colors.service';

interface KanbanCardForm {
  name: FormControl<KanbanCardInterface['name'] | null>;
  start: FormControl<KanbanCardInterface['start'] | null>;
  end: FormControl<KanbanCardInterface['end'] | null>;
  deadline: FormControl<KanbanCardInterface['deadline'] | null>;
  status: FormControl<KanbanCardInterface['column'] | null>;
  assignee: FormControl<KanbanCardInterface['assignee'] | null>;
}

@Component({
  selector: 'ui-kanban-card',
  standalone: true,
  templateUrl: './kanban-card.component.html',
  styleUrls: ['./kanban-card.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
  imports: [
    CommonModule,
    ReactiveFormsModule,
    TranslateModule,
    InplaceStringInputComponent,
    InplaceCalendarInputComponent,
    SingleEntityInplaceInputComponent,
    CIconComponent,
  ],
})
export class KanbanCardComponent {
  private readonly colors = this.kanbanStatusColorsService.colors;
  private readonly _secondaryText: string = this.kanbanStatusColorsService.colors.secondaryText;

  private readonly _avatarLink: WritableSignal<string> = signal('');
  private readonly _deadlineColor: WritableSignal<string> = signal(this._secondaryText);
  private readonly _deadlineHelpText: WritableSignal<string | undefined> = signal(undefined);
  private readonly _endColor: WritableSignal<string> = signal(this._secondaryText);
  private readonly _endHelpText: WritableSignal<string | undefined> = signal(undefined);
  private readonly _showStatusIcon: WritableSignal<boolean> = signal(false);
  private readonly _startColor: WritableSignal<string> = signal(this._secondaryText);
  private readonly _startHelpText: WritableSignal<string | undefined> = signal(undefined);
  private readonly _statusIcon: WritableSignal<string> = signal('');
  private readonly _statusIconColor: WritableSignal<string> = signal(this.colors[KanbanCardStatusEnum.CANCELLED]);
  private readonly cardAssigneeType: WritableSignal<'user' | 'department'> = signal('user');
  private readonly cardType: WritableSignal<KanbanCardType> = signal(KanbanCardType.WORK);
  private readonly executorOptions: WritableSignal<SelectItemGroup[]> = signal([]);

  readonly secondaryText: string = this._secondaryText;
  readonly fixedHeight = 22;

  readonly avatarLink: Signal<string> = this._avatarLink.asReadonly();
  readonly deadlineColor: Signal<string> = this._deadlineColor.asReadonly();
  readonly deadlineHelpText: Signal<string | undefined> = this._deadlineHelpText.asReadonly();
  readonly endColor: Signal<string> = this._endColor.asReadonly();
  readonly endHelpText: Signal<string | undefined> = this._endHelpText.asReadonly();
  readonly showAvatar: Signal<boolean> = computed(() => !!this._avatarLink() && !!this.cardForm.controls.assignee.value);
  readonly showStatusIcon: Signal<boolean> = this._showStatusIcon.asReadonly();
  readonly startColor: Signal<string> = this._startColor.asReadonly();
  readonly startHelpText: Signal<string | undefined> = this._startHelpText.asReadonly();
  readonly statusIcon: Signal<string> = this._statusIcon.asReadonly();
  readonly statusIconColor: Signal<string> = this._statusIconColor.asReadonly();

  readonly cardForm: FormGroup<KanbanCardForm> = new FormGroup<KanbanCardForm>({
    assignee: new FormControl<KanbanCardInterface['assignee'] | null>(null),
    deadline: new FormControl<KanbanCardInterface['deadline'] | null>(null),
    end: new FormControl<KanbanCardInterface['end'] | null>(null),
    name: new FormControl<KanbanCardInterface['name'] | null>(null),
    start: new FormControl<KanbanCardInterface['start'] | null>(null),
    status: new FormControl<KanbanCardInterface['column'] | null>(null),
  });

  readonly nameIcon: Signal<string> = computed(() => {
    const type = this.cardType();

    switch (type) {
      case KanbanCardType.WORK:
        return 'ci-hammer';
      case KanbanCardType.MILESTONE:
        return 'ci-flag-line';
      default:
        console.error('Unknown card type ' + type);

        return 'ci-hammer';
    }
  });

  readonly assigneeIcon: Signal<string> = computed(() => {
    const type = this.cardAssigneeType();

    switch (type) {
      case 'user':
        return 'ci-user';
      case 'department':
        return 'ci-work-group';
      default:
        console.error('Unknown assignee type ' + type);

        return 'ci-user';
    }
  });

  constructor(private readonly kanbanStatusColorsService: KanbanStatusColorsService, private readonly elementRef: ElementRef) {}

  @Input({ required: true }) set card(card: KanbanCardInterface) {
    this.initProps(card);
  }

  private initProps(card: KanbanCardInterface) {
    this.cardForm.controls.name.setValue(card.name);
    this.cardForm.controls.start.setValue(card.start);
    this.cardForm.controls.end.setValue(card.end);
    this.cardForm.controls.deadline.setValue(card.deadline);
    this.cardForm.controls.status.setValue(card.column);
    this.cardForm.controls.assignee.setValue(card.assignee);

    this.cardType.set(card.type);

    if (card.endColor) {
      this._endColor.set(card.endColor);
      this._endHelpText.set(card.endHint);
    } else {
      this._endColor.set(this.secondaryText);
      this._endHelpText.set(undefined);
    }

    if (card.deadlineColor) {
      this._deadlineColor.set(card.deadlineColor);
      this._deadlineHelpText.set(card.deadlineHint);
    } else {
      this._deadlineColor.set(this.secondaryText);
      this._deadlineHelpText.set(undefined);
    }

    if (card.startColor) {
      this._startColor.set(card.startColor);
      this._startHelpText.set(card.startHint);
    } else {
      this._startColor.set(this.secondaryText);
      this._startHelpText.set(undefined);
    }

    this.setExecutor(card.rawModel.toUser, card.rawModel.toDepartment);

    if (card.status) {
      const icon = this.convertCardStatusToIcon(card.status);

      switch (card.status) {
        case KanbanCardStatusEnum.CANCELLED:
          this._statusIcon.set(icon);
          this._showStatusIcon.set(true);
          this._statusIconColor.set(this.colors[KanbanCardStatusEnum.CANCELLED]);
          break;
        case KanbanCardStatusEnum.PAUSED:
          this._statusIcon.set(icon);
          this._showStatusIcon.set(true);
          this._statusIconColor.set(this.colors[KanbanCardStatusEnum.PAUSED]);
          break;
      }
    }

    this._avatarLink.set(card.avatarLink ? card.avatarLink : 'assets/images/default_photo.png');

    this.setMode(!!card.hideMode);
  }

  private setMode(hideMode: boolean) {
    if (hideMode) {
      this.elementRef.nativeElement.classList.add('hide-mode');
    } else if (this.elementRef.nativeElement.classList.contains('hide-mode')) {
      this.elementRef.nativeElement.classList.remove('hide-mode');
    }
  }

  private setExecutor(toUser: UserBaseModel | undefined, toDepartment: DepartmentBaseModel | undefined) {
    if (toUser && toDepartment) {
      throw new Error('Can not set toUser and toDepartment at the same time');
    }

    if (toUser) {
      const items = new UserItemsClass([toUser]);
      this.executorOptions.update((values) => items.setItemGroup(values));
      this.cardAssigneeType.set('user');
    }

    if (toDepartment) {
      const items = new DepartmentItemsClass([toDepartment]);
      this.executorOptions.update((values) => items.setItemGroup(values));
      this.cardAssigneeType.set('department');
    }

    const controlValue = this.executorOptions()[0]?.items[0]?.value;

    if (controlValue) {
      this.cardForm.controls.assignee.setValue(controlValue);
    }
  }

  private convertCardStatusToIcon(status: KanbanCardStatusEnum): string {
    switch (status) {
      case KanbanCardStatusEnum.CANCELLED:
        return 'ci-cancelled';
      case KanbanCardStatusEnum.PAUSED:
        return 'ci-paused';
      default:
        return 'ci-hammer';
    }
  }
}
