import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';

import { WorkModel, WorkStatusEnum, WorkTypeEnum, getDeadlineHint, getEndEffHint, startBaseHint } from 'data-access/sdk/internal-api';
import { BaseAdapter } from 'utils/shared-services';

import { KanbanCardInterface } from '../interfaces/kanban-card.interface';
import { KanbanColumnEnum } from '../interfaces/kanban-column.enum';
import { KanbanCardType } from '../interfaces/kanban-card-type.enum';
import { KanbanCardStatusEnum } from '../interfaces/kanban-card-status.enum';

@Injectable({
  providedIn: 'root',
})
export class KanbanCardAdapterService extends BaseAdapter<KanbanCardInterface, WorkModel> {
  constructor() {
    super();
  }

  convert(dto: WorkModel): Observable<KanbanCardInterface> {
    const endHint = getEndEffHint(dto.endEff, dto.status, dto.workType);
    const startHint = startBaseHint(dto.startBase, dto.status);
    const deadlineHint = getDeadlineHint(dto.endEff, dto.deadline, dto.status);

    const model: KanbanCardInterface = {
      id: dto.id,
      assignee: dto.toUser || dto.toDepartment,
      avatarLink: dto.toUser ? dto.toUser.avatar : undefined,
      deadline: dto.deadline,
      deadlineColor: deadlineHint.color,
      deadlineHint: deadlineHint.text,
      end: dto.endEff,
      endColor: endHint.color,
      endHint: endHint.text,
      name: dto.name,
      start: this.convertStart(dto),
      startColor: startHint.color,
      startHint: startHint.text,
      column: this.convertColumn(dto.status) ?? this.convertError('No status found'),
      type: dto.workType ? this.convertType(dto.workType) : this.convertError('No work type found'),
      status: this.convertStatus(dto.status) ?? this.convertError('No status found'),
      rawModel: dto,
    };

    return this.createConvertObservable(model);
  }
  reverse(model: KanbanCardInterface): WorkModel {
    throw new Error('Method not implemented.');
  }

  private convertStart(dto: WorkModel): string | undefined {
    return dto.workType === WorkTypeEnum.TS ? dto.startBase : undefined;
  }

  private convertColumn(status: WorkStatusEnum): KanbanColumnEnum | undefined {
    switch (status) {
      case WorkStatusEnum.BL:
        return KanbanColumnEnum.BACKLOG;
      case WorkStatusEnum.PS:
      case WorkStatusEnum.CR:
        return KanbanColumnEnum.TODO;
      case WorkStatusEnum.WP:
        return KanbanColumnEnum.IN_PROGRESS;
      case WorkStatusEnum.CN:
      case WorkStatusEnum.AC:
        return KanbanColumnEnum.DONE;
      case WorkStatusEnum.DN:
        return KanbanColumnEnum.IN_REVIEW;
      default:
        return undefined;
    }
  }

  private convertType(type: WorkTypeEnum): KanbanCardType | undefined {
    switch (type) {
      case WorkTypeEnum.TS:
        return KanbanCardType.WORK;
      case WorkTypeEnum.MS:
        return KanbanCardType.MILESTONE;
      default:
        return undefined;
    }
  }

  private convertStatus(status: WorkStatusEnum): KanbanCardStatusEnum | undefined {
    switch (status) {
      case WorkStatusEnum.BL:
        return KanbanCardStatusEnum.BACKLOG;
      case WorkStatusEnum.PS:
        return KanbanCardStatusEnum.PAUSED;
      case WorkStatusEnum.WP:
        return KanbanCardStatusEnum.IN_PROGRESS;
      case WorkStatusEnum.CR:
        return KanbanCardStatusEnum.TODO;
      case WorkStatusEnum.CN:
        return KanbanCardStatusEnum.CANCELLED;
      case WorkStatusEnum.AC:
        return KanbanCardStatusEnum.DONE;
      case WorkStatusEnum.DN:
        return KanbanCardStatusEnum.IN_REVIEW;
      default:
        return undefined;
    }
  }
}
