import { Injectable } from '@angular/core';
import { Observable, forkJoin, map, of } from 'rxjs';

import { WorkModel, WorkStatusEnum, getDeadlineHint, getEndEffHint, startBaseHint } from 'data-access/sdk/internal-api';

import { KanbanCardStatusEnum } from '../interfaces/kanban-card-status.enum';
import { KanbanColumnEnum } from '../interfaces/kanban-column.enum';
import { KanbanColumnInterface } from '../interfaces/kanban-column.interface';
import { KanbanStatusColorsService } from './kanban-status-colors.service';
import { KanbanColumnSortService } from './kanban-column-sort.service';
import { KanbanCardAdapterService } from './kanban-card-adapter.service';
import { KanbanCardInterface } from '../interfaces/kanban-card.interface';

@Injectable({
  providedIn: 'root',
})
export class KanbanColumnsService {
  private readonly colors = this.kanbanStatusColorsService.colors;

  constructor(
    private readonly kanbanCardAdapterService: KanbanCardAdapterService,
    private readonly kanbanStatusColorsService: KanbanStatusColorsService,
    private readonly kanbanColumnSortService: KanbanColumnSortService
  ) {}
  worksToColumns$(works: WorkModel[]): Observable<KanbanColumnInterface[]> {
    const works$ = works.length > 0 ? works.map((work) => this.kanbanCardAdapterService.convert(work)) : [of({} as KanbanCardInterface)];

    return forkJoin(works$).pipe(
      map((cards: KanbanCardInterface[]) => {
        const arr: KanbanColumnInterface[] = [];

        arr.push({
          status: KanbanCardStatusEnum.BACKLOG,
          title: 'KANBAN_BOARD.COLUMNS.BACKLOG',
          titleIcon: 'ci-backlog',
          titleIconColor: this.colors.BACKLOG,
          items: cards.filter((card) => card.column === KanbanColumnEnum.BACKLOG),
        });

        const todoCards = cards
          .filter((card) => card.column === KanbanColumnEnum.TODO)
          .sort((a, b) => this.kanbanColumnSortService.todoColumnByPriority(a, b));
        arr.push({
          status: KanbanCardStatusEnum.TODO,
          title: 'KANBAN_BOARD.COLUMNS.TODO',
          titleIcon: 'ci-todo',
          titleIconColor: this.colors.TODO,
          items: todoCards,
        });

        arr.push({
          status: KanbanCardStatusEnum.IN_PROGRESS,
          title: 'KANBAN_BOARD.COLUMNS.IN_PROGRESS',
          titleIcon: 'ci-in-progress',
          titleIconColor: this.colors.IN_PROGRESS,
          items: cards.filter((card) => card.column === KanbanColumnEnum.IN_PROGRESS),
        });

        arr.push({
          status: KanbanCardStatusEnum.IN_REVIEW,
          title: 'KANBAN_BOARD.COLUMNS.IN_REVIEW',
          titleIcon: 'ci-on-check',
          titleIconColor: this.colors.IN_REVIEW,
          items: cards.filter((card) => card.column === KanbanColumnEnum.IN_REVIEW),
        });

        const doneCards = cards
          .filter((card) => card.column === KanbanColumnEnum.DONE)
          .sort((a, b) => this.kanbanColumnSortService.doneColumnByPriority(a, b));
        arr.push({
          status: KanbanCardStatusEnum.DONE,
          title: 'KANBAN_BOARD.COLUMNS.DONE',
          titleIcon: 'ci-completed',
          titleIconColor: this.colors.DONE,
          items: doneCards,
        });

        arr.forEach((column) => {
          column.items.forEach((card) => {
            card.hideMode = this.getHideMode(card.status);
          });
        });

        return arr;
      })
    );
  }

  updateCards(columns: KanbanColumnInterface[]): KanbanColumnInterface[] {
    return columns.map((column) => {
      column.items.forEach((item) => {
        // update status
        item.column = this.convertCardStatusToColumn(column.status);
        item.status = column.status;
        item.hideMode = this.getHideMode(column.status);
        item.rawModel.status = this.convertCardStatusToWorkStatus(column.status);

        // update hints
        const work = item.rawModel;
        const endHint = getEndEffHint(work.endEff, work.status, work.workType);
        const startHint = startBaseHint(work.startBase, work.status);
        const deadlineHint = getDeadlineHint(work.endEff, work.deadline, work.status);
        item.endColor = endHint.color;
        item.endHint = endHint.text;
        item.startColor = startHint.color;
        item.startHint = startHint.text;
        item.deadlineColor = deadlineHint.color;
        item.deadlineHint = deadlineHint.text;
      });

      return column;
    });
  }

  private getHideMode(status: KanbanCardStatusEnum): boolean {
    switch (status) {
      case KanbanCardStatusEnum.BACKLOG:
      case KanbanCardStatusEnum.PAUSED:
      case KanbanCardStatusEnum.DONE:
      case KanbanCardStatusEnum.CANCELLED:
        return true;
      default:
        return false;
    }
  }

  private convertCardStatusToColumn(status: KanbanCardStatusEnum): KanbanColumnEnum {
    switch (status) {
      case KanbanCardStatusEnum.BACKLOG:
        return KanbanColumnEnum.BACKLOG;
      case KanbanCardStatusEnum.TODO:
        return KanbanColumnEnum.TODO;
      case KanbanCardStatusEnum.IN_PROGRESS:
        return KanbanColumnEnum.IN_PROGRESS;
      case KanbanCardStatusEnum.IN_REVIEW:
        return KanbanColumnEnum.IN_REVIEW;
      case KanbanCardStatusEnum.DONE:
        return KanbanColumnEnum.DONE;
      case KanbanCardStatusEnum.CANCELLED:
        return KanbanColumnEnum.DONE;
      case KanbanCardStatusEnum.PAUSED:
        return KanbanColumnEnum.IN_PROGRESS;
      default:
        throw new Error('Unknown status ' + status);
    }
  }

  private convertCardStatusToWorkStatus(status: KanbanCardStatusEnum): WorkStatusEnum {
    switch (status) {
      case KanbanCardStatusEnum.BACKLOG:
        return WorkStatusEnum.BL;
      case KanbanCardStatusEnum.PAUSED:
        return WorkStatusEnum.PS;
      case KanbanCardStatusEnum.IN_PROGRESS:
        return WorkStatusEnum.WP;
      case KanbanCardStatusEnum.TODO:
        return WorkStatusEnum.CR;
      case KanbanCardStatusEnum.IN_REVIEW:
        return WorkStatusEnum.DN;
      case KanbanCardStatusEnum.CANCELLED:
      case KanbanCardStatusEnum.DONE:
        return WorkStatusEnum.AC;
      default:
        throw new Error('Unknown status ' + status);
    }
  }
}
