import { Point } from './point.class';

export class Line {
  containerElement: HTMLElement;

  lineElement: HTMLElement | undefined;
  gradientElement: HTMLElement | undefined;
  textElement: HTMLElement | undefined;

  readonly lineHeight = 6;
  readonly gradientHeight = 34;
  readonly textYOffset = 9;

  readonly lineContainerId: string;
  readonly textId: string;

  readonly textClass = 'line-text';

  constructor(
    readonly hashId: string,
    readonly textColor: string,
    readonly lineColor: string,
    readonly bgGradientColor: string,
    readonly lineStartOffset: number,
    readonly lineEndOffset: number,
    readonly text?: string,
    readonly from?: Point,
    readonly to?: Point,
    readonly subLines: Line[] = [],
    readonly zIndex: number = 1
  ) {
    this.lineContainerId = this.hashId + '-line-container';
    this.textId = this.hashId + '-line-text';

    this.containerElement = document.createElement('div');
    this.containerElement.setAttribute('id', this.lineContainerId);
    this.containerElement.style.position = 'absolute';
    this.containerElement.style.zIndex = zIndex.toString();

    const fromXOffset = from ? from.x + from.xPointCenterOffset : undefined;
    const fromYOffset = from ? from.y + from.yPointCenterOffset : undefined;
    const toXOffset = to ? to.x + to.xPointCenterOffset : undefined;
    const toYOffset = to ? to.y + to.yPointCenterOffset : undefined;
    const lineWidth = toXOffset ? (typeof fromXOffset === 'number' ? toXOffset - fromXOffset : this.lineEndOffset) : this.lineEndOffset;

    const y = fromYOffset ? fromYOffset - this.lineHeight / 2 + 1 : toYOffset ? toYOffset - this.lineHeight / 2 + 1 : 0;
    this.containerElement.style.top = y + 'px';
    this.containerElement.style.left = fromXOffset + 'px';
    this.containerElement.style.width = lineWidth + 'px';
    this.containerElement.style.height = this.lineHeight + this.gradientHeight + 'px';

    const { lineElement, gradientElement, textElement } = this.create(lineWidth);
    this.lineElement = lineElement;
    this.gradientElement = gradientElement;
    this.textElement = textElement;

    this.containerElement.appendChild(lineElement);

    if (gradientElement) {
      this.containerElement.appendChild(gradientElement);
    }

    this.containerElement.appendChild(textElement);
  }

  private create(
    lineWidth: number,
    lineHeight = this.lineHeight,
    lineColor = this.lineColor,
    yOffset = this.textYOffset,
  ) {
    const lineElement = document.createElement('div');

    lineElement.style.position = 'absolute';
    lineElement.style.left = 0 + 'px';
    lineElement.style.width = lineWidth + 'px';
    lineElement.style.height = lineHeight + 'px';
    lineElement.style.background = lineColor;

    return {
      lineElement,
      gradientElement: this.createGradient(lineWidth, 0),
      textElement: this.createText(lineWidth, yOffset),
    };
  }

  private createGradient(
    lineWidth: number,
    y: number,
    gradient: string = this.bgGradientColor,
    height = this.gradientHeight,
    lineHeight = this.lineHeight
  ) {
    const gradientElement = document.createElement('div');

    gradientElement.style.position = 'absolute';
    gradientElement.style.left = 0 + 'px';
    gradientElement.style.width = lineWidth + 'px';
    gradientElement.style.height = height + 'px';
    gradientElement.style.top = y + lineHeight + 'px';
    gradientElement.style.background = gradient;

    return gradientElement;
  }

  private createText(
    lineWidth: number,
    y: number,
    id = this.textId,
    textClass = this.textClass,
    color: string = this.textColor,
    text: string | undefined = this.text
  ) {
    const textContainerElement = document.createElement('div');

    textContainerElement.style.position = 'absolute';
    textContainerElement.style.display = 'flex';
    textContainerElement.style.left = 0 + 'px';
    textContainerElement.style.width = lineWidth + 'px';
    textContainerElement.style.top = y + 'px';

    if (text) {
      const textBlockElement = document.createElement('div');

      textBlockElement.setAttribute('id', id);
      textBlockElement.setAttribute('class', textClass);

      textBlockElement.style.margin = 'auto';
      textBlockElement.style.textAlign = 'center';
      textBlockElement.style.padding = '2px 4px';
      textBlockElement.style.color = color;

      textBlockElement.innerText = text;

      textContainerElement.appendChild(textBlockElement);
    }

    return textContainerElement;
  }
}
