import { OverlayPanel } from 'primeng/overlaypanel';

import { Point } from '../point.class';

export class RhombPoint extends Point {
  containerElement: HTMLElement = document.createElement('div');

  pointElement: SVGSVGElement | undefined;
  lineElement: SVGSVGElement | undefined;

  textContainer: SVGSVGElement | undefined;
  textElement: SVGTextElement | undefined;
  textBgElement: SVGRectElement | undefined;

  readonly textElementPaddingX = 4;
  readonly textElementPaddingY = 2;

  readonly pointBgColor = 'white';
  readonly pointWidth = 28;
  readonly pointHeight = 28;
  readonly xPointCenterOffset = 14;
  readonly yPointCenterOffset = 14;

  readonly lineStokeWidth = 2;

  readonly pointContainerId: string;
  readonly pointCompletedId: string;
  readonly pointId: string;
  readonly lineId: string; // need for gradient!
  readonly textContainerId: string;
  readonly textId: string;
  readonly textBgId: string;

  readonly textClass = 'rhomb-text';

  constructor(
    readonly hashId: string,
    readonly completed: boolean,
    readonly pointColor: string,
    readonly textBgColor: string,
    readonly textColor: string,
    x: number,
    y: number,
    zIndex: number,
    readonly text?: string,
    readonly overlayPanel?: OverlayPanel,
    readonly overlayEventFn?: (point: Point) => void,
    readonly overlayData?: any,
    readonly textLineHeight = 30,
  ) {
    super();

    this.hashId = hashId;
    this.pointId = 'rhomb-' + this.hashId;
    this.pointCompletedId = this.pointId + '-completed';
    this.pointContainerId = this.pointId + '-container';
    this.lineId = this.pointId + '-line';
    this.textContainerId = this.pointId + '-text-container';
    this.textId = this.pointId + '-text';
    this.textBgId = this.pointId + '-text-bg';

    this.containerElement.setAttribute('id', this.pointContainerId);

    this.create();

    super.setElementPosition(x, y, zIndex);
  }

  create(): void {
    if (this.pointElement || this.textContainer || this.textElement || this.lineElement) {
      throw new Error('Point already created');
    }

    this.pointElement = this.createPointElement(this.completed);

    if (this.text) {
      this.lineElement = this.createLineElement(1, 0);

      const { textContainer, textElement } = this.createTextElement(this.text);
      this.textContainer = textContainer;
      this.textElement = textElement;

      this.containerElement.appendChild(this.lineElement);
      this.containerElement.appendChild(this.textContainer);
    }

    this.containerElement.appendChild(this.pointElement);

    setTimeout(() => this.reRender());
  }

  reRender(): void {
    if (!this.textElement || !this.textContainer || !this.pointElement || !this.lineElement) {
      throw new Error('Point not created');
    }

    // calculate after rendering
    const textWidth = this.textElement.getComputedTextLength();
    const textHeight = this.textElement.getBoundingClientRect().height;

    this.textElement.setAttribute('x', this.textElementPaddingX.toString());
    this.textElement.setAttribute('y', textHeight.toString());

    if (this.textBgElement) {
      this.textBgElement.remove();
    }

    this.textBgElement = this.createTextBgElement(textWidth, textHeight);

    this.textContainer.setAttribute('width', this.textBgElement.width.baseVal.value.toString());
    this.textContainer.setAttribute('height', this.textBgElement.height.baseVal.value.toString());

    this.textContainer.style.top = this.pointHeight + this.textLineHeight + 'px';
    this.textContainer.style.left = this.pointWidth / 2 - (textWidth + this.textElementPaddingX * 2) / 2 + 'px';
    this.lineElement.style.top = this.pointWidth + 'px';
    this.lineElement.style.left = this.pointWidth / 2 - this.lineStokeWidth / 2 + 'px';

    this.textContainer.insertBefore(this.textBgElement, this.textElement);
  }

  private createPointElement(
    completed: boolean,
    id = this.pointId,
    svgPointHeight = this.pointHeight,
    svgPointWidth = this.pointWidth
  ): SVGSVGElement {
    const pointElement = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
    pointElement.setAttribute('fill', 'none');
    pointElement.setAttribute('width', svgPointWidth.toString());
    pointElement.setAttribute('height', svgPointHeight.toString());
    pointElement.setAttribute('id', id);

    pointElement.style.position = 'absolute';

    if (completed) {
      this.createCompleteRect(pointElement);
    } else {
      this.createUncompleteRect(pointElement);
    }

    pointElement.style.position = 'absolute';

    return pointElement;
  }

  private createUncompleteRect(pointElement: SVGSVGElement, pointColor: string = this.pointColor, svgBgColor = this.pointBgColor) {
    const rect1 = document.createElementNS('http://www.w3.org/2000/svg', 'rect');
    rect1.setAttribute('width', '20');
    rect1.setAttribute('height', '20');
    rect1.setAttribute('rx', '2.57128');
    rect1.setAttribute('transform', 'matrix(0.707111 -0.707103 0.707111 0.707103 0 14.1426)');
    rect1.setAttribute('fill', pointColor);

    const rect2 = document.createElementNS('http://www.w3.org/2000/svg', 'rect');
    rect2.setAttribute('width', '20');
    rect2.setAttribute('height', '20');
    rect2.setAttribute('rx', '2.57128');
    rect2.setAttribute('transform', 'matrix(0.707111 -0.707103 0.707111 0.707103 0 14.1426)');
    rect2.setAttribute('stroke', pointColor);

    const rect3 = document.createElementNS('http://www.w3.org/2000/svg', 'rect');
    rect3.setAttribute('width', '14.4144');
    rect3.setAttribute('height', '14.4144');
    rect3.setAttribute('rx', '1.28564');
    rect3.setAttribute('transform', 'matrix(0.707111 -0.707103 0.707111 0.707103 3.99988 14)');
    rect3.setAttribute('fill', svgBgColor);

    pointElement.appendChild(rect1);
    pointElement.appendChild(rect2);
    pointElement.appendChild(rect3);
  }

  private createCompleteRect(
    pointElement: SVGSVGElement,
    id = this.pointCompletedId,
    pointBgColor: string = this.pointBgColor,
    pointColor: string = this.pointColor
  ) {
    const outerRect = document.createElementNS('http://www.w3.org/2000/svg', 'rect');
    outerRect.setAttribute('x', '0.707111');
    outerRect.setAttribute('width', '19');
    outerRect.setAttribute('height', '19');
    outerRect.setAttribute('rx', '2.07128');
    outerRect.setAttribute('transform', 'matrix(0.707111 -0.707103 0.707111 0.707103 0 15.6426)');
    outerRect.setAttribute('fill', pointBgColor);
    outerRect.setAttribute('stroke', pointColor);
    outerRect.setAttribute('id', id);

    const innerRect = document.createElementNS('http://www.w3.org/2000/svg', 'rect');

    innerRect.setAttribute('width', '13');
    innerRect.setAttribute('height', '13');
    innerRect.setAttribute('rx', '1.28564');
    innerRect.setAttribute('transform', 'matrix(0.707111 -0.707103 0.707111 0.707103 4.6 15)');
    innerRect.setAttribute('fill', pointColor);

    const mark = document.createElementNS('http://www.w3.org/2000/svg', 'path');

    // need manual calculation (if change innerRectSize)!
    mark.setAttribute('d', 'M11 15L13.011 17L17 13');
    mark.setAttribute('stroke', pointBgColor);
    mark.setAttribute('stroke-width', '2');
    mark.setAttribute('stroke-linecap', 'round');
    mark.setAttribute('stroke-linejoin', 'round');

    pointElement.appendChild(outerRect);
    pointElement.appendChild(innerRect);
    pointElement.appendChild(mark);
  }

  private createTextElement(
    text: string,
    textColor: string = this.textColor,
    containerId = this.textContainerId,
    textId = this.textId,
    textClass = this.textClass
  ) {
    const textContainer = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
    textContainer.setAttribute('id', containerId);
    textContainer.style.position = 'absolute';

    const textElement = document.createElementNS('http://www.w3.org/2000/svg', 'text');
    textElement.setAttribute('id', textId);
    textElement.setAttribute('class', textClass);
    textElement.setAttribute('fill', textColor);
    textElement.innerHTML = text;

    textContainer.appendChild(textElement);

    return {
      textContainer,
      textElement,
    };
  }

  private createTextBgElement(
    textWidth: number,
    textHeight: number,
    id = this.textBgId,
    textBgColor: string = this.textBgColor,
    paddingX: number = this.textElementPaddingX,
    paddingY: number = this.textElementPaddingY
  ) {
    const textBgElement = document.createElementNS('http://www.w3.org/2000/svg', 'rect');
    textBgElement.setAttribute('x', '0');
    textBgElement.setAttribute('y', '0');
    textBgElement.setAttribute('width', (textWidth + paddingX * 2).toString());
    textBgElement.setAttribute('height', (textHeight + paddingY * 2).toString());
    textBgElement.setAttribute('fill', textBgColor);
    textBgElement.setAttribute('class', id);

    return textBgElement;
  }

  private createLineElement(
    x: number,
    y: number,
    id = this.lineId,
    color: string = this.pointColor,
    lineHeight = this.textLineHeight,
    lineStokeWidth = this.lineStokeWidth
  ) {
    const gradientId = id + '-gradient';

    const lineContainer = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
    lineContainer.setAttribute('width', lineStokeWidth.toString());
    lineContainer.setAttribute('height', lineHeight.toString());
    lineContainer.setAttribute('id', id);

    // line to text
    const lineElement = document.createElementNS('http://www.w3.org/2000/svg', 'line');

    lineElement.setAttribute('x1', x.toString());
    lineElement.setAttribute('y1', y.toString());
    lineElement.setAttribute('x2', x.toString());
    lineElement.setAttribute('y2', (y + lineHeight).toString());
    lineElement.setAttribute('stroke', `url(#${gradientId})`);
    lineElement.setAttribute('stroke-width', lineStokeWidth.toString());

    lineContainer.appendChild(lineElement);

    // gradient
    const defs = document.createElementNS('http://www.w3.org/2000/svg', 'defs');
    const linearGradientElement = document.createElementNS('http://www.w3.org/2000/svg', 'linearGradient');

    linearGradientElement.setAttribute('id', gradientId);
    linearGradientElement.setAttribute('gradientUnits', 'userSpaceOnUse');

    // gradient rotation
    linearGradientElement.setAttribute('x1', '0');
    linearGradientElement.setAttribute('y1', '0');
    linearGradientElement.setAttribute('x2', '0');
    linearGradientElement.setAttribute('y2', '59');

    const stopElement1 = document.createElementNS('http://www.w3.org/2000/svg', 'stop');

    stopElement1.setAttribute('offset', '0');
    stopElement1.setAttribute('stop-color', color);

    const stopElement2 = document.createElementNS('http://www.w3.org/2000/svg', 'stop');

    stopElement2.setAttribute('offset', '1');
    stopElement2.setAttribute('stop-color', color);
    stopElement2.setAttribute('stop-opacity', '0.1');

    linearGradientElement.appendChild(stopElement1);
    linearGradientElement.appendChild(stopElement2);
    defs.appendChild(linearGradientElement);

    lineContainer.appendChild(defs);

    lineContainer.style.position = 'absolute';

    return lineContainer;
  }
}
