import { CommonModule } from '@angular/common';
import {
  ChangeDetectionStrategy,
  Component,
  ElementRef,
  Input,
  ViewChild,
  Signal,
  ChangeDetectorRef,
  WritableSignal,
  signal,
  computed,
  Output,
  EventEmitter,
  AfterViewInit,
} from '@angular/core';
import { OverlayPanel, OverlayPanelModule } from 'primeng/overlaypanel';
import { TranslateModule } from '@ngx-translate/core';
import * as moment from 'moment';

import {
  ProjectMilestoneModel,
  ProjectStageModel,
  ProjectStageStateEnum,
  ProjectMilestoneIndicatorEnum,
  ProjectMilestoneStatusEnum,
} from 'data-access/sdk/internal-api';
import { hashFromString } from 'utils/common-helpers';
import { ResizeObserverDirective } from 'utils/shared-directives';

import { PointFabricService } from '../../services/point-fabric.service';
import { Line } from '../../classes/line.class';
import { Point } from '../../classes/point.class';
import { LineFabricService } from '../../services/line-fabric.service';
import { PointOverlayService } from '../../services/point-overlay.service';
import { DumbPoint } from '../../classes/points/dumb-point.class';
import { debounceTime, filter } from 'rxjs';
import { ColorStatusEnum } from '../../interfaces/color-status.enum';
import { PointOverlayDataInterface } from '../../interfaces/point-overlay-data.interface';
import { PointTypeEnum } from '../../interfaces/point-type.enum';
import { UiMilestonePointOverlayComponent } from '../ui-milestone-point-overlay/ui-milesstone-point-overlay.component';

@Component({
  selector: 'ui-milestones-timeline',
  standalone: true,
  templateUrl: './ui-milestones-timeline.component.html',
  styleUrls: ['./ui-milestones-timeline.component.scss'],
  imports: [CommonModule, TranslateModule, ResizeObserverDirective, OverlayPanelModule, UiMilestonePointOverlayComponent],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class UiMilestonesTimelineComponent implements AfterViewInit {
  private points: Point[] = [];
  private lines: Line[] = [];

  private firstPointDay = 0;
  private oneDayInPx: WritableSignal<number> = signal(1);

  private readonly yOffset = 17;
  private readonly xLineEndOffset = 80;
  private readonly textLineHeightOffset = 20;

  private readonly _infinityMode: WritableSignal<boolean> = signal(false);
  private readonly _nowMarkOffsetChangedWithDebounce = new EventEmitter<number>();
  private readonly _nowMarkId = 'pointNow';

  @ViewChild('pointSingleOverlay') pointSingleOverlay!: OverlayPanel;

  @Input() nowDate: string | undefined = moment().format('YYYY-MM-DD'); // for debug
  @Input() bgColor = '#fafafa';
  @Input() addEvenOffset = false;
  @Input() infinityModeOffset = 500;
  @Input() fixedDayInPx: number | undefined;
  @Input() basicTextLineHeight = 30;
  @Input() disableDefaultError = false;

  @Output() nowMarkOffset = this._nowMarkOffsetChangedWithDebounce.asObservable().pipe(filter(Boolean), debounceTime(50));

  readonly _stages: WritableSignal<ProjectStageModel[]> = signal([]);
  readonly canCreateTimeline: Signal<boolean> = computed(() => {
    const timelineMilestones = this._stages().map((stage) => stage.endMilestone);
    timelineMilestones.pop(); // last stage have not milestone (normal)
    const allStagesHaveMilestones = timelineMilestones.every((milestone) => milestone);
    return this.stages?.length > 0 && allStagesHaveMilestones;
  });

  readonly activePointOverlayData: Signal<PointOverlayDataInterface> = this.pointOverlayService.activePointOverlayData;

  constructor(
    readonly elementRef: ElementRef<HTMLElement>,
    private readonly pointFabric: PointFabricService,
    private readonly lineFabric: LineFabricService,
    private readonly pointOverlayService: PointOverlayService,
    private readonly changeDetectorRef: ChangeDetectorRef
  ) {}

  get stages(): ProjectStageModel[] {
    return this._stages();
  }

  @Input({ required: true }) set stages(value: ProjectStageModel[]) {
    this._stages.set(value);
  }

  @Input() set infinityMode(value: boolean) {
    if (value) {
      this._infinityMode.set(true);
    }
  }

  ngAfterViewInit(): void {
    // this.render(); // not needed bcs resizeObserver emit first value
  }

  /**
   * Generates a timeline based on the given stages, container element, points, and current date.
   * The generated timeline is appended to the container element and the points are added to the points array.
   *
   * @param stages - An array of milestone stages.
   * @param containerElement - The container element to append the timeline to.
   * @param points - An array of points to add the generated points to.
   * @param nowDate - The current date.
   */
  generateTimeline(
    stages: ProjectStageModel[] = this.stages,
    containerElement: HTMLElement = this.elementRef.nativeElement,
    points = this.points,
    nowDate = this.nowDate,
    basicTextLineHeight = this.basicTextLineHeight,
    textLineHeightOffset = this.textLineHeightOffset,
    addEvenOffset = this.addEvenOffset,
    xLineEndOffset = this.xLineEndOffset,
    infinityModeOffset = this.infinityModeOffset,
    yOffset = this.yOffset,
    firstPointDay = this.firstPointDay,
    nowMarkId = this._nowMarkId
  ) {
    const canCreateNowMark = !stages.every((stage) => stage.state === ProjectStageStateEnum.DONE);
    const nowMarkDay: WritableSignal<number | undefined> = signal<number | undefined>(undefined);
    const currentDay = moment(nowDate).diff(moment(0), 'days') - this.firstPointDay;
    const isMarkedAsFeatureCheck = computed(() => typeof nowMarkDay() === 'number');
    const lastPointDate = stages[stages.length - 2].endMilestone;
    const lastPointDay: number = lastPointDate ? moment(lastPointDate.endEff).diff(moment(0), 'days') - firstPointDay : 0;
    const lastAxisOffsetX = lastPointDay * this.oneDayInPx() + xLineEndOffset * 2 + (this._infinityMode() ? infinityModeOffset : 0);
    const firstAxisOffsetX = this._infinityMode() ? 0 - infinityModeOffset : 0;
    const isNowMarkLaterLastLine = currentDay > lastPointDay;
    const currentDayOffset: number = this.oneDayInPx() * currentDay + xLineEndOffset;

    stages.reduce((prevPoint: Point | undefined, currentStage: ProjectStageModel, index: number, arr: ProjectStageModel[]) => {
      const isEven = index % 2 === 0;
      const textLineHeight = !addEvenOffset || isEven ? basicTextLineHeight : basicTextLineHeight + textLineHeightOffset;
      const isFirstStage = index === 0;
      const isLastStage = index === arr.length - 1;
      const prevStage = stages[index - 1];
      const currentStageLineColor = this.convertToLineColor(currentStage, prevStage);
      const currentStageTextColor = this.convertToTextColor(currentStage);
      const lineId = hashFromString(currentStage.name).toString();
      const sublines: Line[] = []; // Create an array to store the sublines
      const isPointCompleted = currentStage.state === ProjectStageStateEnum.DONE;
      const pointType = isPointCompleted ? PointTypeEnum.FilledRhomb : PointTypeEnum.Rhomb;
      const endPointDay: number | undefined = currentStage.endMilestone
        ? moment(currentStage.endMilestone.endEff).diff(moment(0), 'days') - firstPointDay
        : undefined;
      const pointOffsetX: number | undefined =
        typeof endPointDay !== 'undefined' ? endPointDay * this.oneDayInPx() + xLineEndOffset : undefined;
      const startPointDay = isFirstStage ? 0 : moment(prevStage.endMilestone?.endEff).diff(moment(0), 'days') - firstPointDay;
      const isCurrentStage =
        typeof startPointDay !== 'undefined' && typeof endPointDay !== 'undefined'
          ? startPointDay <= currentDay && currentDay <= endPointDay
          : false;
      // debugger;
      const isNowMarkEarlerFirstLine = currentDay < startPointDay;
      const currentDayPointX: number = isNowMarkLaterLastLine
        ? this._infinityMode()
          ? currentDayOffset
          : lastAxisOffsetX - xLineEndOffset / 2
        : isNowMarkEarlerFirstLine
        ? this._infinityMode()
          ? 0 - currentDayOffset
          : xLineEndOffset / 2
        : currentDayOffset;

      if (currentStage.endMilestone) {
        // create point (add to points array later bcs now mark can be created earler and
        // we havn't changeColorMethod() in Point class bcs winter is coming)
        const createpointFn = () => {
          if (!currentStage.endMilestone || typeof pointOffsetX === 'undefined') {
            throw new Error('End milestone or pointOffsetX is undefined');
          }

          return this.pointFabric.create<PointOverlayDataInterface>({
            id: `${currentStage.endMilestone.id}-${hashFromString(currentStage.endMilestone.name)}`,
            colorStatus: this.convertToPointColor(currentStage.endMilestone),
            text: currentStage.endMilestone.name,
            offsetX: pointOffsetX,
            offsetY: yOffset,
            pointType,
            overlayPanel: this.pointSingleOverlay,
            overlayData: {
              endBase: currentStage.endMilestone.endEff,
              milestoneName: currentStage.endMilestone.name,
              overdueDaysBase: currentStage.endMilestone.overdueDaysBase,
              toUser: currentStage.endMilestone.toUser,
              toDepartment: currentStage.endMilestone.toDepartment,
              status: currentStage.endMilestone.status,
            },
            textLineHeight,
          });
        };

        let point = createpointFn();

        // create line if it is the first line in timeline
        if (isFirstStage) {
          const dumpStartPoint = new DumbPoint();
          dumpStartPoint.setElementPosition(firstAxisOffsetX, yOffset, 0);

          if (canCreateNowMark && isNowMarkEarlerFirstLine) {
            const nowMarkPoint = this.addPoint(nowMarkId, ColorStatusEnum.BLUE, currentDayPointX, yOffset, PointTypeEnum.Mark);

            const currentLine = this.addLine('toNowPoint', ColorStatusEnum.BLUE, undefined, dumpStartPoint, nowMarkPoint);
            const dumbLine = this.addLine('dumbLine', ColorStatusEnum.GREY, undefined, nowMarkPoint, point);
            sublines.push(currentLine, dumbLine);
            nowMarkDay.set(currentDay);
          }

          this.addLine(
            lineId,
            isMarkedAsFeatureCheck() ? ColorStatusEnum.GREY : currentStageLineColor,
            currentStageTextColor,
            dumpStartPoint,
            point,
            currentStage.name,
            sublines,
            true,
            false,
            false
          );
        }

        // create now mark point
        if (canCreateNowMark && !isFirstStage && !isLastStage && isCurrentStage) {
          if (!prevStage.endMilestone) {
            throw new Error('End milestone not found');
          }

          const currentDayPointColor = this.convertCurrenMarkColor(prevStage.endMilestone);
          const nowMarkPoint = this.addPoint(nowMarkId, currentDayPointColor, currentDayPointX, yOffset, PointTypeEnum.Mark);
          const nowMarkLine = this.addLine(
            'toNowPointSubline',
            this.convertCurrenMarkColor(prevStage.endMilestone),
            undefined,
            prevPoint,
            nowMarkPoint
          );
          const dumbLine = this.addLine('dumbSublineLine', ColorStatusEnum.GREY, undefined, nowMarkPoint, point);
          sublines.push(nowMarkLine, dumbLine);
          nowMarkDay.set(currentDay);
        }

        // basic line create
        if (!isFirstStage && !isLastStage) {
          this.addLine(
            lineId,
            isMarkedAsFeatureCheck() ? ColorStatusEnum.GREY : currentStageLineColor,
            currentStageTextColor,
            prevPoint,
            point,
            currentStage.name,
            sublines
          );
        }

        // recreate point bcs must be grey (if now mark was earler)
        point = createpointFn();

        // add point to container
        containerElement.appendChild(point.containerElement);
        points.push(point);

        return point;
      }

      // create line if it is the last line in timeline
      if (isLastStage) {
        const lastDumbPoint = new DumbPoint();
        lastDumbPoint.setElementPosition(lastAxisOffsetX, yOffset, 0);

        if (canCreateNowMark && isNowMarkLaterLastLine) {
          if (!prevPoint) {
            throw new Error('Prev point not found');
          }

          const nowMarkPoint = this.addPoint(
            nowMarkId,
            prevStage && prevStage.endMilestone ? this.convertCurrenMarkColor(prevStage.endMilestone) : ColorStatusEnum.BLUE,
            currentDayPointX,
            yOffset,
            PointTypeEnum.Mark
          );

          const currentLine = this.addLine(
            'toNowPointSubline',
            prevStage && prevStage.endMilestone ? this.convertCurrenMarkColor(prevStage.endMilestone) : ColorStatusEnum.BLUE,
            undefined,
            prevPoint,
            nowMarkPoint
          );
          const dumbLine = this.addLine('dumbSublineLine', ColorStatusEnum.GREY, undefined, nowMarkPoint, lastDumbPoint);
          sublines.push(currentLine, dumbLine);
          nowMarkDay.set(currentDay);
        }

        this.addLine(
          lineId,
          isMarkedAsFeatureCheck() ? ColorStatusEnum.GREY : currentStageLineColor,
          currentStageTextColor,
          prevPoint,
          lastDumbPoint,
          currentStage.name,
          sublines,
          false,
          true,
          false
        );
      }

      return undefined;
    }, undefined);
  }

  /**
   * Remove all points and lines from the container elements,
   * recalculate the axis, and generate the timeline.
   *
   * TODO: change delete-create to set x and y in Point
   */
  render() {
    if (!this.canCreateTimeline()) {
      console.warn('No stages found on render');
      return;
    }

    this.setComponentHeight();

    this.points.forEach((point) => {
      point.containerElement.remove();
    });

    this.lines.forEach((line) => line.containerElement.remove());

    this.calculateAxis();
    this.generateTimeline();
    this.pointOverlayService.createEvents(this.points, this.changeDetectorRef);

    const nowMark = this.getNowMark();

    if (nowMark) {
      this._nowMarkOffsetChangedWithDebounce.emit(nowMark.x);
    }
  }

  private setComponentHeight() {
    const minHeight = this.yOffset / 2 + 30 + this.basicTextLineHeight + this.textLineHeightOffset * 0.9 * 2;
    this.elementRef.nativeElement.style.height = minHeight + 'px';
  }

  private getNowMark(): Point | undefined {
    return this.points.find((point) => point.hashId === this._nowMarkId);
  }

  /**
   * Calculates the axis based on the given element reference.
   *
   * @param stages - The array of milestone stages. Defaults to the stages property of the current instance.
   * @param elementRef - The element reference. Defaults to the native element reference of the current instance.
   */
  private calculateAxis() {
    const containerWidth = this.elementRef.nativeElement.offsetWidth - this.xLineEndOffset * 2;

    const milestones: ProjectMilestoneModel[] = this.stages.map((stage) => stage.endMilestone).filter(Boolean) as ProjectMilestoneModel[];

    const pointDates: moment.Moment[] = milestones.map((work) => moment(work.endEff));
    const days = pointDates[pointDates.length - 1].diff(pointDates[0], 'days');

    if (!!this.fixedDayInPx) {
      this.oneDayInPx.set(this.fixedDayInPx);
    } else {
      this.oneDayInPx.set(containerWidth / days);
    }

    this.firstPointDay = pointDates[0].diff(moment(0), 'days');
  }

  /**
   * Adds a point to the element and points array.
   *
   * @param id - The ID of the point.
   * @param colorStatus - The color status of the point.
   * @param offsetX - The X offset of the point.
   * @param offsetY - The Y offset of the point.
   * @param pointType - The type of the point.
   * @param [text] - The text of the point.
   * @param [container=this.elementRef.nativeElement] - The container element to append the point to.
   * @param [points=this.points] - The array to store the point in.
   * @return The created point.
   */
  private addPoint(
    id: string,
    colorStatus: ColorStatusEnum,
    offsetX: number,
    offsetY: number,
    pointType: PointTypeEnum,
    text?: string,
    container: HTMLElement = this.elementRef.nativeElement,
    points: Point[] = this.points,
    textLineHeight = this.basicTextLineHeight
  ) {
    const point = this.pointFabric.create({
      id,
      colorStatus,
      text,
      offsetX,
      offsetY,
      pointType,
      textLineHeight,
    });

    container.appendChild(point.containerElement);

    points.push(point);

    return point;
  }

  /**
   * Adds a line to the container element and lines array.
   *
   * @param id - The id of the line.
   * @param colorStatus - The color status of the line.
   * @param point1 - The first point of the line.
   * @param point2 - The second point of the line.
   * @param [text] - The text of the line.
   * @param [sublines=[]] - The sublines of the line.
   * @param [container=this.elementRef.nativeElement] - The container element.
   * @param [xLineEndOffset=this.xLineEndOffset] - The x line end offset.
   * @return The added line.
   */
  private addLine(
    id: string,
    colorStatus: ColorStatusEnum,
    textColor: ColorStatusEnum = ColorStatusEnum.GREY,
    point1: Point | undefined,
    point2: Point | undefined,
    text?: string,
    sublines: Line[] = [],
    isFirst: boolean = false,
    isLast: boolean = false,
    isEndSubline: boolean = false,
    container: HTMLElement = this.elementRef.nativeElement,
    xLineEndOffset: number = this.xLineEndOffset,
    bgColor = this.bgColor,
    manualOffGradient: boolean = this._infinityMode()
  ) {
    const line = this.lineFabric.create(
      id,
      colorStatus,
      textColor,
      xLineEndOffset,
      point1,
      point2,
      text,
      sublines,
      isFirst,
      isLast,
      isEndSubline,
      bgColor,
      manualOffGradient
    );

    container.appendChild(line.containerElement);

    this.lines.push(line);

    if (sublines.length > 0) {
      sublines.forEach((subline) => {
        container.appendChild(subline.containerElement);

        this.lines.push(subline);
      });
    }

    return line;
  }

  private convertToTextColor(stage: ProjectStageModel): ColorStatusEnum {
    switch (stage.state) {
      case ProjectStageStateEnum.CURRENT:
        return ColorStatusEnum.BLUE;
      case ProjectStageStateEnum.DONE:
        return ColorStatusEnum.GREEN;
      case ProjectStageStateEnum.FUTURE:
        return ColorStatusEnum.GREY;
      default:
        // TODO #errorService
        console.error(`Unknown text color (state): ${stage}`);
        return ColorStatusEnum.GREY;
    }
  }

  /**
   * Convert the timeline stage state and end milestone indicator to a color status.
   *
   * @param stage - The timeline stage state.
   * @param endMilestone - The end milestone.
   * @returns The color status corresponding to line.
   */
  private convertToLineColor(stage: ProjectStageModel, prevStage: ProjectStageModel): ColorStatusEnum {
    switch (stage.state) {
      case ProjectStageStateEnum.CURRENT:
        return ColorStatusEnum.BLUE;
      case ProjectStageStateEnum.DONE:
        return ColorStatusEnum.GREEN;
      case ProjectStageStateEnum.FUTURE: {
        if (!prevStage.endMilestone) {
          console.error(`Unknown line color (prev stage endMilestone): ${prevStage}`);

          return ColorStatusEnum.GREY;
        }

        switch (prevStage.endMilestone.indicator) {
          case ProjectMilestoneIndicatorEnum.GREEN:
            return ColorStatusEnum.BLUE;
          case ProjectMilestoneIndicatorEnum.AMBER:
            return ColorStatusEnum.AMBER;
          case ProjectMilestoneIndicatorEnum.RED:
            return ColorStatusEnum.RED;
          default:
            return ColorStatusEnum.GREY;
        }
      }
      default:
        // TODO #errorService
        console.error(`Unknown line color (state): ${stage}`);

        return ColorStatusEnum.GREY;
    }
  }

  /**
   * Converts the work model to a corresponding color status.
   *
   * @param milestone - The work model to convert.
   * @returns The color status corresponding to point.
   */
  private convertToPointColor(milestone: ProjectMilestoneModel): ColorStatusEnum {
    if (milestone.status === ProjectMilestoneStatusEnum.DN || milestone.status === ProjectMilestoneStatusEnum.AC) {
      return ColorStatusEnum.GREEN;
    }

    switch (milestone.indicator) {
      case ProjectMilestoneIndicatorEnum.GREEN:
        return ColorStatusEnum.GREY;
      case ProjectMilestoneIndicatorEnum.AMBER:
        return ColorStatusEnum.AMBER;
      case ProjectMilestoneIndicatorEnum.RED:
        return ColorStatusEnum.RED;
      default:
        return ColorStatusEnum.GREY;
    }
  }

  /**
   * Converts the color of a previous milestone to the corresponding color of the current milestone.
   *
   * @param prevMilestone - The previous milestone.
   * @returns The color of the current milestone.
   */
  private convertCurrenMarkColor(prevMilestone: ProjectMilestoneModel): ColorStatusEnum {
    const prevColor = this.convertToPointColor(prevMilestone);

    switch (prevColor) {
      case ColorStatusEnum.GREEN:
        return ColorStatusEnum.BLUE;
      case ColorStatusEnum.AMBER:
        return ColorStatusEnum.AMBER;
      case ColorStatusEnum.RED:
        return ColorStatusEnum.RED;
      default:
        // TODO #errorService
        console.error(`Unknown indicator: ${prevMilestone.indicator}`);
        return ColorStatusEnum.BLUE;
    }
  }
}
