import { Injectable } from '@angular/core';

import { hexToRgbA } from 'utils/common-helpers';

import { ColorStatusEnum } from '../interfaces/color-status.enum';
import { Point } from '../classes/point.class';
import { Line } from '../classes/line.class';

@Injectable({
  providedIn: 'root',
})
export class LineFabricService {
  /**
   * Creates a new Line object with the given parameters.
   *
   * @param id - The ID of the line.
   * @param lineColorStatus - The color status of the line.
   * @param xEndOffset - The x end offset of the line.
   * @param from - The starting point of the line (optional).
   * @param to - The ending point of the line (optional).
   * @param text - The text of the line (optional).
   * @param subLines - The sub-lines of the line (defaults to an empty array).
   * @param bgGradientColor - The background gradient color of the line (optional).
   * @param zIndex - The z-index of the line (defaults to 1).
   * @returns A new Line object.
   */
  create(
    id: string,
    lineColorStatus: ColorStatusEnum,
    textColorStatus: ColorStatusEnum,
    xEndOffset: number,
    from?: Point,
    to?: Point,
    text?: string,
    subLines: Line[] = [],
    isFirst: boolean = false,
    isLast: boolean = false,
    isEndSubline: boolean = false,
    globalBgColor: string = '#fafafa',
    manualOffGradient: boolean = false,
    zIndex: number = 1
  ) {
    let lineColor: string = this.mapStatusColorToColor(lineColorStatus);
    const textColor = this.mapStatusColorToColor(textColorStatus);
    const bgGradientColor = this.getGradientColor(lineColor, globalBgColor, isFirst, isLast, isEndSubline, manualOffGradient);

    if (manualOffGradient) {
      lineColor = this.getLineColor(subLines.length > 0 ? 'transparent' : lineColor, isFirst, isLast, globalBgColor);
    }

    if (isFirst || isLast) {
      zIndex = 10;
    }

    return new Line(id, textColor, lineColor, bgGradientColor, xEndOffset, xEndOffset, text, from, to, subLines, zIndex);
  }

  private mapStatusColorToColor(colorStatus: ColorStatusEnum) {
    switch (colorStatus) {
      case ColorStatusEnum.BLUE:
        return '#1E88E5';
      case ColorStatusEnum.GREEN:
        return '#71AF74';
      case ColorStatusEnum.RED:
        return '#F58787';
      case ColorStatusEnum.AMBER:
        return '#FCC066';
      default:
        return '#CACED9';
    }
  }

  /**
   * Returns a gradient for the line from a given side and color.
   *
   * @param color - The base color for the line.
   * @param isFirst - Whether the line has a left point.
   * @param isLast - Whether the line has a right point.
   * @returns The line color.
   */
  private getLineColor(color: string, isFirst: boolean, isLast: boolean, globalBgColor: string) {
    if (isFirst) {
      return this.generateLineLinarGradent('fromStart', color, globalBgColor);
    } else if (isLast) {
      return this.generateLineLinarGradent('toEnd', color, globalBgColor);
    }

    return color;
  }

  /**
   * Returns a gradient for the linear space of the bottom line from a given side and color.
   *
   * @param color - The base color.
   * @param isFirst - Indicates if the gradient has a left point.
   * @param isLast - Indicates if the gradient has a right point.
   * @returns The gradient color.
   */
  private getGradientColor(
    color: string,
    globalBgColor: string,
    isFirst: boolean,
    isLast: boolean,
    isEndSubline: boolean,
    manualOffGradient: boolean
  ) {
    if (isFirst && !manualOffGradient) {
      return this.generateGradientLinarGradent('fromStart', color, globalBgColor);
    } else if (isLast && !manualOffGradient) {
      return this.generateGradientLinarGradent('toEnd', color, globalBgColor);
    }

    const opacity = isEndSubline ? 1 : 0.5;
    return `linear-gradient(to bottom, ${hexToRgbA(color, opacity)}, transparent)`;
  }

  /**
   * Generates a gradient for the line from a given side and color.
   *
   * @param gradientVector - The side of the gradient. Can be 'start' or 'end'.
   * @param color1 - The color value to be used in the gradient.
   * @param color2 - The color value to be used in the gradient.
   * @return The generated linear gradient CSS value.
   */
  private generateLineLinarGradent(gradientVector: 'toEnd' | 'fromStart', color1: string, color2: string) {
    const to = gradientVector === 'toEnd' ? 'right' : 'left';
    const result = `linear-gradient(to ${to}, ${color1}, ${color2})`;
    return result;
  }

  /**
   * Generates a gradient for the linear space of the bottom line from a given side and color.
   *
   * @param gradientVector - The side to apply the gradient ('start' or 'end').
   * @param color1 - The color to use in the gradient.
   * @param color2 - The color to use in the gradient.
   * @return The CSS linear gradient string.
   */
  private generateGradientLinarGradent(gradientVector: 'toEnd' | 'fromStart', color1: string, color2: string) {
    const to = gradientVector === 'toEnd' ? 'right' : 'left';
    return `linear-gradient(to bottom ${to}, ${hexToRgbA(color1, 0.5)}, ${color2} 60%)`;
  }
}
