import { Injectable } from '@angular/core';

import { hexToRgbA } from 'utils/common-helpers';

import { ColorStatusEnum } from '../interfaces/color-status.enum';
import { CreatePointInterface } from '../interfaces/create-point.interface';
import { PointTypeEnum } from '../interfaces/point-type.enum';
import { Point } from '../classes/point.class';
import { Mark } from '../classes/points/mark.class';
import { RhombPoint } from '../classes/points/rhomb.class';

@Injectable({
  providedIn: 'root',
})
export class PointFabricService {
  readonly basicZIndex = 50;

  /**
   * Creates a new Point object based on the provided parameters.
   *
   * @param id - The ID of the point.
   * @param colorStatus - The status of the color.
   * @param text - The text to be displayed on the point.
   * @param offsetX - The X offset of the point.
   * @param offsetY - The Y offset of the point.
   * @param pointType - The type of the point.
   * @param overlayPanel - The overlay panel to be displayed on the point.
   * @param basicZIndex - The basic z-index value.
   * @returns A new Point object.
   * @throws Error if the point type is invalid.
   */
  create<OverlayData = any>(
    {
      id,
      colorStatus,
      text,
      offsetX,
      offsetY,
      pointType,
      overlayPanel,
      overlayEventFn,
      overlayData,
      textLineHeight,
    }: CreatePointInterface<OverlayData>,
    basicZIndex = this.basicZIndex
  ): Point {
    let pointColor: string, textColor: string, textBgColor: string;

    // Set color values based on the color status
    switch (colorStatus) {
      case ColorStatusEnum.BLUE: {
        const color = '#1E88E5';
        pointColor = color;
        textColor = color;
        textBgColor = this.getColorWithAlpha(color);
        break;
      }
      case ColorStatusEnum.GREEN: {
        const color = '#71AF74';
        pointColor = color;
        textColor = color;
        textBgColor = this.getColorWithAlpha(color);
        break;
      }
      case ColorStatusEnum.RED: {
        const color = '#DA4A4A';
        pointColor = color;
        textColor = color;
        textBgColor = this.getColorWithAlpha(color);
        break;
      }
      case ColorStatusEnum.AMBER: {
        const color = '#FCC066';
        pointColor = color;
        textColor = color;
        textBgColor = this.getColorWithAlpha(color);
        break;
      }
      default: {
        const color = '#CACED9';
        pointColor = color;
        textColor = color;
        textBgColor = this.getColorWithAlpha(color);
        break;
      }
    }

    let point: Point;

    // Create a new point object based on the point type
    switch (pointType) {
      case PointTypeEnum.Rhomb: {
        point = new RhombPoint(
          id,
          false,
          pointColor,
          textBgColor,
          textColor,
          offsetX,
          offsetY,
          basicZIndex,
          text,
          overlayPanel,
          overlayEventFn,
          overlayData,
          textLineHeight
        );
        break;
      }
      case PointTypeEnum.FilledRhomb: {
        point = new RhombPoint(
          id,
          true,
          pointColor,
          textBgColor,
          textColor,
          offsetX,
          offsetY,
          basicZIndex,
          text,
          overlayPanel,
          overlayEventFn,
          overlayData,
          textLineHeight
        );
        break;
      }
      case PointTypeEnum.Mark: {
        point = new Mark(id, pointColor, offsetX, offsetY, basicZIndex - 1, overlayPanel, overlayEventFn, overlayData);
        break;
      }
      default: {
        throw new Error('Invalid point type');
      }
    }

    return point;
  }

  /**
   * Returns the color with the specified opacity.
   * @param color - The color to modify in HEX format.
   * @param opacity - The opacity value (0-1). Default is 10% (0.1).
   * @returns The modified color in the rgba format.
   */
  private getColorWithAlpha(color: string, opacity: number = 0.1): string {
    return hexToRgbA(color, opacity);
  }
}
