import {AfterViewInit, ChangeDetectorRef, Directive, Input, TemplateRef, ViewContainerRef} from '@angular/core';
import {Store} from '@ngxs/store';
import {Observable, of, tap} from 'rxjs';
import {switchMap, takeUntil} from 'rxjs/operators';

import {Unsubscriber} from 'utils/shared-services';
import {GlobalRulesSelectors, PermissionModelsNamesShared, ProjectRulesSelectors} from 'data-access/store';


type Action = 'create' | 'read' | 'update' | 'delete' | string;
type PermissionType = 'global' | 'project' | 'projectGroup';

/**
 * Директива для проверки прав на выполнение действий.
 *
 * Старайтесь использовать директиву только в контейнерах, фасадах и других "умных" сущностях.
 * В "тупых" компонентах используйте прокидывание параметра из стора.
 */
@Directive({
  selector: '[hasPermission]',
  providers: [Unsubscriber],
  standalone: true,
})
export class HasPermissionDirective implements AfterViewInit {
  private permissionType!: PermissionType;
  private action!: Action;
  private permissionModelName!: PermissionModelsNamesShared;

  private isHidden = true;

  constructor(
    private readonly store: Store,
    private readonly unsubscriber: Unsubscriber,
    private readonly templateRef: TemplateRef<any>,
    private readonly viewContainer: ViewContainerRef,
    private readonly changeDetectorRef: ChangeDetectorRef,
  ) {
  }

  @Input('hasPermission') set props(props: [Action, PermissionModelsNamesShared, PermissionType?]) {
    this.action = props[0];
    this.permissionModelName = props[1];
    this.permissionType = props[2] || 'global';
  }

  ngAfterViewInit(): void {
    this.getObservable()
      .pipe(
        takeUntil(this.unsubscriber.destroy$),
        tap((access) => this.updateView(access)),
      )
      .subscribe();
  }

  private updateView(viewStatus: boolean): void {
    if (viewStatus) {
      if (this.isHidden) {
        this.viewContainer.createEmbeddedView(this.templateRef);
        this.isHidden = false;
      }
    } else {
      this.isHidden = true;
      this.viewContainer.clear();
    }

    this.changeDetectorRef.detectChanges();
  }

  private getObservable(
    modelName: PermissionModelsNamesShared = this.permissionModelName,
    action: Action = this.action,
  ): Observable<boolean> {
    let selectAction: any;

    switch (action) {
      case 'create':
        selectAction = this.getSelectorsClass().canCreate(modelName);
        break;
      case 'read':
        selectAction = this.getSelectorsClass().canRead(modelName);
        break;
      case 'update':
        selectAction = this.getSelectorsClass().canUpdate(modelName);
        break;
      case 'delete':
        selectAction = this.getSelectorsClass().canDelete(modelName);
        break;
      default:
        selectAction = this.getSelectorsClass().customRule(action, modelName);
    }

    return this.store.select<boolean | any>(selectAction)
      .pipe(
        switchMap((customRuleValue) => {
          if (typeof customRuleValue !== 'boolean') {
            console.warn(`Cannot find custom rule (${action}_${modelName}) in permissions store.`);
          }
          return of(customRuleValue);
        }),
      );
  }

  private getSelectorsClass(permissionType: PermissionType = this.permissionType) {
    switch (permissionType) {
      case 'global':
        return GlobalRulesSelectors;
      case 'project':
        return ProjectRulesSelectors;
      case 'projectGroup':
        throw new Error('Not implemented');
      default:
        throw new Error(`Unknown permission type: ${permissionType}`);
    }
  }
}
