import {finalize, forkJoin, Observable, of} from 'rxjs';

export abstract class BaseAdapter<Model, DTO> {
  errors: string[] = [];

  /**
   * Convert DTO to model.
   *
   * @example
   *  // create adapters observables
   *  const employees$: Observable<UserBaseModel[]> = this.createModelArrObs(dto.rel_employees, this.userBaseAdapterService);
   *
   *  // if has base adapter (else can use forkJoin at start stream)
   *  return this.departmentBaseAdapterService.convert(dto)
   *    .pipe(
   *      // if has base adapter (else can use forkJoin at start stream)
   *      withLatestFrom(employees$),
   *      switchMap(([baseModel, employees]) => {
   *        const model: DepartmentModel = {
   *          ...baseModel,
   *          parentDepartment: dto.parent_department ?? undefined,
   *          subDepartments: dto.subdepartments ?? [],
   *
   *         // relation ids
   *        employeesIds: dto.employees ?? [],
   *
   *         // relation models
   *         employees,
   *       };
   *
   *       return this.createConvertObservable(model);
   *     }),
   *   );
   */
  abstract convert(dto: DTO): Observable<Model>;

  abstract reverse(model: Model): DTO;

  // eslint-disable-next-line @typescript-eslint/member-ordering
  protected convertError(error: any, errors: string[] = this.errors): any {
    console.error(error);
    errors.push(error);
    // TODO message
    return undefined;
  }

  /**
   * Create convert observable.
   *
   * ## Interface checking
   * Function will throw error (use catchError) if use validator.
   *
   * ## Validation
   * Use "throw new Error('Error message')" to throw error.
   *
   * ### Validation example and recommendations
   * 0. Use ?? operator.
   * 1. if value can't be undefined, use validator. Example: prop: dto.prop ?? this.convertError('Error message')
   * 2. if value can be undefined, don't use validator. Example: dto.prop ?? undefined
   * 3. if value has array type use empty array as default. Example: dto.prop ?? []
   * 4. if value has boolean type use default value. Example: dto.prop ?? false (or true if need default "true")
   * 5. if prop is "order" recommend use 0 (zero) as default.
   *
   * @example
   * const model: DepartmentModel = {
   *   ...baseModel,
   *   parentDepartment: dto.parent_department ?? undefined,
   *   subDepartments: dto.subdepartments ?? [],
   *
   *   // relation ids
   *   employeesIds: dto.employees ?? [],
   *
   *   // relation models
   *   employees,
   * };
   *
   * return this.createConvertObservable(model);
   */
  // eslint-disable-next-line @typescript-eslint/member-ordering
  protected createConvertObservable(model: Model): Observable<Model> {
    return new Observable<Model>(subscriber => {
      const errors: string[] = this.errors;

      if (errors.length > 0) {
        subscriber.error(errors);
      } else {
        subscriber.next(model);
        subscriber.complete();
      }
    });
  }

  // eslint-disable-next-line @typescript-eslint/member-ordering
  protected createModelArrObs<CreateModel, CreateDTO>(
    arr: CreateDTO[] | undefined, adapter: BaseAdapter<CreateModel, CreateDTO>,
  ): Observable<CreateModel[]> {
    return arr && arr.length > 0
      ? forkJoin(arr.map((item) => adapter.convert(item)))
      : of([]);
  }

  // eslint-disable-next-line @typescript-eslint/member-ordering
  protected createModelObs<CreateModel, CreateDTO>(
    model: CreateDTO | undefined, adapter: BaseAdapter<CreateModel, CreateDTO>,
  ) {
    return model
      ? adapter.convert(model)
        .pipe(
          finalize(() => {
            this.errors = [];
          }),
        )
      : of(undefined)
      ;
  }
}
